function [z,V,C,I,Pg,E,H,FL,BT] = adcpupk(fname,NrecSkip,NrecOut,NfileOut)
%
% [z, V, C, I, Pg, E, H, FL, BT] = adcpupk(fname, NrecSkip, NrecOut, NfileOut)
%
% Unpacks the WaveADCP binary data file specified by the string 'fname',
% and returns structures containing the four velocities V (mm/s),
% correlations C (count), intensities I (count), and percent good Pg (count)
% at vertical distances from the ADCP given by z (cm).  If a data type is
% not contained in the data file, we return an empty matrix for that variable.
%
% Other quantities, such as ensemble number, yearday, pressure, attitude,
% heading, ..., etc. are returned in the "environmental" structure E.
% Yearday is given in the sense of a timeline (i.e. E.Yday = 0.5 is
% 1200 on 1 January).  Impossible yeardays are returned as NaNs.
% The structures H and FL contain the RecordHeader and FixedLeader
% information (these are constant in time for WaveADCP ensembles),
% and the structure BT contains the bottom-tracking information
%
% Short and long records are deleted.  Records with bad checksums
% are retained, and their indices returned in H.iBadCksum.  Data that
% have been flagged BAD by the ADCP have the value -32768.
%
% The optional arguments NrecSkip (default is 0), NrecOut (default is inf)
% and NfileOut control how the file is broken up into output files.
% If no output arguments are specified, the output variables are written
%   to NfileOut files each containing NrecOut records.  The files have the
%   names 'fname'_001.mat, 'fname'_002.mat, etc. (if NrecOut and NfileOut
%   are not specified, the entire file is unpacked to a single '.mat' file).
% If output arguments are specified, NfileOut is disregarded, and NrecOut
%   records are returned.  In this case, if NrecOut is not specified,
%   the entire input file is unpacked into the output variables.

% Requires: Find1stHdr, ParseHdr, UpkRecs, ParseVL, ParseFL,
%           ParseBT, twoscvt, date2ydN

% Copyright 2/21/02 E. Terray  Woods Hole Oceanographic Institution

% The record of upgrades and bug fixes is appended at the end of this file


                                       % parse input arguments

if( nargin < 2 ),  NrecSkip = 0;  end       % skip 0 records from BOF

if( nargout ~= 0 )                          % if output args requested
   NfileOut = 0;                            %  don't create output files
   if( nargin < 3 )                         %  unpack NrecOut records
      NrecOut = inf;
   end
end

if( nargout == 0 )                          % if no output arguments
   if( nargin < 4 )                         %  if NfileOut not specified
      NfileOut = inf;                       %    create files until done
   end
   if( nargin < 3 )                         %  if NrecOut & NfileOut not given
      NrecOut = inf;                        %    unpack the entire input file
      NfileOut = 1;                         %    and write to a single output
   end
end

                                       % get record structure from 1st good hdr

RecDl = [127,127];                          % RDI 2-byte rec delimiter [7F,7F]
hdr   = Find1stHdr(fname,RecDl);            % find the 1st valid record header
Hdr   = ParseHdr( hdr, 5 );                 % augment to a 5 byte rec delimiter

RDL  = Hdr.RecDel;                          % recs now defined by matching RDL
RecL = Hdr.RecLen + 2;                      % record length (including CkSum)

NbRead = RecL * NrecOut;                    % # bytes to read
NbSkip = RecL * NrecSkip;                   % # bytes to skip from BOF

fp = fopen(fname,'rb');                 % open input as binary, read-only

if( NbSkip > 0 )                        % skip records if requested
   status = fseek(fp,NbSkip,'bof');
   if( status == -1 )
      fclose( fp );
      error('could not skip %d records in %s\n',NrecSkip,fname);
   else
      fprintf('\n');
      fprintf('Skipping %d records',NrecSkip);
   end
end

                                       % read & process input file
nfiles = 0;
while( ~feof(fp) )

   [B,M] = fread(fp,NbRead,'uchar');        % get NrecOut records

   if( M < RecL ),  break;  end             % exit if less than 1 rec returned
   
   [ z, V, C, I, Pg, E, H, FL, BT ] = UpkRecs( B, fname, Hdr );

   if( NfileOut == 0 )                      % if no output files, exit
      break;
   else                                     % else, write output files
      nfiles = nfiles + 1;                  %  increment file count
      [path,oname,ext] = fileparts(fname);  %  get root name of output file

      if( NfileOut > 1 )
         oname = [oname, '_', sprintf('%03d', nfiles)];
      end

      save(oname,'fname','z','V','C','I','Pg','E','H','FL','BT');

      fprintf('Saving data to %s.mat \n', oname);

      if( nfiles == NfileOut )
         break;
      end
   end
end

fclose( fp );

fprintf('\n');
if( nfiles > 1 )
   fprintf('Unpacked %s to %d .mat files\n\n',fname,nfiles);
elseif( nargout == 0 )
   fprintf('Unpacked %s to a single .mat file\n\n',fname);
end



%_______________________________ UpkRecs ____________________________

function [ z, V, C, I, Pg, E, H, FL, BT ] = UpkRecs( B, fname, H )
%
% [ z, V, C, I, Pg, E, H, FL, BT ] = UpkRecs( B, fname, H )
%
% Unpacks the WaveADCP binary data in the vector B read from the file
% specified by the string 'fname'.  H is the structure containing the
% RDI file header information.  Structures are returned containing
% the four along-beam components of velocity V (mm/s), correlation
% C (count), intensity I (count), and percent good Pg (count) at
% vertical distances above (below) the transducer given by z (cm).
% If a data type is not contained in the data file, we return an
% empty matrix for that variable.
%
% Other variables, such as ensemble number, yearday, pressure, attitude,
% heading, ..., etc. are returned in the VariableLeader structure E.
% Yearday is given in the sense of a timeline (i.e. E.Yday = 0.5 is
% 1200 on 1 January).  Yeardays corresponding to dates that are out of
% range are returned as NaNs.
%
% The structures H and FL contain the RecordHeader and FixedLeader
% (these are constant in time for WaveADCP ensembles).
%
% The structure BT contains the bottom-tracking information
%
% Note: Short or long records are deleted.  Records with bad checksums
% are retained, and their indices returned in H.iBadCksum.  Data that
% have been flagged BAD by the ADCP have the value -32768.
%
% If no output arguments are specified, the output variables are written
% to a .mat file having the same name as 'fname'.
%
% Note: The raw ADCP binary data files are imported in a single read.
% Sufficient memory must be available to do this.

% Requires: ParseVL, ParseFL, ParseBT, twoscvt, date2ydN


                                       % read the binary ADCP data

RDL  = H.RecDel;                            % records defined by matching RDL
RecL = H.RecLen + 2;                        % record length (including CkSum)


                                       % find all records bracketed by RDL

N = length( RDL ) - 1;                      % find pointers p to valid records 
p = find( B(1:end-N) == RDL(1) );           %   find ALL bytes matching RDL(1)
for n = 1:N                                 %   loop over remaining RDL bytes
   k = find( B(p+n) ~= RDL(n+1) );          %     if next byte ~= RDL byte
   p(k) = [];                               %     excise that record from p
end                                         %   p points to start of records
                                            %     bracketed by RDL
dp = diff( [p(:);length(B)+1] );            %   lengths of these records

k = find( dp ~= RecL );                     % find recs having the wrong length
Nbadsize = length( k );
if( Nbadsize )                              % if rec is short/long - delete it
   for( n = Nbadsize:-1:1 )                 %   start at the end of the B array
      kn = k(n);
      pn = p(kn);                           %   ptr to start of n_th short rec
      q = (0:dp(kn)-1) + pn;                %   indices all elements within
      B(q) = [];                            %   delete the record
      p(kn) = [];                           %     and expunge the pointer
   end
end
H.NBadSizeRecs = Nbadsize;                  % save # short/long recs in Header

Nrec = length( p ) - 1;                     % # records of correct length

fprintf('\n');
fprintf('Unpacking %d records of length %d in file %s \n',Nrec,RecL,fname);
if( Nbadsize )
   fprintf('... deleted %d records having the wrong size \n',Nbadsize);
end

                                       % extract good data and reshape
                                       %   -->  B(rec_no,[Hdr,Fl,Vl,V,C,I])

B = reshape( B(p(1)-1+(1:Nrec*RecL)), RecL, Nrec )';


                                       % find records having bad checksums

ck = sum( B(:,1:end-2)' )';                 % computed checksum
ck = rem( ck, 65536 );                      %   take wrapping into account
CK = B(:,end)*256 + B(:,end-1);             % true checksum (i.e. from ADCP)
k = find( ck - CK );                        % find records with bad checksums

H.NbadCksum = length( k );                  % save # bad and their indices
H.iBadCksum = k;                            %  to the header structure

if( H.NbadCksum )
   fprintf('... found %d records with bad checksums\n',H.NbadCksum);
end


k  = H.DataPtr(1) + [1:H.DataLen(1)];       % parse the FixedLeader

FL = ParseFL( B(1,k) );

k = H.DataPtr(2) + [1:H.DataLen(2)];        % parse the VariableLeader
E = ParseVL( B(:,k) );

z = (0:FL.NumRangeBins-1)'*FL.RangeBinLen;  % vertical distances from
z = z + FL.CenterRangeBin1;                 %  transducer to range bins (cm)

V  = [];  C  = [];  I = [];                 % initialize data to "missing"
Pg = [];  BT = [];

for n = 3:H.DataNum                         % loop over data V, C, I, Pg, BT

   dptr = H.DataPtr(n);                               % end of n-1 data item

   dtype = 256*B(1,dptr+1) + B(1,dptr+2);             % id of n_th data type

   switch dtype                                  % consider each data type

      case{ 1 }                                       % Velocity (2 bytes)
         lsb = dptr + (3:8:H.DataLen(n));                  % pointers to
         msb = lsb + 1;                                    %   (lsb,msb)

         fprintf('... extracting velocity \n');

         V.b1 = twoscvt( B(:,msb  )*256 + B(:,lsb  ) );    % beam 1 vel (mm/s)
         V.b2 = twoscvt( B(:,msb+2)*256 + B(:,lsb+2) );    %  "   2  "    "
         V.b3 = twoscvt( B(:,msb+4)*256 + B(:,lsb+4) );    %  "   3  "    "
         V.b4 = twoscvt( B(:,msb+6)*256 + B(:,lsb+6) );    %  "   4  "    "

         V.id = B(:,dptr+2);
         if( all(V.id == dtype) ),  V.id = dtype;  end

         V.units = 'V.bn(t,z) = velocity along the n_th beam (mm/s)';


      case{ 2 }                                       % Correlation (1 byte)
         lsb = dptr + (3:4:H.DataLen(n));

         fprintf('... extracting correlation \n');

         C.b1 = B(:,lsb  );                                % beam 1 correl (%)
         C.b2 = B(:,lsb+1);                                %  "   2    "
         C.b3 = B(:,lsb+2);                                %  "   3    "
         C.b4 = B(:,lsb+3);                                %  "   4    "

         C.id = B(:,dptr+2);
         if( all(C.id == dtype) ),  C.id = dtype;  end

         C.units = 'C.bn(t,z) = correlation along the n_th beam (% = count*100/255)';

      case{ 3 }                                       % Intensity (1 byte)
         lsb = dptr + (3:4:H.DataLen(n));

         fprintf('... extracting intensity \n');

         I.b1 = B(:,lsb  );                                % beam 1 intensity
         I.b2 = B(:,lsb+1);                                %  "   2     "
         I.b3 = B(:,lsb+2);                                %  "   3     "
         I.b4 = B(:,lsb+3);                                %  "   4     "

         I.id = B(:,dptr+2);
         if( all(I.id == dtype) ),  I.id = dtype;  end

         I.units = 'I.bn(t,z) = intensity (counts) along the n_th beam (~0.45dB/count)';

      case{ 4 }                                       % Percent Good (1 byte)
         lsb = dptr + (3:4:H.DataLen(n));

         fprintf('... extracting percent good \n');

         Pg.b1 = B(:,lsb  );                               % beam 1 %-good
         Pg.b2 = B(:,lsb+1);                               %  "   2   "
         Pg.b3 = B(:,lsb+2);                               %  "   3   "
         Pg.b4 = B(:,lsb+3);                               %  "   4   "

         Pg.id = B(:,dptr+2);
         if( all(Pg.id == dtype) ),  Pg.id = dtype;  end

         Pg.units = 'Pg.bn(t,z) = Percent Good along the n_th beam (% = count)';

      case{ 6 }                                       % Bottom-Track (79 bytes)
         lsb = dptr + (1:H.DataLen(n));

         fprintf('... extracting bottom-track \n');

         BT = ParseBT( B(:,lsb) );
   end
end



%_______________________________ Find1stHdr ____________________________

function Hdr = Find1stHdr( fname, RDL )
%
% Hdr = Find1stHdr( fname, RDL )
%
% Returns the first valid header in the byte array of data, B.
% A valid header is defined as the run of bytes bracketed by
% two record delimiters, and having the expected length.  RDL
% is a vector of bytes specifying the record delimiter.  The expected
% header length is decoded from the bytes following the leading RDL.


Nbytes = 5000;                         % # bytes to scan for 1st header

if( length(RDL) < 2 )
   error('record delimiter must be at least 2 bytes');
end
                                       % get the 1st header
fp = fopen(fname,'rb');                     % open as binary, read-only
[B,M] = fread(fp,Nbytes,'uchar');           % read the data as a byte stream B
fclose(fp);                                 % and close the data file


                                       % find pointers p to start of
N = length( RDL ) - 1;                      %   all runs matching RDL
p = find( B(1:end-N) == RDL(1) );           % find all bytes matching 1st RDL
for n = 1:N                                 % loop over remaining RDL bytes
   k = find( B(p+n) ~= RDL(n+1) );          %   if next rec byte ~= RDL byte
   p(k) = [];                               %   excise start byte from p
end

Lrec = diff(p);                        % length of each possible record


for n = 1:length(p)-1                  % find a valid header
   pn = p(n);                               % loop over possible records
   RecLn = B(pn+3)*256 + B(pn+2) + 2;       %   decode the expected rec length
   if( Lrec(n) == RecLn )                   %   if matches the computed length
      HdrLn = B(pn+7)*256 + B(pn+6);        %     find the header length
      Hdr = B(pn+[0:HdrLn-1]);              %     grab the header
      return;                               %     and return
   end
end

error('could not find a valid header');


%_______________________________ ParseHdr ____________________________

function H = ParseHdr( Hdr, Nrdl )
%
% H = ParseHdr( Hdr, Nrdl )
%
% Parses the RDI record header contained in the byte array Hdr into
% the structure H.
%
% Nrdl is the number of bytes at the start of the record to use as
% the record delimiter (this is returned in H.RecDel).  This parameter
% is optional - the default is a 5 byte delimiter.


if( nargin < 2 ),  Nrdl = 5;  end           % check for default argument

H.RecDel = Hdr(1:Nrdl)';                    % use first Nrdl bytes to pattern
                                            %  match for valid records

H.RecLen = Hdr(4)*256 + Hdr(3);             % record length (bytes) -
                                            %  excluding 2-byte checksum

H.DataNum = Hdr(6);                         % # data types (Fixed- and
                                            %  Variable-Leaders + various
                                            %  of V, C, I, %_Good)

ptr = reshape(Hdr(7:end),2,H.DataNum);      % absolute ending byte addresses
                                            %  of data types [lsb;msb]

H.DataPtr = ptr(2,:)*256 + ptr(1,:);        % abs end byte addresses (decimal)

H.DataLen = diff([H.DataPtr,H.RecLen-2]);   % byte length of data types
                                            %  FixedL, VariableL, V, C, I

H.HdrLen = H.DataPtr(1);                    % header length (bytes)



%_______________________________ ParseVL ____________________________

function E = ParseVL( VL )
%
% E = ParseVL( VL )
%
% Returns the environmental variables data structure E by parsing
% the variable leader VL.

% Requires twoscvt.m,  date2ydN.m


E.Ens  = VL(:,4)*256 + VL(:,3);
de   = diff( E.Ens );
de( find(de == -65535) ) = 1;
E.Ens = cumsum([E.Ens(1);de]);

% incr = VL(:,12);

year = VL(1,5);                                       % year
if( year < 80 )
   E.Year = 2000 + year;
else
   E.Year = 1900 + year;
end

E.RTC = VL(:,6:11);                                   % [mm dd hh mi ss ss/100]

VL(:, 9) = VL(:,9) + VL(:,10)/60 + VL(:,11)/6000;     % fractional minutes
E.Yday = date2ydN( VL(:,5:9) );                       % yearday

dt = mean(diff(E.Yday)) * 24*3600;                    % sample interval (sec)
E.dt = round( dt * 100 ) / 100;                       %  round to 0.01 s

E.Cs = VL(:,16)*256 + VL(:,15);                       % sound speed (m/s)
if( all(E.Cs == E.Cs(1)) )
   E.Cs = E.Cs(1);
end

E.Xdepth = ( VL(:,18)*256 + VL(:,17) ) / 10;          % transducer depth (m)
if( all(E.Xdepth == E.Xdepth(1)) )
   E.Xdepth = E.Xdepth(1);
end

E.Hdg   = ( VL(:,20)*256 + VL(:,19) ) / 100;          % heading (deg_T)
E.Pitch = twoscvt( VL(:,22)*256 + VL(:,21) ) / 100;   % pitch   (deg)
E.Roll  = twoscvt( VL(:,24)*256 + VL(:,23) ) / 100;   % roll    (deg)

if( any(VL(:,32) ~= VL(1,32)) )                       % std_dev heading (deg)
   E.StdHdg = VL(:,32);
else
   E.StdHdg = VL(1,32);
end

if( any(VL(:,33) ~= VL(1,33)) )                       % std_dev pitch (deg)
   E.StdPitch = VL(:,33) / 10;
else
   E.StdPitch = VL(1,33) / 10;
end

if( any(VL(:,34) ~= VL(1,34)) )                       % std_dev roll (deg)
   E.StdRoll = VL(:,34) / 10;
else
   E.StdRoll = VL(1,34) / 10;
end

E.Salinity  = VL(:,26)*256 + VL(:,25);                % salinity (ppt)
if( all(E.Salinity == E.Salinity(1)) )
   E.Salinity = E.Salinity(1);
end

E.degC = twoscvt( VL(:,28)*256 + VL(:,27) ) / 100;    % temperature (degC)
if( all(E.degC == E.degC(1)) )
   E.degC = E.degC(1);
end

[N,M] = size( VL );
if( M < 56 )                                          % if no pressure sensor
   E.P = [];                                          %   return
   E.StdP = [];                                       %   nulls
else                                                  % else
   E.P = ( VL(:,52)*245^3 + VL(:,51)*256^2 + ...      %   return pressure (Pa)
           VL(:,50)*256   + VL(:,49)     ) * 10;      %   and
   if( all(E.P == E.P(1)) )
      E.P = E.P(1);
   end
                                                      %   std pressure (Pa)
   E.StdP = ( VL(:,56)*245^3 + VL(:,55)*256^2 + ...
              VL(:,54)*256   + VL(:,53)     ) * 10;
   if( all(E.StdP == E.StdP(1)) )
      E.StdP = E.StdP(1);
   end
end

%_______________________________ ParseFL ____________________________

function FL = ParseFL( Fl )
%
% FL = ParseFL( Fl );
%
% Parses the FixedLeader, Fl, and returns selected fields in the
% structure FL.  The original binary FixedLeader is returned for
% future use in FL.BinaryFixedLeader.


FL.CPU_VerRev         = [Fl(3), Fl(4)];          % [Version#, Revision#]

switch( rem(Fl(5),8) )                           % bits 0 - 2
   case 0,  FL.FrequencyKHz =   75;
   case 1,  FL.FrequencyKHz =  150;
   case 2,  FL.FrequencyKHz =  300;
   case 3,  FL.FrequencyKHz =  600;
   case 4,  FL.FrequencyKHz = 1200;
   case 5,  FL.FrequencyKHz = 2400;
end

switch( rem(fix(Fl(5)/8),2) )                    % bit 3
   case 0,  FL.BeamGeometry = 'concave';
   case 1,  FL.BeamGeometry = 'convex';
end

switch( fix(Fl(5)/128) )                         % bit 7
   case 0,  FL.BeamDirection = 'downward';
   case 1,  FL.BeamDirection = 'upward';
end

switch( rem(Fl(6),4) )                           % bits 0 - 1
   case 0,    FL.BeamAngle = 15;
   case 1,    FL.BeamAngle = 20;
   case 2,    FL.BeamAngle = 30;
   otherwise, FL.BeamAngle = '?';
end

FL.NumBeams           = Fl(9);                   % # beams used to calc vel

FL.NumRangeBins       = Fl(10);                  % WN (1-128)

FL.PingsPerEns        = Fl(12)*256 + Fl(11);     % WP (1-16384)

FL.RangeBinLen        = Fl(14)*256 + Fl(13);     % WS  [cm]  (1-6400)

FL.BlankAfterXmit     = Fl(16)*256 + Fl(15);     % WF  [cm]  (1-9999)

FL.LowCorrThresh      = Fl(18);                  % WC  [counts] (0-255)

FL.NumCodeReps        = Fl(19);                  % # code reps (0-255)

FL.MinPercentGood     = Fl(20);                  % WG  [%]  (0-100)

FL.ErrorVelThresh     = Fl(22)*256 + Fl(21);     % WE  [mm/s]  (0-5000)

FL.NomSecBetweenPings = Fl(23)*60 + Fl(24) + Fl(25)/100;   % sec (nominal)

FL.CoordTransform     = dec2base(Fl(26),2,8);    % EX  [bits] coord transform

FL.HeadingAlignment   = Fl(28)*256 + Fl(27);     % EA [deg/100] (-179.99-180)
FL.HeadingAlignment   = twoscvt(FL.HeadingAlignment);

FL.HeadingBias        = Fl(30)*256 + Fl(29);     % EB [deg/100] (-179.99-180)
FL.HeadingBias        = twoscvt(FL.HeadingBias);

FL.SensorSource       = dec2base(Fl(31),2,8);    % EZ  [bits] sensor source

FL.SensorsAvailable   = dec2base(Fl(32),2,8);    % sensors availabile (8-bits)

FL.CenterRangeBin1    = Fl(34)*256 + Fl(33);     % [cm]  (0-65535)

FL.XmitPulseLength    = Fl(36)*256 + Fl(35);     % WT  [cm]  (0-65535)

FL.WaterTrackBins     = Fl(37:38);               % WL, WP [start,end]_bin#

FL.FalseTargetThresh  = Fl(39);                  % WA  [counts]  (0-255)

FL.CodeRepetitionLag  = Fl(42)*256 + Fl(41);     % [cm]  (0-65535)

FL.CPU_SerNum         = sprintf('%02X ',Fl(43:50)');   % bytes

L = length( Fl );
if( L >= 52 )
   FL.SystemBandWidth = Fl(52)*256 + Fl(51);     % WB  (0,1)
end

if( L >= 53 )
   FL.SystemPower     = Fl(53);                  % [counts]  (0-255)
end

FL.Units = 'Distances [cm]; Velocities [mm/s]; Angles [deg/100]';


FL.BinaryFixedLeader  = Fl;                      % raw binary FixedLeader



%_______________________________ ParseBT ____________________________

function Bt = ParseBT( BT )
%
% Bt = Parse( BT )
%
% Parses the Bottom-Track data (id 6) contained in the Nt x 81
% element array BT, where Nt is the number of ensembles.  The
% data is returned in the structure Bt.

% E. Terray  20 May, 2002

% NOTE: Fields 13:16 and 63:66 are reserved by RDI for future use


Bt.id                 = BT(:,1)*256 + BT(:,2);     % B-T data type "id"

                                                   % B-T setup info
Bt.Setup.PingPerEns   = BT(:,4)*256 + BT(:,3);
Bt.Setup.ReacqDelay   = BT(:,6)*256 + BT(:,5);
Bt.Setup.MinCorr      = BT(:,7);
Bt.Setup.MinAmpl      = BT(:,8);
Bt.Setup.MinPg        = BT(:,9);
Bt.Setup.Mode         = BT(:,10);
Bt.Setup.RcvrGain     = BT(:,77);                    % rcvr gain adjust (dB)
Bt.Setup.MaxTrkDepth  = BT(:,72)*256 + BT(:,71);     % max tracking depth (dm)
Bt.Setup.MaxTrkDepth  = Bt.Setup.MaxTrkDepth * 10;   %   convert --> cm
Bt.Setup.MaxErrVel    = BT(:,12)*256 + BT(:,11);
Bt.Setup.MaxErrVel    = twoscvt(Bt.Setup.MaxErrVel);

                                                   % Beams 1-4
Bt.R        = BT(:,18:2:24)*256 + BT(:,17:2:23);     % distance to bdry (cm)
Bt.V        = BT(:,26:2:32)*256 + BT(:,25:2:31);     % velocity (mm/s)
Bt.V        = twoscvt( Bt.V );                       %   convert to signed vels
Bt.C        = BT(:,33:36);                           % correl (100% / 255cnt)
Bt.I        = BT(:,73:76);                           % intensity (0.45 dB/cnt)
Bt.Pg       = BT(:,41:44);                           % percent good (%)
Bt.MatchAmp = BT(:,37:40);                           % matching amplitude

MsbRange    = BT(:,78:81);                         % 1 cnt = 65536 cm

Bt.WaterRefLayer.MinSize  = ( BT(:,46)*256 + BT(:,45) ) * 10;   % cm
Bt.WaterRefLayer.NearBdry = ( BT(:,48)*256 + BT(:,47) ) * 10;   % cm
Bt.WaterRefLayer.FarBdry  = ( BT(:,50)*256 + BT(:,49) ) * 10;   % cm
Bt.WaterRefLayer.C        = BT(:,59:62);                        % counts
Bt.WaterRefLayer.I        = BT(:,63:66);                        % counts
Bt.WaterRefLayer.Pg       = BT(:,67:70);                        % percent
Bt.WaterRefLayer.V        = BT(:,52:2:58)*256 + BT(:,51:2:57);  % mm/s
Bt.WaterRefLayer.V        = twoscvt( Bt.WaterRefLayer.V );      % cvt to signed

Bt.Units = 'All distances are in cm, velocities in mm/s, intensities in counts';

if( all( Bt.id == Bt.id(1) ) )
   Bt.id = Bt.id(1);
else
   fprintf('    >> not all Bottom-Track id are identical \n');
end

if( all( Bt.Setup.PingPerEns == Bt.Setup.PingPerEns(1) ) )
   Bt.Setup.PingPerEns = Bt.Setup.PingPerEns(1);
else
   fprintf('    >> not all Bottom-Track PingPerEns are identical \n');
end

if( all( Bt.Setup.ReacqDelay == Bt.Setup.ReacqDelay(1) ) )
   Bt.Setup.ReacqDelay = Bt.Setup.ReacqDelay(1);
else
   fprintf('    >> not all Bottom-Track ReacqDelay are identical \n');
end

if( all( Bt.Setup.MinCorr == Bt.Setup.MinCorr(1) ) )
   Bt.Setup.MinCorr = Bt.Setup.MinCorr(1);
else
   fprintf('    >> not all Bottom-Track MinCorrelation are identical \n');
end

if( all( Bt.Setup.MinAmpl == Bt.Setup.MinAmpl(1) ) )
   Bt.Setup.MinAmpl = Bt.Setup.MinAmpl(1);
else
   fprintf('    >> not all Bottom-Track MinAmplitude are identical \n');
end

if( all( Bt.Setup.MinPg == Bt.Setup.MinPg(1) ) )
   Bt.Setup.MinPg = Bt.Setup.MinPg(1);
else
   fprintf('    >> not all Bottom-Track MinPercentGood are identical \n');
end

if( all( Bt.Setup.Mode == Bt.Setup.Mode(1) ) )
   Bt.Setup.Mode = Bt.Setup.Mode(1);
else
   fprintf('    >> not all Bottom-Track Mode are identical \n');
end

if( all( Bt.Setup.MaxErrVel == Bt.Setup.MaxErrVel(1) ) )
   Bt.Setup.MaxErrVel = Bt.Setup.MaxErrVel(1);
else
   fprintf('    >> not all Bottom-Track MaxErrVel are identical \n');
end

if( all( Bt.WaterRefLayer.MinSize == Bt.WaterRefLayer.MinSize(1) ) )
   Bt.WaterRefLayer.MinSize = Bt.WaterRefLayer.MinSize(1);
else
   fprintf('    >> not all Bottom-Track WaterRefLayer MinSize are identical \n');
end

if( all( Bt.WaterRefLayer.NearBdry == Bt.WaterRefLayer.NearBdry(1) ) )
   Bt.WaterRefLayer.NearBdry = Bt.WaterRefLayer.NearBdry(1);
else
   fprintf('    >> not all Bottom-Track  WaterRefLayer NearBdry are identical \n');
end

if( all( Bt.WaterRefLayer.FarBdry == Bt.WaterRefLayer.FarBdry(1) ) )
   Bt.WaterRefLayer.FarBdry = Bt.WaterRefLayer.FarBdry(1);
else
   fprintf('    >> not all Bottom-Track WaterRefLayer FarBdry are identical \n');
end

if( all( Bt.Setup.MaxTrkDepth == Bt.Setup.MaxTrkDepth(1) ) )
   Bt.Setup.MaxTrkDepth = Bt.Setup.MaxTrkDepth(1);
else
   fprintf('    >> not all Bottom-Track MaxTrkDepth are identical \n');
end

if( all( Bt.Setup.RcvrGain == Bt.Setup.RcvrGain(1) ) )
   Bt.Setup.RcvrGain = Bt.Setup.RcvrGain(1);
else
   fprintf('    >> not all Bottom-Track ShallowWaterGain are identical \n');
end

if( all( MsbRange(:) == MsbRange(1,1) ) )
   MsbRange = MsbRange(1,:);
   Bt.R = Bt.R + MsbRange(1,1)*65536;              % distance to bdry (cm)
else
   Bt.R = Bt.R + MsbRange(:,ones(1,4))*65536;
end

if( all( Bt.I(:) == Bt.I(1,1) ) )
   Bt.I = Bt.I(1,:);
end

if( all( Bt.Pg(:) == Bt.Pg(1,1) ) )
   Bt.Pg = Bt.Pg(1,:);
end

if( all( Bt.WaterRefLayer.V(:) == Bt.WaterRefLayer.V(1,1) ) )
   Bt.WaterRefLayer.V = Bt.WaterRefLayer.V(1,:);
end

if( all( Bt.WaterRefLayer.C(:) == Bt.WaterRefLayer.C(1,1) ) )
   Bt.WaterRefLayer.C = Bt.WaterRefLayer.C(1,:);
end

if( all( Bt.WaterRefLayer.I(:) == Bt.WaterRefLayer.I(1,1) ) )
   Bt.WaterRefLayer.I = Bt.WaterRefLayer.I(1,:);
end
if( all( Bt.WaterRefLayer.Pg(:) == Bt.WaterRefLayer.Pg(1,1) ) )
   Bt.WaterRefLayer.Pg = Bt.WaterRefLayer.Pg(1,:);
end


%_______________________________ twoscvt ____________________________

function x = twoscvt( x, N )
%
% x = twoscvt( x, N )
%
% Converts N-byte integers in 2's complement form to numbers in the
% range [-(256^N)/2 : 256^N)/2-1].  N must be even, but is optional
% (the default value is 2 -i.e. 16-bit integers).
%
% For example:  [FFFF 8000 7FFF 0000] ---> [ -1  -32768  32767  0 ]


if( nargin < 2 ),  N = 2;  end         % use default if N is not specified

if( rem(N,2) )
   error('the number of bytes in the integers must be even')
end

M = 256^N;                             % FF....FF

i = find( x > M/2-1 );                 % x > 32767 for N = 2

x(i) = x(i) - M;


%_______________________________ date2ydN ____________________________

function Yd = date2ydN( yy, mo, dd, hh, mi )
%
% Yd = date2ydN( yy, mo, dd, hh, mi )  or  Yd = date2ydN( [yy,mo,dd,hh,mi] )
%
% Returns the fractional yearday given the month, day, year and
% (optionally) the hour and (fractional) minutes.  The date & time
% information may be passed as separate vectors, or as a single array.
% If separate arguments, and all dates are in a single year, yy may
% be a scalar.  If mi is absent, we assume hh is the decimal hour.
% If hours (hh) also are not specified, an integer yearday is returned.
%
% For example:  date2ydN( 1994, 1, 1, 6 ) = 0.25
%     whereas:  date2ydN( 1994, 1, 1 )    = 1
%
% If any of the input values are out of range, the corresponding
% yeardays are set to NaN.
%
% See also yd2date                                          [3.5, 4.0, 5.1]


% N.Galbraith & E.Terray 3/94, 1/98, 3/02
% Speed: 2 x datenum, 120 x non-vectorized


Hours_Flag = 0;

if( nargin == 1 )                                % if a single array is passed
    if( length(yy(1,:)) > 3 )                    %    get the hours, if
        hh = yy(:,4);                            %    specified, and set the
        Hours_Flag = 1;                          %    hours flag to true
        if( length(yy(1,:)) > 4 )                %    if minutes explicitly
            hh = hh + yy(:,5)/60;                %    given, add them to hours
        end
    end
    mo = yy(:,2);                                %    then extract the other
    dd = yy(:,3);                                %    date & time parameters
    yy = yy(:,1);                                %    note: must do yy last!!
end

if( nargin > 3 ),  Hours_Flag = 1;   end         % if hours are given, set flag
if( nargin > 4 ),  hh = hh + mi/60;  end         % if mi given, add to hours

[nr,nc] = size( mo );
if( length(yy) == 1 )                            % make sure yy has the same
    yy = yy(ones(nr,nc));                        %   length as mo, dd etc.
end
                                       % now do the conversion to yearday

mdays = [0 31 28 31 30 31 30 31 31 30 31 30];    % days in months Jan thru Nov
cumdays = cumsum( mdays ) - 1;                   % cumulative # days, less 1
if( nr > nc ),  cumdays = cumdays(:);  end       % force row or column form

isleap = ( (rem(yy,4) == 0) & (mo > 2) );        % 1 if a leap year, else 0

k = find( mo < 1 | mo > 12 );                    % find months out of range
mo(k) = 1;                                       % set to Jan as a place holder

if( Hours_Flag )                                 % if hours are specified
    Yd = cumdays(mo) + dd + hh/24 + isleap;      %   use this form
    i  = find( hh < 0 | hh > 24 );               %   set Yd to NaN if hours
    Yd(i) = NaN;                                 %     are out or range
else                                             % else if no hours
    cumdays = cumdays + 1;                       %   bump the days by 1
    Yd = cumdays(mo) + dd + isleap;              %   so that Jan 1 is Day 1
end


if( exist('mi') == 1 )                           % if the variable mi exists
    i  = find( mi < 0 | mi > 60 );               %   set Yd to NaN if minutes
    Yd(i) = NaN;                                 %     are out of range
end

i = find( dd < 1 | dd > 31 );                    % set Yd to NaN if days
Yd(i) = NaN;                                     %   or month are out
Yd(k) = NaN;                                     %   of range

if( any(isnan(Yd)) )
    fprintf('Warning: some dates out of range ==> yeardays set to NaN\n');
end


%_______________________________ Bug Fixes ______________________________

% 3/03/02 E. Terray
%   Files containing a partial record at the beginning are seen by the
%   unpacker to have all bad checksums.  This behavior is due to a bug
%   where we failed to skip the partial record.  Repaired by
%   replacing  B = reshape( B(1:Nrec*RecL), RecL, Nrec )';
%          by  B = reshape( B(p(1)-1+(1:Nrec*RecL)), RecL, Nrec )';

% 5/20/02 E. Terray
%   Added capability to unpack Bottom-Track info - data type id = 6

% 6/16/02 E. Terray
%   Added capability to read large files sequentially & unpack into
%      smaller output files

% 3/19/03 E. Terray
%   Changed Nbytes in function Find1stHdr to 5000 from 1000
%   Added line 83:  if( nfiles == NfileOut ),  break;  end
%   Changed the order of the arguments NrecSkip, NrecOut, NfileOut
%     and added help concerning their role

% 3/20/03 E.Terray
%   Changed the input argument parsing
%   Added the test in the while loop if fread returns < RecL bytes
%   Changed names in hte case of multiple output files to fname_nnn.mat

% 6/15/03 E. Terray
%   Fixed p in UpkRecs to exclude the last little bit between
%   p(end) and length(B) by changing Nrec = length( p ) to
%   Nrec = length( p ) - 1   [see fix on 3/03/02]

% 11/18/04 E. Terray
%   Augmented the fixed leader parsing to include almost everything

% 11/24/04 E. Terray
%   Added twoscvt to ParseBT to correctly convert +/- ranged data

