% corags.m
%**********
% Purpose: Correct a position fix file for the horizontal offset
%          between the gps antenna and the shipboard ADCP transducer.
%          The correction reduces or eliminates glitches in the
%          ADCP absolute reference layer velocity estimate at
%          station arrivals and departures.  The improvement is
%          evident when the offset is 10 m or more, and the fixes
%          are from a PPS (so-called P-code, or military accuracy)
%          receiver.
%
% Usage
% Input files:
%          1) ASCII flat fix file, such as the .ags output of ubprint
%          2) ASCII flat file with time and heading at the end of
%             an ensemble, such as the output of lst_hdg.
%
% Output file:
%          A corrected fix file suitable for input to refabs.
%          Suggested extension: .agt if the source was .ags;
%          the "t" is for "transducer".
%
% Input parameters: You should enter the following distances in meters;
%                   - along the fore-aft axis:
%                     delta_y is the forward distance from the antenna to
%                     the transducer; positive if the transducer is
%                     forward of the antenna, negative otherwise.
%                   - athwardships:
%                     delta_x is the athwartships distance from the antenna
%                     to the transducer; positive if the transducer is
%                     to starboard of the antenna.
%
% For instance, aboard the R/V Knorr in 1995, the antenna and the transducer
% were both bounted on the port side, with the antenna aft of the transducer.
% The values were:
% delta_y = 13.; delta_x = 1.5,  (x_ant = - 3.5; x_trans = -2.)

% Written in March 96 by EK

%>>>>>>>>>>>>>>>>>>>>>>>>>>> input parameters >>>>>>>>>>>>>>>>>>>>>>>>>>>>>%

% input files (ags and hdg)
agsfile = '../../nav/alltmp.ags'   % output of ubprint
cags = 7;                          % number of columns into agsfile
hdgfile = 'a9512.hdg'              % output of lst_hdg or other heading source
nhdgcol = 3;                       % number of columns into hdgfile
hdgcol = [1 3]                     % columns with decimal day and instantaneous
                                   %   heading, such as last_heading, in degrees
enl = 10;                          % extend range of heading times
                                   % (common value: 10 secondes)

% output file (corrected ags)
outfile = 'allcor.agt'

% meters, antenna to transducer
delta_y = 13.;                     % positive forward
delta_x =  1.5;                    % positive to starboard

% miscelleanous for plots
cruise = 'KN9512';
year_base = '1995';

%>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>%
% position data
[tags, lon, lat] = read_asc(agsfile);     %time, longitude, latitude

% heading data
[thdg, hdg] = read_asc(hdgfile, nhdgcol, hdgcol) %time, last heading

% Extend range of heading times slightly:
nags = length(tags);
nhdg = length(thdg);
thdg(1) = thdg(1) - enl/86400;
thdg(nhdg) = thdg(nhdg) + enl/86400;
if ( tags(1) < thdg(1) | tags(nags) > thdg(nhdg) ),
     disp('Warning: some ags times outside of the heading time range')
     disp('Warning: check the ags/hdg files and/or enlarge the enl variable')
     return
end;

% heading in radians, positive CCW from north
theta = - hdg * (pi/180);

% Correction and Interpolation to ags times
delta = (delta_x +i*delta_y)*exp(i*theta);
corx_m = table1([ thdg real(delta) ], tags);
cory_m = table1([ thdg imag(delta) ], tags);

% transform meters to decimal degrees
corx_deg = m_to_lon(corx_m',lat');
cory_deg = m_to_lat(cory_m',lat');

% apply correction to the fixes
loncor = lon + corx_deg';
latcor = lat + cory_deg';

%array = [ tags loncor latcor gps(:,4:7)];
array = [ tags loncor latcor ];
eval([ 'save ' outfile ' array -ascii'])

% plot the horizontal corrections vs ags time
gtit = [ cruise ' : Offsets between GPS antenna and shipboard ADCP transducer'];
xlab = [ year_base ' Decimal Days' ];
figure(1)
clf
subplot(211)
plot(tags,corx_m,'.')
grid
axis([floor(tags(1)) ceil(tags(nags))  -20 20])
xlabel(xlab)
ylabel('Longitude (m)')
title(gtit)
subplot(212)
plot(tags,cory_m,'.')
grid
axis([floor(tags(1)) ceil(tags(nags))  -20 20])
xlabel(xlab)
ylabel('Latitude (m)')
orient tall
print corags
