/***************************************************************

   SET_TOP.C

   This program knocks out bad top bins of underway or on-station
    profiles
   by setting first_good_bin field of ACCESS_VARIABLES_TYPE
   to the desired bin number.

                     Willa Zhu
                     89-01-13

Sat  98/06/13 EF
   Modernized with dbget_f_cnf and dbmove_cnf;
   changed so that a time range of "all" works.

99/07/01 EF
   Added ability to select the on-station rather than the
   underway profiles.  If speed_threshold is positive, editing
   will be applied to underway profiles; if negative, then to
   on-station profiles.

----------------------------------------------------------------

control file structure:

     dbname:    (e.g. WEP)
     speed_threshold: [ 3 ]  { (m/s) of underway limit }
     ref_bin_range: [reference_bins] [ 5 ] to  [ 20 ]
                             { bin range; first bin is 1 }
     first_good_bin:  [ 3 ]  { first_good_bin number }
     time_ranges:
     (list of YMDHMS time pairs:)
        (yy/mm/dd hh:mm:ss) to (yy/mm/dd hh:mm:ss)
        .
        .
****************************************************************/

#include "geninc.h"
#include "adcp.h"    /* ACCESS_VARIABLES_TYPE */
#include "data_id.h"
#include "vector.h"  /* array_mean() */
#include "ioserv.h"  /* check_error(), get_parameters() */
#include "use_db.h"  /* db*_cnf(), get_time_range(), print_time_range() */

#define  sqr(x)  (x)*(x)
#define  good_float(x)  ( (x) < ADJ_BADFLOAT )

/*
Main routine
*/

#if PROTOTYPE_ALLOWED
void do_it(FILE *fp_cnt)
#else
void do_it(fp_cnt)
FILE *fp_cnt;
#endif
{
   int    search_error, IERR;
   unsigned int N;
   ACCESS_VARIABLES_TYPE access;
   YMDHMS_TIME_TYPE start, end,            /* for time range */
                    this_time;  /* current profile time */
   float *u, *v, mean_u, mean_v;


   /*---------- end of auto declarations for do_it -----------------*/

/***************************************************************
   static variables needed for initialization of
   the parameter array

*/

static   FILE_NAME_TYPE dbname;
static   int   STEPS = 1;
static   int first_good, bin_range[2];
static   double speed_limit;
static   short underway;
static   int edit_underway;
static   PARAMETER_LIST_ENTRY_TYPE parameters[] =
   {
      { " dbname:",          " %79s", dbname,       TYPE_STRING },
      { " speed_threshold:", " %lf" , &speed_limit, TYPE_DOUBLE },
      { " ref_bin_range:",   " %d"  , bin_range,    TYPE_INT    },
      { " to",               " %d"  , bin_range+1,  TYPE_INT    },
      { " first_good_bin:",  " %d"  , &first_good,  TYPE_INT    },
      { " time_ranges:"   ,  NULL   , NULL,         0           },
      { NULL,                NULL,    NULL,         0           }
   };                   /* parameters[] */


/***************************************************************/

   edit_underway = 1; /* default; original case */
   if (get_parameters(fp_cnt, parameters, NULL))  exit(-1);
   print_parameters(stdout, parameters, NULL, "\n");
   if (speed_limit < 0)
   {
      edit_underway = 0;
      speed_limit = - speed_limit;
   }
   check_error( (
      (u = (float *) calloc(bin_range[1], sizeof(FLOAT))) == NULL ||
      (v = (float *) calloc(bin_range[1], sizeof(FLOAT))) == NULL ),
      "insufficient memory for u and v arrays");
   if (dbopen_cnf(1, dbname, READ_WRITE, DIR_IN_MEMORY)) exit(-1);
/*
---> Start the loop through time ranges listed in the control file.
*/
   while (get_time_range(fp_cnt, &start, &end) == 1)
   {
      print_time_range(stdout, &start, &end);
      search_error = dbsrch_cnf(TIME_SEARCH, (char *)&start);
      if (search_error != 0 && search_error != SEARCH_BEFORE_BEGINNING)
      {
         printf("The starting time of this range produced a search error.\n");
         printf("set_top is aborting.\n");
         goto close_db;
      }
      IERR = 0;
/*
---> Start the loop through profiles within the time range.
*/
      while (!IERR && check_time(&this_time, &start, &end))
      {
         N = bin_range[1];
         if (dbget_f_cnf(U, u, &N, "getting u")) goto close_db;
         if (dbget_f_cnf(V, v, &N, "getting v")) goto close_db;
         mean_u = array_mean((u+bin_range[0]-1), (N-bin_range[0]+1));
         mean_v = array_mean((v+bin_range[0]-1), (N-bin_range[0]+1));
         underway = (sqrt(sqr(mean_u)+sqr(mean_v)) >= speed_limit);
         if ( (edit_underway && underway) || (!edit_underway && !underway) )
         {
            N = sizeof(ACCESS_VARIABLES_TYPE);
            if (dbget_cnf(ACCESS_VARIABLES, (char *) &access, &N,
               "access")) goto close_db;
            access.first_good_bin = first_good;
            if (dbput_cnf(ACCESS_VARIABLES, (char *) &access, &N,
               "access")) goto close_db;
         }
         IERR = dbmove_cnf(STEPS);   /* IERR is checked at the start of the loop */
      }  /* end of loop through profiles within a time range */
   }  /* end of loop through time ranges read from control file */

   close_db:
      dbclose_cnf();
      printf("\n SET_TOP completed.\n");
}
