#!/usr/bin/python

'''A mechanism for repeatedly running a set of tasks.

   I originally intended to use threadgroup for this, but
   it seems that rsync simply won't work when run from a
   python thread, no matter how it is run--via os.system,
   os.popen, etc.

   This module uses signals to provide a timeout, so that
   a hung task will be deleted from the queue instead of
   blocking the queue.  Therefore it must be used only in
   the main thread--signals do not work in secondary threads.
'''

# rsync doesn't work in a python thread, so here is a non-threaded
# alternative to threadgroup.

import sys, os, os.path, commands, time
import signal

class Timeout(Exception):
   pass

def timeout_handler(signum, frame):
   raise Timeout()

class periodic:
   def __init__(self, function, args = (), kwargs = {},
                interval = 60, initial = None, at_exit = 0):
      self.function = function
      self.args = args
      self.kwargs = kwargs
      self.interval = interval
      self.at_exit = at_exit
      self.started = time.time()
      self.time_to_run = self.started + interval
      if initial:
         t = ((int(self.started) // 86400) * 86400) + initial
         while t < self.started + 0.2 * interval:
            t += interval
         self.time_to_run = t

   def run(self):
      now = time.time()
      if now >= self.time_to_run:
         self.time_to_run += self.interval
         out = apply(self.function, self.args, self.kwargs)
         if type(out) is str and len(out) > 2:
            print out
         ended = time.time()
         while ended > self.time_to_run:
            self.time_to_run += self.interval

class periodic_system(periodic):
   def __init__(self, cmd, **kw):
      print "periodic_system: %f :  %s" % (time.time(), cmd)
      function = commands.getoutput
      args = (cmd,)
      periodic.__init__(self, function, args = args, **kw)


class repeater:
   def __init__(self, condition = None, poll_interval = 1, timeout = 3600):
      if condition is None:
         self.condition = lambda : 1 # loop forever
      else:
         self.condition = condition
      self.poll_interval = poll_interval
      self.timeout = timeout
      self.tasks = []
      self.running = 1

   def add(self, task):
      self.tasks.append(task)

   def start(self):
      alarm_handler = signal.getsignal(signal.SIGALRM)
      signal.signal(signal.SIGALRM, timeout_handler)
      while self.condition() and self.running:
         ibad = None
         for i, task in enumerate(self.tasks):
            if self.condition() and self.running:
               signal.alarm(self.timeout)
               try:
                  task.run()
               except Timeout:
                  print "Timeout in repeater.start: %d" % i
                  ibad = i
               except Exception, arg:
                  print arg
               signal.alarm(0)
         if ibad is not None:
            del(self.tasks[ibad])
            print "Deleted task %d" % ibad
            if len(self.tasks) == 0:
               self.running = 0
         time.sleep(self.poll_interval)
      for task in self.tasks:
         if task.at_exit:
            try:
               task.run()
            except Exception, arg:
               print arg
      signal.signal(signal.SIGALRM, alarm_handler)

   def stop(self):
      '''Stop the loop.
      This might be useful only in a gui context, which might
      in turn require other changes.
      '''
      self.running = 0


#####################################################################

def main():
   '''Example for testing.

   This illustrates capturing python output and output
   from system commands, and redirecting it to a file.
   It also shows the use of a sentinal file created by
   the program, the deletion of which stops the loop.
   '''
   sys.stdout = open('test.tmp', 'w')
   open("/tmp/repeater_test", 'w').close() # make a flag file
   def file_present():
      return os.path.exists("/tmp/repeater_test")
   def printer(s):
      print s, time.time()
   R = repeater(condition = file_present, poll_interval = 0.2, timeout = 5)
   #R.add(periodic_system("ls -ltr | tail -2 >> test2.tmp", interval = 2))
   R.add(periodic_system("ls -ltr | tail -2", interval = 2))
   R.add(periodic(printer, args = ("The time is",), interval=10,
                                          initial=10, at_exit=1))
   R.add(periodic_system("sleep 6", interval = 10))
   R.start()
   sys.stdout.close()

if __name__ == "__main__":
   main()


