#!/usr/bin/python

'''
   asc2bin.py, experimental version of serasc2bin.py,
   presently for UHDAS files only.
   Does NOT parse ascii files from other logging systems: Use serasc2bin.py

   Usage: asc2bin.py [<options>] [-y 2001] ...
                   [-o outputdirectory] file1 [file2 ...]

      Reads ascii files written by  UHDAS and writes corresponding
      files in binary double precision.  Output file name is the
      input file name with '.rbin' appended, unless the
      --message and/or --add_extension options are used.
      By default,
      if a file of that name already exists, the corresponding
      input file will not be processed.  Use '--redo' option
      to reprocess in this case.  Use '--update' to append to
      an rbin file if it already exists.  This can be used to
      continually update an rbin file as the source file is
      being written.

      Gzipped input files are handled transparently, without
      being explicitly gunzipped; the '.gz' extension is ignored
      when generating the output filename, unless the '-a' option
      is used.

      -h or --help:     print this message and exit
      -r or --redo:     process all input files, regardless of
                           whether an output file already exists
      -u or --update:   process incrementally
      -s or --seconds:  with -u, timeout in seconds (floating point)
      -o or --outdir:   directory for output files
      -m or --message:  desired output file type (without the dot)
      -y or --yearbase: 4-digit yearbase
      -l or --log_yearbase" 4-digit yearbase (yearbase used for $UNIXD
                            during logging). defaults to same as
                            yearbase if unspecified
      -v or --verbose:  print short diagnostics
      -V or --Verbose:  echo line number and line, so we can see
                           where it chokes
      -S or --showbad:  echo line number and line when they are
                           rejected, so we can see what junk is in the
                           file
      -a or --add_extension: Add the message extension to the input
                           file name, instead of replacing the input
                           file extension
      -t or --type:     'rvdas', 'uh', 'vmdas',...  ('uh' is default)
      -c or --count:   for vmdas only:
                       VmDAS may have more than one serial message per
                       timestamp (PADCP); the timestamp comes at the
                       end of a group.  Use "--count" to specify which
                       messages to store.  The field "count" starts with
                       zero just prior to the PADCP message and decreases
                       backwards in time.
                       count switch:     outputs
                         first         (output the serial message at
                                            the beginning of the collection
                                            for the ensemble;
                                            corresponds to raw data
                                            structure nav.txy1)
                         last           (output the serial message at the
                                            end (nav.txy2)).  DEFAULT
                         all            (output all serial messages)


      NOTE: the first entry, 'u_dday', is the time of the PC doing the logging.
      For UHDAS, this is the linux clock time
      For rvdas, it is a GGA timestamp
      For VmDAS, it is the PC clock time.  Subtract sec_pc_minus_utc to get UTC


      File or message types:

         gps  GGA messages.  .gps.bin fields are logger decimal day,
                                    GGA decimal day, lon degrees,
                                    lat degrees.
                                    ??GGA message (e.g., INGGA or GPGGA)
         gga  same as gps
         spd  soundspeed     .spd.bin fields are logger decimal day,
                                    soundspeed in m/s.

         hdg  gyro heading   .hdg.bin fields are logger decimal day,
                                    heading in degrees.
                                    HEHRC or HEHDT messages are read.

         hnc  gyro heading, NO CHECKSUM
                                    .hdg.bin fields are logger decimal day,
                                    heading in degrees.
                                    HEHRC or HEHDT messages are read.

         adu  Ashtech $PASHR,ATT    .adu.bin fields are logger decimal day,
                                    ATT decimal day (GPS days in week),
                                    heading, pitch, roll, mrms, brms,
                                    reacq flag.

         pmv  POS/MV $PASHR   .pmv.bin fields are logger decimal day,
                                    GGA decimal day, heading, pitch,
                                    roll, heave, accuracy of roll,
                                    of pitch, of heading, heading flag,
                                    IMU flag.
                                    GAMS flag: (0=no aiding, 1=GPS
                                       aiding, 2 = GAMS aiding)
                                    IMU flag: (0 = no IMU; 1 = using IMU )

         rdi (various) $PRDID  .rdi.bin fields are logger decimal day,
                                    pitch, roll, heading.
                                    There is no error checking.

         psxn20 Seapath $PSXN,20   logger decimal day,
                                    quality for:  horizontal, height,
                                    heading, roll&pitch.

         psxn23 Seapath $PSXN,23   logger decimal day,
                                    roll, pitch, heading, heave

         sea                       logger decimal day,
                                    roll, pitch, heading, heave,
                                    quality for:  height,
                                    heading, roll&pitch.  (The psxn23
                                    fields preceed the psxn20 fields.)

         gps_sea                   same as gga, but with horizontal quality
                                   column from psxn20 appended

         psathpr CSI $PSAT,HPR     logger decimal day, GPS decimal day,
                                   heading, pitch, roll,
                                   flag (1 for GPS, 0 for gyro)

   Note: there is a slight difference in the logger decimal day for the
   Seapath gga file versus the gps_sea.  In the latter, the time stamp
   is the one from the psxn20 message, which arrives immediately after
   the gga.  Fixing this would be more trouble than it is worth.

'''
# 2006/05/18 EF: added CSI psathpr message
# 2005/03/28 EF: Reworked for UHDAS; new elapsed time fields;
#    logging module.

# 2004/09/30 EF: added sys.byteorder to binfile header,
#    to match binfile.py, write_bin.m, and read_bin.m,
#    all of which were modified to handle non-native order

# 2004/09/24 EF: switched from dday_frac to dday;
#   eliminate duplicate times; added dday to sea record

# 2004/09/18 EF: cleanup, documentation tweaks;
#   transparent handling of gzipped input files;
#   changed order of columns in vmdas output;
#   combined handling of psxn20 and psxn23.
#   Added hdop field to gga.
#   Added log_yearbase option from JH version;
#   fixed yearbase bug in get_vmdas_dday.
#   Added gps_sea combined message.


# 2003/11/01 JH: added vmdas

# 2003/06/25 EF: merged changes from rvdas; the implementation
# of the different file reading functions might be improved
# to reduce duplication of code, but we have a working framework
# for reading any files of this sort, e.g. SCS files as well
# as rvdas.

# 2003/06/23 EF: rvdas version working.
#  profiling shows that the checksum routine is the biggest
#  single time sink.
#  Added the Pyrex version of the checksum routines; ~20% speedup.
#
#  Changed variable name "str" to "str_" to avoid conflict
#  with built-in.


# 2002/08/18 EF: Added pmv support; generalized gga; --message option.
# bug in translate_file_incr fixed 2001/04/16


import sys, os, array, string, time, copy
import logging
L = logging.getLogger('asc2bin')

from  binfile import binfile_a
from linefile_tail import linefile2s

def usage():
   print __doc__
   sys.exit()

def split_rvdas(line):
   h, t = line.split(' ', 1)
   return h.strip(), t.strip()

def get_rvdas_dday(timetag, yearbase, epoch_t):
   parts = timetag.split(':', 3)
   yr2, yd = parts[0].split('+')
   year = 2000 + int(yr2)
   yd = int(yd)
   hr = int(parts[1])
   mn = int(parts[2])
   sec = float(parts[3])
   dsec = hr * 3600 + mn * 60 + sec
   dday = (yd - 1) + dsec / 86400.0
   if year != yearbase:
      yearbase_dif_days = round((time.mktime((year, 1, 1, 0, 0, 0, 0, 0, 0)) -
                           epoch_t) / 86400.0)
      dday = dday + yearbase_dif_days
   return dday

def get_vmdas_dday(str_, yearbase, epoch_t):
   if str_[:7] != '$PADCP,':
      return None
   ### timetag is:   $PADCP,2137,20030909,000148.26,-148.11
   parts = str_.split(',')
   if len(parts) != 5:
      return None

   ens_num = parts[1]
   ymd = parts[2]
   hms = parts[3]
   sec_pc_minus_utc = float(parts[4])
   year = int(ymd[0:4])
   mon =  int(ymd[4:6])
   day =  int(ymd[6:8])
   hr =   int(hms[0:2])
   mn = int(hms[2:4])
   sec = int(hms[4:6])
   sec100 = int(hms[7:])

   # PC time
   dday = float(time.mktime((year, mon, day, hr, mn, sec, 0, 0, 0)) +
                sec100/100.0 -  epoch_t)/86400.0
   return  [dday, ens_num, sec_pc_minus_utc]


## Here and in checksums.pyx, the use of ValueError is convenient
## but not particularly good practice; it would be better to define
## a ChecksumError so as to distinguish between bad checksums and
## other reasons for failure in the message parsing.

## The general strategy is that the message reading functions return
## None if the requested message is not the one in the string, and
## raise an exception if it is the right message but fails in any other
## way.  This includes checksum errors and missing fields.

try:
   from checksums import NMEA as NMEAchecksum
except ImportError:
   def NMEAchecksum(str_):
      if len(str_) < 10:
         raise ValueError
      #str_ = str_.strip()
      bytes = array.array('B')
      bytes.fromstring(str_[1:-3])   # exclude the $ and the *
      cs = string.atoi('0x' + str_[-2:], 16) # May want to change for Py 2.x
      bcs = reduce(lambda x, y: x ^ y, bytes)
      if cs != bcs:
         raise ValueError

try:
   from checksums import HERC as HERCchecksum
except ImportError:
   def HERCchecksum(str_):
      if len(str_) < 10:
         raise ValueError
      #str_ = str_.strip()
      bytes = array.array('B')
      bytes.fromstring(str_[:-3])   # exclude the * but include the $
      cs = string.atoi('0x' + str_[-2:], 16) # May want to change for Py 2.x
      bcs = reduce(lambda x, y: x ^ y, bytes)
      if cs != bcs:
         raise ValueError

def frac_to_dday(u_dday, dday_frac):
   return u_dday + divmod(dday_frac - u_dday + 0.5, 1)[1] - 0.5

def hms_to_sec(hms):
   hh = float(hms[0:2])
   mm = float(hms[2:4])
   ss = float(hms[4:])
   return ss + 60.0 * (mm + 60.0 * hh)

def dm_to_deg(dm, let):
   idec = dm.index('.')
   deg = float(dm[:(idec-2)])
   mn = float(dm[(idec-2):])
   if let in 'SW':
      return -(deg + mn/60.0)
   else:
      return (deg + mn/60.0)

def get_degrees(str_):
   if str_[:6] == '$HEHRC':
      HERCchecksum(str_)
      return [float(str_[6:11])/100.0, ]
   elif str_[3:7] == 'HDT,':
      NMEAchecksum(str_)
      return [float(str_.split(',')[1]) ]
   else:
      raise ValueError

def get_degrees_nochecksum(str_):
   if str_[:6] == '$HEHRC':
      return [float(str_[6:11])/100.0, ]
   elif str_[3:7] == 'HDT,':
      return [float(str_.split(',')[1]) ]
   else:
      raise ValueError

def get_spd(str_):
   return [float(str_), ]


def get_gga(str_):
   ''' returns [t, x, y, qual, hdop] in days, decimal degrees'''
   if str_[3:7] != 'GGA,':
      return None
   NMEAchecksum(str_)
   fields = str_[7:].split(',')
   hms, latdm, ns, londm, ew, qual, nsat, hdop = fields[:8]
   return [hms_to_sec(hms)/86400.0, dm_to_deg(londm, ew),
                               dm_to_deg(latdm, ns), int(qual), float(hdop)]

def get_rdi(str_):
   ''' returns [pitch, roll, heading]'''
   if str_[:7] != '$PRDID,':
      return None
   #NMEAchecksum(str_)
   # chop off the checksum till we write the code for it
   str_ = str_[:-3]
   pitch, roll, heading = str_[7:].split(',', 3)
   return [float(pitch), float(roll), float(heading)]

def get_att(str_):
   if str_[:11] != '$PASHR,ATT,':
      return None
   #str_ = NMEAchecksum(str_)
   NMEAchecksum(str_)
   f = map(float, str_[11:-3].split(','))
   #f = map(float, str_[11:].split(','))  # version if NMEAchecksum chopped
   f[0] = f[0] / 86400.0
   return f

def get_posmv_pashr(str_):
   if str_[:7] != '$PASHR,':
      return None
   NMEAchecksum(str_)
   fields = str_[7:-3].split(',')
   del fields[2]
   f = map(float, fields[1:])
   day = hms_to_sec(fields[0])/86400.0
   return [day,] + f

def get_psxn20(str_):
   if str_[:9] != '$PSXN,20,':
      return None
   NMEAchecksum(str_)
   fields = str_[9:-3].split(',')
   f = map(float, fields)
   return f

def get_psxn23(str_):
   if str_[:9] != '$PSXN,23,':
      return None
   NMEAchecksum(str_)
   #checksums.NMEA(str_)
   fields = str_[9:-3].split(',')
   f = map(float, fields)
   return f

def get_psathpr(str_):
   if str_[:10] != '$PSAT,HPR,':
      return None
   NMEAchecksum(str_)
   fields = str_[10:-3].split(',')
   day = hms_to_sec(fields[0])/86400.0
   if fields[4] == 'N':
      q = 1
   else:
      q = 0
   f = [day]
   for field in fields[1:4]:
      try:
         f.append(float(field))
      except ValueError:
         f.append(1e38)
   f.append(q)
   return f


# Following are three dictionaries, each of which has the
# output file types as keys.

function_dict = {'gps': get_gga, # alias for gga
                 'gga': get_gga,
                 'spd': get_spd,
                 'hdg': get_degrees,
                 'hnc': get_degrees_nochecksum,
                 'adu': get_att,
                 'rdi': get_rdi,
                 'pmv': get_posmv_pashr,
                 'psxn20' : get_psxn20,
                 'psxn23' : get_psxn23,
                 'psathpr' : get_psathpr,
                 'sea' : None, # These will be filled in later,
                 'gps_sea' : None}  # when an instance copy of the
                                     # dictionary is made. The get_sea
                                     # function needs to store state
                                     # in the instance.

# for uhdas and rvdas
field_dict1 = {'gps': ['u_dday', 'dday', 'lon', 'lat', 'quality', 'hdop'],
              'gga': ['u_dday', 'dday', 'lon', 'lat', 'quality', 'hdop'],
              'pmv': ['u_dday', 'dday', 'heading',
                      'roll', 'pitch', 'heave',
                      'acc_roll', 'acc_pitch', 'acc_heading',
                      'flag_GAMS', 'flag_IMU'],
              'spd': ['u_dday', 'sndspd'],
              'hdg': ['u_dday', 'heading'],
              'hnc': ['u_dday', 'heading'],
              'adu': ['u_dday', 'dday', 'heading',
                                'pitch', 'roll', 'mrms', 'brms',
                                'reacq'],
              'rdi': ['u_dday', 'pitch', 'roll', 'heading'],
              'psxn20' : ['u_dday', 'horiz_qual', 'height_qual',
                          'head_qual', 'rp_qual'],
              'psxn23' : ['u_dday', 'roll', 'pitch', 'heading', 'heave'],
              'psathpr': ['u_dday', 'dday', 'heading',
                                    'pitch', 'roll', 'flag_gps'],
              'sea' : ['u_dday', 'dday', 'roll', 'pitch', 'heading', 'heave',
                           'height_qual', 'head_qual', 'rp_qual'],
              'gps_sea' : ['u_dday', 'dday', 'lon', 'lat',
                           'quality', 'hdop', 'horiz_qual']
                           }


# For VMDAS include three extra pieces of information from the $PADCP string.
# 'count' is message index relative to the $PADCP, such that 0 means
# the message immediately before the string, -1 means the message before
# that, etc.  Which message or messages is/are provided depends on the
# 'count' argument to the asc2bin initializer, which is used only when
# parsing VMDAS data.

field_dictN = copy.deepcopy(field_dict1)
for value in field_dictN.values():
   value += ['count', 'ens_num', 'sec_pc_minus_utc']

# Append the monotonic elapsed time field (2005/03/29)
for key, value in field_dict1.iteritems():
   if key is not 'rdi':
      value.append('m_dday')

class asc2bin:
   def __init__(self, yearbase = None,
                  log_yearbase = None,
                  outfiledir = './',
                  message = None,
                  type = 'uh',
                  redo = 0,
                  update = 0,
                  showbad = 0,
                  verbose = 0,
                  sleepseconds = 10,
                  keep_running = None,
                  logname = 'asc2bin'
                  ):
      logging.basicConfig()  # Only acts if a handler is not already present.
      L.setLevel(logging.INFO)
      self.yearbase = yearbase
      self.log_yearbase = log_yearbase
      self.verbose = verbose
      self.showbad = showbad
      self.outfiledir = outfiledir
      self.redo = redo
      self.update = update
      if update:
         self.sleepseconds = sleepseconds
      else:
         self.sleepseconds = 0
      self.keep_running = keep_running
      self.message = message
      self.type = type
      self.function_dict = function_dict.copy()
      self.function_dict['sea'] = self.get_sea
      self.function_dict['gps_sea'] = self.get_gps_sea
      self.get_field = self.function_dict[self.message]
      if self.type == 'vmdas':
         self.fields = field_dictN[self.message]
      else:
         self.fields = field_dict1[self.message]
      self.recordlength = len(self.fields)
      if self.log_yearbase is None: #it was unset
         self.log_yearbase = self.yearbase
      if self.message in ['gps', 'gga', 'pmv', 'sea', 'gps_sea', 'psathpr']:
         self.check_dday = self.check_gga_dday
      elif self.message == 'adu':
         self.check_dday = self.check_adu_dday
      else:
         self.check_dday = lambda x: x
      if self.redo:
         self.mode = 'w'
      else:
         self.mode = 'a'
      # Information saved from message to message and across file boundaries:
      self.last_dday = -100000
      self.last_psxn20 = None
      self.last_gga = None
      # Counters
      self.Nbad = 0
      self.Ngood = 0

      self.log = logging.getLogger(logname)  #gets root logger if empty

      if not os.access(self.outfiledir, os.F_OK):
         os.makedirs(self.outfiledir, 0775)
      self.epoch_t = time.mktime((self.yearbase, 1, 1, 0, 0, 0, 0, 0, 0))
      epoch_tlog = time.mktime((self.log_yearbase, 1, 1, 0, 0, 0, 0, 0, 0))
      self.yearbase_dif_days = round((epoch_tlog - self.epoch_t) / 86400.0)

      if self.type[:2] == 'uh':
         self.translate_file = self.translate_file_2
         self.translate_file_incr = self.translate_file_2
         self.get_dday = self.get_UNIX_dday
      else:
         raise NameError, self.type

   def get_UNIX_dday(self, str_):
      if str_[:7] == '$UNIXD,':
         fields = str_[7:].split(',')
         log_dday = float(fields[0])  # this was dday for log_yearbase
         return [log_dday + self.yearbase_dif_days, float(fields[-1])]
      elif str_[:7] == '$UNIXT,':
         ut = float(str_[7:])
         return [(ut - self.epoch_t)/86400.0, 0.0]
      else:
         raise ValueError



   def check_gga_dday(self, r):
      r[1] = frac_to_dday(r[0], r[1])
      if abs(r[1] - self.last_dday) < 1e-7: # around 0.01 s
         return []
      self.last_dday = r[1]
      return r

   def check_adu_dday(self, r):
      r[1] = frac_to_dday(r[0], r[1]) - 0.00013888888888888889 # 12 seconds
      if abs(r[1] - self.last_dday) < 1e-7: # around 0.01 s
         return []
      self.last_dday = r[1]
      return r


   def get_sea(self, str_):
      f = get_gga(str_)
      if f:
         self.last_gga = f
         return None
      f = get_psxn20(str_)
      if f:
         self.last_psxn20 = f
         return None
      f = get_psxn23(str_)
      if f and self.last_psxn20 and self.last_gga:
         return self.last_gga[0:1] + f + self.last_psxn20[1:]

   def get_gps_sea(self, str_):
      f = get_gga(str_)
      if f:
         self.last_gga = f
         return None
      f = get_psxn20(str_)
      if f and self.last_gga:
         return self.last_gga + f[0:1]

   def make_record(self, timestring, datastring):
      rec = None
      try:
         timestring = timestring.strip()
         datastring = datastring.strip()
         rec2 = self.get_field(datastring)
         if rec2:
            rec1 = self.get_dday(timestring)
            if rec1:
               rec = self.check_dday(array.array('d', rec1[:1] + rec2 + rec1[1:]))
               assert len(rec) == self.recordlength
               self.Ngood += 1
      except:
         self.Nbad += 1
         if self.showbad and self.Nbad < 10:
            L.warn( 'Rejected %s %s\n   %s %s\n %s\n' % (self.filename,
                                                      self.message,
                                                      timestring,
                                                      datastring, str(rec)))
         rec = None
      return rec


   def translate_file_2(self, filename, outfilename):
      '''The time tag is on the line preceeding the message.
      '''
      self.filename = filename
      self.Ngood = 0
      self.Nbad = 0
      # append to UH
      f_in = linefile2s(filename, sync = '$UNIXD',
                        timeout = self.sleepseconds,
                        keep_running = self.keep_running)
      f_out = binfile_a(outfilename, mode=self.mode, name = self.message,
                       columns = self.fields)

      n_written = f_out.count()
      self.log.debug('filename: %s, outfilename: %s, n_written: %d' %
                      (filename, outfilename, n_written))
      if n_written > 0:    # appending to an existing binfile
         f_out.seek(-1, 2)
         last_written = f_out.read(1)
         self.log.debug('last_written: %s' % str(last_written))
         # Read n_written records to position the input file:
         # (this is the earliest possible location; it might need
         #  to be advanced much farther, if there is more than one
         #  message type in the file)
         t, d = f_in.read_records(n_written)[-2:]  # change if read_records changes
         rec = self.make_record(t, d)
         self.log.debug('rec: %s' % str(rec))
         # Because of the Python bug, a record might have been
         # missed, so we compare the monotonic time fields.
         while rec is None or rec[-1] < last_written[-1]:
            lines = f_in.read_record()
            if lines:
               rec = self.make_record(*lines)
               self.log.debug('searching, rec: %s' % str(rec))
            else:
               self.log.debug('no more lines, and rec != last_written')
               break

      if n_written == 0:    #New file: read as much as possible in one chunk.
         lines = f_in.read_records()
         n = len(lines)/2
         self.log.debug('New file, n = %d' % n)
         if n > 0:
            recs = array.array('d')
            j = 0
            for ii in range(0, 2*n, 2):
               rec = self.make_record(lines[ii], lines[ii+1])
               if rec:
                  recs += rec
                  j += 1
            if j > 0:
               f_out.write(recs)
               n_written += j

      # Incremental read
      self.log.debug('Starting incremental, n_written = %d' % n_written)
      for (timeline, dataline) in f_in:
         self.log.debug('%s %s' % (timeline, dataline))
         rec = self.make_record(timeline, dataline)
         if rec:
            self.log.debug('rec is %s' % str(rec))
            f_out.write(rec)
            n_written += 1

      f_in.close()
      f_out.close()
      self.log.debug('End translate_file_2, n_written = %d' % n_written)
      return n_written



   def translate_files(self, filenames):
      #self.__dict__.update(kw)
      n = 0
      try:
         for filename in filenames:
            if (not self.update) or self.verbose:
               L.info(filename)
            fpath, fname = os.path.split(filename)
            fname_base, log_ext = os.path.splitext(fname)
            if log_ext == '.gz':
               fname_base, log_ext = os.path.splitext(fname_base)
            out_ext = self.message or log_ext[1:]

            outfilename = os.path.join(self.outfiledir,
                                 fname_base + '.' + out_ext + '.rbin')
            needs_processing = self.redo or not os.access(outfilename, os.F_OK)
            if self.update and not needs_processing:
               if os.path.getmtime(filename) > os.path.getmtime(outfilename):
                  needs_processing = True
            if needs_processing:
               if self.verbose: L.info( ' ---> %s' % outfilename)
               n = self.translate_file(filename, outfilename)
            else:
               if self.verbose:
                  L.info('no action')
            if (not self.update) or self.verbose:
               L.info('Ngood = %d   Nbad = %d' % (self.Ngood, self.Nbad))
      except KeyboardInterrupt:
         raise
      except:
         raise
      return n  # number of records written to the last file

os.environ['TZ'] = 'UTC'


def main():
   import getopt

   try:
      options, args = getopt.getopt(sys.argv[1:], 'SvVhro:y:l:us:m:c:t:',
                           ['showbad', 'verbose', 'Verbose', 'help',
                            'redo', 'outdir=', 'yearbase=', 'log_yearbase=',
                            'update', 'seconds=',
                            'message', 'count=',
                            'type='])
   except getopt.GetoptError:  # Before Python 2.x this was getopt.error
      usage()

   if len(args) == 0:
      usage()

   opts = {}

   for o, a in options:
      if o in ('-h', '--help'):
         usage()
      elif o in ('-r', '--redo'):
         opts['redo'] = 1
      elif o in ('-o', '--outdir'):
         opts['outfiledir'] = a
      elif o in ('-y', '--yearbase'):
         opts['yearbase'] = int(a)
      elif o in ('-l', '--log_yearbase'):
         opts['log_yearbase'] = int(a)
      elif o in ('-v', '--verbose'):
         opts['verbose'] = 1
      elif o in ('-V', '--Verbose'):
         opts['verbose'] = 2
      elif o in ('-S', '--showbad'):
         opts['showbad'] = 1
      elif o in ('-u', '--update'):
         opts['update'] = 1
      elif o in ('-s', '--seconds'):
         opts['sleepseconds'] = string.atof(a)
      elif o in ('-m', '--message'):
         opts['message'] = a
      elif o in ('-c', '--count'):
         opts['count'] = a
      elif o in ('-t', '--type'):
         opts['type'] = a


   t0 = time.clock()

   if 'type' in opts.keys():
      if opts['type'] == 'vmdas':
         if opts['count'] not in ('first', 'last', 'all'):
            print "option 'count' must be 'first', 'last', or 'all'"
            print "you chose '%s'" % opts['count']
            sys.exit()

   import glob
   filenames = []
   for arg in args:
      filenames += glob.glob(arg)

   asc2bin(**opts).translate_files(filenames)

   t1 = time.clock()
   print 'Elapsed cpu time:   %.1f s' % (t1 - t0)


if __name__ == '__main__':
   main()

