#!/usr/bin/python

''' Binfile class, with read and write methods.
    (Write methods are incomplete.)

    A binfile is a very simple file format for storing 2-D arrays
    of double precision numbers, with a minimal self-describing
    ascii header.

   See also comments at bottom.

   2005/03/25
   Added append support.

'''

import string, os, stat, sys, struct, time, array

_str = '''Binfile %(name)s with %(IOmethod)s IO, mode %(mode)s,
    byteorder %(byteorder)s,
    %(count)d records of %(ncolumns)d fields:
        '''

# String byteorder as returned by sys.byteorder, indexed by '<' or '>'
_sbyteorder = {'<': 'little', '>': 'big'}
# The reverse:
_cbyteorder = {'little': '<', 'big': '>'}
# System byteorder as a character:
_sysbyteorder = _cbyteorder[sys.byteorder]

_pytype = {'i1':'b', 'u1':'B', 'i2':'h', 'u2':'H', 'i4':'i', 'u4':'I',
           'f4':'f', 'f8':'d'}

class binfile_s:
   '''Uses Python byte strings for input/output. Does *not* take
      care of byte-swapping.  See the binfile subclass.
   '''
   def __init__(self, filename, mode = 'r', name = 'anon', columns = [],
                type = 'f8',
                timeout = 0.0, interval = 0.1, runflag = None):
      self.filename = filename
      #file_exists  = os.path.exists(filename)
      assert mode == 'r' or mode == 'w' or mode == 'a'
      self.mode = mode   # Logical mode; not necessarily actual mode.
      if mode == 'a':
         mode = 'a+'     # ...so we can read the header.
      if sys.platform == 'win32':
         mode += 'b'
      self.file = open(filename, mode)
      self.fileno = self.file.fileno()
      self.name = name
      self.columns = columns
      self.ncolumns = len(columns)
      self.byteorder = sys.byteorder
      self.type = type
      self.nl_header = self.ncolumns + 2 # first line, columns, byteorder
      if self.mode == 'r':
         self._read_header()
      elif self.mode == 'a':
         try:
            self.file.seek(0)
            self._read_header()
            self.file.seek(0,2)
         except:
            self.file.seek(0)
            self._write_header()
      else:
         self._write_header()
      self.typelen = int(self.type[1])
      self.record_length = self.typelen * self.ncolumns
      self.cursor = 0  # for sequential reading of single records
      if self.mode == 'a':
         self.cursor = self.count()
      if self.byteorder == _sysbyteorder:
         prefix = '@'
      else:
         prefix = self.byteorder
      self._fmt = prefix + '%d' + _pytype[self.type]
      self.record_fmt = self._fmt % self.ncolumns
      self.IOmethod = 'string'
      self._fstr = _str + '\n        '.join(self.columns) + '\n'
      self.interval = interval
      self.nloops = int(timeout / interval)
      if runflag is not None:
         self.runflag = runflag
      else:
         self.runflag = lambda : True

   def __iter__(self):
      return self

   def __del__(self):
      if self.file:
         self.file.close()

   def __str__(self):
      d = self.__dict__.copy()
      d['count'] = self.count()
      return self._fstr % d

   def close(self):
      self.file.close()
      self.file = None

   def _read_header(self):
      line1 = self.file.readline()
      fields = string.split(line1)
      self.ncolumns = string.atoi(fields[0])
      self.nl_header = string.atoi(fields[1])
      self.name = fields[2]
      self.columns = []
      for ii in range(self.ncolumns):
         self.columns.append(string.split(self.file.readline())[0])
      nleft = self.nl_header - self.ncolumns - 1
      if nleft:
         dtypestring = self.file.readline().strip()
         if dtypestring in _cbyteorder.keys():
            self.byteorder = _cbyteorder[dtypestring]
            self.type = 'f8'
         else:
            self.byteorder = dtypestring[0]
            self.type = dtypestring[1:]
         nleft -= 1
         #assert nleft == 0
      for ii in range(nleft):   # nleft = 0 with present version.


         self.file.readline()
      self.offset = self.file.tell()

   def _write_header(self):
      self.file.write('%d %d %s\n' % (self.ncolumns, self.nl_header, self.name))
      for r in self.columns:
         self.file.write(r + '\n')
      self.byteorder = _sysbyteorder
      self.file.write('%s%s\n' % (self.byteorder, self.type))
      self.file.flush()
      self.offset = self.file.tell()

   def write(self, s):
      nrec = len(s) // self.record_length
      assert nrec * self.record_length == len(s)  # quick error check
      self.file.write(s)
      self.file.flush()
      self.cursor += nrec
      return nrec

   def seek(self, index, whence=0):
      nrec = self.count()
      if whence == 0:        # relative to start
         irec = index
      elif whence == 1:      # relative to cursor
         irec = self.cursor + index
      elif whence == 2:      # relative to end
         if index > 0:       # index should be negative
            raise IndexError
         irec = nrec + index
      else:
         raise ValueError
      irec = min(irec, nrec)
      irec = max(irec, 0)
      self.file.seek(self.offset + irec * self.record_length)
      self.cursor = irec

   def next(self):
      s, n = self.read_n(1)
      if n == 0:
         raise StopIteration
      return s

   def read_n(self, n = None):
      '''Read n records, or to end of file.
         Returns string, number of records read.
      '''
      iloop = self.nloops
      n_available = self.count() - self.cursor
      while n_available == 0 and iloop != 0 and self.runflag():
         time.sleep(self.interval)
         n_available = self.count() - self.cursor
         iloop -= 1
      if n is None:   # Or we could just set it to a large number.
         n = n_available
      else:
         n = min(n, n_available)
      s = self.file.read(n * self.record_length)
      self.cursor += n
      return s, n

   def read(self, n = None):
      '''Analogous to file object read; returns available
         file contents up to n records.
         Also see read_n, which will often be more useful.
      '''
      s, n = self.read_n(n)
      return s

   def count(self):
      ''' Return the number of complete records in the file. '''
      if self.mode != 'r':
         self.file.flush()
      nbytes = os.fstat(self.fileno)[stat.ST_SIZE]
      return int((nbytes - self.offset) // self.record_length)

class binfile(binfile_s):
   '''Uses lists for input/output, takes care of byte order'''
   def __init__(self, *args, **kwargs):
      binfile_s.__init__(self, *args, **kwargs)
      self.IOmethod = "list"

   def read_n(self, n = None):
      '''Read n records, or to end of file. '''
      s, nrec = binfile_s.read_n(self, n)
      return struct.unpack(self._fmt % (self.ncolumns * nrec) , s), nrec

   def write(self, rec):
      s = struct.pack(self._fmt % len(rec), *rec)
      return binfile_s.write(self, s)

class binfile_a(binfile_s):
   '''Uses arrays for input/output, does not handle foreign byte order'''
   def __init__(self, *args, **kwargs):
      binfile_s.__init__(self, *args, **kwargs)
      self.IOmethod = "array"

   def read_n(self, n = None):
      '''Read n records, or to end of file. '''
      s, nrec = binfile_s.read_n(self, n)
      a = array.array('d')
      a.fromstring(s)
      return a, n

   def write(self, rec):
      '''this one does not do byte-swapping...'''
      return binfile_s.write(self, rec.tostring())


#def open(filename, mode = 'r', name = '', columns = []):
#   return binfile(filename, mode, name, columns)
#
#def open_s(filename, mode = 'r', name = '', columns = []):
#   return binfile_s(filename, mode, name, columns)

# The above would create havoc with "from binfile import *",
# and also  prevent one from using the builtin "open" call
# in this module.

####################################################


from optparse import OptionParser

def main():

   parser = OptionParser()
   parser.add_option("-e", "--ends", action="store_true", dest="ends",
                     help="print first, last record", default=False)
   parser.add_option("-c", "--columns",
                     action="store_true", dest="header", default=False,
                     help="print header, including column labels")

   parser.add_option("-a", "--all",
                     action="store_true", dest="all", default=False,
                     help="print all records")

   (options, args) = parser.parse_args()


   for fn in args:
      b = binfile(fn)
      fmt = " %12.6g" * b.ncolumns
      if options.header:
         print b
      if options.ends:
         print fmt % b.read(1)
         b.seek(-1, 2)
         print fmt % b.read(1)
      if options.all:
         b.seek(0)
         #for record in b:
         #   print fmt % record
         # The previous method appears to be about as fast as
         # the following; the limiting factor may be the
         # print statement, or it may be that slicing is
         # slow.
         records = b.read()
         nrec = len(records) // b.ncolumns
         for irec in range(0, nrec):
            print fmt % records[irec*b.ncolumns:(irec+1)*b.ncolumns]

if __name__ == "__main__":
   main()

