#!/usr/bin/env python
## bltmon2.py
## This version works reasonably well, displaying data on a
## time axis relative to the present, while maintaining
## independent X axes for each panel.
# Next: try keeping a single X vector for all panels.

import sys, os
import string
import Tkinter
import Pmw
import math
import fcntl, FCNTL, time

import binfile

os.environ['TZ'] = 'UTC'
epoch_t = time.mktime((2001, 1, 1, 0, 0, 0, 0, 0, 0))


class MonChart(Pmw.MegaWidget):
   def __init__(self, parent=None, **kw):
      # Define the megawidget options.
      optiondefs = (
         ('size',      100,   Pmw.INITOPT),
         ('variables',   ['unnamed'],   Pmw.INITOPT),
      )
      self.defineoptions(kw, optiondefs)
      # Initialise the base class (after defining the options).
      Pmw.MegaWidget.__init__(self, parent)
      # Create the graph.
      self.createWidgets()
      # Check keywords and initialise options.
      self.initialiseoptions(MonChart)
      self.pack()   # if MegaWidget, not MegaToplevel

   def createWidgets(self):
      # Create vectors for use as x and y data points.
      self._vectorSize = self['size']
      self._variables = self['variables']
      self._npanels = len(self._variables)
      self._vector_x = Pmw.Blt.Vector()
      self._vector_x0 = Pmw.Blt.Vector()
      self._vector_y = []
      for i in range(self._npanels):
         self._vector_y.append(Pmw.Blt.Vector())
      interior = self.interior()
      controlFrame = Tkinter.Frame(interior)
      controlFrame.pack(side = 'bottom', fill = 'x', expand = 0)
      # Create the graphs.
      self._graph = []
      for i in range(self._npanels):
         self._graph.append(Pmw.Blt.Stripchart(interior,
                             title = self._variables[i]))
         # size of the whole widget in pixels
         self._graph[i].configure(height = 200,
                                  width = 800,
                                  title = '',
                                  topmargin = 5,
                                  leftmargin = 75,
                                  rightmargin = 10)
         self._graph[i].pack(side = 'top', expand = 1, fill = 'both')
         self._graph[i].legend_configure(hide = 1)
         self._graph[i].yaxis_configure(title = self._variables[i],
                                        shiftby = 2,
                                        loose = 1)
         self._graph[i].xaxis_configure(command=self.noticklabel,
                                        min = -self._vectorSize,
                                        max = 0)
         self._graph[i].line_create(
            'dummy',
            linewidth = 1,
            #symbol = '',
            symbol = 'square',
            pixels = 2,
            #fill = 'red',
            #outline = 'red',
            xdata=self._vector_x,
            ydata=self._vector_y[i])
      self._graph[-1].xaxis_configure(command='',
                                      title = 'Seconds before now')

   def noticklabel(self, graph, value):
      return ''

   def xaxisSeconds(self, graph, value):
      return '%.0f' % value

   def xaxisCommand(self, graph, value):
      try:
         dday = string.atof(value)
         ddfrac = dday - int(dday)
         hr = int(ddfrac * 24)
         minsec = 60 * (ddfrac * 24 - hr)
         min = int(minsec)
         sec = int(60 * (minsec - min))
         return '%02d:%02d:%02d' % (hr, min, sec)
      except ValueError:
         return '%f' % value

   def _update_data(self, x, y, dday_now):
      vx0 = self._vector_x0
      vx  = self._vector_x
      vylist = self._vector_y
      nv = self._vectorSize
      n_new = len(x)
      n_old = len(vx0)
      if n_new >= nv:
         vx0[:] = x[-nv:]
         for i in range(len(vylist)):
            vylist[i][:] = y[i][-nv:]
      else:
         n_total = n_old + n_new
         if n_total <= nv:
            vx0[:] = vx0 + x
            for i in range(len(vylist)):
               vylist[i][:] = vylist[i] + y[i]
         else:
            n_keep = nv - n_new
            vx0[:] = vx0[-n_keep:] + x
            for i in range(len(vylist)):
               vylist[i][:] = vylist[i][-n_keep:] + y[i]
      vx.expr('( %s - %f ) * 86400' % (str(vx0), dday_now))

   def update_data(self, x, y, dday_now):
      nv = self._vectorSize
      n_new = len(x)
      n_old = len(self._vector_x0)
      if n_new >= nv:
         self._vector_x0[:] = x[-nv:]
         for i in range(self._npanels):
            self._vector_y[i][:] = y[i][-nv:]
      else:
         n_total = n_old + n_new
         if n_total <= nv:
            self._vector_x0[:] = self._vector_x0 + x
            for i in range(self._npanels):
               self._vector_y[i][:] = self._vector_y[i] + y[i]
         else:
            n_keep = nv - n_new
            self._vector_x0[:] = self._vector_x0[-n_keep:] + x
            for i in range(self._npanels):
               self._vector_y[i][:] = self._vector_y[i][-n_keep:] + y[i]
      self._vector_x.expr('( %s - %f ) * 86400' % (
                           str(self._vector_x0), dday_now))


class Monitor:
   def __init__(self, parent):
      if not Pmw.Blt.haveblt(parent):
         message = 'Sorry\nThe BLT package has not been\n' + \
               'installed on this system.\n' + \
               'Please install it and try again.'
         w = Tkinter.Label(parent, text = message)
         w.pack(padx = 8, pady = 8)
         return
      self.parent = parent
      self.chart = MonChart(self.parent, size = 200,
                           variables = ['heading', 'pitch', 'roll'])
      #self.chart.focus()

   def start(self, file):
      self.binfile = binfile.binfile(file, 'r')
      self.nrec = 0
      self.parent.after(1000, self.run)

   def run(self):
      self.update()
      self.parent.after(500, self.run)

   def update(self, fp = None, mask = None):
      nrec = self.binfile.count()
      print "nrec, self.nrec", nrec, self.nrec
      dday_now = (time.time() - epoch_t)/86400.
      if nrec > self.nrec:
         records = self.binfile.read_records(self.nrec, nrec - self.nrec)
         if len(records):
            t = []
            hpr = [[], [], []]
            for ii in range(len(records)):
               if records[ii][-1] == 0.0:
                  t.append(records[ii][0])
                  hpr[0].append(records[ii][2])
                  hpr[1].append(records[ii][3])
                  hpr[2].append(records[ii][4])
            self.nrec = self.nrec + len(records)
            self.chart.update_data(t, hpr, dday_now)

######################################################################
# Create demo in root window for testing.
if __name__ == '__main__':
   root = Tkinter.Tk()
   Pmw.initialise(root, fontScheme = 'pmw1')
   root.title('Ashtech Pitch, Roll, Heading')
   widget = Monitor(root)
   widget.start('/mnt/peale1/binfiles/adu2/nbp2001_106_00000.adu.bin')
   root.mainloop()
   #widget.disconnect()
