#!/usr/bin/env python

''' Continuous incremental translation of ascii to binary files.

    This is a driver for the serasc2bin routines, providing
    multi-threaded incremental processing of ascii files as
    they are being logged, always working on the most recent
    file.

    2004/06/13 EF: changed gzipping strategy; note that it
    requires a change in the fileglob.

'''

import sys, os, glob, threading, time
import serasc2bin

sys.path.append('../py_common')
from threadgroup import *



class Translate(ThreadGroup):
   default_opts = { 'update' : 1,
                    'redo'   : 0,
                    'gzip'   : 0,
                    'maxsleeps' : 10,
                    'sleepseconds': 2,
                    'dir_asc_base':'',
                    'dir_bin_base':'',
                    'fileglob': '*_???_?????.???',  # 3-char extension
                    'subdir_msgs':{'ashtech': ('adu', 'gga'),
                                   'gyro': ('hdg',)}}


   def __init__(self, **kw):
      ThreadGroup.__init__(self)
      self.opts = Translate.default_opts
      self.opts.update(kw)
      if len(self.opts) > Translate.default_opts:
         raise ValueError("Translate input options don't match defaults")
      self.dir_asc_base = self.opts['dir_asc_base']
      self.dir_bin_base = self.opts['dir_bin_base']
      self.subdir_messages = self.opts['subdir_messages']
      self.fileglob = self.opts['fileglob']
      self.sleepseconds = self.opts['sleepseconds']
      for subdir in self.subdir_messages.keys():
         dir_in = os.path.join(self.dir_asc_base, subdir)
         dir_out = os.path.join(self.dir_bin_base, subdir)
         if not os.path.isdir(dir_out):
            os.makedirs(dir_out)
         primary = 1   # flag for first thread in a subdirectory
         for message in self.subdir_messages[subdir]:
            T = threading.Thread(target = self.translate,
                              args = (dir_in, dir_out, message, primary))
            T.setDaemon(1)
            self.add(T)
            print subdir, message, primary
            print T
            primary = 0  # next one is not primary

   def translate(self, dir_in, dir_out, message, primary):
      optdict = {'outfiledir': dir_out, 'message': message,
                 'timer': self.timer}
      for key in ['update', 'maxsleeps', 'sleepseconds', 'yearbase',
                  'verbose', 'showbad', 'redo']:
         if self.opts.has_key(key):
            optdict[key] = self.opts[key]
      optdict_old = optdict.copy()
      optdict_old['update'] = 0
      print dir_out, message, primary
      first_time = 1
      while self.running:
         ascfiles = glob.glob(os.path.join(dir_in, self.fileglob))
         ascfiles.sort()
         for _file in ascfiles[:-1]:
            if first_time:
               serasc2bin.asc2bin(optdict = optdict_old).translate_files([_file])
               first_time = 0
         n = -1
         # The following was ascfiles[-1:], to do the last file
         # only.  This should be enough.  There seems to be a failure
         # mode in which rbinning stops, though, and using [-2:] should
         # provide a means of recovery with no significant penalty.
         for _file in ascfiles[-1:]:
            n = serasc2bin.asc2bin(optdict = optdict).translate_files([_file])
         ziplist = ascfiles[:-2]
         if len(ziplist) > 0 and primary and self.opts['gzip']:
            cmd = '(nice gzip %s ) &' % ' '.join(ziplist)
            print cmd
            os.system(cmd)
         if n < 1:         # Nothing new in the last file, or no file, ...
            self.timer(self.sleepseconds)  #  so wait before trying again.

      # On exit, zip anything left.
      print "translate exiting, ", message, dir_out
      ziplist = glob.glob(os.path.join(dir_in, self.fileglob))
      ziplist.sort()
      if len(ziplist) > 0 and primary and self.opts['gzip']:
         cmd = '(sleep 2; nice gzip %s ) &' % ' '.join(ziplist)
         print cmd
         os.system(cmd)


   def stop(self):
      serasc2bin.haltflag = 1
      ThreadGroup.stop(self)

   def start(self):
      serasc2bin.haltflag = 0
      ThreadGroup.start(self)


## Sample use of a Translate object:

if __name__ == '__main__':
   T = Translate(dir_asc_base = './test/raw',
                 dir_bin_base = './test/binfiles',
                 subdir_messages = {'ashtech': ('gps', 'adu')})
   T.start()
   while T.running:
      try:
         time.sleep(1)
      except:  # Ctl-C
         T.stop()


