'''
   Modified file class; its next method uses readline() instead
   of the readahead buffer in file.next(), which doesn't work in
   a "tail -f " mode.

   Gzipped files are read transparently.

'''

import os, stat, time
from pyfiletail import Filetail

class linefile1:
   def __init__(self, filename,
                timeout = 0.0, interval = 0.1, keep_running = None):
      fname_base, ext = os.path.splitext(filename)
      self.file = Filetail()
      if ext == '.gz':
         temp_file = os.tmpfile()
         temp_file.write(os.popen('zcat %s' % filename).read())
         self.file.connect(temp_file)
      else:
         self.file.open(filename)
      self.filename = filename
      self.interval = interval
      self.nloops = int(timeout / interval)
      if keep_running is not None:
         self.keep_running = keep_running
      else:
         self.keep_running = lambda : True

   def __iter__(self):
      return self

   def __del__(self):
      if self.file:
         self.file.close()

   def close(self):
      self.file.close()
      self.file = None

   def next(self):
      s = self.file.readline()
      if len(s) == 0:
         raise StopIteration
      return s

   def read_n(self, n):
      '''Read n bytes.
         Returns string, number of bytes read.
      '''
      s = ''
      n_to_read = n
      for ii in range(self.nloops):
         ss = self.file.read(n_to_read)
         ns = len(ss)
         if ns > 0:
            s += ss
            n_to_read -= ns
         if n_to_read <= 0 or not self.keep_running():
            break
         time.sleep(self.interval)
      return s, len(s)


   def readline(self):
      s = ''
      for ii in range(self.nloops):
         s = self.file.readline()
         if s or not self.keep_running():
            break
         time.sleep(self.interval)
      return s

   def readlines(self):
      lines = []
      for ii in range(self.nloops):
         lines = self.file.readlines()
         if lines or not self.keep_running():
            break
         time.sleep(self.interval)
      return lines


class linefile2s(linefile1):
   ''' Return lines a pair at a time, synchronizing by matching
       a string at the start of the first line.  The synchronization
       string is given by the kwarg 'sync'.  A KeyError will be
       raised if the kwarg is not supplied.
   '''
   def __init__(self, *args, **kwargs):
      self.sync = kwargs.pop('sync')
      linefile1.__init__(self, *args, **kwargs)
      self.lines = []

   def update(self):
      new = self.readlines()
      if len(new) == 0:
         return
      ii = len(self.lines)
      for line in new:
         if ii % 2 == 1 or line.startswith(self.sync):
            self.lines.append(line)
            ii += 1


   def read_record(self):
      if len(self.lines) < 2:
         self.update()
      if len(self.lines) < 2:
         return []
      rec = self.lines[:2]
      del(self.lines[:2])
      return rec

   def read_records(self, n = None):
      '''This may be of little use; or perhaps it should
         return a list of pairs, or a pair of lists...
      '''
      if len(self.lines) < 2:
         self.update()
      if len(self.lines) < 2:
         return []
      nrec = len(self.lines) // 2
      if n is not None:
         nrec = min(n, nrec)
      rec = self.lines[:(2*nrec)]
      del(self.lines[:(2*nrec)])
      return rec

   def next(self):
      rec = self.read_record()
      if len(rec) == 0:
         raise StopIteration
      return rec



