#!/usr/bin/python

import os, sys, shutil, pwd, string, glob
from optparse import OptionParser


usage = string.join([\
  "Start in the cruise directory, with subdirectories raw and rbin.\n"
  "Sweep through all (or specified) raw directories,\n",
  "making rbin files in the parallel 'rbin' directory\n",
  " -y, --yearbase        required\n",
  " -l, --log_yearbase    for old Palmer data\n",
  " -d, --uhdas_dir       path to directory containing raw,rbin\n",
  " -i, --ser_inst        specify one instrument  ('ashtech', 'posmv',....)\n",
  " -m, --message         specify message ('pmv', 'gps'...)\n",
  " -t, --type            logger type: 'uh' (default, uses asc2bin.py;\n",
  "                                    'olduh', 'vmdas', 'rvdas' (uses ser_asc2bin.py)\n",
  " -o, --outdir          output directory (default is uhdas_dir/rbin/inst\n",
  "     --test            print commands to run; exit\n",
  "usage: uhdas\n",
  "%prog -d uhdas_dir -i inst_dir -m msg -o outdir -y 2005 -t uh\n",
  "\n ",
  "eg: %prog -d uhdas_dir -y 2005",'\n',
  "\n",
  "\n",
  " NOTE: If uhdas logging includes monotonic time stamp, it is type 'uh';\n",
  "       Otherwise it is 'olduh', and is called by serasc2bin.py.\n",
  ])                  
                     

parser = OptionParser(usage)
parser.add_option("-y",  "--yearbase", dest="yearbase",
                  help="yearbase (required)")
parser.add_option("-l",  "--log_yearbase", dest="log_yearbase",
                  help="log_yearbase for generation 1 polar data (usually 2003). only works with -t olduh")
parser.add_option("-d",  "--uhdas_dir", dest="uhdas_dir",
                  help="uhdas base directory, contains 'raw' and 'rbin'")
parser.add_option("-i", "--ser_inst", dest="ser_inst",
                  help="serial instrument, eg. 'ashtech', 'posmv'")
parser.add_option("-m",  "--message", dest="suffix",
                  help="message name, eg. 'gps' or 'pmv' for posmv")
parser.add_option("-t",  "--type", dest="loggertype",
                  help="logger type: 'uh', 'olduh', 'vmdas', 'rvdas'")
parser.add_option("-o",  "--outdir", dest="outdir",
                  help="output directory: default is parallel rbin directory")
parser.add_option("--test", dest="test",
                  action="store_true",  default= False)

(options, args) = parser.parse_args()

if options.uhdas_dir == None:
   parser.error('must specify uhdas directory')
if options.yearbase == None:
   parser.error('must specify yearbase')

if options.loggertype == None:
   loggertype = 'uh'
else:
   loggertype = options.loggertype
   
## fixed values
if loggertype == 'uh':
   parser = 'asc2bin.py '
   gunzip      = 0
   gzip        = 0
else:
   gunzip      = 1
   gzip        = 1
   if loggertype == 'olduh':
      parser  = 'serasc2bin.py -t uh '
   else:
      parser =  'serasc2bin.py -t %s ' % (loggertype)
   
adcps       = ('os38', 'os75', 'wh300', 'nb150')

## possible ser_insts and extensions (list)
path_ext ={}
path_ext['gyro']     =   ['hdg',]            # gyro heading
path_ext['ashtech']  =   ['adu', 'gps']      # [attitudes, positions] from adu
path_ext['posmv']    =   ['pmv', 'gps']      # [attitude, positions]  from posmv
path_ext['seapath']  =   ['sea', 'gps_sea']  # [attitude, position]   from seapath
path_ext['gpsnav']   =   ['gps',]            # positions from some gps sensor
path_ext['simrad']   =   ['gps',]            # positions from simrad
path_ext['sndspd']   =   ['spd',]            # sound speed sensor

## old directory name       ## new directory name
oldpath_key = {}
oldpath_key['adu2']     =   'ashtech'    # positions from adu
oldpath_key['pcode']    =   'gpsnav'    # positions from pcode


# variables from options
yearbase    = options.yearbase
log_yearbase = yearbase
if options.log_yearbase != None:
   log_yearbase = options.log_yearbase
uhdas_dir   = options.uhdas_dir
ser_inst    = options.ser_inst
outdir      = options.outdir

if __name__ == '__main__':

  cruiseid = os.path.basename(uhdas_dir)
  rawdir = os.path.join(uhdas_dir,'raw')
  if outdir == None:
    outdir = os.path.join(uhdas_dir,'rbin')
  else:
    if not os.path.exists(outdir):
      print 'cannot find specified output directory %s' % (outdir)
      print 'directory must exist before this program will work'
      sys.exit()
  print 'about to write rbin files to subdirectories of %s' % (outdir)
        
  
  inst_dirs = []
  if ser_inst == None:
    # find all possible
    testdirs = glob.glob(os.path.join(uhdas_dir,'raw','*'))
    for ii in range(0,len(testdirs)):
      testdir = os.path.basename(testdirs[ii])
      if testdir not in adcps and testdir in path_ext.keys():
        inst_dirs.append(testdir)
        print 'found instrument directory: %s' % (testdir)
      if testdir in oldpath_key.keys():
        print 'found OBSOLETE instrument directory name: %s' % (testdir)
        print 'rename %s to %s before continuing\n' % \
              (testdir, oldpath_key[testdir])
        sys.exit()
  else:
    # use the one given, check first
    inst_dir = os.path.join(rawdir, ser_inst)
    if os.path.exists(inst_dir):
      inst_dirs = [os.path.basename(inst_dir),]
      print 'found instrument directory: %s' % (inst_dir)
    else:
      print 'cannot find specified instrument directory %s' % (ser_inst)
  
  for inst_dir in inst_dirs:
    testdir = os.path.join(outdir, inst_dir)
    if os.path.exists(testdir):
       print 'not making %s' % (testdir)
    else:
       print 'making %s' % (testdir)
       if not options.test:
          os.mkdir(testdir)
  
  ## 
  for diri in range(0,len(inst_dirs)):
    inst_dir = inst_dirs[diri]
    full_instdir = os.path.join(rawdir, inst_dir)
    filelist = glob.glob(os.path.join(full_instdir, '*'))
    filelist.sort()
  
    outdir_inst = os.path.join(outdir, inst_dir)
    if os.path.exists(outdir_inst):
      print 'not making directory ', outdir
    else:
      print 'making  directory %s' % (outdir_inst)
      if not options.test:
         os.mkdir(outdir)
        
  
    if gunzip == 1:
      ## gunzip first
      cmd = 'gunzip %s' %  (os.path.join(full_instdir,'*.gz'))
      print 'about to run: %s' % (cmd)
      if not options.test:
         os.system(cmd)  
  

    if options.suffix == None:
       suffixes = path_ext[inst_dir]
    else:
       suffixes = [options.suffix,]

    for msg in suffixes:
      print 'message is ', msg
      if parser == 'asc2bin':
         cmd = '%s -r  -y %s -m %s  -o %s %s/*' % \
               (parser, yearbase, msg, outdir_inst, full_instdir)
      else:
         cmd = '%s -r  -y %s -l %s -m %s  -o %s %s/*' % \
               (parser, yearbase, log_yearbase, msg, outdir_inst, full_instdir)
         
      print 'about to run: %s' % (cmd)
      if not options.test:
         os.system(cmd)
  
    if gzip == 1:
      ## gzip after
      cmd = 'gzip %s' %  (os.path.join(full_instdir,'*'))
      print 'about to run: %s' % (cmd)
      if not options.test:
         os.system(cmd)
      
  
  
  
        
  
