
import struct, sys, os, os.path
from ascfile import ascfile

''' Low level access to UHDAS data recorded from an RDI NB.

    The header is decoded to locate the velocity data,
    which is then returned as a string, to be decoded by another
    module (e.g., nbspeed.pyx).
'''


class PingParser:
    '''Hold a single ping of binary data, and provide the
       velocity part as a binary string.
    '''
    def __init__(self):
        self.offV = None  # sanity check flag
        self.data = None

    def decode_header(self, s):
        ''' Get and save the offsets from a header.
            s is a block of binary data, including at least the header.
        '''
        (nb, nL, nV, nSW, nA, nPG, nS) = struct.unpack('>7H', s)
        self.offV = 14 + nL
        self.nV = nV
        self.offSW = self.offV + nV
        self.nSW = nSW
        self.offA = self.offSW + nSW
        self.nA = nA
        self.offPG = self.offA + nA
        self.nPG = nPG
        self.offS = self.offPG + nPG
        self.nS = nS

    def set_data(self, s):
        '''Set the data to 's' and decode the header.
           Header is decoded only if it has not been done once.
           There is no checking to see if it has changed.
        '''
        self.data = s
        if self.offV is None:
            self.decode_header(s[0:14])

    def get_V(self):
        '''Return the velocity part of the data block,
           still as a binary string with no decoding.
        '''
        return self.data[self.offV:(self.offV + self.nV)]

    def get_V_stream(self, s):
        '''Decode header and return velocity;
           for streams where the header may change.
        '''
        self.data = s
        self.decode_header(s[0:14])
        return self.data[self.offV:(self.offV + self.nV)]

class PingFromRawLog(ascfile):
    '''Use a UHDAS raw log file to access the raw data, one ping at a time.
    '''
    def __init__(self, filename):
        '''The .raw.log filename must be provided; the .raw filename
           will be derived from it.
        '''
        ascfile.__init__(self, filename)
        self.rawfilename = os.path.splitext(filename)[0] # chop off the .log
        self.rawfile = open(self.rawfilename, 'rb')

    def __del__(self):
        if self.rawfile:
            self.rawfile.close()
        ascfile.__del__(self)


    def next(self):
        #(udday, offset, nbytes, ens_num, inst_dday) = ascfile.next(self)
        fields = ascfile.next(self)
        (udday, offset, nbytes, ens_num) = fields[:4]
        self.rawfile.seek(int(offset))
        ens = self.rawfile.read(int(nbytes))
        #print int(ens_num)
        return ens


if __name__ == "__main__":
    from nbspeed import get_speed
    nb = PingFromRawLog(sys.argv[1])
    if len(sys.argv) > 2:
        H = float(sys.argv[2])
    else:
        H = 0.0
    nbp = PingParser()

    count = 0
    for record in nb:
        nbp.set_data(record)
        v = nbp.get_V()
        (U, V, n) = get_speed(v, 0, 50, H)
        print U, V, n
        count += 1
        if count == 5:
            break
