#!/usr/bin/python

''' Binfile class, with read and write methods.
    (Write methods are incomplete.)

    A binfile is a very simple file format for storing 2-D arrays
    of double precision numbers, with a minimal self-describing
    ascii header.

   See also comments at bottom.

'''

import string, os, stat, sys, struct

_str = '''Binfile %(name)s with %(IOmethod)s IO, mode %(mode)s,
    byteorder %(byteorder)s,
    %(count)d records of %(ncolumns)d fields:
        '''

class binfile_s:
   '''Uses Python byte strings for input/output. Does *not* take
      care of byte-swapping.  See the binfile subclass.
   '''
   def __init__(self, filename, mode = 'r', name = '', columns = []):
      self.filename = filename
      file_exists  = os.path.exists(filename)
      assert mode == 'r' or mode == 'w'
      self.file = file(filename, mode + 'b') # for Windows
      self.mode = mode
      self.name = name
      self.columns = columns
      self.ncolumns = len(columns)
      if mode[0] == 'r':
         self.byteorder = "unknown"
      else:
         self.byteorder = sys.byteorder
      self.nl_header = self.ncolumns + 2 # first line, columns, byteorder
      if mode == 'r':
         self._read_header()
         if self.byteorder == sys.byteorder or self.byteorder == "unknown":
            self.sfmt_prefix = '@'
         elif self.byteorder == 'little':
            self.sfmt_prefix = '<'
         elif self.byteorder == 'big':
            self.sfmt_prefix = '>'
         else:
            raise ValueError, 'unrecognized byteorder string: %s' % self.byteorder
      else:
         self._write_header()
      self.record_length = 8 * self.ncolumns
      self.cursor = 0  # for sequential reading of single records
      self.fmt = self.sfmt_prefix + '%dd' % self.ncolumns
      self.IOmethod = 'string'
      self._fstr = _str + '\n        '.join(self.columns) + '\n'

   def __iter__(self):
      return self

   def __del__(self):
      if self.file:
         self.file.close()

   def __str__(self):
      d = self.__dict__.copy()
      d['count'] = self.count()
      return self._fstr % d

   def close(self):
      self.file.close()
      self.file = None

   def _read_header(self):
      line1 = self.file.readline()
      fields = string.split(line1)
      self.ncolumns = string.atoi(fields[0])
      self.nl_header = string.atoi(fields[1])
      self.name = fields[2]
      self.columns = []
      for ii in range(self.ncolumns):
         self.columns.append(string.split(self.file.readline())[0])
      nleft = self.nl_header - self.ncolumns - 1
      if nleft:
         self.byteorder = self.file.readline().strip()
         nleft -= 1
         #assert nleft == 0
      for ii in range(nleft):   # nleft = 0 with present version.
         self.file.readline()
      self.offset = self.file.tell()

   def _write_header(self):
      self.file.write('%d %d %s\n' % (self.ncolumns, self.nl_header, self.name))
      for r in self.columns:
         self.file.write(r + '\n')
      self.file.write('%s\n' % sys.byteorder)
      self.file.flush()
      self.offset = self.file.tell()

   def write(self, s):
      nrec = len(s) // self.record_length
      assert nrec * self.record_length == len(s)  # quick error check
      self.file.write(s)
      self.file.flush()
      self.cursor += nrec
      return nrec

   def seek(self, index, whence=0):
      self.file.seek(0, 2)
      nrec = (self.file.tell() - self.offset) // self.record_length
      if whence == 0:        # relative to start
         irec = index
      elif whence == 1:      # relative to cursor
         irec = self.cursor + index
      elif whence == 2:      # relative to end
         if index > 0:       # index should be negative
            raise IndexError
         irec = nrec + index
      else:
         raise ValueError
      irec = min(irec, nrec)
      irec = max(irec, 0)
      self.file.seek(self.offset + irec * self.record_length)
      self.cursor = irec

   def next(self):
      s = self.next_string()
      if len(s) == 0:
         raise StopIteration
      return s
      #return struct.unpack(self.fmt, s)

   def read(self, n = None):
      '''Read n records, or to end of file. '''
      if n is None:   # Or we could just set it to a large number.
         cursor = self.cursor
         self.file.seek(0,2)
         n = (self.file.tell() - self.offset) // self.record_length
         n -= cursor
         self.seek(cursor)
      s, nrec = self.next_string_n(n)
      return s
      #return struct.unpack('%dd' % self.ncolumns * nrec, s)


   def next_string(self):
      s = self.file.read(self.record_length)
      if len(s) != self.record_length:
         self.file.seek(self.offset + self.cursor * self.record_length)
         return ''
      self.cursor += 1
      return s

   def next_string_n(self, n=1):
      nbytes = n * self.record_length
      s = self.file.read(nbytes)
      nrec = len(s) // self.record_length
      if nrec < n:
         self.file.seek(self.offset + (self.cursor + nrec) * self.record_length)
         s = s[:nrec*self.record_length]
      self.cursor += nrec
      return s, nrec

   def count(self):
      ''' Return the number of complete records in the file. '''
      if self.mode == 'w': # It may not make sense to call count in this case.
         self.file.flush()
      nbytes = os.fstat(self.file.fileno())[stat.ST_SIZE]
      return (nbytes - self.offset) / self.record_length

class binfile(binfile_s):
   '''Uses lists for input/output, takes care of byte order'''
   def __init__(self, *args, **kwargs):
      binfile_s.__init__(self, *args, **kwargs)
      self.IOmethod = "list"

   def next(self):
      s = binfile_s.next(self)
      return struct.unpack(self.fmt, s)

   def read(self, n = None):
      '''Read n records, or to end of file. '''
      s = binfile_s.read(self, n)
      nrec = len(s) // (self.ncolumns * 8)
      return struct.unpack(self.sfmt_prefix + '%dd' % self.ncolumns * nrec, s)

   def write(self, rec):
      s = struct.pack('%dd' % len(rec), *rec)
      return binfile_s.write(self, s)



## If needed, we can add classes for Numeric, numarray, and
## array IO.

def open(filename, mode = 'r', name = '', columns = []):
   return binfile(filename, mode, name, columns)

def open_s(filename, mode = 'r', name = '', columns = []):
   return binfile_s(filename, mode, name, columns)

####################################################


from optparse import OptionParser

def main():

   parser = OptionParser()
   parser.add_option("-e", "--ends", action="store_true", dest="ends",
                     help="print first, last record", default=False)
   parser.add_option("-c", "--columns",
                     action="store_true", dest="header", default=False,
                     help="print header, including column labels")

   parser.add_option("-a", "--all",
                     action="store_true", dest="all", default=False,
                     help="print all records")

   (options, args) = parser.parse_args()


   for fn in args:
      b = binfile(fn)
      fmt = " %12.6f" * b.ncolumns
      if options.header:
         print b
      if options.ends:
         print fmt % b.read(1)
         b.seek(-1, 2)
         print fmt % b.read(1)
      if options.all:
         b.seek(0)
         #for record in b:
         #   print fmt % record
         # The previous method appears to be about as fast as
         # the following; the limiting factor may be the
         # print statement, or it may be that slicing is
         # slow.
         records = b.read()
         nrec = len(records) // b.ncolumns
         for irec in range(0, nrec):
            print fmt % records[irec*b.ncolumns:(irec+1)*b.ncolumns]

if __name__ == "__main__":
   main()

