#!/usr/bin/python

'''

   Usage: serasc2bin.py [<options>] [-y 2001] ...
                   [-o outputdirectory] file1 [file2 ...]

      Reads ascii files written by RVDAS (Palmer and Gould)
      or UH logging system (we can add others later)
      and writes corresponding files
      in binary double precision.  Output file name is the
      input file name with '.rbin' appended, unless the
      --message and/or --add_extension options are used.
      By default,
      if a file of that name already exists, the corresponding
      input file will not be processed.  Use '--redo' option
      to reprocess in this case.  Use '--update' to append to
      an rbin file if it already exists.  This can be used to
      continually update an rbin file as the source file is
      being written.

      Gzipped input files are handled transparently, without
      being explicitly gunzipped; the '.gz' extension is ignored
      when generating the output filename, unless the '-a' option
      is used.

      -h or --help:     print this message and exit
      -r or --redo:     process all input files, regardless of
                           whether an output file already exists
      -u or --update:   process incrementally
      -n or --nsleep:   with -u, number of sleep cycles before
                           timing out
      -s or --seconds:  with -u, sleep time in seconds (floating point)
      -o or --outdir:   directory for output files
      -m or --message:  desired output file type (without the dot)
      -y or --yearbase: 4-digit yearbase
      -l or --log_yearbase" 4-digit yearbase (yearbase used for $UNIXD
                            during logging). defaults to same as
                            yearbase if unspecified
      -v or --verbose:  print short diagnostics
      -V or --Verbose:  echo line number and line, so we can see
                           where it chokes
      -S or --showbad:  echo line number and line when they are
                           rejected, so we can see what junk is in the
                           file
      -a or --add_extension: Add the message extension to the input
                           file name, instead of replacing the input
                           file extension
      -t or --type:     'rvdas', 'uh', 'vmdas',...  ('uh' is default)
      -c or --count:   for vmdas only:
                       VmDAS may have more than one serial message per
                       timestamp (PADCP); the timestamp comes at the
                       end of a group.  Use "--count" to specify which
                       messages to store.  The field "count" starts with
                       zero just prior to the PADCP message and decreases
                       backwards in time.
                       count switch:     outputs
                         first         (output the serial message at
                                            the beginning of the collection
                                            for the ensemble;
                                            corresponds to raw data
                                            structure nav.txy1)
                         last           (output the serial message at the
                                            end (nav.txy2)).  DEFAULT
                         all            (output all serial messages)


      NOTE: the first entry, 'u_dday', is the time of the PC doing the logging.
      For UHDAS, this is the linux clock time
      For rvdas, it is a GGA timestamp
      For VmDAS, it is the PC clock time.  Subtract sec_pc_minus_utc to get UTC


      File or message types:

         gps  GGA messages.  .gps.bin fields are logger decimal day,
                                    GGA decimal day, lon degrees,
                                    lat degrees.
                                    ??GGA message (e.g., INGGA or GPGGA)
         gga  same as gps
         spd  soundspeed     .spd.bin fields are logger decimal day,
                                    soundspeed in m/s.

         hdg  gyro heading   .hdg.bin fields are logger decimal day,
                                    heading in degrees.
                                    HEHRC or HEHDT messages are read.

         adu  Ashtech $PASHR,ATT    .adu.bin fields are logger decimal day,
                                    ATT decimal day (GPS days in week),
                                    heading, pitch, roll, mrms, brms,
                                    reacq flag.

         pmv  POS/MV $PASHR   .pmv.bin fields are logger decimal day,
                                    GGA decimal day, heading, pitch,
                                    roll, heave, accuracy of roll,
                                    of pitch, of heading, heading flag,
                                    IMU flag.
                                    GAMS flag: (0=no aiding, 1=GPS
                                       aiding, 2 = GAMS aiding)
                                    IMU flag: (0 = no IMU; 1 = using IMU )

         rdi (various) $PRDID  .rdi.bin fields are logger decimal day,
                                    pitch, roll, heading.
                                    There is no error checking.

         psxn20 Seapath $PSXN,20   logger decimal day,
                                    quality for:  horizontal, height,
                                    heading, roll&pitch.

         psxn23 Seapath $PSXN,23   logger decimal day,
                                    roll, pitch, heading, heave

         sea                       logger decimal day,
                                    roll, pitch, heading, heave,
                                    quality for:  height,
                                    heading, roll&pitch.  (The psxn23
                                    fields preceed the psxn20 fields.)

         gps_sea                   same as gga, but with horizontal quality
                                   column from psxn20 appended

   Note: there is a slight difference in the logger decimal day for the
   Seapath gga file versus the gps_sea.  In the latter, the time stamp
   is the one from the psxn20 message, which arrives immediately after
   the gga.  Fixing this would be more trouble than it is worth.

'''
# 2004/09/30 EF: added sys.byteorder to binfile header,
#    to match binfile.py, write_bin.m, and read_bin.m,
#    all of which were modified to handle non-native order

# 2004/09/24 EF: switched from dday_frac to dday;
#   eliminate duplicate times; added dday to sea record

# 2004/09/18 EF: cleanup, documentation tweaks;
#   transparent handling of gzipped input files;
#   changed order of columns in vmdas output;
#   combined handling of psxn20 and psxn23.
#   Added hdop field to gga.
#   Added log_yearbase option from JH version;
#   fixed yearbase bug in get_vmdas_dday.
#   Added gps_sea combined message.


# 2003/11/01 JH: added vmdas

# 2003/06/25 EF: merged changes from rvdas; the implementation
# of the different file reading functions might be improved
# to reduce duplication of code, but we have a working framework
# for reading any files of this sort, e.g. SCS files as well
# as rvdas.

# 2003/06/23 EF: rvdas version working.
#  profiling shows that the checksum routine is the biggest
#  single time sink.
#  Added the Pyrex version of the checksum routines; ~20% speedup.
#
#  Changed variable name "str" to "str_" to avoid conflict
#  with built-in.


# 2002/08/18 EF: Added pmv support; generalized gga; --message option.
# bug in translate_file_incr fixed 2001/04/16


import sys, os, array, string, time, gzip, copy

def usage():
   print __doc__
   sys.exit()

def split_rvdas(line):
   h, t = line.split(' ', 1)
   return h.strip(), t.strip()

def get_rvdas_dday(timetag, yearbase):
   #uses globals: epoch_t
   parts = timetag.split(':', 3)
   yr2, yd = parts[0].split('+')
   year = 2000 + int(yr2)
   yd = int(yd)
   hr = int(parts[1])
   mn = int(parts[2])
   sec = float(parts[3])
   dsec = hr * 3600 + mn * 60 + sec
   dday = (yd - 1) + dsec / 86400.0
   if year != yearbase:
      yearbase_dif_days = round((time.mktime((year, 1, 1, 0, 0, 0, 0, 0, 0)) -
                           epoch_t) / 86400.0)
      dday = dday + yearbase_dif_days
   return dday

def get_vmdas_dday(str_, yearbase):
   if str_[:7] != '$PADCP,':
      return None
   #uses globals: epoch_t
   ### timetag is:   $PADCP,2137,20030909,000148.26,-148.11
   parts = str_.split(',')
   if len(parts) != 5:
      return None

   ens_num = parts[1]
   ymd = parts[2]
   hms = parts[3]
   sec_pc_minus_utc = float(parts[4])
   year = int(ymd[0:4])
   mon =  int(ymd[4:6])
   day =  int(ymd[6:8])
   hr =   int(hms[0:2])
   mn = int(hms[2:4])
   sec = int(hms[4:6])
   sec100 = int(hms[7:])

   # PC time
   dday = float(time.mktime((year, mon, day, hr, mn, sec, 0, 0, 0)) +
                sec100/100.0 -  epoch_t)/86400.0
   return  [dday, ens_num, sec_pc_minus_utc]


## Here and in checksums.pyx, the use of ValueError is convenient
## but not particularly good practice; it would be better to define
## a ChecksumError so as to distinguish between bad checksums and
## other reasons for failure in the message parsing.

## The general strategy is that the message reading functions return
## None if the requested message is not the one in the string, and
## raise an exception if it is the right message but fails in any other
## way.  This includes checksum errors and missing fields.

try:
   from checksums import NMEA as NMEAchecksum
except ImportError:
   def NMEAchecksum(str_):
      if len(str_) < 10:
         raise ValueError
      bytes = array.array('B')
      bytes.fromstring(str_[1:-3])   # exclude the $ and the *
      cs = string.atoi('0x' + str_[-2:], 16) # May want to change for Py 2.x
      bcs = reduce(lambda x, y: x ^ y, bytes)
      if cs != bcs:
         raise ValueError

try:
   from checksums import HERC as HERCchecksum
except ImportError:
   def HERCchecksum(str_):
      if len(str_) < 10:
         raise ValueError
      bytes = array.array('B')
      bytes.fromstring(str_[:-3])   # exclude the * but include the $
      cs = string.atoi('0x' + str_[-2:], 16) # May want to change for Py 2.x
      bcs = reduce(lambda x, y: x ^ y, bytes)
      if cs != bcs:
         raise ValueError

def frac_to_dday(u_dday, dday_frac):
   return u_dday + divmod(dday_frac - u_dday + 0.5, 1)[1] - 0.5

def hms_to_sec(hms):
   hh = float(hms[0:2])
   mm = float(hms[2:4])
   ss = float(hms[4:])
   return ss + 60.0 * (mm + 60.0 * hh)

def dm_to_deg(dm, let):
   idec = string.find(dm, '.')
   deg = float(dm[:(idec-2)])
   mn = float(dm[(idec-2):])
   if let in 'SW':
      s = -1
   else:
      s = 1
   return s * (deg + mn/60.0)

def get_degrees(str_):
   if str_[:6] == '$HEHRC':
      HERCchecksum(str_)
      return [float(str_[6:11])/100.0, ]
   elif str_[3:7] == 'HDT,':
      NMEAchecksum(str_)
      return [float(string.split(str_, ',')[1]), ]
   else:
      raise ValueError

def get_spd(str_):
   return [float(str_), ]

def get_UNIX_dday(str_):
   # uses globals: epoch_t and yearbase_dif_days
   if str_[:7] == '$UNIXD,':
      log_dday = float(str_[7:])  # this was dday for log_yearbase
      return log_dday + yearbase_dif_days
   elif str_[:7] == '$UNIXT,':
      ut = float(str_[7:])
      return (ut - epoch_t)/86400.0
   else:
      raise ValueError

def get_gga(str_):
   ''' returns [t, x, y, qual, hdop] in days, decimal degrees'''
   if str_[3:7] != 'GGA,':
      return None
   NMEAchecksum(str_)
   fields = string.split(str_[7:], ',')
   hms, latdm, ns, londm, ew, qual, nsat, hdop = fields[:8]
   return [hms_to_sec(hms)/86400.0, dm_to_deg(londm, ew),
                               dm_to_deg(latdm, ns), int(qual), float(hdop)]

def get_rdi(str_):
   ''' returns [pitch, roll, heading]'''
   if str_[:7] != '$PRDID,':
      return None
   #NMEAchecksum(str_)
   # chop off the checksum till we write the code for it
   str_ = str_[:-3]
   pitch, roll, heading = string.split(str_[7:], ',', 3)
   return [float(pitch), float(roll), float(heading)]

def get_att(str_):
   if str_[:11] != '$PASHR,ATT,':
      return None
   NMEAchecksum(str_)
   f = map(float, string.split(str_[11:-3], ','))
   f[0] = f[0] / 86400.0
   return f

def get_posmv_pashr(str_):
   if str_[:7] != '$PASHR,':
      return None
   NMEAchecksum(str_)
   fields = string.split(str_[7:-3], ',')
   del fields[2]
   f = map(float, fields[1:])
   day = hms_to_sec(fields[0])/86400.0
   return [day,] + f

def get_psxn20(str_):
   if str_[:9] != '$PSXN,20,':
      return None
   NMEAchecksum(str_)
   fields = string.split(str_[9:-3], ',')
   f = map(float, fields)
   return f

def get_psxn23(str_):
   if str_[:9] != '$PSXN,23,':
      return None
   NMEAchecksum(str_)
   #checksums.NMEA(str_)
   fields = string.split(str_[9:-3], ',')
   f = map(float, fields)
   return f


def write_header(src, rows):
   h = []
   nrows = len(rows)
   nlines = nrows + 2   # could change if we put in more info
   h.append('%(nrows)d %(nlines)d %(src)s' % vars())
   for r in rows:
      h.append(r)
   h.append(sys.byteorder)
   return string.join(h, '\n') + '\n'

# Following are three dictionaries, each of which has the
# output file types as keys.

function_dict = {'gps': get_gga, # alias for gga
                 'gga': get_gga,
                 'spd': get_spd,
                 'hdg': get_degrees,
                 'adu': get_att,
                 'rdi': get_rdi,
                 'pmv': get_posmv_pashr,
                 'psxn20' : get_psxn20,
                 'psxn23' : get_psxn23,
                 'sea' : None, # These will be filled in later,
                 'gps_sea' : None}  # when an instance copy of the
                                     # dictionary is made. The get_sea
                                     # function needs to store state
                                     # in the instance.

# for uhdas and rvdas
field_dict1 = {'gps': ['u_dday', 'dday', 'lon', 'lat', 'quality', 'hdop'],
              'gga': ['u_dday', 'dday', 'lon', 'lat', 'quality', 'hdop'],
              'pmv': ['u_dday', 'dday', 'heading',
                      'roll', 'pitch', 'heave',
                      'acc_roll', 'acc_pitch', 'acc_heading',
                      'flag_GAMS', 'flag_IMU'],
              'spd': ['u_dday', 'sndspd'],
              'hdg': ['u_dday', 'heading'],
              'adu': ['u_dday', 'dday', 'heading',
                                'pitch', 'roll', 'mrms', 'brms',
                                'reacq'],
              'rdi': ['u_dday', 'pitch', 'roll', 'heading'],
              'psxn20' : ['u_dday', 'horiz_qual', 'height_qual',
                          'head_qual', 'rp_qual'],
              'psxn23' : ['u_dday', 'roll', 'pitch', 'heading', 'heave'],
              'sea' : ['u_dday', 'dday', 'roll', 'pitch', 'heading', 'heave',
                           'height_qual', 'head_qual', 'rp_qual'],
              'gps_sea' : ['u_dday', 'dday', 'lon', 'lat',
                           'quality', 'hdop', 'horiz_qual']
                           }


# For VMDAS include three extra pieces of information from the $PADCP string.
# 'count' is message index relative to the $PADCP, such that 0 means
# the message immediately before the string, -1 means the message before
# that, etc.  Which message or messages is/are provided depends on the
# 'count' argument to the asc2bin initializer, which is used only when
# parsing VMDAS data.

field_dictN = copy.deepcopy(field_dict1)
for value in field_dictN.values():
   #value[1:1] = ['count', 'ens_num', 'sec_pc_minus_utc']
   value += ['count', 'ens_num', 'sec_pc_minus_utc']



haltflag = 0  # for interrupting the incremental conversion

class asc2bin:
   def __init__(self, yearbase = None, log_yearbase = None,
                  verbose = 0, showbad = 0,
                  outfiledir = './', redo = 0, update = 0,
                  maxsleeps = 2, sleepseconds = 10, message = None,
                  count = 'last', optdict = {}):
      self.yearbase = yearbase
      self.log_yearbase = log_yearbase
      self.verbose = verbose
      self.showbad = showbad
      self.outfiledir = outfiledir
      self.redo = redo
      self.update = update
      self.maxsleeps = maxsleeps
      self.sleepseconds = sleepseconds
      self.message = message
      self.last_dday = -100000
      self.timer = time.sleep  # default; override comes in via optdict
      self.add_extension = 0
      self.type = 'uh'
      self.count = count
      self.last_psxn20 = None
      self.last_gga = None
      self.function_dict = function_dict.copy()
      self.function_dict['sea'] = self.get_sea
      self.function_dict['gps_sea'] = self.get_gps_sea
      for k in optdict.keys():
         setattr(self, k, optdict[k])
      if self.log_yearbase == None: #it was unset
         self.log_yearbase = self.yearbase
      if self.message in ['gps', 'gga', 'pmv', 'sea', 'gps_sea']:
         self.check_dday = self.check_gga_dday
      elif self.message == 'adu':
         self.check_dday = self.check_adu_dday
      else:
         self.check_dday = lambda x: x

      if not os.access(self.outfiledir, os.F_OK):
         os.makedirs(self.outfiledir, 0775)
      global epoch_t, yearbase_dif_days
      epoch_t = time.mktime((self.yearbase, 1, 1, 0, 0, 0, 0, 0, 0))
      epoch_tlog = time.mktime((self.log_yearbase, 1, 1, 0, 0, 0, 0, 0, 0))
      yearbase_dif_days = round((epoch_tlog - epoch_t) / 86400.0)

      if self.type[:2] == 'uh':
         self.translate_file = self.translate_file2
         self.translate_file_incr = self.translate_file_incr2
         self.get_dday = get_UNIX_dday
      elif self.type == 'rvdas':
         self.translate_file = self.translate_file1
         self.translate_file_incr = self.translate_file_incr1
         self.split_line = split_rvdas
         self.get_dday = get_rvdas_dday
      elif self.type == 'vmdas':
         self.translate_file = self.translate_file3
         self.translate_file_incr = self.translate_file_incr3
         self.get_dday = get_vmdas_dday
      else:
         raise NameError, self.type

   def check_gga_dday(self, r):
      r[1] = frac_to_dday(r[0], r[1])
      if abs(r[1] - self.last_dday) < 1e-7: # around 0.01 s
         return []
      self.last_dday = r[1]
      return r

   def check_adu_dday(self, r):
      r[1] = frac_to_dday(r[0], r[1]) - 0.00013888888888888889 # 12 seconds
      if abs(r[1] - self.last_dday) < 1e-7: # around 0.01 s
         return []
      self.last_dday = r[1]
      return r


   def get_sea(self, str_):
      f = get_gga(str_)
      if f:
         self.last_gga = f
         return None
      f = get_psxn20(str_)
      if f:
         self.last_psxn20 = f
         return None
      f = get_psxn23(str_)
      if f and self.last_psxn20 and self.last_gga:
         return self.last_gga[0:1] + f + self.last_psxn20[1:]

   def get_gps_sea(self, str_):
      f = get_gga(str_)
      if f:
         self.last_gga = f
         return None
      f = get_psxn20(str_)
      if f and self.last_gga:
         return self.last_gga + f[0:1]


   def translate_file1(self, filename, outfilename):
      '''The time tag and the message are on a single line.
      '''
      # redo RVDAS
      ext = string.split(outfilename, '.')[-2]
      field_dict = field_dict1
      try:
         get_field = self.function_dict[ext]
         header = write_header(ext, field_dict[ext])
         nfields = len(field_dict[ext])
      except KeyError:
         print 'filename <%s> has unrecognized extension <%s>\n' % (outfilename, ext)
         return -1
      split_line = self.split_line
      get_dday = self.get_dday
      a = array.array('d')
      i = 0
      for line in self.open(filename):
         i += 1
         if self.verbose > 1:
            print '%d %s' % (i, line),
         try:
            timetag, nmea = split_line(line)
            fieldlist = get_field(nmea)
            if fieldlist:
               fieldlist.insert(0, get_dday(timetag, self.yearbase))
               fieldlist = self.check_dday(fieldlist)
               a.fromlist(fieldlist)
         except ValueError:
            if self.showbad:
               print '%s: BAD: %d %s' % (ext, i, lines[i]),

      fo = open(outfilename, 'wb')
      fo.write(header)
      a.tofile(fo)
      fo.close()
      return len(a) / nfields

      # Pre-allocating the full size of the array and then using
      # the insert method is slower, not faster, than using the
      # append method.


   def translate_file_incr1(self, filename, outfilename):
      '''The time tag and the message are on a single line.
      '''
      # append to RVDAS
      field_dict = field_dict1
      ext = string.split(outfilename, '.')[-2]
      try:
         get_field = self.function_dict[ext]
         header = write_header(ext, field_dict[ext])
         nfields = len(field_dict[ext])
      except KeyError:
         print 'filename <%s> has unrecognized extension <%s>\n' % (outfilename, ext)
         return -1
      split_line = self.split_line
      get_dday = self.get_dday
      f = self.open(filename, 'r')
      fo = open(outfilename, 'ab')
      fo_end = fo.tell()
      if fo_end == 0:
         # New output file: initialize.
         fo.write(header)
         fo_end = fo.tell()
         fo_start = fo_end
      else:
         # The output file already has something in it, so
         # we want to resume at the end.  We have to read the
         # contents to figure out where the new data start in
         # the input file.
         fo_read = open(outfilename, 'r')
         head = fo_read.readline()
         fo_nl = string.atoi(string.split(head)[1])
         for i in range(1, fo_nl):
            fo_read.readline()
         fo_start = fo_read.tell()
         fo_read.close()
      if self.verbose:
         print "%s: bin start: %d   end: %d" % (ext, fo_start, fo_end)
      nrecords_written = (fo_end - fo_start) / (nfields * 8)
      if self.verbose:
         print "%s: records already in the file: %d " % (ext, nrecords_written)
      lines = []
      ii = 0
      first_read = 1
      n_written = 0
      while not haltflag:             # Unconditionally halt if flagged?
         newl = f.readlines()
         if self.verbose:
            print "%s: new lines read: %d" % (ext, len(newl))   ##
         if len(newl) == 0:
            ii = ii + 1
            if ii > self.maxsleeps or haltflag:
               break
            try:
               self.timer(self.sleepseconds)
            except IOError:        # How does this occur? Signal?
               break
            continue
         ii = 0
         lines = lines + newl
         if self.verbose:
            print "%s: lines in 'lines': %d "% (ext, len(lines)) ##
         nl = len(lines)
         if nl < 1:
            continue
         a = array.array('d')
         for i in range(0, nl):
            if self.verbose > 1:
               print '%s: %d %s' % (ext, i, lines[i]),
            try:
               timetag, nmea = split_line(lines[i])
               fieldlist = get_field(nmea)
               if fieldlist:
                  fieldlist.insert(0, get_dday(timetag, self.yearbase))
                  fieldlist = self.check_dday(fieldlist)
                  a.fromlist(fieldlist)
            except:
               if self.showbad:
                  print '%s: BAD: %d %s' % (ext, i, lines[i]),
         if first_read:
            # Discard the records already in the bin file by
            # assigning the corresponding slice of a to an
            # empty array; equivalent to a[:n] = [] for a list.
            a[:(nrecords_written * nfields)] = array.array('d')
            first_read = 0
         nw = len(a) / nfields
         n_written = n_written + nw
         a.tofile(fo)
         fo.flush()
         lines[:nl] = []
         if self.verbose:
            print "%s: Records added to file: %d" % (ext, nw)
         self.timer(self.sleepseconds)

      return n_written


   def translate_file2(self, filename, outfilename):
      '''The time tag is on the line preceeding the message.
      '''
      # redo UH
      field_dict = field_dict1
      ext = string.split(outfilename, '.')[-2]
      try:
         get_field = self.function_dict[ext]
         header = write_header(ext, field_dict[ext])
         nfields = len(field_dict[ext])
      except KeyError:
         print 'filename <%s> has unrecognized extension <%s>\n' % (outfilename, ext)
         return -1
      get_dday = self.get_dday
      f = self.open(filename, 'r')
      lines = f.readlines()
      f.close()
      a = array.array('d')
      for i in range(0, len(lines)-1, 2):
         if self.verbose > 1:
            print '%d %s' % (i, lines[i]),
            print '%d %s' % (i+1, lines[i+1]),
         try:
            fieldlist = get_field(string.rstrip(lines[i+1]))
            if fieldlist:
               fieldlist.insert(0, get_dday(lines[i]))
               fieldlist = self.check_dday(fieldlist)
               a.fromlist(fieldlist)
         except ValueError:
            if self.showbad:
               print '%s: BAD: %d %s' % (ext, i, lines[i]),
               print '%s: BAD: %d %s' % (ext, i+1, lines[i+1]),
         except:
            print "Some other exception in ", filename

      fo = open(outfilename, 'wb')
      fo.write(header)
      a.tofile(fo)
      fo.close()
      return len(a) / nfields

      # Pre-allocating the full size of the array and then using
      # the insert method is slower, not faster, than using the
      # append method.


   def translate_file_incr2(self, filename, outfilename):
      '''The time tag is on the line preceeding the message.
      '''
      # append to UH
      field_dict = field_dict1
      ext = string.split(outfilename, '.')[-2]
      try:
         get_field = self.function_dict[ext]
         header = write_header(ext, field_dict[ext])
         nfields = len(field_dict[ext])
      except KeyError:
         print 'filename <%s> has unrecognized extension <%s>\n' % (outfilename, ext)
         return -1
      get_dday = self.get_dday
      f = self.open(filename, 'r')
      fo = open(outfilename, 'ab')
      fo_end = fo.tell()
      if fo_end == 0:
         fo.write(header)
         fo_end = fo.tell()
         fo_start = fo_end
      else:
         fo_read = open(outfilename, 'r')
         head = fo_read.readline()
         fo_nl = string.atoi(string.split(head)[1])
         for i in range(1, fo_nl):
            fo_read.readline()
         fo_start = fo_read.tell()
         fo_read.close()
      if self.verbose:
         print "%s: bin start: %d   end: %d" % (ext, fo_start, fo_end)
      nrecords_written = (fo_end - fo_start) / (nfields * 8)
      if self.verbose:
         print "%s: records already in the file: %d " % (ext, nrecords_written)
      lines = []
      ii = 0
      first_read = 1
      n_written = 0
      while not haltflag:             # Unconditionally halt if flagged?
         newl = f.readlines()
         if self.verbose:
            print "%s: new lines read: %d" % (ext, len(newl))   ##
         if len(newl) == 0:
            ii = ii + 1
            if ii > self.maxsleeps or haltflag:
               break
            try:
               self.timer(self.sleepseconds)
            except IOError:        # How does this occur? Signal?
               break
            continue
         ii = 0
         lines = lines + newl
         if self.verbose:
            print "%s: lines in 'lines': %d "% (ext, len(lines)) ##
         if len(lines) < 2:
            continue
         nl = (len(lines) / 2) * 2
         a = array.array('d')
         for i in range(0, nl, 2):
            if self.verbose > 1:
               print '%s: %d %s' % (ext, i, lines[i]),
               print '%s: %d %s' % (ext, i+1, lines[i+1]),
            try:
               fieldlist = get_field(string.rstrip(lines[i+1]))
               if fieldlist:
                  fieldlist.insert(0, get_dday(lines[i]))
                  fieldlist = self.check_dday(fieldlist)
                  a.fromlist(fieldlist)
            except ValueError:
               if self.showbad:
                  print '%s: BAD: %d %s' % (ext, i, lines[i]),
                  print '%s: BAD: %d %s' % (ext, i+1, lines[i+1]),
         if first_read:
            a[:(nrecords_written * nfields)] = array.array('d')
            first_read = 0
         nw = len(a) / nfields
         n_written = n_written + nw
         a.tofile(fo)
         fo.flush()
         lines[:nl] = []
         if self.verbose:
            print "%s: Records added to file: %d" % (ext, nw)
         self.timer(self.sleepseconds)

      fo.close()
      f.close()
      return n_written


   def translate_file3(self, filename, outfilename):
      '''A series of message lines preceed a time stamp line.
      This is presently specific to VMDAS data, but could be
      generalized.
      '''

      field_dict = field_dictN
      ext = string.split(outfilename, '.')[-2]
      try:
         get_field = self.function_dict[ext]
         header = write_header(ext, field_dict[ext])
         nfields = len(field_dict[ext])
      except KeyError:
         print 'filename <%s> has unrecognized extension <%s>\n' % (outfilename, ext)
         return -1
      get_dday = self.get_dday
      f = self.open(filename, 'r')
      lines = f.readlines()
      f.close()
      a = array.array('d')

      # for each field that occurs BEFORE the next time stamp,
      # store the entries in a list and
      # then print all messages with the timestamp


      msglist = []  ## accrete messages here

      for i, line in enumerate(lines):
         try:
            # Is it the message? If so, append it to msglist.
            fieldlist = get_field(string.rstrip(line))
            if fieldlist != None:
               msglist.append(fieldlist)
               continue   # Get another line.

            # It's not the message, so look for a $PADCP timestamp.
            lst =  get_dday(line, self.yearbase)
            if lst != None and msglist != []:
               # Now we have at least one message followed by
               # a timestamp, so assemble the message(s) and
               # timestamp.
               dday = lst[0]
               ens_num = lst[1]
               sec_pc_minus_utc = lst[2]
               mlistlen = len(msglist)
               if self.count == 'first':
                  numrange = (0,)
               elif self.count == 'last':
                  numrange = (mlistlen - 1,)
               else:
                  numrange = range(mlistlen)
               for num in numrange:
                  r = [dday] + msglist[num]
                  r += [ float(num - mlistlen + 1),   # 'count'
                         float(ens_num),
                         float(sec_pc_minus_utc)]
                  r = self.check_dday(r)
                  a.fromlist(r)
               msglist = []  #reinitialize msglist
         except ValueError:
            if self.showbad:
               print '%s: BAD: %d %s' % (ext, i, line),

      #now write it all out to disk

      fo = open(outfilename, 'wb')
      fo.write(header)
      a.tofile(fo)
      fo.close()
      return len(a) / nfields

   def translate_file_incr3(self, filename, outfilename):
      print 'cannot increment at this time\n'
      return -1

   def translate_files(self, filenames, **kw):
      self.__dict__.update(kw)
      #for k in kw.keys():
      #   setattr(self, k, kw[k])
      n = 0
      try:
         for filename in filenames:
            if (not self.update) or self.verbose:
               print filename
            fpath, fname = os.path.split(filename)
            fname_base, log_ext = os.path.splitext(fname)
            if log_ext == '.gz':
               fname_base, log_ext = os.path.splitext(fname_base)
               self.open = gzip.open
            else:
               self.open = open
            out_ext = self.message or log_ext[1:]

            if self.add_extension:
               outfilename = os.path.join(self.outfiledir,
                                 fname + '.' + out_ext + '.rbin')
            else:
               outfilename = os.path.join(self.outfiledir,
                                 fname_base + '.' + out_ext + '.rbin')
            if self.update:
               if self.verbose: print ' ---> ', outfilename
               n = self.translate_file_incr(filename, outfilename)
            elif self.redo or not os.access(outfilename, os.F_OK):
               if self.verbose: print ' ---> ', outfilename
               n = self.translate_file(filename, outfilename)
            else:
               if self.verbose: print 'no action'
      except KeyboardInterrupt:
         raise
      except:
         raise
      return n  # number of records written to the last file

os.environ['TZ'] = 'UTC'


def main():
   import getopt

   try:
      options, args = getopt.getopt(sys.argv[1:], 'SvVhro:y:l:un:s:m:ac:t:',
                           ['showbad', 'verbose', 'Verbose', 'help',
                            'redo', 'outdir=', 'yearbase=', 'log_yearbase=',
                            'update', 'nsleep=', 'seconds=',
                            'message', 'add_extension', 'count=',
                            'type='])
   except getopt.GetoptError:  # Before Python 2.x this was getopt.error
      usage()

   if len(args) == 0:
      usage()

   opts = {}

   for o, a in options:
      if o in ('-h', '--help'):
         usage()
      elif o in ('-r', '--redo'):
         opts['redo'] = 1
      elif o in ('-o', '--outdir'):
         opts['outfiledir'] = a
      elif o in ('-y', '--yearbase'):
         opts['yearbase'] = int(a)
      elif o in ('-l', '--log_yearbase'):
         opts['log_yearbase'] = int(a)
      elif o in ('-v', '--verbose'):
         opts['verbose'] = 1
      elif o in ('-V', '--Verbose'):
         opts['verbose'] = 2
      elif o in ('-S', '--showbad'):
         opts['showbad'] = 1
      elif o in ('-u', '--update'):
         opts['update'] = 1
      elif o in ('-n', '--nsleep'):
         opts['maxsleeps'] = string.atoi(a)
      elif o in ('-s', '--seconds'):
         opts['sleepseconds'] = string.atof(a)
      elif o in ('-m', '--message'):
         opts['message'] = a
      elif o in ('-a', '--add_extension'):
         opts['add_extension'] = 1
      elif o in ('-c', '--count'):
         opts['count'] = a
      elif o in ('-t', '--type'):
         opts['type'] = a


   t0 = time.clock()

   if 'type' in opts.keys():
      if opts['type'] == 'vmdas':
         if opts['count'] not in ('first', 'last', 'all'):
            print "option 'count' must be 'first', 'last', or 'all'"
            print "you chose '%s'" % opts['count']
            sys.exit()

   import glob
   filenames = []
   for arg in args:
      filenames += glob.glob(arg)

   asc2bin(optdict = opts).translate_files(filenames)

   t1 = time.clock()
   print 'Elapsed cpu time:   %.1f s' % (t1 - t0)


if __name__ == '__main__':
   main()

