#!/usr/bin/env python

'''
   Usage: serasc2bin.py [-hr] [-y 2001] [-o outputdirectory] file1 [file2 ...]

      Reads ascii files written by ser_asc
      and writes corresponding files
      in binary double precision.  Output file name is the
      input file name with '.rbin' appended.  By default,
      if a file of that name already exists, the corresponding
      input file will not be processed.  Use '--redo' option
      to reprocess in this case.

      -h or --help: print this message and exit
      -r or --redo: process all input files, regardless of
                     whether an output file already exists
      -o or --outdir: directory for output files
      -y or --yearbase: 4-digit yearbase
      -v or --verbose: echo line number and line, so we can see
                     where it chokes
      -s or --showbad: echo line number and line when they are
                     rejected, so we can see what junk is in the
                     file

      -m or --musthave: dummy variable for calling consistency

      File types:

          .sea  Seapath       .sea.rbin fields include are logger decimal day,
                                    GGA dday fraction, heading, pitch, roll,
                                    heave, height, lon, lat,  status

                                    Fields not included in this version are:
                                    V, U, Down, Roll_Rate, Pitch_Rate, Yaw_Rate.

'''


import sys, os, array, string, struct, time, math


# Use string.rstrip(str) to remove trailing whitespace first.

def hms_to_sec(hms):
   hh = float(hms[0:2])
   mm = float(hms[2:4])
   ss = float(hms[4:])
   return ss + 60.0 * (mm + 60.0 * hh)

def get_seabin(msg):
   LLUnits = 90/math.pow(2,30)
   HPRUnits = 90/math.pow(2,14)
   fmt = '>ciB3i6hH3hHH'
   (header, timesecs, timehun, lat, lon, height, heave, V, U, down, \
    roll, pitch, heading, roll_rate, pitch_rate, yaw_rate, status, CS) = \
   struct.unpack(fmt, msg)
   # take the colon field (hours) and make it be seconds
   ascdate = time.asctime(time.gmtime(timesecs))
   hhcmmcss = ascdate.split()[3]
   hhmmss = string.join(hhcmmcss.split(':'), '')
   secs = hms_to_sec(hhmmss) + timehun/100
   #print secs
   # check the checksum,
   # reformat time, then return everything except header and checksum
   fulllist =  [secs/86400,
                lat*LLUnits,
                lon*LLUnits,
                height/100.0,
                heave/100.0,
                V/100.0,
                U/100.0,
                down/100.0,
                roll*HPRUnits,
                pitch*HPRUnits,
                360-heading*HPRUnits,
                roll_rate*HPRUnits,
                pitch_rate*HPRUnits,
                yaw_rate*HPRUnits,
                status]
   #print fulllist
   #sys.exit()
   return   [secs/86400,
             360 - heading*HPRUnits,
             pitch*HPRUnits,
             roll*HPRUnits,
             heave/100.0,
             height/100.0,
             lon*LLUnits,
             lat*LLUnits,
             status]


def write_header(src, rows):
   h = []
   nrows = len(rows)
   nlines = nrows + 1   # could change if we put in more info
   h.append('%(nrows)d %(nlines)d %(src)s' % vars())
   for r in rows:
      h.append(r)
   return string.join(h, '\n') + '\n'




functions = {'sea': get_seabin}

headers = {'sea': write_header('sea', ['u_dday', 'dday_frac',  'heading',
                                       'pitch', 'roll', 'heave', 'height',
                                       'lon', 'lat', 'status'])}


def translate_file(filename, outfilename):
   ext = string.split(filename, '.')[-1]
   try:
      get_field = functions[ext]
      header = headers[ext]
      if verbose:
         print header
   except KeyError:
      print 'filename <%s> has unrecognized extension <%s>\n' % (filename, ext)
      return
   if (ext == 'sea'): #binary seapath data.  read log file first,
                      # get byte location from log file

      # put lines from the log file into array "alines"
      logfile =  '%s.log' % filename
      print 'opening logfile ', logfile
      fa = open(logfile, 'r')
      alines = fa.readlines()
      fa.close()

      fb = open(filename, 'r')

      # start here.
      # read the log file, get the number of bytes to read and where to start
      a = array.array('d')
      for i in range(0, len(alines)-1):
         if verbose:
            print '%d %s' % (i, alines[i]),
         try:
            [ut, startbyte, numbytes] = string.split(alines[i])
            msg = fb.read(int(numbytes))
            if len(msg) != int(numbytes):
               break
            fieldlist = get_field(msg)
            if verbose:
               print [float(ut)] + fieldlist
            if fieldlist:
               a.append(float(ut))
               a.fromlist(fieldlist)
         except ValueError:
            raise
            if showbad:
               print '%d %s' % (i, alines[i]),
               print '%d %s' % (i+1, alines[i+1]),

      fb.close()

      fo = open(outfilename, 'wb')
      fo.write(header)
      a.tofile(fo)
      fo.close()
   else:
      f = open(filename, 'r')
      lines = f.readlines()
      f.close()
      a = array.array('d')
      for i in range(0, len(lines)-1, 2):
         if verbose:
            print '%d %s' % (i, lines[i]),
            print '%d %s' % (i+1, lines[i+1]),
         try:
            fieldlist = get_field(string.rstrip(lines[i+1]))
            if fieldlist:
               a.append(get_seconds(lines[i]))
               a.fromlist(fieldlist)
         except ValueError:
            if showbad:
               print '%d %s' % (i, lines[i]),
               print '%d %s' % (i+1, lines[i+1]),

      fo = open(outfilename, 'wb')
      fo.write(header)
      a.tofile(fo)
      fo.close()

   # Pre-allocating the full size of the array and then using
   # the insert method is slower, not faster, than using the
   # append method.

def usage():
   print __doc__
   sys.exit()

import getopt

try:
   options, args = getopt.getopt(sys.argv[1:], 'm:svhro:y:',
                        ['musthave','showbad', 'verbose', 'help',
                         'redo', 'outdir=', 'yearbase='])
except getopt.GetoptError:
   usage()

if len(args) == 0:
   usage()

outfiledir = './'
redo = 0
yearbase = 2001
verbose = 0
showbad = 0
musthave = 'junk'

for o, a in options:
   if o in ('-h', '--help'):
      usage()
   elif o in ('-r', '--redo'):
      redo = 1
   elif o in ('-m', '--musthave'):
      musthave = a
   elif o in ('-o', '--outdir'):
      outfiledir = a
   elif o in ('-y', '--yearbase'):
      yearbase = int(a)
   elif o in ('-v', '--verbose'):
      verbose = 1
   elif o in ('-s', '--showbad'):
      showbad = 1

os.environ['TZ'] = 'UTC'
epoch_t = time.mktime((yearbase, 1, 1, 0, 0, 0, 0, 0, 0))
t0 = time.clock()

for filename in args:
   print filename,
   fpath, fname = os.path.split(filename)
   outfilename = os.path.join(outfiledir, fname + '.rbin')
   if redo or not os.access(outfilename, os.F_OK):
      print ' ---> ', outfilename
      translate_file(filename, outfilename)
   else:
      print 'no action'

t1 = time.clock()
print 'Elapsed time:   %.1f s' % (t1 - t0)




