function refsm_core(dbname, fixfile, yearbase, varargin)
% refsm_core(dbname, fixfile, yearbase, varargin)
% This is a wrapper to adcp_nav, which is designed to replace smoothr:
%    (i.e. adcp_nav tweaks positions to achieve a smoother ocean velocity)
% In general, a stub (refsm.m) should exist in and be run from the nav/
%    subdirectory of the adcp processing tree.  A template for refsm.m 
%    exists in the demo/nav directory though it is not implimented in the demo.
%
% refsm_core does the following:
%  - runs getmat (on the WHOLE database) to extract data (prefix is 'refsm_')
%  - load getmat output and fix file 
%  - run adcp_nav
%  - write the output to a file suitable for use with putnav
%
% varargin inputs are name, value pairs with the following defaults
%
%  name              default            what it is
% -------          -----------         ----------------------------
% min_pg                30     % only use velocity if PG greater than this
% min_maxpg             70     % skip profile if no PG exceeds this
% bl_half_width         3      % For filtering velocity.
% xy_bl_half_width      6      % For high-pass filtering position adjustments
% ref_top               1      % top bin reference layer 
% ref_bot               20     % bottom of reference layer
% ensemble_secs         300    % length if ensemble (seconds)
% fix_t_mismatch        10     % seconds between ensemble time and fix time
% min_xy_frac           0.05   % data fraction of filter weight integral,
% min_uv_frac           0.5    %   ...  _xy_ for position, _uv_ for velocity
% max_dt                310    % If a velocity profile time exceeds the
%                              %   previous one by more than this many
%                              %   seconds, there is a gap; don't use it.
% lon_split             []     % split longitude here (so it doesn't jump at 
%                              %   at the date line).  Should not be necessary
%                              %   (there is an automatic routine to try this)
% dbpath          '../adcpdb'  % path to get to dbname
% prefix          'refsm_'     % prefix for getmat output
% cleanup              1       % delete getmat output files when through
%                              %     (they can be large)
%
%
% refsm.sm is written with slightly different columns:
%    1       2      3      4      5      6    7      8
%   dday  dx(km) dy(km)  shipu  shipv   lon  lat  was_adjusted
%
%%the OLD SMOOTHR columns were:
%% [dday refu    refv   shipu shipv    lon  lat       ?]     
%
% usage: run from the nav directory in the adcp processing tree as
%    refsm_core('a0101', 'a0101.ags', 2001);
% output is "refsm.sm"

%% 2001 script, JH modified from EF original
%% 2002 05/24   JH added varargin
%% 2002 06/27   JH added unwrap_lon


cfg.min_pg           = 30;           
cfg.min_maxpg        = 70;           
cfg.bl_half_width    = 3;            
cfg.xy_bl_half_width = 6;            
cfg.ref_top          = 1;            
cfg.ref_bot          = 20;           
cfg.ensemble_secs    = 300;          
cfg.fix_t_mismatch   = ceil((10/cfg.ensemble_secs)*cfg.ensemble_secs); 
cfg.min_xy_frac      = 0.05;         
cfg.min_uv_frac      = 0.5;          
cfg.cleanup          = 1;            
cfg.max_dt           = cfg.ensemble_secs + cfg.fix_t_mismatch;
cfg.dbpath           = '../adcpdb';
cfg.prefix           = 'refsm_';
cfg.lon_split        = [];
cfg.shipuv           = [];
%---------------------------------------------------------

%test inputs
sm_filename = 'refsm.sm';


if (~exist('dbname'))
   error('variable "dbname" not set: perhaps refsm.m is out of date?')
end



if (exist([fullfile(cfg.dbpath,dbname),'dir.blk']) ~= 2)
   error(sprintf('codas database %s does not exist\n', dbname))
end

if (exist(fixfile) ~=2)
   fprintf('fix file should be output from ubprint (*.ags),\n')
   fprintf('   ldcodas (*.gps1, *.gps2), or other navigation source.\n')
   error(sprintf('fix file %s does not exist\n', fixfile))
end

if ( (yearbase < 1970) | (yearbase > 2070))
   error(sprintf('yearbase should have 4 digits, probably %d is wrong\n',...
                 yearbase))
end

%add some more things to cfg structure
cfg.yearbase         = yearbase;
cfg.dbname           = fullfile(cfg.dbpath,dbname);
cfg.fixfile          = fixfile;
%

cfg = fillstruct(cfg, varargin, 0);

% get navigation
[fixes.dd, fixes.x, fixes.y] = read_asc(fixfile);
%make sure it doesn't jump at the date line
fixes.x = unwrap_lon(fixes.x, cfg.lon_split); 


% run getmat on the whole codasdb dataset
testfile = [cfg.prefix, 'u'];
if ~exist(testfile, 'file'),
   eval(['!getmat -f ', cfg.prefix, ' ',  ...
         cfg.dbname, ' 0 0 100000 ', int2str(cfg.ref_bot)]);
end

%% changed slightly for windows...
fprintf('loading getmat output\n')
SU=load([cfg.prefix, 'u.mat']);           %added 8/24/2004 TAB
SV=load([cfg.prefix, 'v']);
SPG=load([cfg.prefix, 'pg']);
SPF=load([cfg.prefix, 'pf']);
SOTHER=load([cfg.prefix, 'other']);

profs.dd = to_day(cfg.yearbase,SOTHER.TIME).';  % Row vector
profs.u = SU.U;        %clear U
profs.v = SV.V;        %clear V
profs.pg = SPG.PGOOD;   %clear PGOOD
profs.pf = SPF.PFLAG;   %clear PFLAG
%%% end of changed part

if cfg.cleanup,
   delete([cfg.prefix, '*.mat'])
end

[fixed, uv] = adcp_nav(fixes, profs, cfg);


ii = cfg.ref_top:cfg.ref_bot;


if (~isempty(cfg.shipuv))
   %%% debugging %%%
   keyboard
   profs1 = profs;
   [ss.dday, profs1.uship, profs1.vship]=read_asc(cfg.shipuv);
   
   profs1.uship =profs1.uship';
   profs1.vship =profs1.vship';
   
   %% check that tabling is not necessary
   %   profs.uship = table1([ss.dday ss.uship], profs.dd)';
   %   profs.vship = table1([ss.dday ss.uship], profs.dd)';


   [fixed1, uv1] = adcp_nav(fixes, profs1, cfg);
   uv_reflayer_adj1 = mstdgap(uv1.adj(ii, :));
   uv_reflayer_raw1 = mstdgap(uv1.raw(ii, :));
   uv_reflayership_adj1 = mstdgap(uv1.adj_uvship(ii, :));
   uv_reflayership_raw1 = mstdgap(uv1.raw_uvship(ii, :));
end   
%%% debugging %%%



%these are the adjusted and unadjusted reference layers: profiles applied first
uv_reflayer_adj = mstdgap(uv.adj(ii, :));
uv_reflayer_raw = mstdgap(uv.raw(ii, :));

sm_table = ones(8, length(profs.dd));
sm_table(1,:) = profs.dd;
%%% change these [2 and 3] from their old meanings
% make these indicate how far points have been moved (meters)
sm_table(2,:) = lon_to_m(fixed.x_adj - fixed.x, [fixed.y_av(1) fixed.y_av]);
sm_table(3,:) = lat_to_m(fixed.y_adj - fixed.y, [fixed.y_av(1) fixed.y_av]);
%the rest are the same:
sm_table(4,:) = fixed.u_adj;
sm_table(5,:) = fixed.v_adj;
sm_table(6,:) = fixed.x_adj;
sm_table(7,:) = fixed.y_adj;
sm_table(8,:) = fixed.was_adjusted;


fid = fopen(sm_filename, 'w');
ff = '%12.6f  %7.3f %7.3f  %7.3f %7.3f %12.6f %12.6f %4.2f\n';
bigstring = sprintf(ff, sm_table);
newstring = strrep(bigstring, ' NaN', '1e38');
fprintf(fid, '%s', newstring);
fclose(fid);

   