function xducerxy_core(varargin)
% xducerxy_core(varargin)
%**********
% Purpose: Correct a position fix file for the horizontal offset
%          between the gps antenna and the shipboard ADCP transducer.
%          The correction reduces or eliminates glitches in the
%          ADCP absolute reference layer velocity estimate at
%          station arrivals and departures.  The improvement is
%          evident when the offset is 10 m or more, and the fixes
%          are from a PPS (so-called P-code, or military accuracy)
%          receiver.
%
% NOTE: defaults are set to run this from the nav/ directory
%
% Input: dbname (required), varargin name, value pairs
%
%       name          what                                     default
%
%       dbpath        path to database                        ../adcpdb 
%       dbname        prefix to database                      from *dir.blk
%       fixfile       ASCII flat fix file                     [dbname '.ags']
%                     eg. output of ubprint
%                     requires [dday lon lat ....]
%       dx            - athwardships:                               0
%                        delta_x is the athwartships
%                        distance from the antenna
%                        to the transducer; positive if
%                        the transducer is to starboard
%                        of the antenna.
%       dy            - along the fore-aft axis:                    0
%                        delta_y is the forward distance 
%                        from the antenna to the transducer; 
%                        positive if the transducer is
%                        forward of the antenna, negative 
%                        otherwise.
%       enl          move the first and last ensemble times         10
%                        out by this amount in case headings
%                        do not exactly match
%       outfile      write corrected positions to this file    [dbname '.agt']
%       plotit       [0 (false) or 1 (true)] make a figure          0
%       hdgsrc       heading source.  Options:
%                    string, assumes an ascii file as from
%                       lst_hdg
%                    1 (integer), run lst_hdg                       1
%                       and read output ([dbname '.hdg'])
%                    0 (integer), read output of lst_hdg
%                       from a previous run:  ([dbname '.hdg'])
%
%       yearbase     required only if running lst_hdg               [];
%
% For instance, aboard the R/V Knorr in 1995, the antenna and the transducer
% were both mounted on the port side, with the antenna aft of the transducer.
% Usage would be 
% xducerxy_core('dx', 1.5, 'dy', 13.0, 'yearbase', 1995);
% 

% Written in March 96 by EK
% made into a function, adapted to fillstruct, JH 2003/04/29


cfg.yearbase = [];
cfg.fixfile = [];
cfg.dbpath = '../adcpdb';
cfg.dbname = [];
cfg.dx = 0;
cfg.dy = 0;
cfg.outfile = [];
cfg.plotit = 0;
cfg.hdgsrc = 1;
cfg.enl = 10;

%

cfg = fillstruct(cfg, varargin, 0);

if (isempty(cfg.dbname))
   dirblklist = dirs(fullfile(cfg.dbpath, '*dir.blk'));
   if (isempty(dirblklist))
      fprintf('no database found in path %s\n', cfg.dbpath)
      help(mfilename)
      return
   else
      if (length(dirblklist) > 1)
         fprintf('found more than one database.  Block files:\n')
         for ii=1:length(dirblklist)
            fprintf('%s\n', dirblklist(ii).name)
         end
         return
      end
   end
   % one dbname
   cfg.dbname = dirblklist(1).name;
   cfg.dbname(end-6:end) = [];
end

if isempty(cfg.fixfile)
   cfg.fixfile = [cfg.dbname '.ags'];
end

if isempty(cfg.outfile)
   cfg.outfile = [cfg.dbname '.agt'];
end

if isstr(cfg.hdgsrc)
   run_lsthdg = 0;
   if ~exist(cfg.hdgsrc,'file')
      fprintf('cannot file heading input file %s\n', cfg.hdgsrc)
      return
   end
else
   %assume it is 0 or 1
   if (cfg.hdgsrc)
      run_lsthdg = 1;
   else
      run_lsthdg = 0;
   end
   %overwrite with default name
   cfg.hdgsrc = [cfg.dbname '.hdg'];
end

if (isempty(cfg.yearbase))
   help(mfilename)
   fprintf('must set yearbase to run lst_hdg\n')
   return
end


if (run_lsthdg)
   lsthdg(cfg.dbpath, cfg.dbname, cfg.yearbase)
end
                                   
                                   
%>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>%
% position data


data = read_asc(cfg.fixfile);
tags =  data(:,1);
lon  =  data(:,2);
lat  =  data(:,3);


data = read_asc(cfg.hdgsrc);
thdg = data(:,1);
hdg = data(:,3);

% Extend range of heading times slightly:
nags = length(tags);
nhdg = length(thdg);
thdg(1) = thdg(1) - cfg.enl/86400;
thdg(nhdg) = thdg(nhdg) + cfg.enl/86400;
if ( tags(1) < thdg(1) | tags(nags) > thdg(nhdg) ),
   fprintf('extending heading times out by %d seconds\n', cfg.enl)
   fprintf('heading file start time %s, end time %s\n',...
           to_date(cfg.yearbase, thdg(1),'s'),...
           to_date(cfg.yearbase, thdg(end),'s'))

   fprintf('fix file start time     %s, end time %s\n',...
           to_date(cfg.yearbase, tags(1),'s'),...
           to_date(cfg.yearbase, tags(end),'s'))

     disp('Aborting: some ags times outside of the heading time range')
     disp('Warning: check the ags/hdg files and/or enlarge the enl variable')
     return
end;

% heading in radians, positive CCW from north
theta = - hdg * (pi/180);

% Correction and Interpolation to ags times
delta = (cfg.dx +i*cfg.dy)*exp(i*theta);
corx_m = table1([ thdg real(delta) ], tags);
cory_m = table1([ thdg imag(delta) ], tags);

% transform meters to decimal degrees
corx_deg = m_to_lon(corx_m',lat');
cory_deg = m_to_lat(cory_m',lat');

% apply correction to the fixes
loncor = lon + corx_deg';
latcor = lat + cory_deg';

%array = [ tags loncor latcor gps(:,4:7)];
txy = [ tags loncor latcor ];
fid = fopen( cfg.outfile, 'w');
fprintf('writing output to file %s\n', cfg.outfile)


fprintf(fid,'%% %s\n', nowstr);
fprintf(fid,'%% dy = %f, (transducer forward of antenna : positive)\n', cfg.dy);
fprintf(fid,'%% dx = %f, (transducer starboard of antenna: positive)\n',cfg.dx);

fprintf(fid, '%10.7f %10.6f %10.6f\n', txy');
fclose(fid);



if (cfg.plotit)
   % plot the horizontal corrections vs ags time
   titlestr = sprintf('%s: Offsets between GPS antenna and ADCP transducer',...
                      cfg.dbname)
   figure(1)
   clf
   subplot(211)
   plot(tags,corx_m,'.')
   grid
   axis([floor(tags(1)) ceil(tags(nags))  -20 20])
   xlabel('decimal days')
   ylabel('Longitude (m)')
   title(titlestr)
   subplot(212)
   plot(tags,cory_m,'.')
   grid
   axis([floor(tags(1)) ceil(tags(nags))  -20 20])
   xlabel('decimal days')
   ylabel('Latitude (m)')
   orient tall
   %printit('corags')
end


%--------------------------------------------
function lsthdg(dbpath, dbname, yearbase)

fid = fopen('lst_hdg.tmp', 'w');
fprintf(fid,'dbname:      %s\n', fullfile(dbpath, dbname));
fprintf(fid, '  output:      %s.hdg\n', dbname);
fprintf(fid, '  step_size=   1\n');
fprintf(fid,'  year_base=   %d\n', yearbase);
fprintf(fid, '  time_ranges:\n');
fprintf(fid, '        all\n');
fclose(fid);

!lst_hdg lst_hdg.tmp
