function varargout = m_vec_ell(s, x, y, varargin)
%
%    M_VEC_ELL draws fancy arrows and/or ellipses on a map.
%
%    [H1, H2] = m_vec_ell(S, LON, LAT, VARARGIN) draws arrows and/or
%    ellipses as patch objects on a map created by the M_Map package.
%
%    m_vec_ell(...) returns up two handles in the following order:
%      HPV: handle to the vector patch objects
%      HPE: handle to the ellipse patch objects
%      HT: text object handle for the key label; see below.
%
%      Input arguments:
%
%      S:  scale factor, arrow data units per inch.
%          It is based on the figure PaperPosition, so be
%          sure to set this (e.g. via "landscape" or "portrait")
%          before calling m_vec_ell.
%      LON, LAT: longitude, latitude of the arrows
%          LON and LAT must have the same dimensions, but can be
%          scalars or vectors; if scalars, multiple arrows can
%          be plotted at a single location.
%
%      VARARGIN can consist of any of the following:
%        U,V   U,V,C   Z,U,V,C    if arrows are to be drawn,
%      followed by optional arrow and ellipse parameters,
%      followed by optional patch parameters.
%        U,V are vectors containing the east and north components
%                 of the arrows.
%        C is an optional colorspec for all arrows, or an
%                 array of CData, one value per arrow.
%                 Defaults to black. Arrows only.
%        Z is a height in axes data units: it is probably not
%                 useful for trying to make 3-D plots (for which
%                 m_map is not designed), but may be useful in
%                 controlling what is drawn on top of what.
%                 (Doesn't seem to work, though.)
%      for ellipses only--no arrows--VARARGIN optionally starts
%          with Z and then includes ellipse parameters below:
%
%      optional parameters: keyword-value pairs, shown
%                 here with default values:
%         <arrows/quiverplots>:
%         'headangle',60     degrees: angle of arrow tip
%         'headwidth',NaN    points: direct specification of
%                                width, instead of headangle
%         'headlength',5     points: length of tip; set to 0
%                                to omit arrowhead entirely
%         'shaftwidth',1     points: width of arrow shaft
%         'centered', 0     1 = to make x,y the arrow
%                                center instead of its tail
%         <ellipses>:
%         'semi-major',[]    real (magnitude only) or complex
%                            vector (magnitude & angle)
%         'semi_minor',[]    real or complex (magnitude or i*0)
%         'angles',[]        angles for ellipses if data not complex
%         'circles',[]       supply radius values only
%         'location', 'll'   centered on lat long; 'tip' to center
%                            the ellipses or circles on arrow tip
%         'colors','none'    no fill; can also be char, RGB, or
%                            colordata; for ellipses or circles only
%         <other>:
%         'key', ''          make a labelled horizontal arrow
%                                if the string is not empty;
%                                then the string labels the
%                                arrow, and the second argument
%                                returned, ht, is the handle of
%                                the string.
%      optional patch parameters: any valid patch properties
%         may be specified here; they are passed directly
%         to the patch function.
%
%    m_vec_ell called without any parameters generates a demonstration plot.

%
%  Mon  98/02/16 Eric Firing, efiring@soest.hawaii.edu
%  Ellipse add-on CLA Dec.'00
%  2001/03/05 EF changed to get xlims, ylims from MAP_VAR_LIST
%                instead of using get(gca, ...).  This eliminates
%                the need to use m_grid before m_vec.
%                Also, other fixes and cleanups were done.


global MAP_PROJECTION  MAP_VAR_LIST

if nargin == 0, m_vec_demo; return; end

if nargin < 5,   % Minimum: s,x,y + 2 varargin
   help('m_vec_ell');
   error('not enough input arguments');
end


%%%%% INITIALIZE VARIABLES

if isempty(MAP_PROJECTION),
  error('No Map Projection initialized - call M_PROJ first!');
end;

hpe = []; hpv = []; ht = [];

% Default arrow parameters:
u = [];
v = [];
centered = 0;
headlength = 5/72;
headwidth  = NaN;
headangle = 40;
shaftwidth = 1/72;
c = 'k';
key = '';

% Default ellipse parameters:
uvar = []; % semi-major
vvar = []; % semi-minor
angles = [];
circles = [];
location = 'll';
colors = 'none';

clip = 'off'; % Clipping in m_ll2xy for finding the angles; this was
              % 'on' for a long time, but I think 'off' makes more sense.


%% Mandatory input arguments:
UVperIn = s;  % Data units per inch.
x = x(:);     % All data arrays are converted to column vectors.
y = y(:);

%% Begin the slightly complicated parsing of arguments.

nvarargin = length(varargin);

% Find the index of the first string argument in varargin.
istr0 = 0;
for ii = 1:nvarargin
   if ischar(varargin{ii}), istr0 = ii; break, end
end
if istr0 > 0,
   kvargs = varargin(istr0:end);
   numargs = varargin(1:(istr0-1));
else
   kvargs = {};
   numargs = varargin;
end

if rem(length(kvargs), 2) == 1, % odd, so first must be colorspec
   c = kvargs{1};
   kvargs(1) = [];
end

% Now kvargs contains only keyword/value pairs.
%
nkvargs = length(kvargs);
ipatch0 = 0;
for ikvp = 1:2:nkvargs
   kv = lower(kvargs{ikvp});
   value = kvargs{ikvp + 1};
   if     strcmp(kv, 'headlength')
      headlength = value/72;
   elseif strcmp(kv, 'headwidth')
      headwidth = value/72;
   elseif strcmp(kv, 'headangle')
      headangle = value;
   elseif strcmp(kv, 'shaftwidth')
      shaftwidth = value/72;
   elseif strcmp(kv, 'centered')  
      centered = value;
   elseif strcmp(kv, 'semi-major')
      if isreal(value)
         uvar = value;
      else
         uvar = abs(value);
         ang = angle(value);
      end
      uvar = uvar(:);
   elseif strcmp(kv, 'semi-minor')
      if isreal(value)
         vvar = value;
      else
         vvar = abs(value);
      end
      vvar = vvar(:);
   elseif strcmp(kv, 'angles')
      ang = value;
      ang = ang(:);
   elseif strcmp(kv, 'circles')
      value = value(:);
      uvar = value;
      vvar = value;
      ang = zeros(length(uvar),1);
   elseif strcmp(kv, 'location')
      location = value;
   elseif strcmp(kv, 'colors')
      colors = value;
   elseif strcmp(kv, 'key')
      key = value;
      clip = 'off'; % Can put key outside the map.
   else
      kvargs = kvargs(ikvp:end);
      break;  % no match; break out; remainder must be patch params
   end
   ipatch0 = ikvp + 2;
end

if ipatch0 > nkvargs,
   kvargs = {}; % no patch params.
end

n_numeric = length(numargs);
% If Z is an input argument, it will be the first of 1 or 4;
% store it and remove it from the array of arguments.
if n_numeric == 1 | n_numeric == 4,
   z = numargs{1}(:);
   numargs(1) = [];
   n_numeric = n_numeric - 1;
else
   z = zeros(size(x));
end

% There should be 0, 2, or 3 left:  {} or {U, V} or {U, V, C}
if n_numeric >= 2,
   u = numargs{1}(:);
   v = numargs{2}(:);
end
if n_numeric == 3,
   c = numargs{3};
   if length(c(:)) == length(u),
      c = c(:);
   end
end

%% End of argument parsing.

if ~isempty(uvar) & (isempty(ang) | isempty(vvar))
   help('m_vec_ell')
   error('insufficient ellipse specification')
end

if ~ischar(colors) & length(colors(:)) == length(uvar),
   colors = colors(:);
end

% Calculate the headwidth if it is not given explicitly:
if isnan(headwidth) & headangle < 170 & headangle > 0
   headwidth = headlength * tan(headangle*pi/180);
end
headwidth = max([headwidth; shaftwidth]);

if (length(x) == 1 & length(y) == 1 & length(u) > 1)
   x = x(ones(size(u)));
   y = y(ones(size(u)));
end
%%%%% END INITIALIZE VARIABLES


%%%%% INITIALIZE PLOT

OrigAxUnits = get(gca,'Units');
if OrigAxUnits(1:3) == 'nor'
   OrigPaUnits = get(gcf, 'paperunits');
   set(gcf, 'paperunits', 'inches');
   figposInches = get(gcf, 'paperposition');
   set(gcf, 'paperunits', OrigPaUnits);
   axposNor = get(gca, 'position');
   axWidLenInches = axposNor(3:4) .* figposInches(3:4);
else
   set(gca, 'units', 'inches');
   axposInches = get(gca, 'position');
   set(gca, 'units', OrigAxUnits);
   axWidLenInches = axposInches(3:4);
end

% Multiply inches by the following to get data units:
scX = diff(MAP_VAR_LIST.xlims)/axWidLenInches(1);
scY = diff(MAP_VAR_LIST.ylims)/axWidLenInches(2);
sc = max([scX;scY]);  %max selects the dimension limited by plot box.
%%%%% END INITIALIZE PLOT


[xs, ys] = m_ll2xy(x,y, 'clip', 'off');
Y = ys;  % change of variable names for historical reasons
X = xs;



%%%%% ELLIPSES FIRST

%% Pre-plot any ellipses or circles before the arrows:
if ~isempty(uvar)      % ellipse data exist
   uvar = uvar*sc/UVperIn; % plot data units conversion
   vvar = vvar*sc/UVperIn;

   if strcmp(location, 'tip');
   % vector data exist & ellipses require adjustment data re: arrows tips
      uvmag = abs(u + i*v);
      L = uvmag*sc/UVperIn;
      [xsp, ysp] = m_ll2xy(x+0.1*u./uvmag, ...
                           y+0.1*v.*cos(y*pi/180)./uvmag, 'clip', clip);
      % Vector angles in the Cartesian data-unit system of m_map:
      Ang = angle( (xsp-xs) + i*(ysp-ys) );
   end

   % revise angles in the Cartesian data-unit system of m_map:
   [newx, newy] = m_ll2xy(x + 0.1 * cos(ang), ...
                          y + 0.1 * sin(ang) .* cos(y*pi/180), 'clip', clip);
   ang = angle( (newx - xs) + i*(newy - ys) );
   ang = ang(:); % Check whether this is needed to make column, like uvar.
   el_nell = length(uvar);
   el_npts = 50;
   theta = (1:el_npts)*(2*pi)/el_npts;
   el_nV = el_npts * el_nell;

   % This will be reshaped later to (el_nV,3).
   elVerts = zeros(el_npts, el_nell, 3);  % fix statement for vectors also

   ellipse = (uvar * cos(theta) + i * vvar * sin(theta));
   % Rotate
   rr = exp(i * ang);
   ellipse = ellipse .* rr(:, ones(1, el_npts));

   % should be el_npts by el_nell
   ellipse = ellipse.';

   % Translate:
   origin = X + i * Y;
   if ~strcmp(location, 'll')
      origin = origin + L .* exp(i * Ang);  %% Dimensions right?
   end
   origin = origin(:).';
   origin = origin(ones(el_npts, 1), :);

   ellipse = ellipse + origin;

   elVerts(:, :, 1) = real(ellipse);
   elVerts(:, :, 2) = imag(ellipse);
   elVerts(:, :, 3) = z(:, ones(1, el_npts)).';

   elVerts = reshape(elVerts, el_nV, 3);

   elFaces = (1:el_nV).';
   elFaces = reshape(elFaces, el_npts, el_nell).';



   hpe = patch('Faces', elFaces, 'Vertices', elVerts);

   %% ellipses are coming out as circles, and with no color........

   if ischar(colors) & strcmp(colors, 'none'),
      set(hpe, 'EdgeColor', 'k', 'FaceColor', 'none', 'LineWidth', 0.1);
   elseif ischar(colors) | (size(colors,1) == 1 & size(colors,2) == 3),
      set(hpe, 'EdgeColor', colors, 'FaceColor', colors, 'LineWidth', 0.1);
   else
      set(hpe, 'EdgeColor', 'none', 'FaceColor','Flat', ...
        'FaceVertexCdata', colors);
   end

   if ~isempty(kvargs)
      set(hpe, kvargs{:});
   end
else
   hpe = [];
end
%%%%% END ELLIPSES


%%%%% ARROW VECTORS

if ~isempty(u)      % arrow vector data exist
   HPV = zeros(length(u));
   Width = shaftwidth*sc;
   HeadWidth = headwidth*sc;
   HeadLength = headlength*sc;

   uvmag = abs(u + i*v);
   % Arrow lengths in plot data units:
   L = uvmag*sc/UVperIn;

   [xsp, ysp] = m_ll2xy(x+0.1*u./uvmag, ...
                        y+0.1*v.*cos(y*pi/180)./uvmag, 'clip', clip);
   % Vector angles in the Cartesian data-unit system of m_map:
   Ang = angle( (xsp-xs) + i*(ysp-ys) );
   if ~isempty(key), Ang = 0; end

   nvec = length(L);
   Zero = zeros(nvec,1);

   % Normal arrow dimensions:
   HL = Zero+HeadLength;
   HW = Zero+HeadWidth;
   W =  Zero+Width;

   % Distinguish zero-length vectors from non-zero:
   mm = (L < 100*eps);
   i_zero = find(mm);
   i_nonzero = find(~mm);
   % Don't plot if length is zero.
   if ~isempty(i_zero)
      HL(i_zero) = NaN; HW(i_zero) = NaN; W(i_zero) = NaN;
   end

   if ~isempty(i_nonzero)
      ii = i_nonzero;
      if HeadLength == 0,   %% square end; no arrowhead
         W(ii)  = Width;
         HW(ii) = Width;
         HL(ii) = Zero;
      else
         % If the arrow length is less than the headlength,
         % omit the arrow shaft and just plot a head scaled
         % to the length.
         i_short = ii( find(L(ii) < HeadLength) );
         W(i_short)  = 0;
         HL(i_short) = L(i_short);
         HW(i_short) = HL(i_short) * (HeadWidth/HeadLength);
      end
   end


   % It is not clear whether non-zero elevations will be useful;
   % that depends on whether the entire m_map system will work
   % with 3-D views, as in making a surface plot of topography.
   % Then one might want to have a current profile represented
   % as a stack of vectors at appropriate heights above the topog.
   Z = z(:);


   Xzero = Zero;
   if centered,
      Xzero = -L/2;
   end


   nV = 7*nvec;  % number of Vertices: 7 per arrow.
   Vert = zeros(nvec,3);

   Vert(1:7:nV,:) =  [Xzero,  W/2, Z ];       % back corner of shaft
   Vert(2:7:nV,:) = Vert(1:7:nV,:) + ...
      [(L-HL), Zero,  Zero];    % shaft-head junction
   Vert(3:7:nV,:) = Vert(2:7:nV,:) + ...
      [Zero, (HW-W)/2,  Zero];  % point of barb
   Vert(4:7:nV,:) = [Xzero + L, Zero, Z];     % tip of arrow

   %% This could be done more efficiently with fancier indexing, but
   %% it is probably not a bottleneck, hence not worth the trouble.

   % Reflect the top half to get the bottom half.
   % First replicate:
   Vert(5:7:nV,:) = Vert(3:7:nV,:);
   Vert(6:7:nV,:) = Vert(2:7:nV,:);
   Vert(7:7:nV,:) = Vert(1:7:nV,:);
   % Then negate y to reflect:
   Vert(5:7:nV,2) = -Vert(5:7:nV,2);
   Vert(6:7:nV,2) = -Vert(6:7:nV,2);
   Vert(7:7:nV,2) = -Vert(7:7:nV,2);

   % Make an index array for operating on all vertices of each vector:
   ii = (1:nvec);
   ii = ii(ones(7,1),:);
   ii = ii(:);

   %% Rotate:
   Vxy = exp(i*Ang(ii)).*(Vert(:,1) + i*Vert(:,2));

   %% Translate:
   Vxy = Vxy + X(ii) + i*Y(ii);

   %%
   Vert(:,1) = real(Vxy);
   Vert(:,2) = imag(Vxy);

   Faces = [1:nV].';            %Top
   Faces = reshape(Faces,7,nvec).';

   % Extremely narrow patches don't show up on the screen (although they seem
   % to be printed OK) when EdgeColor is 'none', so when the arrows are all
   % the same color, set the EdgeColor to be the same as FaceColor.
   hpv = patch('Faces', Faces, 'Vertices', Vert, 'tag', 'm_vec');
   if ischar(c) | (size(c,1) == 1 & size(c,2) == 3),
      set(hpv, 'EdgeColor', c, 'FaceColor', c, 'LineWidth', 0.1);
   else
      set(hpv, 'EdgeColor', 'none', 'FaceColor','Flat', ...
         'FaceVertexCdata', c);
   end
   if ~isempty(kvargs)
      set(hpv, kvargs{:});
   end
else
   hpv = [];  % no data
end
%%%%% END ARROW VECTORS

if ~isempty(key) % Key (Legend)
   ht = text(X(1), Y(1)-0.5*HW, Z(1), key, ...
      'color', c, 'horiz','left','vert','top', 'tag','m_vec_ell', ...
      'clipping','off');
   set(hpv,'clipping','off')
   set(hpe,'clipping','off')
else
   ht = [];  %no Key (Legend)
end

varargout = {};
ii = 1;
if ~isempty(hpv), varargout{ii} = hpv; ii = ii + 1; end;
if ~isempty(hpe), varargout{ii} = hpe; ii = ii + 1; end;
if ~isempty(ht), varargout{ii} = ht; ii = ii + 1; end;

%-----------------------------------------------------------------------

function m_vec_demo()

    % This is a demonstration of m_vec_ell.
    % Sun  98/02/22 Eric Firing

    % Set up the figure and axes at the start, so that the vector lengths
    % will come out right when the figure is printed:

    orient tall
    % Main axes, for the map.
    ha1 = axes;
    pos = get(ha1,'position');
    pos1 = pos;
    pos1(4) = pos1(4) - 0.1;
    pos1(2) = pos1(2) + 0.1;
    set(ha1,'position',pos1)

    % Colormap axes.  Make them smaller than for the default colormap.
    pos2 = [0.3  0  0.4  0];
    pos2(2) = pos(2);
    pos2(4) = 0.02;
    % This positioning of the colormap is actually not very good as
    % printed, but I am not going to fiddle with it any more now.
    % This is just a typical difficulty with normalized coordinates
    % combined with fixed DataAspectRatio; if
    % something looks reasonable on the screen, it is likely to look
    % wrong when printed.

    ha2 = axes('position', pos2);

    axes(ha1); % Back to the main axes.

    m_proj('ortho','lat',48','long',-123', 'rad', 10, ...
         'rec', 'off' );
    m_coast('patch',[0.9 0.95 0.9]);
    m_grid('linest','-','xtick',[-135:5:-110]);
    title('Demonstration of m\_vec\_ell')

    %% The following form, centered and without heads, is suitable for
    %% the square roots of the principle axes of variance ellipses,
    %% for example.  Color is given as a single RGB triplet; if it were
    %% given as an array of triplets, the two lines could be different
    %% colors; unfortunately, as of Matlab 5.1, if you do this the plot
    %% will be forced into Zbuffer mode with the warning:
    %%  "RGB color data not yet supported in Painter's mode"
    %% This example also illustrates specification of a black patch
    %% boundary, via direct patch property specifications following any
    %% vector parameter options.
    hpv1 = m_vec_ell(100, [-133 -133], [49 49], [0 50], [100 0.0],...
     [0.7 0.8 0.9],'centered',1, ...
     'shaftwidth', 5, 'headlength', 0, ...
     'semi-major',[30 0], 'semi-minor', [10 0], 'angles',[pi/2 0], 'colors','y', ...
     'EdgeColor','k');

    %% Here are three vectors, no color specified, all properties defaulted'
    %% plus an uncolored ellipse at the tip of the second arrow only.
    hpv2 = m_vec_ell(100, [-128 -128 -128], [46 46 46], ...
       [0 25*sqrt(2) 50], [50 25*sqrt(2) 0], ...
       'semi-major',[0 + i+0; 10*sqrt(2) + i*10*sqrt(2); 0 + i*0], ...
       'semi-minor',[0 + i+0; 6*sqrt(2) + i*0; 0 + i*0], ...
       'location', 'tip');

    incs = (1:20)/20;
    vlat = 42 + incs*3;
    vlon = -127 - incs*2;
    uu = 50*sin(incs*2*pi);
    vv = 50*cos(incs*2*pi);

    %% This simulates a set of vectors such as currents measured
    %% with an ADCP along a cruise track.  Color is specified by letter,
    %% in this case. The arrows and heads are about as skinny as they
    %% can reasonably be.
    hpv3 = m_vec_ell(100, vlon, vlat, uu, vv, 'm', ...
     'shaftwidth', 0.2, 'headlength', 2.5);

    %% Now let's make a similar vectors, but with colors based on the
    %% colormap, simulating SST, for example:
    vlon = vlon - 3;
    sst = 12 - incs * 4;
    hpv4 = m_vec_ell(100, vlon, vlat, uu, vv, sst);
    colorbar(ha2);
    axes(ha2); xlabel('SST'); axes(ha1);
    %% These vectors will not show up well on the screen because they
    %% have no edges, but they will print adequately.  There does not
    %% seem to be any easy way to get around this; I would have to
    %% completely change the way the patches are specified.  Ordinarily,
    %% one would probably use fatter than default vectors with this
    %% method anyway, so that the colors are clear when printed; and in
    %% this case the screen display will look OK also.

    %% Here is a set of colored circles.
    vlon = [-115 -116 -117 -118]; % this set is to produce
    vlat = [44 44.5 45 45.5];
    sst = sst(1:4);
    hpe6 = m_vec_ell(100, vlon, vlat, 'circles', [17 19 16 12],...
       'colors', sst);


    %% Key: Note that it can be on or off the map. The vector and
    %% text are the same color by default. Handles to both the vector
    %% (patch) and the label (text) are returned so that you can
    %% change the font, color, etc.
    [hpv5, htv5] = m_vec_ell(100, -115, 38, 50, 0, 'b', 'key', '50 cm s^{-1}');
    set(htv5,'FontSize',8);

    return


