function r = load_raw(instclass, file, params, varargin)
% function load_raw(instclass, file, params, ...)
%
% Writes the command (*.cmd) and binary (*.bin) files
% required by ldcodas, starting from ue4 or EF RAW nb logging
% Each call to load_raw processes a single input file.
%                 ** Will change to process whole set.
% It also writes out files with (t x y) in a format suitable
% for refabs input.  The first fix from the ensemble goes to
% *.gps1, the last to *.gps2.
%
% Required arguments:
%   instclass:              instrument class (nb, bb, os, son)
%   file: name of RAW
%   params: a structure with fields
%     params.h_align      : Heading alignment in degrees;
%                             RDI sign convention
%     params.velscale      : multiply velocities by this factor
%     params.interval     : Averaging interval in seconds (default 300)
%     params.end_dday     : Averaging interval end times (as from a 
%                             codas3 database; eg. to match pingdata times)  
%                             default empty (disabled).  
%                             Overrides params.interval if set.  
%                             indices, not just the first and last bin.
%     params.geometry     : 'convex' (OS) 'concave' (NB) 
%     params.yearbase     : yearbase (required for nbuh and sonuh)
%     params.ducer_depth      : add to transducer pressure;
%                             (nbuh and sonuh start with tr_pressure = 0)
%
%     params.use_refavg   : def:1 (0, use simple reference layer average)
%     params.min_pings    : Minimum number of pings for an average;
%                             only relevant at the end of file with
%                             "finish" set to 1 (see below)
%     params.pingedit     :         level of pingediting to apply
%                                   'none' (or 0)   no pingediting at all
%                                   'ampbt'         amp and bottom only
%                                   'all'  (or 1)   amp and bottom, and water
%                                   'debug'         return both answers
%                                                   'none' and 'all' as
%                                                  a.enu_noedit, a.enul_pedit
%                                                   (a.enu is a.enu_pingedit)
%     params.min_pg       : Within the reference layer, use only
%                             the depths with at least this percent good;
%                             if there are not at least 2 such levels,
%                             don't use a reference layer, just calculate
%                             a simple average.
%                             NOTE: decrease this significantly if pingediting
%     params.use_3_beam   : Not yet implemented; only 4-beam solutions
%                             are used at present
%     params.instrument   : used only to select specific irregular transformations
%                         :        (two exist: "kmson", "kmos")
%
%   >>The first four param fields are required;<<
%   the remainder have defaults.  Read the source code (pingedit, load_raw) to 
%   see what they are, or eliminate all uncertainty by specifying all
%   param fields explicitly.
%
%   Other fields will likely be added as additional screening
%   methods are implemented.
%
% Optional named arguments: (keyword, value pairs)
%   "redo" , num          : redo averages for this file?: (1) force (0) do not 
%                         :      (default is 0: do not overwrite files)
%   "ldcodas_path", bindir: directory to write *.cmd, *.bin
%   "leftover", b         : remaining profiles from last file
%   "finish", num         : 1 to finish, 0 to leave "leftover" for next call
%  ("second_set", num     : 1 for second set, otherwise 0 (default) OBSOLETE)
%   "pingtype",           : (default = 'first')
%                         :  options -
%                         : 'first' (same as 'second_set' == 0) only use bb pings
%                         : 'bb'    (same as 'second_set' == 0) only use bb pings
%                         : 'second' (same as 'second_set' == 1) only use 2nd set
%                         : 'nb'    use logic to test if file is nb only or 
%                         :         interleaved.  Only use nb pings
%   "heading_offset", dh  : transducer heading offset in degrees
%     "matfile", [0 or 1] : def:0 write a matfile with the averages
%  showverbose", [0 or 1] : def:0 (1: show raw data -- timeconsuming)
%     "gbindir", path     : path to gbin dirs (default is  '../gbin')
%     "attstats" [cell]   : get attitude statistics (default: none
%                               'attstats', {'adu2'} %ashtech stats
%                               'attstats', {'adu2', 'posmv'} % and posmv
%
%        (second_set is normally 0, the default; set it to 1 only
%        if you have interleaved BB and NB pings and want to load
%        the NB part of the data set; leave it at 0 for the BB data,
%        or for any case when only BB or NB pings were collected.)
%
%  pingediting parameters come from get_pedefaults.m and are overridden as
%  elements of a substructure 'editcfg', eg:
%      params.editcfg.rl_range = [1:10];
%
% 2000/09/22 EF
% 2001/01/25 JH adding instclass, morphing for NBADCP and gbin;
%                    get nav fields from gbin files for sonuh and nbuh
%            


%%%% not yet implimented:
%   "use_for_heading", str: default 'heading' (from best)
%                         : other options: 'gyro.heading' 
%                         :     'posmv.heading' (NOTE: must add {'posmv'}
%                         :            to 'attstats'
%                         :     'adu2.heading' (NOTE: must add {'adu2'}
%                         :            to 'attstats'
%
%%%% undocumented: (for testing only) 
%
% 'max_BLKprofiles'       :     number of profiles allowed per block.
%                               (make smaller to test 'newday')


global fid_bin fid_cmd

if (nargin <= 4)
   help(mfilename)
   error('too few variables')
end

fid_cmd = 0;  % initialize so it can serve as a "files are open" flag

fid_log = fopen('load_raw.log','a');
blkinfo_logfile = 'BLKINFO.log';
cc = clock;
fprintf(fid_log,'running load_raw on %s at %s\n',...
        file, to_date(cc(1), to_day(cc(1), cc)));
clear cc

%set defaults for varargin structure 
cfg.redo = 0;
cfg.ldcodas_path = './';
cfg.nav_path = './';     
cfg.att_path = './';
cfg.use_for_heading = 'heading';
cfg.second_set = [];
cfg.pingtype = []; %defaults to 'first';
cfg.leftover = [];
cfg.finish = 1;
cfg.verbose = 0;
cfg.gbindir = '../gbin';
cfg.attstats = [];
cfg.header = [];
cfg.path_ext = {'time',    'best';   
                'time',    'tim'};
cfg.matfile = 0;
p.use_3_beam = 0;    % 3-beam not yet implemented
p.min_pings = 10;
p.h_bias = 0; %hardwire here; deal with offset in h_align

cfg.max_BLKprofiles = 399;  % max number of profiles to use per block

cfg = fillstruct(cfg, varargin, 0);

if ~isemptys(cfg.second_set)
   if isemptys(cfg.pingtype)
      if (cfg.second_set==1)
         fprintf('replacing ''second_set'',1  with  ''pingtype'', ''second''\n')
         cfg.pingtype = 'second';
      end
      if (cfg.second_set==0)
         fprintf('replacing ''second_set'',0  with  ''pingtype'', ''first''\n')
         cfg.pingtype = 'first';
      end
   else
      fprintf('options ''second_set'' (%d) and ''pingtype'' (%s) conflict\n',...
              cfg.second_set, cfg.pingtype)
      fprintf('''second_set'' is obsolete.  Use pingtype instead. \n')
      fprintf('try: help load_raw\n')
      fprintf('exiting\n')
      r=[];
      return
   end
end
if isemptys(cfg.pingtype)
   cfg.pingtype = 'first';
end
cfg = rmfield(cfg, 'second_set');

% rename parameters to match code
ldcodas_path = cfg.ldcodas_path;
nav_path = cfg.nav_path;     %don't write nav yet
leftover = cfg.leftover;
finish = cfg.finish;


% get statistics if requested: have to use path_ext in get_serfields
for ii=1:length(cfg.attstats)
   if (strcmp(cfg.attstats(ii), 'posmv'))
      cfg.path_ext(2 + ii, 1) = cfg.attstats(ii);
      cfg.path_ext(2 + ii, 2) = {'pmv'};
   elseif (strcmp(cfg.attstats(ii), 'adu2'))
      cfg.path_ext(2 + ii, 1) = cfg.attstats(ii);
      cfg.path_ext(2 + ii, 2) = {'adu'};
   else
      fprintf('cannot add attitude statistics for unsupported type "%s"\n',...
              cfg.attstats{ii});
   end
end


if nargin < 4 | ~ischar(file) | ~isstruct(params),
   help load_raw
   fprintf('At least 3 arguments are required:\n')
   fprintf('an instrument class, a filename and a structure.')
   error('too few input arguments ')
end


if isfield(params, 'end_dday')
   if ~isemptys(params.end_dday)
      params.interval = 300;
   end
end


if (~isfield(params, 'h_align') | ...
    ~isfield(params, 'velscale') | ...    
    ~isfield(params, 'interval') | ...
    ~isfield(params, 'ducer_depth') | ...
    ~isfield(params, 'yearbase'));
   help(mfilename)
   fprintf('''params'' requires more fields.  You have only: \n');
   params
   fprintf('must have: h_align, velscale, interval, yearbase, ducer_depth \n')
   fclose(fid_log);  fid_log = 0;
   return
end
if isa(instclass, 'nb') & ~isfield(params, 'beamangle')
   fprintf('NB instrument must have explicit beamangle in params structure\n')
   fprintf('exiting\n')
   fclose(fid_log);  fid_log = 0;
   return
end

%%------ set up file I/O info ----

p = fillstruct(p, params, 1);
if (isemptys(p.yearbase))
   r=[];
   fprintf('MUST specify yearbase\n')
   fclose(fid_log);  fid_log = 0;
   return
end


[a_path, rawfilebase, a_ext] = fileparts(file);
cmdfile = fullfile(ldcodas_path, [rawfilebase  a_ext '.cmd']);
binfile = fullfile(ldcodas_path, [rawfilebase  a_ext '.bin']);
nav1file = fullfile(nav_path, [rawfilebase     a_ext '.gps1']);  
nav2file = fullfile(nav_path, [rawfilebase     a_ext '.gps2']);  
attfile =  fullfile(nav_path, [rawfilebase     a_ext '.attstats.bin']);  
matfile = fullfile(ldcodas_path, [rawfilebase  '_raw'   '.mat']);


if ~cfg.redo
   if (   (exist(nav1file, 'file') == 2 ) | ...
          (exist(nav2file, 'file') == 2 ) | ...
          (exist(cmdfile, 'file') == 2 ) | ...
          (exist(binfile, 'file') == 2 ) )
      fprintf('at least one of the following files already exists:\n')
      fprintf('%s, %s, %s, %s.  Not processing.\n',...
              nav1file, nav2file, cmdfile, binfile)
      r = [];
      fclose(fid_log);  fid_log=0;
      return
   end
end


%% ----- get corrected decimal days and index ranges for averaging ----

% Read the first and last profile to get the configuration, get start and 
% end dates (uncorrected instrument times), initialize the structure  
% for the average.  This is very fast. (AND return the fid)



if isa(instclass, 'os')
   data1 = read(instclass, file, 'yearbase', p.yearbase, 'ends', 1,'second_set',0);
   
   %% only read second set if asked for and reasonable
   if strcmp(cfg.pingtype, 'second') | ...
         (strcmp(cfg.pingtype, 'nb') & strcmp(data1.config.pingtype, 'bb'))
      data2 = read(instclass, file, 'yearbase', p.yearbase, 'ends', 1,'second_set',1);
   else
      data2 = [];
   end
   
   if strcmp(data1.config.pingtype, cfg.pingtype) | strcmp(cfg.pingtype, 'first')
      fprintf('reading first ping from %s\n', file)
      second_set = 0;
      a = read(instclass, file,  'yearbase', p.yearbase, 'ends', 1,...
               'vars', 'all'); 
   elseif  ~isemptys(data2) & ...
         (strcmp(cfg.pingtype, 'second') | strcmp(cfg.pingtype, 'nb'))
      fprintf('reading second ping from %s\n', file)
      second_set = 1;
      a = read(instclass, file,  'yearbase', p.yearbase, 'ends', 1,...
               'second_set', 1, 'vars', 'all'); %no nav in nb or son
   else
      fprintf('no data of type "%s" in file %s\n', cfg.pingtype, file)
      a = [];
   end
   
else
   a = read(instclass, file,  'yearbase', p.yearbase, 'ends', 1,...
             'vars', 'all'); %no nav in nb or son
end



if isemptys(a)
   r=[];
   return
end


% Transfer input parameters, overriding defaults.
editcfg = get_pedefaults(instclass, a.config.pingtype);
if isfield(params, 'editcfg')
   p.editcfg = fillstruct(editcfg, params.editcfg, 0);
   params = rmfield(params, 'editcfg');
end





if (isemptys(leftover))
   fprintf('leftover is empty\n')
end

if (isemptys(a))
   r=leftover;
   return
end   
   
if (a.ens_num(1) == 1)
   leftover = [];
   fprintf('ensnum = 1 in raw data:\n')
   fprintf('resetting leftover to []\n')
end

if ~(isemptys(leftover))
   if (a.dday(1) - leftover.corr_dday(end) > .5*(p.interval/86400))
      fprintf('too big a gap between leftover and new data:\n')
      fprintf('resetting leftover to []\n')
      leftover = [];
   end
end


% Instruments logged with UH raw logging (sontek, rdi NB) need auxilliary
% data (serially logged navigation, etc).   This also provides the
% time correction.
if (~isfield(a, 'nav')) | isnan(sum(a.nav.txy1))
   use_auxnav = 1;
else
   use_auxnav = 0;
end


%% get the corrected list of decimal days corresponding to the raw
%% adcp data in this file:
if use_auxnav % sonuh and nbuh
   time_path = fullfile(cfg.gbindir, 'time');
   timbin_filename = fullfile(time_path,  sprintf('%s.tim.gbin', rawfilebase));
   tim_bindata = read_bin(timbin_filename);
   if (isemptys(tim_bindata))
      r = [];
      fclose(fid_log);  fid_log=0;
      return
   end
   
   p.dday = tim_bindata(2,:);  %best time
else %VmDAS logging.  nav structure exists
   fprintf('reading all times\n') % this may file with interleaved os(bb+nb)
   if (isa(instclass, 'os'))
      tempraw = read(instclass, file,  'second_set', second_set,...
                     'yearbase', p.yearbase, 'vars', {'ts', 'nav'});
   else
      tempraw = read(instclass, file, 'yearbase', p.yearbase,...
                     'vars', {'ts','nav'});
   end

   %correct times by median of sec_pc_minus_utc
   [rddaymm, rddayss, rddaynn, rddaymdt] = ...
       mstdgap(tempraw.nav.sec_pc_minus_utc);
   p.dday = tempraw.dday - rddaymdt / 86400;

   goodi = find(~isnan(tempraw.dday)); %excise bad dday (from bug in VmDAS)
   if (length(goodi) ~= length(tempraw.dday))    % for a VmDas bug
      tempraw = cutadcp(tempraw, goodi)
   end

   badi = find(isnan(p.dday));
   p.dday(badi)=[];
end

% compare leftover and new data configurations.  set leftover to [] if
% configuration changed. 
if (~isemptys(leftover))
   if chk_rconfig(leftover.config, a.config)
      leftover = [];
   end
end


% check whether p.dday range falls within params.end_dday range
if isfield(p, 'end_dday')
   if ~isemptys(p.end_dday)
      if (p.dday(end) < p.end_dday(1))
         fprintf('not proccessing: %s \n', rawfilebase)
         fprintf('raw file times end before params.end_dday starts\n')
         r=[];
         return
      end
      if (p.dday(1) > p.end_dday(end))
         fprintf('not proccessing: %s \n', rawfilebase)
         fprintf('raw file times start after params.end_dday end\n')
         r=[];
         return
      end
   end
else
   p.end_dday = [];
end

fprintf('calculating index ranges\n')
%% NOTE: these are index ranges to read; 
%% indices less than 1 refer to leftovers (already read)
%% read will ignore those

[index_ranges, leftover_irange] = find_iranges(leftover,  p); 

%% see whether we need to get out now based on interval
stopnow = 0;

if (isemptys(index_ranges)), fprintf('empty index_range\n'); stopnow = 1; end
if (length(index_ranges(1,:)) == 1) %there is only one ensemble
   ir = index_ranges;
   ir(1) = max(1,ir(1));
   if (86400*(p.dday(ir(2))  - p.dday(ir(1))) >   params.interval*1.1)
      fprintf('found one ensemble but it is too long\n')
      stopnow = 1;
      leftover = [];
   end
end



%% see whether we're starting a new block or not   
if exist('blkinfo.mat','file') %load the old blkinfo
   load blkinfo
   
   if chk_rconfig(blkinfo.config, a.config) %configuration change, start new block
      fprintf('config change\n')
      blkinfo.newblock = 1;                 
      blkinfo.blocknum = 1;
      blkinfo.profnum = 0;                  % reset to 0
   elseif  (blkinfo.profnum + length(index_ranges(1,:)) > cfg.max_BLKprofiles)
      fprintf('file will exceed full block, starting new block\n')
      blkinfo.newblock = 1;                 % file hits boundary; start a new block
      blkinfo.blocknum = 1;
      blkinfo.profnum = 0;                  % reset to 0
   elseif blkinfo.blocknum > 1
      fprintf('previous file required more than one block.  starting another\n')
      blkinfo.newblock = 1;                 % last file was huge; start a new block
      blkinfo.blocknum = 1;
      blkinfo.profnum = 0;                  % reset to 0
   else
      blkinfo.newblock = 0;                %update count below
      % leave profnum and blocknum the same
   end
   blkinfo.config = a.config;              % update config
else                                       % initialize 
   fprintf('no blkinfo file exists: initializing\n')
   blkinfo.newblock = 1;                 % instructions for later
   blkinfo.blocknum = 1;                 % 
   blkinfo.profnum = 0;                  % start at 0
   blkinfo.config = a.config;            % create config
end
%% save it at the end of each ensemble



if stopnow
   r=[];
   if (exist('fid_log'))
      if (fid_log), fclose(fid_log);   end;   fid_log=0; 
   end
      
   if (exist('fid_bin'))
      if (fid_bin),   fclose(fid_bin);  end;  fid_bin  =0;
   end
   
   if (exist('fid_nav1'))
      if (fid_nav1),  fclose(fid_nav1); end;  fid_nav  =0;
   end
   
   if (exist('fid_nav2'))
      if (fid_nav2),  fclose(fid_nav2); end;  fid_nav2 =0;
   end
   
   if (exist('fid_cmd'))
      if (fid_cmd),   fclose(fid_cmd);  end;  fid_cmd  =0;
   end
   return
end

%% proceed
NP = size(index_ranges, 2);

if cfg.matfile
   data = initialize_averages(a, NP);
end

if (isemptys(leftover))
   leftoverpings = 0;
   alldday = p.dday;
else
   leftoverpings = length(leftover.corr_dday);
   alldday = [leftover.corr_dday  p.dday];
end


%% ---- 

%get all the auxilliary data for this raw file
% ("data" can simply be data.info.rawfilenames and data.info.ilist)
% must replace ilist!!!
a.info.ilist = [1:length(p.dday)];

if use_auxnav
   aux_struct = get_serfields(a, 'gbindir', cfg.gbindir, ...
                                 'path_ext', cfg.path_ext);
   
   if (isemptys(aux_struct))
      fprintf('empty: could not complete auxilliary (nav) fill\n')
      r = [];   
      fprintf(fid_log,'empty: could not complete auxilliary (nav) fill\n')
      fclose(fid_log);  fid_log=0;
      return;
   end

   if (~isstruct(aux_struct))
      fprintf('not struct: could not complete auxilliary (nav) fill\n')
      r = [];   
      fprintf(fid_log,'not struct: could not complete auxilliary (nav) fill\n')
      fclose(fid_log);  fid_log=0;
      return;
   end

   aux_fnames = fieldnames(aux_struct);
   
   
   if (~ismember('adu2', cfg.attstats))
      noashtech = 1;
   else
      noashtech = 0;
   end
   
   if (~ismember('posmv', cfg.attstats))
      noashtech = 1;
   else
      noashtech = 0;
   end
else
   noashtech = 1;
end



if isstr(p.editcfg.rl_bins)
   if strcmp(p.editcfg.rl_bins,'all')
      p.editcfg.rl_bins = 1:length(a.depth);
   else
      fprintf('ERROR: reference layer string can only be ''all''\n')
      return
   end
end


%%   diagnostics -------------------

print_diagnostics(fid_log, rawfilebase, leftover, p, ...
                           NP, index_ranges, leftover_irange);

%% ---- write once-only database info ---
finfo = a.info; %pass this in to subsequent reads; 
a.config.tr_pressure = a.config.tr_pressure + p.ducer_depth;
a.depth = a.config.tr_pressure + a.depth;
NC = a.config.ncells;
a.ens_threshold = 0;
a.ev_threshold  = p.editcfg.error_velocity_cutoff;
a.top_ref_bin   = p.editcfg.rl_bins(1);
a.bot_ref_bin   = p.editcfg.rl_bins(end);
config1 = mk_config1(a);
c1_index =  config1_ind;
config1(c1_index.avg_interval)   = p.interval;
config1(c1_index.hd_offset)      = p.h_align;
config1(c1_index.heading_bias)   = p.h_bias;
access = mk_access(a);    % Doesn't change.

%% ----  read and average the full profiles -----


for ip = 1:NP
   
   tic

   ilist = [index_ranges(1, ip) : index_ranges(2, ip)];
   
   if (max(ilist) < 1) %the leftover passed in was the whole ensemble
      r = [];
   else
      if (isa(instclass, 'os'))
         r = read(instclass, finfo,  'yearbase',  p.yearbase, ...
                  'second_set', second_set,'ilist', ilist , 'vars', 'all');
      else
         r = read(instclass, finfo,  'yearbase',  p.yearbase, ...
                  'ilist', ilist , 'vars', 'all');
      end         
      r.vel = r.vel*params.velscale;
   end

   % attach corrected time to raw data.
   r.corr_dday = p.dday(ilist(ilist>0));   
   
   if ~isemptys(r.corr_dday)

      %fprintf('%d out of %d (%s) \n', ip, NP, file)
      fprintf('%4d out of %d   ', ip, NP)
      
      if use_auxnav
         aux = cutadcp(aux_struct, ilist(ilist>0));
         
         %assign aux fields to r: (eg. aduheading, pitch, roll,...)
         
         for fnamei = 1:length(aux_fnames);
            fname = aux_fnames{fnamei};
            r = setfield(r, fname, getfield(aux, fname));
         end
         
         %% choose correct heading here
         
         if (~strcmp(cfg.use_for_heading, 'heading'))
            inst_struct = getfield(r, cfg.use_for_heading);  %eg "gyro"
            r = setfield(r, 'heading', getfield(inst_struct, 'heading'));
         end
      end
   end
   
   %now that 'r' is set up, cat it to 'leftover' if appropriate
   % corr_dday gets catted here and ends up in leftover eventually
   if ip == 1
      if (isemptys(r.corr_dday)) 
         r = leftover;   %leftover was a whole ensemble
      else 
          ~isemptys(leftover)
          r = catadcp(leftover, r);
      end
   end
   
   
   fprintf(fid_log,  '%4d out of %4d, indices [%5d %5d]\n', ...
           ip, NP, index_ranges(1, ip), index_ranges(2, ip));
   fprintf(fid_log,'  corrected dd: %10.7f - %10.7f (%s %s)\n',...
           r.corr_dday(1), r.corr_dday(2), ...
           to_date(p.yearbase, r.corr_dday(1)),...
           to_date(p.yearbase, r.corr_dday(end)));
   fprintf(fid_log, 'uncorrected dd: %10.7f - %10.7f (%s %s)\n',...
           r.dday([1 end]), to_date(p.yearbase, r.dday(1)), ...
           to_date(p.yearbase, r.dday(end)));


   a.num_pings = length(r.corr_dday);
   
   
   if ~noashtech
      if (~isemptys(leftover))   
         %get ashtech statistics
         [attstats, att_rownames] = att_stats(r.adu2);
      end
   end
   
   %get beam statistics from data file
   p.verbose = cfg.verbose;


   
   %% avg.m will use this: just reporint it once here
   if ip == 1
      needs_beamenu =  ~isfield(r, 'enu') &...
          ~strcmp(r.config.coordsystem, 'earth');
      if needs_beamenu
         fprintf('rotating beam coordinates to ENU\n')
      else
         fprintf('velocities are already ENU\n')
      end
   end   
   a = avg(instclass, r, a, p);
   %this matlab file holds ensemble dday, profile number, and configuration
   % information so we can determine whether to start a new block



   if (cfg.matfile)
      data = fillfields(data, a, ip);
   end

   
   % all the structures:
   ancil1 = mk_ancil1(r, 1);
   ancil2 = mk_ancil2(r, 1);
   

   nav = ones(4,1) * 1e38;   
   %NAVIGATION structure wants [latitude longitude]
   %but nav structure has [time longitude latitude]
   
   % official codas timestamp is at the END of the averaging interval
   % NOT THIS: r.dday(end); (which is the instrument time).
   % use the corrected time (from alldday, i.e. [leftover.dday p.dday])
   
   if ip == 1,
      fid_nav1 = fopen(nav1file, 'w');
      fid_nav2 = fopen(nav2file, 'w');
      fid_cmd  = fopen(cmdfile, 'w');
      fid_bin  = fopen(binfile, 'w');
      if fid_nav1 == -1, error(['Can''t open ', nav1file]); end
      if fid_nav2 == -1, error(['Can''t open ', nav2file]); end
      if fid_cmd == -1, error(['Can''t open ', cmdfile]); end
      if fid_bin == -1, error(['Can''t open ', binfile]); end
      fprintf(fid_cmd, 'binary_file: %s\n', binfile);
   
   end

   %% break this loop out in case the file is too big (i.e. new block in mid-file)
   if blkinfo.newblock == 1  | ip == 1
      fid_blkinfo = fopen(blkinfo_logfile,'a');
      if blkinfo.newblock == 1
         fprintf(fid_blkinfo,...
                 'filebase= %s newblock= %d profile= %d  start_dday= %10.7f \n',...
                 rawfilebase, blkinfo.blocknum, blkinfo.profnum, a.lastdday);
      else
         fprintf(fid_blkinfo,...
                 'filebase= %s newblock= %d profile= %d  start_dday= %10.7f \n',...
                 rawfilebase, blkinfo.newblock, blkinfo.profnum, a.lastdday);
      end         
      fclose(fid_blkinfo);
   end   
   
   if blkinfo.newblock  %% write info to file
      fprintf(fid_log, 'filebase %s newblock= 1  dday= %10.7f\n', ...
              rawfilebase, a.lastdday);
      fprintf(fid_cmd, 'new_block\n');
      fprintf(fid_cmd, 'dp_mask: 1\n');
      
      bin_float('DEPTH', round(a.depth));
      bin_double('CONFIGURATION_1', config1);
      blkinfo.newblock = 0;
      blkinfo.profnum = 0;                  % reset to 0
   else
      %% update blkprofile info
      blkinfo.profnum = blkinfo.profnum+1;
      blkinfo.newblock = 0;   % instructions for later

      %% adding this JH 03/29: if file is too big, 
      %%   start another blk next time around
      if  (blkinfo.profnum >= cfg.max_BLKprofiles)
         fprintf('file will exceed full block, starting new block\n')
         blkinfo.newblock = 1;   % instructions for later
         blkinfo.blocknum = blkinfo.blocknum + 1;   %block number we're on now
         blkinfo.profnum = 0;                  % reset to 0
      end
   end
   fprintf(fid_log, 'added profile %d for dday %10.7f\n',...
           blkinfo.profnum, a.lastdday);
  
   fprintf(fid_cmd, ['new_profile: ', ...
                     to_date(a.config.yearbase, a.lastdday),...
                     '  /* %d %d */\n'], r.ens_num([1 end]));
   fprintf(fid_cmd, 'depth_range: %d %d\n', round(a.depth([1 end])));
   bin_double('ACCESS_VARIABLES', access);
   bin_double('ANCILLARY_1', ancil1);
   bin_double('ANCILLARY_2', ancil2);
   bin_double('BOTTOM_TRACK', a.bt);
   bin_double('NAVIGATION', nav);
   bin_float('U', a.enu(:, 1));
   bin_float('V', a.enu(:, 2));
   bin_float('W', a.enu(:, 3));
   bin_float('ERROR_VEL', a.enu(:, 4));
   bin_ubyte('AMP_SOUND_SCAT', round(mean(a.amp, 2)));
   bin_ubyte('RAW_AMP', a.amp);
   bin_ubyte('PERCENT_GOOD', a.pg(:, 4));
   bin_ubyte('PERCENT_3_BEAM', a.pg(:, 3));

   %% Maybe we should call it something else, but this will do
   %% for now; it is not correlation, either.
   if (~isfield(a, 'sw'))
      if (isfield(a, 'cor'))
         a.sw = a.cor;
      else
         a.sw = zeros(size(a.amp));
      end
   end
   bin_ubyte('SPECTRAL_WIDTH', round(mean(a.sw, 2)));
   bin_ubyte('RAW_SPECTRAL_WIDTH', a.sw);
   bin_ubyte('PROFILE_FLAGS', zeros(1, NC));
   % .....
   
   %make sure there are values for navigation 
   %txy1 is the first good fix of the ensemble
   posi = min(find(~isnan(sum(r.nav.txy1))));
   if (~isemptys(posi))
      fprintf(fid_nav1, '%12.7f %12.7f %12.7f\n', r.nav.txy1(1, posi),...
              r.nav.txy1(2, posi), r.nav.txy1(3, posi));
   end
   %txy2 is the last good fix of the ensemble
   posi = max(find(~isnan(sum(r.nav.txy2))));
   if (~isemptys(posi))
      fprintf(fid_nav2, '%12.7f %12.7f %12.7f\n', r.nav.txy1(1, posi),...
              r.nav.txy1(2, posi), r.nav.txy1(3, posi));
   end
   
   if (~noashtech)
      if (ip == 1)
         attstruct = write_bin(attfile, attstats, 'att_stats', att_rownames); 
      else
         attstruct = write_bin(attstruct, attstats);
      end
   end

   elapsed_time = toc;
   fprintf('time used: %4.2fs\n', elapsed_time)
end


if (cfg.matfile)
   save6(matfile, 'data')
end


%close the att file
if (~noashtech)
   if (exist('attstruct'))
      fclose(attstruct.fid); attstruct.fid = 0;
      fprintf('closing the att bin file %s\n', attfile)
   else
      if (isemptys(index_ranges))
         fprintf(fid_log, 'No full profiles\n');
      end
      fprintf(fid_log, 'NOTE: no ashtech file to close\n');
   end
end


if finish,
   fprintf(fid_log, 'finishing: r=[];\n');
   r = [];
else
   %we already read, averaged,  and wrote info for all full ensembles that 
   % existed.  Now 
   
   if (isemptys(leftover_irange))
      r = [];
      fprintf(fid_log, 'no leftover\n');
   else
      ilist = [leftover_irange(1):leftover_irange(end)];
      
      if (isa(instclass, 'os'))
         r = read(instclass, finfo,  'yearbase',  p.yearbase, ...
                  'second_set', second_set, 'ilist', ilist , 'vars', 'all');
      else
         r = read(instclass, finfo,  'yearbase',  p.yearbase, ...
                  'ilist', ilist , 'vars', 'all');
      end         
         
      
      fprintf('reading leftover: %d pings.\n', length(r.dday))
      %% add corrected time to leftover
      if (isemptys(r.dday))
         fprintf('r.dday is empty: in load_raw now, reading leftovers\n')
         fclose(fid_log);                         fid_log=0; 
         if (fid_bin),   fclose(fid_bin);  end;  fid_bin  =0;
         if (fid_nav1),  fclose(fid_nav1); end;  fid_nav  =0;
         if (fid_nav2),  fclose(fid_nav2); end;  fid_nav2 =0;
         if (fid_cmd),   fclose(fid_cmd);  end;  fid_cmd  =0;
         return
      else
         r.corr_dday = p.dday(leftover_irange(1):leftover_irange(2));
      end

      goodi = find(~isnan(r.dday)); %excise bad dday (from bug in VmDAS)
      if (length(goodi) ~= length(r.dday))    % for a VmDas bug
         fprintf('deleting %d bad pings (dday was NaN)\n', ...
                 length(r.dday)-length(goodi))
         r = cutadcp(r, goodi);
      end
      
      fprintf('%d out of %d (%s) \n', ip, NP, file)
      
      if (use_auxnav)
         aux = cutadcp(aux_struct, ilist(ilist>0));
   
         for fnamei = 1:length(aux_fnames);
            fname = aux_fnames{fnamei};
            r = setfield(r, fname, getfield(aux, fname));
         end
      end
         
         
      %now that 'r' is set up, cat it to 'leftover' if appropriate
      if (NP == 0) & ~isemptys(leftover)
         r = catadcp(leftover, r);
         fprintf(fid_log, 'there WAS a leftover from the previous file,  \n');
         fprintf(fid_log, 'but the new file only had %d pings: passing the \n');
         fprintf(fid_log, 'leftover through as a bigger leftover\n');
      end
   
      fprintf(fid_log, 'leftover, ilist = [%d  %d]\n', ilist(1), ilist(end));
      fprintf(fid_log, 'leftover dday range is %10.7f - %10.7f (%s - %s)\n', ...
              r.corr_dday([1 end]),to_date(r.config.yearbase,r.corr_dday(1)),...
              to_date(r.config.yearbase,r.corr_dday(end)));

      if (fid_cmd)
         fprintf(fid_cmd, '/* leftover: %d %d */\n', r.ens_num([1 end]));
      else
         fid_cmd = fopen(cmdfile, 'a');
         fprintf('leftover generated from this read.\n')
         fclose(fid_cmd);  fid_cmd=0;
      end
   end
end


% update this now.  Assumes things didn't fail while processing the file
blkinfo.dday = a.lastdday;
blkinfo.leftover = r;
save6 blkinfo blkinfo;
eval(sprintf('save6 %s_blkinfo.mat blkinfo', rawfilebase));


if exist('fid_bin') 
   if fid_bin
      fclose(fid_bin);   
      fid_bin = 0;
   end
end
if exist('fid_nav1') 
   if fid_nav1
      fclose(fid_nav1);  
      fid_nav1 = 0;
   end
end
if exist('fid_nav2') 
   if fid_nav2
      fclose(fid_nav2);  
      fid_nav2 = 0;
   end
end
if exist('fid_cmd') 
   if fid_cmd
      fclose(fid_cmd);   
      fid_cmd       = 0;
   end
end
if exist('fid_log') 
   if fid_log
      fclose(fid_log);   
      fid_log = 0;
   end
end


%===================================================
function print_diagnostics(fid_log, rawfilebase, leftover, p, ...
                           NP, index_ranges, leftover_irange);
% after we have index ranges, print info about them

if (isemptys(leftover))
   leftoverpings = 0;
   alldday = p.dday;
else
   leftoverpings = length(leftover.corr_dday);
   alldday = [leftover.corr_dday  p.dday];
end

fprintf(fid_log, '(%d pings from leftover) + (%d pings from %s) = %d\n', ...
        leftoverpings, length(p.dday),   rawfilebase,...
        leftoverpings + length(p.dday));

fprintf('(%d pings from leftover) + (%d pings from %s) = %d\n', ...
        leftoverpings, length(p.dday),   rawfilebase,...
        leftoverpings + length(p.dday));

sec_res = mstdgap(86400*diff(p.dday));
%% print indices and time duration (seconds)


fprintf('indices:   [start   end]        duration\n')
for ip=1:NP
   irange = [index_ranges(1,ip), index_ranges(2,ip)];
   if (irange(2) < 1)
      secs = NaN;
   else
      irange(1) = max(irange(1),1);
      secs = floor(86400*(p.dday(irange(2)) - p.dday(irange(1))));
   end
   if ~isnan(secs)
      fprintf('indices %6d %6d %6d\n', irange(1), irange(2), secs)
   end
   if (isemptys(p.end_dday)) % using strict interval, not predetermined times
      if (secs > p.interval + sec_res)
         fprintf('averaging interval at ip = %d is too big: %f secs\n',...
                 ip, secs)
         fprintf(fid_log, ...
                 'averaging interval at ip = %d is too big: %f secs\n',...
                 ip, secs);
      end
   end
end

if (~isemptys(leftover_irange))
   irange = [leftover_irange(1), leftover_irange(2)];
   if (irange(2) < 1)
      secs = NaN;
   else
      irange(1) = max(irange(1),1);
      secs = floor(86400*(p.dday(irange(2)) - p.dday(irange(1))));
   end
   fprintf('leftover: %6d %6d %6d\n',   irange(1), irange(2), secs);
end


%====================================================

function [ir, lr] =  find_iranges( leftover,  p);
% ir is 2xN index ranges for profile averages
% Start with [leftover.cor_dday, p.dday]; 
% index ranges start at 1 for p.dday (negative indices are for 
%  leftovers passed in
% new leftover_irange describes the part at the end
% the last index range may be incomplete (gets used as lefover)

gapfrac = .5;  %fraction of delta to allow between leftover(end) and dday(1)

yearbase = p.yearbase;
interval = p.interval;
delta = interval/86400;

% use all days together
if (isemptys(leftover) | ((p.dday(1) - leftover.corr_dday(end)) > gapfrac))
   alldday = p.dday;
   first_ipday = 1;
else
   alldday = [leftover.corr_dday  p.dday];
   first_ipday = length(leftover.corr_dday) + 1;
end

%%%

if (isemptys(p.end_dday)) %use the time interval
   % make sure we include leftovers in the allocation
   d0 = alldday(1) : delta : alldday(end);
   % but get rid of first one if it is before p.dday
   n = length(d0);
   ir = zeros(2,n);
   
   % lock in the starting indices
   for ii = 1:n
      ir(1, ii) = min(find(alldday >= d0(ii)));
   end
   ir(2,:) = [ir(1,2:end)-1, length(p.dday)];   
   if (ir(1,1) ~= 1)
      fprintf('something went wrong with determining ensemble indices\n')
      fprintf('index_range(1,1) is not 1\n')
      ir = []; lr = [];
      return
   end
  
else % use the ddays indicated
   edii = find( (p.end_dday >= alldday(1)) & (p.end_dday <= alldday(end)));
   d1 = p.end_dday(edii) ;

   n = length(d1)+1;
   ir = zeros(2,n);

   % lock in the end indices
   for ii = 1:n-1
      ir(2, ii) = max(find(alldday <= d1(ii)));
   end
   ir(2,end) = length(alldday);
   ir(1,:) = [1 ir(2,(1:end-1))+1];

   
   if (length(ir(1,:)) > 1)
      %trim the last one if it has no new info
      if (ir(2,end) == ir(2, end-1))
         ir(:,end) = [];
      end
   end
   
end

if (~isemptys(ir))
   ens1_secs = floor(86400*(alldday(ir(2,1)) - alldday(ir(1,1))));
end

%%%% now shift indices so 1 is the beginning of p.dday
ir(1,:) = ir(1,:) - first_ipday + 1;
ir(2,:) = [ir(1,2:end)-1, length(p.dday)];


%now move the last index range to lr if it is not a whole ensemble
lr = [];
ilist = [ir(1,end) ir(2:end)];  %wasilist = [ir(1,end):ir(2:end)];  
ilist(ilist<=0) = [];

if (~isemptys(ilist))
   if (    (86400*(alldday(ilist(end)) - alldday(ilist(1)))) < interval)
      lr = ir(:,end); %lefover range
      ir(:,end)=[];   %index range
   end
end   

if (~isemptys(ir))
   if (ens1_secs < gapfrac*interval)
      fprintf('NOTE: skipping first ensemble( %d seconds)\n',ens1_secs)
      ir(:,1)=[];
   end
end


%-----------------------------------------------------------


function  data = initialize_averages(a, NP)

dummy = NaN*ones(1,NP);
bigdummy = NaN*ones(a.config.ncells, NP);
bigbigdummy = NaN*ones(a.config.ncells, 4, NP);


data.umeas                 = bigdummy;
data.vmeas                 = bigdummy;
data.w                     = bigdummy;
data.e                     = bigdummy;
if (isfield(a, 'sw'))
   data.sw                 = bigbigdummy;
   data.mean_sw_pedited      = bigdummy;
end
if (isfield(a, 'cor'))
   data.cor                = bigbigdummy;
   data.mean_cor_pedited      = bigdummy;
end
data.amp                   = bigbigdummy;
data.mean_amp_pedited      = bigdummy;
data.pg                    = bigdummy;
data.depth                 = a.depth;
data.num_pings             = bigdummy(1,:);
data.lastdday                  = bigdummy(1,:);
data.lastlon                   = bigdummy(1,:);
data.lastlat                   = bigdummy(1,:);


%% these are means 
data.meanheading               = bigdummy(1,:);       
data.meanpitch                 = bigdummy(1,:);
data.meanroll                  = bigdummy(1,:);
data.meantemperature           = bigdummy(1,:);
data.meansoundspeed            = bigdummy(1,:);
data.meanuship                 = bigdummy(1,:);
data.meanvship                 = bigdummy(1,:);

data.bt.enu                = repmat(dummy,4,1);  %earthcoord
data.bt.range              = repmat(dummy,4,1);
data.bt.pg                 = repmat(dummy,4,1);

data.config.yearbase       = a.config.yearbase;
data.config.nbeams_used    = a.config.nbeams_used;
data.config.ncells         = a.config.ncells;
data.config.num_pings      = a.config.num_pings;
data.config.cell           = a.config.cell;
data.config.blank          = a.config.blank;
data.config.pulse          = a.config.pulse;
data.config.bin1distance   = a.config.bin1distance;
data.config.head_align     = a.config.head_align;
data.config.head_bias      = a.config.head_bias;
data.config.tr_pressure    = a.config.tr_pressure;
data.config.convex         = a.config.convex;
data.config.orientation    = a.config.orientation;
data.config.beamangle      = a.config.beamangle;
data.config.frequency      = a.config.frequency;
data.config.coordsystem    = a.config.coordsystem;


data.config.bt_was_on      = bigdummy(1,:);



%------------------------------------------------------------------
function  data = fillfields(data, a, ip)


data.umeas(:,ip)           = a.enu(:,1);  %earthcoord
data.vmeas(:,ip)           = a.enu(:,2);  %earthcoord
data.w(:,ip)               = a.enu(:,3);  %earthcoord
data.e(:,ip)               = a.enu(:,4);  %earthcoord

if (isfield(a, 'sw'))
   data.sw(:,:,ip)           = a.sw;
end

if (isfield(a, 'cor'))
   data.cor(:,:,ip)          = a.cor;
end

data.amp(:,:,ip)             = a.amp;

data.pg(:,ip)              = a.pg(:,4);

data.num_pings(ip)         = a.num_pings;
data.lastdday(ip)              = a.lastdday;
data.lastlon(ip)               = a.lastlon;
data.lastlat(ip)               = a.lastlat;


data.meanheading(ip)           = a.meanheading;
data.meanpitch(ip)             = a.meanpitch;
data.meanroll(ip)              = a.meanroll;
data.meantemperature(ip)       = a.meantemperature;
data.meansoundspeed(ip)        = a.meansoundspeed;


if (isfield(a.bt,'enu'))
   data.bt.vel(:,ip)          = a.bt.enu;
end
if (isfield(a.bt,'range'))
   data.bt.range(:,ip)        = a.bt.range;
end
if (isfield(a.bt,'pg'))
   data.bt.pg(:,ip)           = a.bt.pg;
end

data.ens_mean.lon(ip)      = a.ens_mean.lon;
data.ens_mean.lat(ip)      = a.ens_mean.lat;
data.ens_mean.uship(ip)    = a.ens_mean.uship;
data.ens_mean.vship(ip)    = a.ens_mean.vship;
 
data.config.bt_was_on(ip)  = a.config.bt_was_on;
