function [dd, config] = get_rndata(instclass, time_range, varargin)
% get raw and navigation data (merged)
% [dd, config] = get_rndata(instclass, time_range, varargin)  %uses time_range
%
% time_range should be like [94 94.0174]   %for 15 minutes
% varargin is a collection of name, value pairs
% 
%    name               what
%  -----------         ----------------------------------------------
%  h_align          :  mounting angle   +   phase            [required] 
%                   :  (OH, from start.cnf) (from watertrk)
%
%  velscale         :  multiply velocities by this          default= 1.0
%  yearbase         :  for decimal day conversions            [required]
%   convex          :  1  (eg. Ocean Surveyor)            default='concave'
%                   :    or 0 (concave, NB shipboard)
%                   :    (for bb or os, presesent in 
%                   :    config field of raw data
%  beamangle        : beam angle (default = []; use value from instrument)
%                   :    (for bb or os, presesent in raw data)
%                   :     ---> need to specify if nb <---
%
%                   :    config field of raw data
% ducer_depth       : for processing, add this to depths        0
%                   :
% instrument        :  (specify 'kmson' iff  Kilo Moana Sontek.)   []
% 
% "second_set"      : 1 for second set, otherwise 0 (default) OBSOLETE
%   "pingtype",     : (default = 'first')
%                   :  options -
%                   : 'first' (same as 'second_set' == 0) only use bb pings
%                   : 'bb'    (same as 'second_set' == 0) only use bb pings
%                   : 'second' (same as 'second_set' == 1) only use 2nd set
%                   : 'nb'    use logic to test if file is nb only or 
%                   :         interleaved.  Only use nb pings
%                   :
% rawfpath          :  relative path to directory 
%                   :    with adcp raw files                     '../raw/';
% pingdir           :  directory with adcp raw files            'adcp_pings'
%                   :
% 'gbindir'         :   relative path to gbin dirs                   '../gbin'       
%                   : NOTE: more likely to be ../gbin/instname
%
% 'path_ext'        : instrument extension  %            cellarray telling which
%                   :    path_ext = ...                   gbin files to load  
%                   : {'time',    'best';    %  default  
%                   :  'time',    'tim';     %  default
%                   :  'pcode',   'pos';     %             
%                   :  'adu2',    'adu';     %             
%                   :  'adu2',    'pos';     %             
%                   :  'posmv',   'pmv';     %             
%                   :  'posmv',   'pos';     %             
%                   :  'gyro',    'hdg';     %             
%                   :  'sndspd',  'spd'};    %             
% 'add_aux'         : include the auxilliary data from path_ext to the output
%                   :         (increases size)  default = 0
%
%   'step'          : use only every 'step' index         [integer]  default=1
%   'notransform'   : (1 means no earth coords )          [0 or 1]   default 0 
%   'return3d',     : (1: also return 3-d vecs)           [0    1]   default 0   
%   'logfpath'      : directory to write logfile            './'      
%
%   'verbose'       :  say some useful things             [0 or 1]   default=0
% -----------------------------------
% %%example: NB Palmer :
% instclass = nb;
% params.h_align = 45.0;  %OH 
% params.yearbase = 2001;
% params.ducer_depth = 7;
% pingdir = 'nb150';
% gbindir = '../gbin/nb150';
% path_ext =  {'time',    'best';    %  default   gbin files to load
%            'time',    'tim'};      %  default
% ddrange = 155 + [0 30*60/86400];
% [dd,config] = get_rndata(instclass, ddrange,...
%                             'h_align', params.h_align, ...
%                             'yearbase', params.yearbase,...
%                             'ducer_depth', params.ducer_depth,...
%                             'gbindir', gbindir,...
%                             'pingdir', pingdir,...
%                             'path_ext', path_ext,...
%                             'return3d',1,...
%                             'convex', 0);
%% 
% %%% do this from the raw/ directory
%
% %then run 
% rlookit(dd, config, 1, [15])  %beam 1, bin 15
% rlookit2(dd, config, [15])  % bin 15
%


cfg.yearbase = [];
cfg.h_align = [];
cfg.velscale = 1;
cfg.convex = 0;
cfg.beamangle = [];
cfg.ducer_depth = 0;
cfg.step = 1;
cfg.second_set = [];
cfg.pingtype = []; %defaults to 'first';
cfg.gbindir = '../gbin';
cfg.rawfpath = '../raw';
cfg.pingdir = 'adcp_pings';
cfg.add_aux = 0;
cfg.logfpath = './';
cfg.instrument = [];
cfg.notransform = 0;
cfg.return3d = 0;
%get these      %instrument extension                
cfg.path_ext = {'time',    'best';   
                'time',    'tim'};

cfg = fillstruct(cfg, varargin, 0);
logfile =  fullfile(cfg.logfpath, 'get_rndata.log');

   

if ~isemptys(cfg.second_set)
   if isemptys(cfg.pingtype)
      if (cfg.second_set==1)
         fprintf('replacing ''second_set'',1  with  ''pingtype'', ''second''\n')
         cfg.pingtype = 'second';
      end
      if (cfg.second_set==0)
         fprintf('replacing ''second_set'',0  with  ''pingtype'', ''first''\n')
         cfg.pingtype = 'first';
      end
   else
      fprintf('options ''second_set'' (%d) and ''pingtype'' (%s) conflict\n',...
              cfg.second_set, cfg.pingtype)
      fprintf('''second_set'' is obsolete.  Use pingtype instead. \n')
      fprintf('try: help load_raw\n')
      fprintf('exiting\n')
      dd=[];
      config=[];
      r=[];
      return
   end
end
if isemptys(cfg.pingtype)
   cfg.pingtype = 'first';
end
cfg = rmfield(cfg, 'second_set');

if isemptys(cfg.yearbase)
   help(mfilename)
   error('must set yearbase')
end

if (isemptys(cfg.h_align))
   help(mfilename)
   error('must set transducer angle correction (h_align)')
end

%now get the data
data = [];


tmpcfg = cfg; tmpcfg.toofarsecs = 10;
rfilelist = guess_rfilelist(tmpcfg, time_range, {'time','tim'},'gbin');
if (isemptys(rfilelist))
   fprintf('%s found no raw files in %s\n', mfilename, cfg.gbindir)
   dd=[];
   config=[];
   return
end


finfo = tr2ilist(rfilelist, 'timerange', time_range, 'rowname',...
                      'bestdday');

if isemptys(finfo)
   fprintf('at %s, no data found in time range %f-%f\n', nowstr, ...
           time_range(1), time_range(2));
   fid = fopen(logfile,'w');
   fprintf(fid, 'at %s, no data found in time range %f-%f\n', nowstr, ...
           time_range(1), time_range(2));
   fclose(fid);
   dd = [];
   config = [];
   return
elseif isemptys(finfo{1,1})
   fprintf('at %s, no data found in time range %f-%f\n', nowstr, ...
           time_range(1), time_range(2));
   fid = fopen(logfile,'w');
   fprintf(fid, 'at %s, no data found in time range %f-%f\n', nowstr, ...
           time_range(1), time_range(2));
   fclose(fid);
   dd = [];
   config = [];
   return
else
   for ii = 1:length(finfo(:,1))
      ilistends = finfo{ii,3};
      ilist = ilistends(1):ilistends(end);
     
      ilist = ilist(1:cfg.step:end);
      
      gbinfile = finfo{ii,1};
      sepi = find(gbinfile == filesep);
      if (~isemptys(sepi))
         gbinfile(1:sepi(end)) = [];
      end
      dots = find(gbinfile == '.');
      barename = gbinfile(1:dots(1)-1);
      rawfilebase = [barename, '.raw'];
      

      fullrawfile = fullfile(cfg.rawfpath,cfg.pingdir, rawfilebase);
      fprintf('reading %d pings [%d-%d] from file %d of %d (%s)\n', ...
              length(ilist), ilist(1), ilist(end), ...
              ii, length(finfo(:,1)), rawfilebase)
      % 2004/01/10 -- JH adding logic to determine second_set or not base on pingtype
      if isa(instclass, 'os')
         data1 = read(instclass, fullrawfile, ...
                      'yearbase', cfg.yearbase, 'ends', 1,'second_set',0);
         data2 = read(instclass, fullrawfile,...
                      'yearbase', cfg.yearbase, 'ends', 1,'second_set',1);
         if strcmp(data1.config.pingtype, cfg.pingtype) | ...
               strcmp(cfg.pingtype, 'first')
            second_set = 0;
            newdata = read(instclass, fullrawfile,...
                           'yearbase', cfg.yearbase, 'ilist', ilist,...
                           'vars', 'all'); 
         elseif strcmp(cfg.pingtype, 'second') | ...
               (strcmp(cfg.pingtype, 'nb') & (~isemptys(data2)))
            fprintf('reading second ping from %s\n', fullrawfile)
            second_set = 1;
            newdata = read(instclass, fullrawfile, ...
                           'yearbase', cfg.yearbase, 'ilist',ilist,...
                           'second_set', 1, 'vars', 'all'); %no nav in nb
         else
            fprintf('no data of type "%s" in file %s\n', cfg.pingtype, ...
                    fullrawfile)
            dd=[];
            config=[];
            return
         end
         
         if isempty(cfg.beamangle)
            cfg.beamangle = newdata.config.beamangle;
         end
         
      else
         
         newdata = read(instclass, fullrawfile,...
                        'beamangle', cfg.beamangle,...
                        'yearbase', cfg.yearbase, 'ilist', ilist,...
                        'vars', 'all'); %no nav in nb or son
         
      end
   
      if (isemptys(newdata))
         dd = [];
         config=[];
         return
      end
      
      if (~isemptys(newdata))
         %add corrected time to structure
         tim_bindata = read_bin(finfo{ii,1});
         if (isemptys(tim_bindata))
            dd = [];
            config=[];
            return
         end
         
         newdata.corr_dday = tim_bindata(2,ilist);  %best time
         %ilist is incorporated from data.info.ilist
         aux_struct = get_serfields(newdata, ...
                                    'path_ext', cfg.path_ext,...
                                    'logfpath', cfg.logfpath,...
                                    'gbindir', cfg.gbindir);
         
         newdata = update_raw(newdata, aux_struct);
         
         if (cfg.add_aux == 1)
            newdata.aux_struct = aux_struct;
         end
         
         if ~isemptys(data)
            if chk_rconfig(data.config, newdata.config)
               fprintf('configuration changed at corr_dday %10.7f\n',...
                       newdata.corr_dday(1))
               fprintf('not appending.\n')
               break
            end
         end
         
         data = catadcp(data, newdata);
      end
   end
end


data.vel = data.vel*cfg.velscale;

if (cfg.notransform)
   dd = data;
   config = [];
else
   if (isemptys(data))
      dd=[];
      config=[];
   else
      
      hdg = data.heading(:);
      
      
      [data.enu data.xyze] = beam_enu(instclass, data.vel,...
                                      cfg.h_align + hdg, 0*hdg, 0*hdg, ...
                                      'error', 1,...
                                      'instrument', cfg.instrument,...
                                      'convex', data.config.convex,...
                                      'beamangle', cfg.beamangle);   
      
      if (isfield(data, 'bt'))
         data.bt.enu = ...
             beam_enu(instclass, data.bt.vel,...
                      cfg.h_align + hdg, 0*hdg, 0*hdg, ...
                      'error', 1,...
                      'instrument', cfg.instrument,...
                      'convex', data.config.convex,...
                      'beamangle', cfg.beamangle);   
      end

   
      [dd, config] = restruct(instclass, data);
      dd.cuv = dd.umeas + i*dd.vmeas;
      
      if (cfg.return3d)
         dd.enu = data.enu;
         dd.xyze = data.xyze;
         dd.vel = data.vel;
         dd.amp = data.amp;
         dd.pg = data.pg;
         if (isfield(data,'sw'))
            dd.sw = data.sw;
         end
         if (isfield(data,'cor'))
            dd.cor = data.cor;
         end
      end
   end
end



dd.depth = dd.depth + cfg.ducer_depth;
