function as = get_serfields(data, varargin)
% aux_struct = get_serfields(data, varargin)
% ("data" can simply be data.info.rawfilebase and data.info.ilist)
%              (raw file name [without path or '.raw'])     (ilist into file)
%              (such as nbp2001_091_52596'            )
%
% read .gbin files where they exist and return a structure suitable for
% adding info to raw data structure (returned by read).  The filename
% root is in data.info.rawfilebase; indices are in data.info.ilist:
% varargin is only used to set paths and extensions.
%
% default: assume adcptree directory structure and paths are relative
% to a subdirectory such as load/.  Override as necessary with varargin
% ASSUMES all gbin files are under one root directory
%
% *** returns empty structure if file is missing *** (eg. as.adu2 = [];)
%
% example: (from load/) ------
%   ilist = 1:300;
%   data = read(nb, fullrawfile, 'yearbase', 2001, 'ilist', ilist,...
%         'vars', 'all')
%   data = update_raw(data, get_serfields(data));
% % -------------------------
%
% varargin can be override paths
%    'gbindir'               '../gbin';             relative path to gbin dirs
%
%    'path_ext' = ...
%               %instrument extension  %            cellarray telling which
%               {'time',    'best';    % [default]   fills nav.txy1 structure
%                'time',    'tim';     % [default]
%                'pcode',   'pos';     %             returns a substructure
%                'adu2',    'adu';     %             with the name of the 
%                'adu2',    'pos';     %             instrument, with fields
%                'posmv',   'pmv';     %             under that.  eg: 
%                'posmv',   'pos';     %             as.gyro.rdata
%                'gyro',    'hdg';     %             as.gyro.rstruct
%                'sndspd',  'spd'};    %             as.adu2.rdata  
%                                      %             as.adu2.rstruct    
%                                                    as.nav.txy1
%                                                    as.nav.txy2
%
%        NOTE:                                      
%             'tim' and 'best' yield best estimates as  as.heading (etc)
%                          as well as filling the as.txy structures
%
%
% 
% logfpath              ''             %             do not write log messages
%                                      %             './'    write logs here
% verbose [0 1]    give error messages if file not found  % 0
%

%JH 2001/04/05

cfg.verbose = 0;


cfg.gbindir = '../gbin';
cfg.logfpath = '';
%get these      %instrument extension                
cfg.path_ext = [];


cfg = fillstruct(cfg, varargin, 0);

base_path_ext = {'time',    'best';   
                 'time',    'tim'};
cfg.path_ext = cat(1, base_path_ext, cfg.path_ext); %ok if repeated

%% now render unique
for ii=1:length(cfg.path_ext(:,1))
   cellstr{ii} = [cfg.path_ext{ii,1}, '_', cfg.path_ext{ii,2}];
end
cellstr = unique(cellstr);
for ii=1:length(cellstr)
   underscore = find(cellstr{ii} == '_');
   path_ext{ii,1} = cellstr{ii}(1:underscore-1);
   path_ext{ii,2} = cellstr{ii}(underscore+1:end);
end

if isempty(cfg.logfpath)
   logfile = '';
else
   logfile =  fullfile(cfg.logfpath, 'get_serfields.log');
end


if (~isfield(data, 'info'))
   if ~isempty(logfile)
      fid = fopen(logfile,'w');
      fprintf(fid, '%s: no "info" structure in data\n', nowstr);
      fclose(fid);
   end
   as = [];
   return
end
infostruct = data.info;

if (~isfield(infostruct, 'rawfilebase') )
   if ~isempty(logfile)
      fid = fopen(logfile,'w');
      fprintf(fid, '%s: data.info  is missing "rawfilebase"\n', nowstr);
      fclose(fid);
   end
   as = [];
   return
end
rawfilebase = data.info.rawfilebase;

if (~isfield(infostruct, 'ilist') )
   if ~isempty(logfile)
      fid = fopen(logfile,'w');
      fprintf(fid, '%s: data.info  is missing "ilist"\n', nowstr);
      fclose(fid);
   end
   as = [];
   return
end
ilist = data.info.ilist; %this gets used 

%get time data
time_pathrow  = find(strcmp(path_ext(:,2), 'tim'));
time_pathrow = time_pathrow(1);
if (~isemptys(time_pathrow))
   clear fullfile
   fullpath = fullfile(cfg.gbindir, path_ext{time_pathrow, 1});
   timefullfile = sprintf('%s.%s.gbin', ...
                         fullfile(fullpath, rawfilebase),...
                         path_ext{time_pathrow, 2});
   if (exist(timefullfile, 'file'))
      timedata = read_bin(timefullfile);
      
      bestdday = timedata(2,ilist);   %best time, not necessarily logger time
      nav.sec_pc_minus_utc =  timedata(3,ilist);
   else
      %show whether we have filename right
      if ~isempty(logfile)
         fid = fopen(logfile,'w');
         fprintf(fid,'tim.gbin file %s does not exist\n',timefullfile);;
         fclose(fid);
      end
      as.nav = [];
   end
end

%% all the rest are optional

%get best position (and other best) data
best_pathrow = find(strcmp(path_ext(:,2), 'best'));
best_pathrow = best_pathrow(1);
if (~isemptys(best_pathrow))
   fullpath = fullfile(cfg.gbindir, path_ext{best_pathrow, 1});
   bestfullfile = sprintf('%s.%s.gbin', ...
                         fullfile(fullpath, rawfilebase),...
                         path_ext{best_pathrow, 2});
   

   if (exist(bestfullfile, 'file'))
      [bestdata, beststruct]  = read_bin(bestfullfile);
      if cfg.verbose
         fprintf('file %s has length %d\n', bestfullfile, ...
                 length(bestdata(1,:)))
      end
      fclose(beststruct.fid);
      lon_row = find(strcmp(beststruct.rows, 'lon'));
      lat_row = find(strcmp(beststruct.rows, 'lat'));
      nav.txy1 = [bestdday ;bestdata([lon_row lat_row],ilist)];
      nav.txy2 = nav.txy1; %NOT GOOD -- should separate these
      as.nav = nav;
      
      %% anything else according to best2time rules (eg. heading, pitch...)
      other_rows = find( (~strcmp(beststruct.rows, 'lon')) & ...
                         (~strcmp(beststruct.rows, 'lat')));
      for rowi = 1:length(other_rows)
         as = setfield(as, beststruct.rows{other_rows(rowi)},...
                           bestdata(other_rows(rowi),ilist));
      end
   else
      %show whether we have filename right
      if ~isempty(logfile)
         fid = fopen(logfile,'w');
         fprintf(fid,'pos.gbin file %s does not exist\n',bestfullfile);
         fclose(fid);
      end
      as.nav = [];
   end
end


% get the rest of the structures requested from this list
%                {'time',    'best'; 
%                'time',    'tim';   
%                'pcode',   'pos';     pcode.gps.gdata,   pcode.gps.gstruct
%                'adu2',    'adu';     adu2.adugdata,     adu2.adu.gstruct
%                'adu2',    'pos';     adu2.pos.,gdata,   adu2.pos.gstruct
%                'posmv',   'pmv';     posmv.pmv.gdata,   posmv.pmv.gstruct
%                'posmv',   'pos';     posmv.pos.gdata,   posmv.pos.gstruct
%                'gyro',    'hdg';     gyro.hdg.gdata,    gyro.hdg.gstruct
%                'sndspd',  'spd'};    sndspd.spd.gdata,  sndspd.spd.gstruct

%remove time rows (already dealt with it)


nottime_rows = find(~strcmp(path_ext(:,1),'time'));
for pathrow= 1:length(nottime_rows)
   otherpath = path_ext{nottime_rows(pathrow), 1};
   otherext  = path_ext{nottime_rows(pathrow), 2};
   fullpath = fullfile(cfg.gbindir, otherpath);
   
   fullfilename = sprintf('%s.%s.gbin', ...
                          fullfile(fullpath, rawfilebase),otherext);
   if (exist(fullfilename, 'file'))
      [gdata, gstruct]  = read_bin(fullfilename);
      fclose(gstruct.fid);
      if cfg.verbose
         fprintf('file %s has length %d\n', fullfilename, length(gdata(1,:)))
      end
      eval(sprintf('as.%s.%s.gdata = gdata(:,ilist);', otherpath , otherext))
      eval(sprintf('as.%s.%s.gstruct = gstruct;', otherpath , otherext))
   else
      % return empty fields
      if ~isempty(logfile)
         fid = fopen(logfile,'w');
         fprintf(fid,'file %s does not exist\n',fullfilename)
         fclose(fid);
      end
      eval(sprintf('as.%s = [];', otherpath))
   end
end

as.bestdday = bestdday;

if ~isstruct(as)
   as = [];
end

