function best2gbin(rawfilebase, varargin)
% best2gbin(rawfilebase, varargin)
%                                
% read .tim.bin files to get best logger times 
%  *** should have already created it with time2gbin ***
% find the matching (logger) time ranges for selected position and
% attitude files, interpolate to get fields  at logger times,  
% write a flat binary file with (ncols-1)xN entries
% output name is 'outfpath/rawfilebase.lld.gbin' for 
% "lon,lat,deg gridded bin"  (these files are screened for errors)
% 
% varargin pairs are:
% 
%      rawdir =  'raw';
%      rbindir = 'rbin';
%      gbindir = 'gbin';
%      logfpath = './';
%      gap_seconds             [NaN positions with    %default 10
%                             gaps longer than this]
%      scootch_sec              [require time shifts to         % .5
%                                     be under this
%      redo                   [0 or 1] redo gbin file            % 0 
%
%      best                     make the rules about what data are 'best'
%                               contains the following structures
%
% (1) here are the fields that get filled (subset of those that have rules)
% best.rules = {'gps', 'heading', 'pitch', 'roll', 'heave'};  %!! in order
% 
% (2) here are the rules that govern filling them
%                                                      
%                     directory       rbin_suffix      
%                                                      
% position
% best.gps_rule  =    {'pcode',          'gps';         %best      
%                      'adu2',           'gps';         %less good 
%                      'posmv',          'gps'};        %worst case
% 
% best.heading_rule = { 'gyro',           'hdg';        %best      
%                       'adu2',           'adu';        %less good 
%                       'posmv',          'pmv'};       %worst case 
% 
% best.pitch_rule =   {'posmv',          'pmv';         %best      
%                      'adu2',           'att'};        %less good  
%                                                      
% best.roll_rule  =   {'posmv',          'pmv';         %best     
%                      'adu2',           'att'};        %less good 
%   
% best.heave_rule  =   {'posmv',          'pmv';        %best     
%                       'adu2',           'att'};       %less good 
% 
% best.sndspd_rule = {'sndspd',           'spd' };      %this is it
% 

%% add an output to say which dataset was used
%JH 2002/08/05

cfg.rawdir =  'raw';
cfg.rbindir = 'rbin';
cfg.gbindir = 'gbin';
cfg.logfpath = '.';              %not currently used
cfg.gap_seconds = 10;
cfg.scootch_sec = .5;
cfg.redo = 0;
cfg.best = [];

cfg = fillstruct(cfg, varargin, 0);


%update with whatever fields were put in varargin 'best'
if (~isemptys(cfg.best))
   cfg = fillstruct(cfg, cfg.best, 1);
end
cfg = rmfield(cfg,'best');


% columns for rules
rule_dircol = 1;       %column for directory
rule_extcol = 2;       %column for file extension
rule_refcol = 3;     %column for reference structure


%editing thresholds
acc_heading_cutoff = .5;
acc_roll_cutoff = .5;
acc_pitch_cutoff = .5;


% set up output
time_gbindir =  fullfile(cfg.gbindir, 'time');
outfile = fullfile(time_gbindir, sprintf('%s.best.gbin', rawfilebase));

if (exist(outfile, 'file') & (~cfg.redo))
   fprintf('not rewriting %s\n', outfile)
   return
end

timebinfile = fullfile(time_gbindir, [rawfilebase, '.tim.gbin'])
if (~exist(timebinfile, 'file'))
   fprintf('cannot find gridded time file %s\n', timebinfile)
   return
end


tic
%% read time info 
timdata = read_bin(timebinfile);
logger_dday = timdata(1,:)';
%get the logger time info from the tim files
badi = find(isnan(logger_dday));
goodi = find(~isnan(logger_dday));
logger_dday = logger_dday(goodi);
logger_timerange = [logger_dday(1) logger_dday(end)];


%number of columns beyond the time stamp
numbest = 2*find(strcmp(cfg.rules, 'gps')) + ...   %lon and lat
          length(find(strcmp(cfg.rules, 'heading'))) + ... 
          length(find(strcmp(cfg.rules, 'pitch')))   + ...   
          length(find(strcmp(cfg.rules, 'roll')))    + ...   
          length(find(strcmp(cfg.rules, 'heave')))   + ...  
          length(find(strcmp(cfg.rules, 'sndspd')))  ;
                                      


%initialize the gbin file:
bestgbin = NaN*ones(numbest, length(timdata(1,:)));
%set up structure (add names from cfg.rules to lon and lat)
bestfnames = {'lon', 'lat', cfg.rules{find(~strcmp(cfg.rules, 'gps'))}};


%% first get the data for each kind of instrument, then extract the parts as 
%% specified by rules (this is to avoid reading attitude files more than
%% once

dir_suffix = {};
count = 1;
for ii=1:length(cfg.rules)
   rulename = sprintf('%s_rule',cfg.rules{ii});
   if (isfield(cfg, rulename))
      rule = getfield(cfg, rulename);
      dir_suffix{count,1}= rule{1};
      dir_suffix{count,2}= rule{2};
      count = count+1;
   else
      fprintf('no rule was specified for data type  %s\n', cfg.rules{ii})
      return
   end
end
% now get rid of repeats
badi = [];
for entrynum = 2:length(dir_suffix(:,1))
   for jj=entrynum-1:-1:1
      if ( strcmp(dir_suffix{entrynum,1}, dir_suffix{jj,1}) & ...
           strcmp(dir_suffix{entrynum,2}, dir_suffix{jj,2}))
         badi = [badi; entrynum];
      end
   end
end
badi = unique(badi);
dir_suffix(badi,:)=[];

% now load the data from each of these instrument types; to be called 
%subsequently.

for dirnum = 1:length(dir_suffix(:,1))
   rbindir = fullfile(cfg.rbindir, dir_suffix{dirnum,1});
   if (exist(rbindir,'dir'))
      [bindata(dirnum).a, bindata(dirnum).s]  = ...
          bin_extract(rbindir, dir_suffix{dirnum,2},  logger_timerange );
      if (~isemptys(bindata(dirnum).a))
         bindata(dirnum).a(1,:) = fix_bintimes(cfg, ...
                                               bindata(dirnum).a, logger_dday);
      end
   else
      bindata(dirnum).a = [];
      bindata(dirnum).s = '';         
      berrfid = fopen('bindata.errs','a');
      fprintf(berrfid, 'written at %s\n', nowstr);
      fprintf('check bindata.errs\n')
      fclose(berrfid);
   end
end

%encapsulate all the flag-checking in a function
bindata = cleanup(bindata, dir_suffix);
   
   
%========= do positions: gps_rule ==================

if (find(strcmp(cfg.rules, 'gps')))
   %now use rule:
   fname = 'lon';
   best_row = find(strcmp(bestfnames, fname));
   rule = cfg.gps_rule;
   rows = userows(rule, dir_suffix);
   
   bestgbin(best_row,:) = apply_rule(bindata(rows), logger_dday, rule, fname);
   fname = 'lat';
   best_row = find(strcmp(bestfnames, fname));
   rule = cfg.gps_rule;
   rows = userows(rule, dir_suffix);
   bestgbin(best_row,:) = apply_rule(bindata, logger_dday, rule, fname);
end

% ========== do heading ===============

if (find(strcmp(cfg.rules, 'heading')))
   fname = 'heading';
   rule = cfg.heading_rule;
   rows = userows(cfg.heading_rule, dir_suffix);
   best_row = find(strcmp(bestfnames, fname));

   %apply constants
   %data = apply_constants(bindata(rows), rule, 'heading');
  
   %now use rule:
   bestgbin(best_row,:) = apply_rule(bindata(rows),  logger_dday, rule, fname);
end

% ========= do pitch =================  

if (find(strcmp(cfg.rules, 'pitch')))
   fname = 'pitch';
   rule = cfg.pitch_rule;
   rows = userows(rule, dir_suffix);
   best_row = find(strcmp(bestfnames, fname));
   
   %now use rule:
   bestgbin(best_row,:) = apply_rule(bindata(rows), logger_dday, rule, fname);
end      

% ========= do roll  =================   

if (find(strcmp(cfg.rules, 'roll')))
   fname = 'roll';
   rule = cfg.roll_rule;
   rows = userows(rule, dir_suffix);
   best_row = find(strcmp(bestfnames, fname));
   
   %now use rule:
   best_row = find(strcmp(bestfnames, fname));
   bestgbin(best_row,:) = apply_rule(bindata(rows), logger_dday, rule, fname);
end      


%========= heave ====================

if (find(strcmp(cfg.rules, 'heave')))
   fname = 'heave';
   rule = cfg.heave_rule;
   rows = userows(rule, dir_suffix);
   best_row = find(strcmp(bestfnames, fname));

   %now use rule:
   best_row = find(strcmp(bestfnames, fname));
   bestgbin(best_row,:) = apply_rule(bindata(rows), logger_dday, rule, fname);
end


 % ========= soundspeed =========== (untested)
if (find(strcmp(cfg.rules, 'sndspd')))
   fname = 'sndspd';
   rule = cfg.snd_spd_rule;
   rows = userows(rule, dir_suffix);
   %now use rule:
   best_row = find(strcmp(bestfnames, fname));
   bestgbin(best_row,:) = apply_rule(bindata(rows), logger_dday, rule, fname);
end


%% write the file
write_bin(outfile, bestgbin, 'best', bestfnames);

toc

%%%%%%%%%%%%%%

function rows = userows(rule,dir_suffix)
rows = [];
for rulerow=1:length(rule(:,1))
   for dsnum = 1:length(dir_suffix(:,1))
      if (strcmp(dir_suffix{dsnum,1}, rule{rulerow, 1}) & ...
          strcmp(dir_suffix{dsnum,2}, rule{rulerow, 2}))
         rows = [rows; dsnum];
      end
   end
end 
rows = unique(rows);


%%%%%%%%%%%%%

function bin_fldday = fix_bintimes(cfg, bindata, logger_dday);
%
% filtered logger dday, in case there are busy cpu cycles:
bindata(1,:) = fix_timestamps(bindata);
nani = find_gaps(logger_dday, bindata(1,:), cfg.gap_seconds);
dt = filt242(diff(bindata(1,:)));
bin_lmean = mean(bindata(1,:));   %want mean of times to match
bin_filt_ldday = bindata(1,1) + cumsum([0 dt]);
%this might only be necessary for small files: make the means match
bin_fldday = bin_filt_ldday - mean(bin_filt_ldday) + bin_lmean;
%make sure shifts over 1/2 don't occur
bin_fldday = unscootch(bin_fldday, bindata(1,:), cfg.scootch_sec);

%%%%%%%%%%%%%%%%
function bindata = apply_constants(bindata, rule, fname)
%now apply constants to offset things (eg. shift adu to match posmv)
% temporarily DO NOT CALL
%
fprintf('called apply_constants: disabled\n')
return


for ii=1:length(rule(:,1))
   %                         reference  instrument
   %                  eg      posmv
   
   [rows, cols]=size(rule);
   % if there is a reference listed and it is not the instrument itself:
   if (rows == 3) 
      if ~strcmp(rule(ii,1), rule(ii,3))
         % reference this isntrument (row 1) to the instrument in row 3
         ref_datarow = find(strcmp(rule(ii,3), rule(:,1)));
         refstruct = bindata(ref_datarow).s;                   %posmv struct
         refdata   = bindata(ref_datarow).a;                   %posmv data
         ref_fnamerow = find(strcmp(refstruct.rows, fname));   %heading row
         
         % instrument being referenced
         bstruct = bindata(ii).s;                              % eg. adu struct
         bdata = bindata(ii).a;                                %     adu data
         fname_row = find(strcmp(bstruct.rows, fname));        % heading row
         
         
         bdata_on_ref = table1([bdata(1,:)' bdata'], refdata(1,:)')';
         [diffmean, diffstd, diffn, diffmed] = ...
             mstdgap(refdata(ref_fnamerow,:) - bdata_on_ref(fname_row,:));
         
         
         bdata(fname_row,:) =  bdata(fname_row,:) + diffmed;
         bindata(ii).a = bdata;                                %     adu data
      end
   end
end

%%%%%%%%%%%%%%
function bestrow = apply_rule(bindata, logger_dday, rule, fname);
% start with worst option, move towards top, replacing if data are good

bestrow = NaN*ones(size(logger_dday'));
for ii=length(rule(:,1)):-1:1
   if (~isemptys(bindata(ii).a))
      gbinned = table1(bindata(ii).a', logger_dday)';
   
      %take one away from index because gbinned does not include timestamp
      rownum = find(strcmp(bindata(ii).s.rows, fname));
      if (~isemptys(rownum))
         rule_fnamerow = rownum  - 1;
         
         goodi = find(~isnan(gbinned(rule_fnamerow,:)));
         if (~isemptys(goodi))
            bestrow(goodi) = gbinned(rule_fnamerow, goodi);
         end
      end
   end
end   

%%%%%%%%%%%%%%%

function bindata = cleanup(bindata, dir_suffix);


   %posmv position
   posmv_row = find(strcmp('posmv', dir_suffix(:,1)));
   if (~isemptys(posmv_row)) %there is a posmv rule
      for datatype = 1:length(posmv_row)
         if (strcmp(dir_suffix{posmv_row(datatype), 2}, 'gps'))
            if (~isemptys(bindata(posmv_row(datatype)).a)) %it has data
                % give it an easier name
                bdata = bindata(posmv_row(dtype)).a;
                bstruct = bindata(posmv_row(dtype)).s;
                lon_row  = find(strcmp(bstruct.rows, 'lon'));
                lat_row  = find(strcmp(bstruct.rows, 'lat'));
                qual_row = find(strcmp(bstruct.rows, 'quality'));
                badi = find( ...
                   (bdata(qual_row, :) == 0) | (bdata(qual_row, :) == 6));
                bdata([lon_row lat_row],badi) = NaN;
                %now put it back in
                bindata(posmv_row(dtype)).a = bdata;
                bindata(posmv_row(dtype)).s = bstruct;
            end
         elseif (strcmp(dir_suffix{posmv_row(datatype), 2}, 'pmv'))
            %posmv attitude
            % use the reaquisition flag to make bad entries into NaNs
            bdata = bindata(posmv_row(dtype)).a;
            bstruct = bindata(posmv_row(dtype)).s;

            % edit (NaN) out bad headings
            heading_row  = find(strcmp(bstruct.rows, 'heading'));
            qual_row = find(strcmp(bstruct.rows, 'acc_heading'));
            badi = find(bdata(qual_row, :) > acc_heading_cutoff);
            bdata(heading_row,badi) = NaN;

            % edit (NaN) out bad pitch
            pitch_row  = find(strcmp(bstruct.rows, 'pitch'));
            qual_row = find(strcmp(bstruct.rows, 'acc_pitch'));
            badi = find(bdata(qual_row, :) > acc_pitch_cutoff);
            bdata(heading_row,badi) = NaN;

            % edit (NaN) out bad roll
            roll_row  = find(strcmp(bstruct.rows, 'roll'));
            qual_row = find(strcmp(bstruct.rows, 'acc_roll'));
            badi = find(bdata(qual_row, :) > acc_roll_cutoff);
            bdata(heading_row,badi) = NaN;

            % throw out heave if all 3 are bad (pretty arbitrary)
            heave_row  = find(strcmp(bstruct.rows, 'heave'));
            badi = find(isnan(bdata(heading_row,:)) & ...
                        isnan(bdata(pitch_row,:)) & ...
                        isnan(bdata(roll_row,:)));
            bdata(heave_row, badi) = NaN;
      
            %now put it back in
            bindata(posmv_row(dtype)).a = bdata;
            bindata(posmv_row(dtype)).s = bstruct;
         else
            fprintf('posmv quality control only coded for position or attitude\n')
         end
      end
   end

   
   %seapath position,    % specific to seapath binary message 11
   seabin_row = find(strcmp('seapath1', dir_suffix(:,1)));
   if (~isemptys(seabin_row)) %there is a seapath1 dir_suffix
      if (~isemptys(bindata(seabin_row).a)) %it has data
         
         %% fix positions differently from attitude
         % give it an easier name
         bdata = bindata(seabin_row).a;
         bstruct = bindata(seabin_row).s;
         lon_row  = find(strcmp(bstruct.rows, 'lon'));
         lat_row  = find(strcmp(bstruct.rows, 'lat'));
         badi = find(...
            abs(bdata(lon_row,:) - boxfilt(bdata(lon_row,:),31)) > 10 | ...
            abs(bdata(lat_row,:) - boxfilt(bdata(lat_row,:),31)) > 10 );
         bdata([lon_row lat_row],badi) = NaN;
         
         %now do heading
         qual_row = find(strcmp(bstruct.rows, 'status'));
         heading_row = find(strcmp(bstruct.rows, 'heading'));      
         badi = find(bdata(qual_row, :) > 1 );
         bdata(heading_row,badi) = NaN;
         %now put it back in
         bindata(seabin_row).a = bdata;
         bindata(seabin_row).s = bstruct;
      
      end
   end



   % adu2 heading
   % use the reaquisition flag to make bad entries into NaNs (reaqfl = 1)
   adu2_row = find(strcmp('adu2', dir_suffix(:,1)));
   if (~isemptys(adu2_row)) %there is an adu2 rule
      % give it an easier name
      bdata = bindata(adu2_row).a;
      bstruct = bindata(adu2_row).s;
      qual_row = find(strcmp(bstruct.rows, 'quality'));
      heading_row = find(strcmp(bstruct.rows, 'heading'));      
      badi = find(bdata(qual_row, :) == 1 );
      bdata(heading_row,badi) = NaN;
      %now put it back in
      bindata(adu2_row).a = bdata;
      bindata(adu2_row).s = bstruct;
   end


   %sound speed (untested)
   sndspd_row = find(strcmp('sndspd', dir_suffix(:,1)));
   if (~isemptys(sndspd_row)) %there is a sound speed rule
                             % give it an easier name
      bdata = bindata.a;
      bstruct = bindata.s;
      sndspd_row  = find(strcmp(bstruct.rows, fname));
      badi = flag_sspd(bindata);
      bdata([sndspd_row],badi) = NaN;
      bindata.a = bdata;
      bindata.s = bstruct;
   end
