function xxx2gbin(rawfilebase, varargin)
% xxx2gbin(rawfilebase, varargin)
%                                
% read .tim.bin files to get best logger times 
%  *** should have already created it with time2gbin ***
% find the matching (logger) time ranges for xxx files, 
% interpolate to get xxx fields  at logger times,  
% write a flat binary file with (ncols-1)xN entries
% output name is 'outfpath/rawfilebase.xxx.gbin' for "gridded bin"
% (these files are screened for some errors; error fields removed)
% (these files are only useful for diagnostic purposes; see 
%     best2gbin for choice of best position, roll, pitch, heading)
% 
% eg. xxx2gbin('nbp2001_094_36014', 'binfpath', 'gyro')
% varargin name, value pairs are:
% 
% tim_gbinpath [path to *.tim.gbin]   (input)    % defaults to ../gbin/time
% xxx_rbinpath [path to *.xxx.rbin]     (input)    % *** required ***
%           xxx is 'hdg', 'gps', 'adu', 'spd'
%           eg: ../rbin/pcode
% outfpath [path to output file]                 % *** required ***
% logfpath  [path to log file]      './' default
% in_fext     [file extension in xxx_rbinpath]    % *** required  ***
%           xxx is 'hdg', 'gps', 'adu', 'spd'
% out_fext    [file extension in xxx_gbinpath]   % *** required  ***
%           xxx is 'hdg', 'pos', 'adu', 'spd'
% gap_seconds [NaN positions with    %default 10
%             gaps longer than this]
% scootch_sec [require time shifts to         % .5
%              be under this
% redo     [0 or 1] redo gbin file            % 0 
%
% eg. running from a subdirectory of adcpr (where rbin and gbin 
%         directories are ../rbin and ../gbin)
%
% xxx2gbin('rb2003_195_64800', 'xxx_rbinpath', '../rbin/seapath1',...
%          'outfpath', '../gbin/seapath1',...
%          'in_fext', 'sea',...
%          'out_fext', 'sea')
%
%


%JH 2001/04/05

cfg.xxx_rbinpath = '';
cfg.tim_gbinpath = 'gbin/time';
cfg.outfpath = '';
cfg.logfpath = './';              %not currently used
cfg.gap_seconds = 10;
cfg.scootch_sec = .5;
cfg.in_fext     = '';
cfg.out_fext     = '';
cfg.redo = 0;

cfg = fillstruct(cfg, varargin, 0);

if (isemptys(cfg.in_fext))
      error('in_fext not set\n')
end

if (isemptys(cfg.out_fext))
   error('out_fext not set')
end

if (isemptys(cfg.xxx_rbinpath))
      error('path to *.xxx.rbin not set')
end

if (isemptys(cfg.outfpath))
      error('path to outfile *.xxx.gbin not set')
end


outfile =     fullfile(cfg.outfpath, ...
                       sprintf('%s.%s.gbin', rawfilebase, cfg.out_fext));

if (exist(outfile, 'file') & (~cfg.redo))
   fprintf('not writing %s\n', outfile)
   return
end


%% read time info 
timefile = fullfile(cfg.tim_gbinpath, [rawfilebase, '.tim.gbin']);
timdata = read_bin(timefile);
logger_dday = timdata(1,:)';
%get the logger time info from the tim files
badi = find(isnan(logger_dday));
goodi = find(~isnan(logger_dday));
logger_dday = logger_dday(goodi);
logger_timerange = [logger_dday(1) logger_dday(end)];

%% get bindata for this particular file, fiddle with time stamps

[bindata, binstruct] = ...
    bin_extract(cfg.xxx_rbinpath, cfg.in_fext,  logger_timerange );
if (isemptys(bindata))
   cfg
   fprintf('%s:\n', timefile)
   fprintf('extracting data from %s/*.%s.rbin\n', cfg.xxx_rbinpath, cfg.in_fext)
   fprintf('no bin data read for logger time range %10.7f - %10.7f\n',...
           logger_timerange(1),logger_timerange(2))
   return
end


%% new: JH 2002/08/19
%fix bad times due to ?? cpu cycles used up??
bindata(1,:) = fix_timestamps(bindata);
%%%


nani = find_gaps(logger_dday, bindata(1,:), cfg.gap_seconds);
dt = filt242(diff(bindata(1,:)));
bin_lmean = mean(bindata(1,:));   %want mean of times to match
bin_filt_ldday = bindata(1,1) + cumsum([0 dt]);
%this might only be necessary for small files: make the means match
bin_fldday = bin_filt_ldday - mean(bin_filt_ldday) + bin_lmean;
%make sure shifts over 1/2 don't occur
bin_fldday = unscootch(bin_fldday, bindata(1,:), cfg.scootch_sec);


%%% rearrange columns if necessary, delete bad attitudes

% use the reaquisition flag to make bad entries into NaNs (reaqfl = 1)
if (strcmp(cfg.in_fext, 'adu'))
   badi = find(bindata(8,:) == 1);
   %       1          2       3       4      5     6      7
   % [logger_dday, attdday, heading, pitch, roll, mrms, brms]
   bindata([2:7],badi) = NaN;
   bindata([2 6 7 8],:) = [];  %lose the att time and flags
   % rearrange it
   % [logger_dday,  heading, pitch, roll]
   %     1             2       3     4
end


% make this act like adu2:
if (strcmp(cfg.in_fext, 'pmv'))
   % manual indicates:
   % reliable heading if (flag_heading == 2) & (flag_IMU == 1) %rows 10 and 11
   %       1            2       3       4      5        6
   % [logger_dday, dday_frac, heading, roll, pitch, heave, ...
   %                        7          8          9          10           11
   %                    acc_roll, acc_pitch, acc_heading, flag_heading, flag_IMU

   % bad heading:
   badi = find(bindata(9,:) > .5);  %% (heading accuracy cutoff )
   bindata(3,badi) = NaN;
   % bad pitch:
   badi = find(bindata(8,:) > .5);  %% (pitch accuracy cutoff )
   bindata(5,badi) = NaN;
   % bad roll
   badi = find(bindata(7,:) > .5);  %% (roll accuracy cutoff )
   bindata(4,badi) = NaN;
   % 
   bindata([2 7:11],:) = [];
   % [logger_dday, heading, pitch, roll, heave] 
end


if (strcmp(cfg.in_fext, 'sea')) %seapath binary message 11
   % look for status message of 1 (indicates bad)
   %       1          2              3           4           5
   %   'u_dday'    'dday_frac'    'heading'    'pitch'    'roll'
   %             6          7           8        9        10
   %          'heave'    'height'    'lon'    'lat'    'status'
   
   qual_row = find(strcmp(binstruct.rows, 'status'));
   badi = find(bindata(qual_row, :) > 1 );
   heading_row = find(strcmp(binstruct.rows, 'heading'));
   bindata(heading_row,badi) = NaN;
   bindata([2 6 7 10],:)=[];
   %[logger_dday heading, pitch, roll, lon,  lat]
end


%remove bad points in speed of sound
if (strcmp(cfg.in_fext, 'spd'))
   badi = flag_sspd(bindata);
   if (~isemptys(badi))
      bin_fldday(badi)=[];
      bindata(:,badi)=[];
   end
end


if (strcmp(cfg.in_fext, 'gps'))
   bindata = bindata([1 3 4],:); % logger_time [gga time] lon, lat
end

%%%%%%%%
% table the data onto the logger ddays
%%%%%%%%


try
   gdata = NaN*ones(length(timdata(1,:)), length(bindata(:,1))-1);
   gdata(goodi,:) = table1([bin_fldday' bindata(2:end,:)'], logger_dday);
   gdata = gdata';
catch
   cfg.in_fext
   cfg.out_fext
   rawfilebase
   disp('not monotonic')
   keyboard
end

   
gdata(:,nani) = NaN;

if (length(gdata(1,:)) ~= length(timdata(1,:)))
   error('incorrect number of indices')
   keyboard
end


%% JH 2002/08 generic names; rely on data source to distinguish
if (strcmp(cfg.out_fext, 'hdg'))
   write_bin(outfile, gdata, 'hdg', {'heading'});
elseif (strcmp(cfg.out_fext, 'pos'))
   write_bin(outfile, gdata, 'pos', {'lon', 'lat'});
elseif (strcmp(cfg.out_fext, 'spd'))
   write_bin(outfile, gdata, 'spd', {'soundspeed'});
elseif (strcmp(cfg.out_fext, 'adu'))
   write_bin(outfile, gdata, 'adu', ...
             { 'heading', 'pitch', 'roll'});
elseif (strcmp(cfg.out_fext, 'pmv'))
   write_bin(outfile, gdata, 'pmv', ...
             { 'heading', 'pitch' , 'roll', 'heave'});
elseif (strcmp(cfg.out_fext, 'sea'))
   write_bin(outfile, gdata, 'sea', ...
             {'heading', 'pitch', 'roll', 'lon',  'lat'});
else
   error('not writing for some reason: output file extension mismatch?')
end


