function [trinfo, rbinstruct, qcmask] = tr2ilist(arg1, varargin)
% TWO modes: (1) % trinfo = tr2ilist(arg1, varargin)
%                  return cellaray with file and time info
%
%            (2)  [trinfo, rbinstruct, qcmask] = tr2ilist(arg1, varargin)
%                return data and structure from rbin read; 
%                data have qc applied or not; mask has 1 for bad values
% [trinfo, rbinstruct, qcmask] = tr2ilist(arg1, varargin)
% (1)
% trinfo = tr2ilist(filelist, varargin)
% input: filelist is a filelist as from dirs('*.rbin','fullfile',1)
% returns a cellarray with each row containing:
%    filename, [startdd enddd], [starti endi], numpts_in_tr
%    (for files that need to be read)
%    [startdd enddd] are decimal day ranges of u_dday from file
%    [starti endi] are index range from this file for time range 
%
% (2)
% subset = tr2ilist(trinfo, varargin)
% input: trinfo is from a previous call of tr2ilist
%        Uses info from trinfo to output a subset of the data
%          that is inside timerange
%
%
% varargin
%
%   name                    what                        default
%  ---------          --------------                --------------
%   'timerange'       dday time range of file             'all'  
%                  string 'all' (returns all)          
%                     [dday1 dday2]
%   
%   'rowname'        row to use as the range           'u_dday'
%   'step'           take every N values (only call #2))   1
%   'sort'           how to sort rows         
%                   'none' (as is)                     'none'
%                   'time' (startdd)
%  'verbose'         [0,1] print messages                0
%
%% (do "help r_row" to access rows using structure)


%% undocumented: 
%  'timebase'        'raw' (as is) or 'UTC' (adjusted    'raw'
%                            by pc_sec_minus_utc
%                            if available
%
% NOTE (1): rbin time bases vary.  See 'serasc2bin.py' for more info  
%      (2): If using tr2ilist with 'timebase', 'UTC', use it in both calls!!


% jh

cfg.sort = 'none';
cfg.timerange = 'all';
cfg.rowname = 'u_dday';
cfg.verbose = 0;
cfg.step = 1;
cfg.timebase = 'raw';


cfg = fillstruct(cfg, varargin, 0);

trinfo = [];
rbinstruct = [];
qcmask = [];

if isemptys(arg1)
   return
end



if isstruct(arg1) %sending in a filelist
   
   if isemptys(arg1)
      fprintf('%s returning: (filelist was empty)\n', mfilename)
      return
   end
   
   badi = [];
   filelist = arg1;

   for ii=1:length(filelist)
      filename = filelist(ii).name;
      if (~exist(filename,'file'))
         badi = [badi; ii];
         fprintf('could not find file %s\n', filename)
      end
   end
   filelist(badi)=[];


   for ifile=1:length(filelist)
      [tmp, rbinstruct]=read_bin(filelist(ifile).name,'last',1);
      if (~isemptys(tmp))
         fclose(rbinstruct.fid);
         continue
      end
   end
   

   rownum = find(strcmp(rbinstruct.rows, cfg.rowname));
   if (isemptys(rownum))
      fprintf('\n\n%s error: you chose rowname %s\n', mfilename, cfg.rowname)
      fprintf('must choose rowname from among these\n')
      rbinstruct.rows
      trinfo = [];
      rbinstruct = [];
      return
   end
   
   badi = [];
   % go through and fill the cellarray; delete unused files later
   trinfo = cell(length(filelist),4);
   for filei = 1:length(filelist)
      filename = filelist(filei).name;
      data = read_bin(filename);         
      
      if (isemptys(data))
         trinfo{filei,1} = filename;
         trinfo{filei,2} = [];
         trinfo{filei,3}=  [];
         trinfo{filei,4}=  [];
         badi = [badi; filei];
      else
         
         
         if strcmp(cfg.timebase, 'UTC')
            if ismember('sec_pc_minus_utc', rbinstruct.rows)
               dtrow = find(strcmp(rbinstruct.rows,'sec_pc_minus_utc' ));
               dt = data(dtrow,:);
               [medfilt_dt, ibig] = medfilt(dt, 3, 20);
               dday = data(rownum,:) - medfilt_dt/86400;
               fprintf('adjusting raw times by about %2.2f seconds to get UTC\n', ...
                       mstdgap(medfilt_dt))
            else
               dday = data(rownum,:);
               fprintf('cound not find sec_pc_minus_utc\n')
            end
         else
            dday = data(rownum,:);
         end
               
         if (isstr(cfg.timerange))
            trange = [min(dday), max(dday)];
         else
            trange = cfg.timerange;
         end
         
         %check whether the file is completely outside the timerange
         if (min(dday) > trange(2))               % data(1) after trange
            badi = [badi; filei];
         elseif (max(dday) < trange(1))
            badi = [badi; filei];
         else
            ii = find(...
               (dday >= trange(1)) & (dday <= trange(2)));
            trinfo{filei,1} = filename;
            trinfo{filei,2} = [min(dday) max(dday) ];
            trinfo{filei,3}=  [min(ii) max(ii)];
            trinfo{filei,4}=  [max(ii) - min(ii)+1];            
         end
      end
   end

   if (~isemptys(badi))
      trinfo(badi,:) = [];
   end

      
elseif iscell(arg1)  %using previous trinfo
   trinfo = arg1;

   [pp,ff,ee]=fileparts(trinfo{1,1});
   % get the pieces if possible
   [junk, junk2, cfg.suffix] = fileparts(ff);
   cfg.suffix(1) = [];
   if strcmp(cfg.suffix, 'tim') | strcmp(cfg.suffix, 'best')
      cfg.apply_qc = 0;
   end
   
   [tmp, rbinstruct]=read_bin(trinfo{1,1},'last',1);
   try
      fclose(rbinstruct.fid);
   catch
      fprintf('cannot close file id \n')
      trinfo = [];
      return
   end
      
   rownum = r_row(rbinstruct, cfg.rowname);
   if (isemptys(rownum))
      fprintf('\n\n%s error: you chose rowname %s\n', mfilename, cfg.rowname)
      fprintf('must choose rowname from among these\n')
      rbinstruct.rows
      trinfo = [];
      return
   end

   badi = [];
   % go through and fill the cellarray; delete unused files at the end
   % definitely delete empty files!

   for filei = 1:length(trinfo(:,1))

      filename = trinfo{filei,1};
      if cfg.verbose, 
         fprintf('tr2ilist: reading file %s\n', filename)
      end
      
      %no time range: delete row from trinfo later
      if (isemptys(trinfo{filei,2}))
         badi = [badi; filei];
      else
         
         % requested time range:
         if (isstr(cfg.timerange))
            req_trange = trinfo{filei,2};
         else
            req_trange = cfg.timerange;
         end
         
         %check whether the file is completely outside the timerange
         file_tr = trinfo{filei,2}; 
         if (file_tr(1) > req_trange(2))     % data(1) after req_trange
            badi = [badi; filei];
         elseif (file_tr(2) < req_trange(1))   % data(2) before req_trange
            badi = [badi; filei];
         else
            data = read_bin(trinfo{filei,1});
         
            if strcmp(cfg.timebase, 'UTC')
               if ismember('sec_pc_minus_utc', rbinstruct.rows)
                  dtrow = find(strcmp(rbinstruct.rows,'sec_pc_minus_utc' ));
                  dt = data(dtrow,:);
                  [medfilt_dt, ibig] = medfilt(dt, 3, 20);
                  dday = data(rownum,:) - medfilt_dt/86400;
                  fprintf('adjusting raw times by about %2.2f seconds to get UTC\n', ...
                       mstdgap(medfilt_dt))
               else
                  dday = data(rownum,:);
                  fprintf('could not find sec_pc_minus_utc\n')
               end
            else
               dday = data(rownum,:);
            end

            ii = find(...
               (dday >= req_trange(1)) & ...
               (dday <= req_trange(2)));
            %update indices to read
            if (isemptys(ii))
               badi = [badi; filei];
            else
               trinfo{filei,3}=  [ii(1) ii(end)];
               trinfo{filei,4}=  [ii(end) - ii(1) + 1];
            end
         end
      end
      if isemptys(trinfo{filei,3}) | isemptys(trinfo{filei,4})
         badi = [badi; filei];
      end            
   end
   
   %delete the bad ones
   
   if (~isemptys(badi))
      trinfo(badi,:) = [];
   end
  
   if (isemptys(trinfo))
      return
   end
   
   %initialize the array; read the remaining ones
   NP = 0;
   for filei = 1:length(trinfo(:,1))
      NP = NP + trinfo{filei,4};
   end
   segment = NaN*zeros(length(rbinstruct.rows), NP);
   maxindex = 0;
   % now fill it
   
   for filei = 1:length(trinfo(:,1))
      data = read_bin(trinfo{filei,1},'irange',trinfo{filei,3});
      
      if cfg.step ~= 1
         data = data(:,  1:cfg.step:end);
      end

      if ~isemptys(data)
         if strcmp(cfg.timebase, 'UTC')
            if ismember('sec_pc_minus_utc', rbinstruct.rows)
               dtrow = find(strcmp(rbinstruct.rows,'sec_pc_minus_utc' ));
               dt = data(dtrow,:);
               [medfilt_dt, ibig] = medfilt(dt, 3, 20);
               data(rownum,:) = data(rownum,:) - medfilt_dt/86400;
            end
         end
         
         count = maxindex + [1:length(data(1,:))];
         if length(segment(:,1)) ~= length(data(:,1))
            fprintf('number of rbin data rows changed midstream\n')
            fprintf('skipping file %s\n',trinfo{filei,1})
            continue
         end
         segment(:,count) = data;
         maxindex = count(end);
      end
   end

   %get rid of bad ones
   nani = find(isnan(segment(1,:)));
   segment(:,nani)=[];
   
   
   %%% unwrap  heading if necessary:
   head_row = find(ismember(rbinstruct.rows, 'heading'));
   if ~isemptys(head_row)
      heading = segment(head_row,:);
      head_tol = 10;
      head = rem(heading + head_tol,360) - head_tol;
      if ~isemptys(find(abs(head) < .5*head_tol)) %there are headings near zero
         segment(head_row,:) = 180*unwrap(head*pi/180)/pi;
      end
   end
   
   %%% unwrap longitudes if necessary (assumes split at date line)
   lon_row = find(ismember(rbinstruct.rows, 'lon'));
   if ~isemptys(lon_row)
      lon = segment(lon_row,:);
      lon_tol = 10;
      lon = rem(lon + lon_tol,360) - lon_tol;
      if ~isemptys(find(abs(lon + 180) < .5*lon_tol)) %there are longitudes near 180
         segment(lon_row,:) = 180*unwrap(lon*pi/180)/pi;
      end
   end
   
   
   if nargout == 3
      qcmask = qc_rbin(segment, rbinstruct);
   end
   
   %rename it for output
   trinfo = segment;
   
else
   help(mfilename)
   fprintf('first argument was not a structure or a cellarray\n')
   fprintf('maybe you tried to use the old tr2ilist. \n')
   fprintf('... its first argument was a time range.  arg1 was:\n')
   arg1
   trinfo = [];
   return
end

if isstruct(arg1) & strcmp(cfg.sort,'time')
   startdd = ones(length(trinfo(:,1)),1);
   for ii=1:length(trinfo(:,1))
      tr = trinfo{ii,2};
      startdd(ii)= tr(1);
   end
   [junk,sortedi] = sort(startdd);
   trinfo = trinfo(sortedi,:);
end

      
      
      



