function mask = correlation_cutoff(data, editcfg)
% mask = correlation_cutoff(data, editcfg)
% who: called by pingedit
% output: returns a mask with 1 where data.cor falls below  
%              editcfg.cor_cutoff in the range of editcfg.cor_testbins
% input (1) data:  structure containing correlation field, i.e.  data.cor
%                  (data.cor can be Nbins x Nbeams x NtimesNbins x Ntimes)
%       (2) editcfg: structure from get_pedefaults, containing fields
%                  'cor_cutoff" and "cor_testbins"
%
% If using one cutoff:
% editcfg.cor_cutoff  applies to the bins in editcfg.cor_testbins
% editcfg.cor_testbins can be the string 'all', or a bin range
% 
% If using multiple cutoffs:
% editcfg.cor_testbins and editcfg.cor_cutoff are cellarrays of valid
%        entries instead of a single valid entry
% 

mask = zeros(size(data.cor));

if ~isfield(data, 'cor') 
   fprintf('no correlation in this dataset\n')

   return
end


if ~isfield(editcfg, 'cor_cutoff') 
   fprintf('no correlation cutoff in editcfg\n')
   return
end

if ~isfield(editcfg, 'cor_testbins') 
   fprintf('no cor_testbins in editcfg\n')
   return
end
%it's a structure; look for fields
if ~isfield(editcfg, 'cor_cutoff') | ~isfield(editcfg, 'cor_testbins')
   fprintf('editcfg must have fields "cor_cutoff" and "cor_testbins"\n')
   return
end
% it has the right fields, check to see whether they match
cellarray_mismatch = 0;
if (iscell(editcfg.cor_cutoff) & ~iscell(editcfg.cor_testbins)) |...
      (~iscell(editcfg.cor_cutoff) & iscell(editcfg.cor_testbins)) 
   cellarray_mismatch = 1;
elseif iscell(editcfg.cor_cutoff)
   if length(editcfg.cor_cutoff) ~= length(editcfg.cor_testbins)
      cellarray_mismatch = 1;
   end
else %not cellarrays -- make cellarrays
   cellarray_mismatch = 0;
   editcfg.cor_cutoff   = {editcfg.cor_cutoff};
   editcfg.cor_testbins = {editcfg.cor_testbins};
end

if cellarray_mismatch
   fprintf('error in editcfg setup:\n')
   fprintf('cellarray mismatch in fields  cor_cutoff and cor_testbins\n')
   fprintf('if one field is acellarray, the other must a cellarray of the same length.\n')
   return
end

% do it


cor = data.cor;
if length(size(cor)) == 3
   mask = zeros(size(squeeze(cor(:,1,:))));
   for beami = 1:length(cor(1,:,1))
      mask = mask | apply_cor(squeeze(cor(:,beami,:)), ...
                              editcfg.cor_cutoff,...   
                              editcfg.cor_testbins );
   end
else
   mask = apply_cor(cor, editcfg.cor_cutoff, editcfg.cor_testbins );
end
      

%%%%%%%%%%%%%%%%%%%%%%%%                       


function mask = apply_cor(cor, cutoff, binrange)

mask = zeros(size(cor));   

if ~iscell(binrange)
   brcell = binrange{1};
   ccell = cutoff{1};

   binrange = brcell;
   cutoff = ccell;
end


for ii=1:length(cutoff)
   if isstr(binrange{ii})
      if strcmp(binrange{ii}, 'all')
         bins = [1 : length(cor(:,1))]';
      else
         fprintf('string %s not allowed for correlation cutoff\n', cutoff{ii})
         return
      end
   else
      br = binrange{ii};
      bins = br(1):br(2);
   end
   mask(cor(bins,:) < cutoff{ii}) = 1;
end



