function editcfg = get_pedefaults(instclass, pingtype)
%% ----------------------------------------------------------------------------
%% "get_pedefaults": return pingedit defaults
%%
%% usage: 
%%  editcfg = get_pedefaults(instclass, pingtype);  %returns structure 
%%   instclass is CLASS, one of:  os, nb, son
%%   pingtype is a STRING, 'son', 'nb', 'bb'  NOT the same as instclass
%%  get_pedefaults               %prints documentation
%%  get_pedefaults(instclass, pingtype)    %prints values 
%
%% eg.
%  editcfg = get_pedefaults(instclass, data.config.pingtype)
%  editcfg = get_pedefaults(os, 'bb')
% 
%% -------------------------------------------------------------------------

%%
%% top section defines the defaults,
%% bottom section prints info.


%% JH 2003
%% 2003/11 NOTE: if changing pingedit defaults (or adding more)
%% change the value in the cfg structure AND document below

%set defaults

if nargin == 0
   str = showdoc;
   for ii=1:length(str(:,1))
      fprintf('%s\n',str{ii})
   end
   help get_pedefaults
   cfg = [];
   return
elseif nargin ~= 2 
   fprintf('must set instclass and pingtype\n')
   help get_pedefaults
   cfg = [];
   return
end

             
% set editcfg fields
editcfg.verbose              = 0;
editcfg.bigstarget_thresh    =  25; 
editcfg.bigstarget_offset    =  3; 
editcfg.bigstarget_nside     =  1; 
editcfg.rl_bins              = 'all';   
editcfg.ocean_uv_cutoff      =  2.5;
editcfg.weakprof_numbins     =  1;   
editcfg.weakprof_testbins    =  [1:10];   % really do want shallow 'all';     
editcfg.refavg_valcutoff     =  1;       
editcfg.refavg_stdcutoff     =  3;       
editcfg.pingedit_functions   =  {'weak_profile', 'refavg_outlier'};
editcfg.useredit_functions   = {};  % user-defined functions

% now specify values for instclass + pingtype configurations

if isa(instclass,'bb')  %bb
   editcfg.cutoff_functions = {...
      'correlation',...
      'error_velocity',...
      'ocean_velocity'};
   editcfg.ampfilt                =        25;
   editcfg.cor_cutoff             =       120;
   editcfg.cor_testbins           =       'all';
   editcfg.error_velocity_cutoff  =        1;
elseif isa(instclass, 'os') 
   if strcmp(pingtype, 'bb')  %osbb
      editcfg.cutoff_functions = {...
         'correlation', ...
         'error_velocity', ...
         'ocean_velocity'};
      editcfg.ampfilt             =        25;
      editcfg.cor_cutoff          =        120;
      editcfg.cor_testbins        =        'all';
      editcfg.error_velocity_cutoff=       1;
   else % osnb 
      editcfg.cutoff_functions = {...
         'correlation', ...
         'error_velocity', ...
         'ocean_velocity'};
      editcfg.ampfilt             =        30;
      editcfg.cor_cutoff          =        120;
      editcfg.cor_testbins        =        'all';
      editcfg.error_velocity_cutoff  =     1;          
   end 
elseif isa(instclass, 'nb') %nb
   editcfg.cutoff_functions = {...
      'spectral_width', ...
      'error_velocity', ...
      'ocean_velocity'};
   editcfg.ampfilt                 =        25; 
   editcfg.spectral_width_cutoff   =       120;  %from the instrument
   editcfg.error_velocity_cutoff   =       .5;          
else
   fprintf('instrument class must be one of: os, bb, nb\n')
   help(mfilename)
   editcfg = [];
   return
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function docstr = showdoc;

%% if pingedit is called without any arguments, make it print documentation
% this is an array of cellarray strings.

docstr =[...
{' Type "help pingedit" for an overview.  '};
{' This is a list of editing parameters, their effect, and their defaults'};
{'                                                                                   '};
{' name         values            default     what                                   '};
{' -------      ---------         --------    ---------------------------------      '};
{' ''verbose''    [0,1]                       be verbose during pingediting          '};
{'                                                                                   '};
{'                                                                                   '};
{'                                                                                   '};
{'                                                                                   '};
{'  Editing of an ensemble of pings takes place in the following stages:           '};
{'  (1) cutoff thresholds: correlation, error velocity, ocean velocity             '};
{'       HELP:                                                                     '};
{'                 help correlation_cutoff                                         '};
{'                 help spectral_width_cutoff                                      '};
{'                 help error_velocity_cutoff                                      '};
{'                 help ocean_uv_cutoff                                            '};
{'                                                                                 '};
{'  (2) pre-pingediting: amplitude despiking, bottom detection                     '};
{'                                                                                 '};
{'       HELP:     help prepedit                                                   '};
{'                                                                                 '};
{'  (3) provision for user-written editing                                         '};
{'                                                                                 '};
{'       HELP:     help useredit                                                   '};
{'                                                                                 '};
{'  (4) after applying #1 and #2, refavg outliers and weak profiles are removed    '};
{'                                                                                 '};
{'       HELP:                                                                     '};
{'                 help weak_profile_edit                                          '};
{'                 help refavg_outlier_edit                                        '};
{'                                                                                 '};
{'                                                                                 '};
{'  procedure:                                                                     '};
{'  - obtain pingedit defaults (type ''help get_pedefaults'' for more information) '};
{'  - override any defaults desired                                                '};
{'  - call pingedit (type ''help pingedit'' for more information)                  '};
{'                                                                                 '};
{'                                                                                 '};
{'   EDITING:                      |        WHAT                                   '};
{'                                 |                                               '};
{'  (1) cutoff thresholds:         |        (flag as bad if)                       '};
{'                                 |                                               '};
{'      correlation_cutoff         |  correlation falls below threshold            '};
{'      spectral_width_cutoff      |  spectral width excedes threshold             '};
{'      error_velocity_cutoff      |  error velocity excedes threshold             '};
{'      ocean_uv_cutoff            |  ocean velocity u,v excede threshold          '};
{'                                 |                                               '};
{'  (2) pre-editing parameters:    |        (what it is)                           '};
{'                                 |                                               '};
{'      bigstarget_thresh          | amplitude threshold: bottom identification    '};
{'      bigstarget_offset          |  how much to smooth the profile first         '};
{'      bigstarget_nside           |  how many bins on each side to look           '};
{'      ampfilt                    |  3-point "median" filter tolerance            '};
{'                                 |                                               '};
{'  (3) useredit                   | (as specified) (type "help useredit")         '};
{'                                 |                                               '};
{'  (4) final pass editing:        |        (what it is)                           '};
{'                                 |                                               '};
{'      refavg_valcutoff           | threshold for "outlier from mean" velocity    '};
{'      refavg_stdcutoff           | threshold for "outlier from mean" vel. stddev '};
{'      weakprofile_testbins       | bins to test if sufficient good data:         '};
{'      weakprofile_numbins        |   number of bins of good data required        '};
{'                                                                                 '};
{'                                                                                 '};
];
