function gotstats = get_rstats(varargin)
% gotstats = test_posmv(varargin)
%
%  UNDER CONSTRUCTION.  Eventually:
%
% generate statistics on rbin data
% operates as specified on one of:
%            - a whole directory of rbin files 
%            - a filelist (cellarray or structure from dirs); 
%                 if not absolute, uses uhdas_dir
%            - a time range (requires uhdas_dir to be specified)
% output: 
% gotstats  is  structure with fields:
%  'rows' (cellarray of statistics names)
%  'stats'  array with one row per per statistic, one column per filename
%  'filenames' is a cellarray, entries correspond to columns in stats
% 
% usage: three examples
% from rbin/posmv 
%        [stats, filenames] = get_rstats;
% from $cruiseid/quality, 
%     [stats, filenames] = get_rstats('uhdas_dir','../rbin/posmv', 'msg', pmv');
% from $cruiseid/quality, 
%     filelist = dirs('../rbin/posmv/*.pmv.rbin');
%     [stats, filelist] = get_rstats('uhdas_dir','../rbin/posmv', 'filelist',...
%                 filelist); %infers msg and instrument from files
%
% varargin:
% name                       default                      what
%-------                 -------------              ---------------
% filelist                   []                explicit filelist, no path:
%                                            - a string  
%                                            - cellarray 
%                                            - structure array (from dirs)
%                                            - [] : use defaults to get all
%                                            - takes precedence over category
%                                                 
%  --- specify files by category: --- 
% instrument                 'posmv'              possibilities: 
%                                                     'pcode',          
%                                                     'adu2',           
%                                                     'posmv',          
%                                                     'seapath1'        
% msg                        'pmv'             possibilities: 
%                                            for pcode, 'gps'
%                                            for posmv, 'gps', 'pmv' 
%                                            for adu2,  'gps', 'adu'
%                                            for seapath1, 'sea'
%
%    --- specify path one of two ways ---
% uhdas_dir                                     path to "rbin" directory 
%                                             (same convention as uhdas)
%                                             defaults to current directory
%                                             i.e. path to files is:
%                                             fullfile(uhdas_dir,'rbin',...
%                                                   instrument)
%
% rfile_dir               []                 path to files.  if not empty, 
%                                              takes precedence over uhdas_dir.
%

% 
%JH 2004/01/10

cfg.instrument = 'posmv';
cfg.msg = 'pmv';
cfg.filelist = [];
cfg.rfile_dir = [];
cfg.uhdas_dir = [];


cfg = fillstruct(cfg, varargin);

stats = [];
filenames = [];

posmv_acc_heading_cutoff = .5;
posmv_acc_roll_cutoff = .5;
posmv_acc_pitch_cutoff = .5;


%set up path to files
if isemptys(cfg.uhdas_dir)
   cfg.uhdas_dir = pwd;
end
if isemptys(cfg.rfile_dir)
   instdir = fullfile(cfg.uhdas_dir, 'rbin', cfg.instrument);
else 
   instdir = cfg.rfile_dir;
end


% find files:
if isemptys(cfg.filelist)
   cfg.filelist = dirs(fullfile(instdir, sprintf('*.%s.rbin', cfg.msg)));
   if isemptys(cfg.filelist)
      fprintf('could not find any %s.rbin files in %s\n',...
              cfg.msg, instdir)
      return
   end
else
   if isstr(cfg.filelist)
      cfg.filelist = dirs(instdir,  filelist);
   elseif iscell(cfg.filelist)
      cfg.filelist = [];
      filecell = cfg.filelist;
      for ii=1:length(filecell)
         filestruct = [];
         filestruct = dirs(instdir, filecell{ii});
         if isemptys(filestruct)
            fprintf('file %s not found in %s\n', filecell{ii}, instdir)
         else
            cfg.filelist(ii) = filestruct;
         end
      end
      if isemptys(cfg.filelist)
         fprintf('none of files in "filelist" were found in %s\n',instdir)
         return
      end
   elseif isstruct(cfg.filelist)
      filelist = cfg.filelist;
      goodfiles = ones(size(filelist));
      for ii=1:length(filelist)
         if ~exist(fullfile(instdir, filelist(ii).name))
            goodfiles(ii) = 0;
         end
      end
      goodi = find(goodfiles==1);
      if isemptys(goodi)
         fprintf('none of files in "filelist" were found in %s\n',instdir)
         return
      else
         cfg.filelist = cfg.filelist(goodi);
      end
   else
      fprint('"filelist" must be string, cellarray, or structure from dirs\n')
      return
   end
end



%% get message name
if isemptys(cfg.msg)
   filename = cfg.filelist(1).name;
   [pp,nn,ee]=fileparts(filename); %basename.msg
   [pp,nn, msgname]=fileparts(nn); %basename.msg
   msgname(1)=[];
   cfg.msg = msgname;
end

msg_mismatch = zeros(size(cfg.filelist));
for ii=1:length(cfg.filelist)
   filename = cfg.filelist(ii).name;
   [pp,nn,ee]=fileparts(filename); %basename.msg
   [pp,nn, msgname]=fileparts(nn); %basename.msg
   msgname(1)=[];
   if ~strcmp(msgname, cfg.msg)
      msg_mismatch(ii) = 1;
   end
end
if length(length(find(msg_mismatch==0)) == length(cfg.filelist))
   fprintf('rbin files are all msg type %s\n', msgname)
   cfg.msg = msgname;
elseif length(find(msg_mismatch==0)) == 0
   % everything matched; do nothing
else
   fprintf('found rbin file with msg type %s, doesn''t match specified\n', ...
           msgname, cfg.msg)
   return
end


%% get instrument name
startdir = pwd;
chdir(instdir)
[pp,instname,ee]=fileparts(pwd);
chdir(startdir)

if ~strcmp(instname, cfg.instrument)
   fprintf('using %s for instrument name\n', instname)
   cfg.instrument = instname;
end

%%%%%%%%%%%%%%%%%%%%%% %%% initialize stats
   
if (strcmp(cfg.instrument, 'posmv'))
   if (strcmp(cfg.msg, 'gps'))
      gotstats.rows = {'startdd', 'numbad', 'numpts'};
   elseif (strcmp(cfg.msg, 'pmv'))
      gotstats.rows = {'startdd', 'numpts', ...
           'num_badhead_acc', 'num_badpitch_acc', 'num_badroll_acc', ...
           'num_GAMS2', 'num_GAMS1', 'num_GAMS0', 'num_IMU1', 'num_IMU0'}';
   end
end
gotstats.filenames = cell(size(cfg.filelist));
gotstats.stats = zeros(length(gotstats.rows), length(cfg.filelist));


%%% posmv
%editing thresholds

for ii=1:length(cfg.filelist)
   [bdata, bstruct] = read_bin(fullfile(instdir,cfg.filelist(ii).name));
   fclose(bstruct.fid);
   gotstats.stats(r_row(gotstats, 'numpts'), ii) = length(bdata(1,:));
   gotstats.stats(r_row(gotstats, 'startdd'), ii) = bdata(r_row(bstruct,'u_dday'),1);
   
   if (strcmp(cfg.instrument, 'posmv'))
      if (strcmp(cfg.msg, 'gps'))
         
         % want to find runs of bad values too
         
         qual_row = find(strcmp(bstruct.rows, 'quality'));
         badi = find( ...
            (bdata(qual_row, :) == 0) | (bdata(qual_row, :) == 6));
         
         gotstats.stats( r_row(gotstats, 'numbad'), ii) = length(badi);
      
      elseif (strcmp(cfg.msg, 'pmv'))

         %posmv attitude
         %  bad headings
         
         badi = find(bdata(r_row(bstruct,'acc_heading'), :) > posmv_acc_heading_cutoff);
         gotstats.stats( r_row(gotstats, 'num_badhead_acc'), ii) = length(badi);
         
         %  bad pitch
         badi = find(bdata(r_row(bstruct, 'acc_pitch'), :) > posmv_acc_pitch_cutoff);
         gotstats.stats( r_row(gotstats, 'num_badpitch_acc'), ii) =length(badi);
   
         % bad roll
         badi = find(bdata(r_row(bstruct,'acc_roll'), :) > posmv_acc_roll_cutoff);
         gotstats.stats( r_row(gotstats, 'num_badroll_acc'), ii) = length(badi);

         % IMU flag (no GAMS)
         qual_row = r_row(bstruct, 'flag_heading');
         if isemptys(qual_row)
            qual_row = r_row(bstruct, 'flag_GAMS');
         end
         gotstats.stats( r_row(gotstats, 'num_GAMS2'), ii) = ...
             length(find(bdata(qual_row,:) == 2));
         gotstats.stats( r_row(gotstats, 'num_GAMS1'), ii) = ...
             length(find(bdata(qual_row,:) == 1));
         gotstats.stats( r_row(gotstats, 'num_GAMS0'), ii) = ...
             length(find(bdata(qual_row,:) == 0));
         
         % IMU flag (no IMU)
         qual_row = find(strcmp(bstruct.rows, 'flag_IMU'));
         gotstats.stats( r_row(gotstats, 'num_IMU1'), ii) = ...
             length(find(bdata(qual_row,:) == 1));
         gotstats.stats( r_row(gotstats, 'num_IMU0'), ii) = ...
             length(find(bdata(qual_row,:) == 0));
      else
         fprintf('posmv quality control only coded for position or attitude\n')
      end
   end
end


   


    