function  [tdh, outdata] = ...
    mk_avgdh(gdata, gstruct, odata, ostruct, instname, varargin)
% [tdh, outdata] = ...
%   mk_avgdh(gdata, gstruct, odata, ostruct, instname,  varargin);
%
% gdata, gstruct  (as from read_bin)
% odata, ostruct (as from read_bin)  
% instname (directory with other heading data: 'posmv', 'seapath', 'ashtech')
%
% output: 
% tdh: do quality control on other heading, return structure with
%     tdh.dday  (timestamp at end of time range used)
%     tdh.ang   (angle correction suitable for use with rotate: (other-gdata))
%         NaN for bad values
%     tdh.ngood (number of good points in the time range)
% (bad values have ang=NaNs, ngood = 0)
% outdata:
%     fields: gyrodday, gyrohead, otherdday, otherhead, other_qcmask
%     (other_qcmask is 0 for OK, 1 for bad)
%
% varargin:
%
% 'enddday'            vector of ensemble endtimes              [] (disabled)
% 'interval'           number of seconds to bin                 60
% 'timefield'          row name to use for timestamps           'udday'
%  'suffix'            suffix of original ascii data            (deduced from
%                                                                instrument)
%                        ('posmv' is *.pmv)             
%                        ('seapath' is *.sea)             
%                        ('adu2' is *.adu)             
%
% 'logfile'            write error messages                     'avgdh.errs'
%(additional error checking):
% 'maxdh'              maximum abs(dh) deviation from the mean    10;
%
%

% JH 2004/02

%% set defaults
cfg.enddday = [];
cfg.interval = 60;
cfg.logfile = 'avgdh.errs';
cfg.timefield = 'u_dday';
cfg.suffix = '';
cfg.maxdh = 20;


tdh = [];
%% input error checking
if ismember(nargin, [0 1 2 3 4])
   help(mfilename);   
   return
end

if ~isstruct(gstruct) | ~isstruct(ostruct)
   help(mfilename);
   fprintf('gstruct and ostruct must be structures\n')
   return   
end

if ~isfield(gstruct, 'rows') | ~isfield(ostruct, 'rows')
   help(mfilename);   
   fprintf('gstruct and ostruct must each have field ''rows''\n')
   return
end

instnames =  {'posmv', 'seapath', 'ashtech'};
if ~ismember(instname, instnames)
   help(mfilename);   
   fprintf('instrument name must be from this list:\n')
   for ii=1:length(instnames)
      fprintf('   %s\n', instnames{ii})
   end
   return
end


% get varargin
cfg = fillstruct(cfg, varargin, 0);
if isemptys(cfg.suffix)
   if strmatch(instname, 'posmv')
      cfg.suffix = 'pmv';
   elseif strmatch(instname, 'ashtech')
      cfg.suffix = 'adu';
   elseif strmatch(instname, 'seapath')
      cfg.suffix = 'sea';
   else
      fprintf('error in program %s, bad instrument name not caught\n',...
              mfilename)
      return
   end
end

% set time ranges
if isemptys(cfg.enddday)
   dday = odata(r_row(ostruct, cfg.timefield),:);
   dt = cfg.interval/86000;
   cfg.enddday = [dday(1): dt : dday(end)];
   if cfg.enddday(end) < dday(end)
      cfg.enddday = [cfg.enddday dday(end)];
   end
end


gdata_ddayrow = r_row(gstruct, cfg.timefield);
gdata_headrow = r_row(gstruct, 'heading');
odata_ddayrow = r_row(ostruct, cfg.timefield);
odata_headrow = r_row(ostruct, 'heading');


fid = fopen(cfg.logfile,'a');
%%%%%%%%% get rid of gyro duplicate timestamps
badi = find(diff(gdata(gdata_ddayrow,:)) <= 0);
count = max(length(badi), 30);
origcount = count;
while(1)
   if ~isemptys(badi)
      fprintf('deleting %d nonmonotonic gyro timestamps\n', length(badi))
      fprintf('nonmonotonic gyro timestamps at (%10.7f)\n', ...
              gdata(gdata_ddayrow,badi))
      fprintf(fid, ...
              'nonmonotonic gyro timestamps at (%10.7f)\n', ...
              gdata(gdata_ddayrow,badi));
      gdata(:,badi)=[];
      badi = find(diff(gdata(gdata_ddayrow,:)) <= 0);
      if isemptys(badi)
         fprintf('no more bad timestamps\n')
         break
      else
         fprintf('WARNING: gyro data still contain nonmonotonic timestamps\n')
      end
      count = count - 1;
   end
   if count < 0 | isemptys(badi)
      break
   end
   paused(.1)
end
badi = find(diff(gdata(gdata_ddayrow,:)) <= 0);
if isemptys(badi)
   if count == origcount
      fprintf('no bad timestamps\n')
   else
      fprintf('no more bad timestamps\n')
   end      
else
   fprintf('FATAL: after %d deletions, gyro data still contain nonmonotonic timestamps\n',...
           origcount)
   return
end

   
   
%%%%%%%%%% also in "other"
badi = find(diff(odata(odata_ddayrow,:)) <= 0);
count = max(length(badi), 30);
origcount = count;
while(1)
   if ~isemptys(badi)
      fprintf('deleting %d duplicate other timestamps\n', length(badi))
      fprintf(fid, ...
              'nonmonotonic other timestamps at (%10.7f)\n', odata(gdata_ddayrow,badi))
      odata(:,badi)=[];
      badi = find(diff(odata(odata_ddayrow,:)) <= 0);
      if isemptys(badi)
         fprintf('no more bad timestamps\n')
         break
      else
         fprintf('WARNING: %s data still contain nonmonotonic timestamps\n', instname)
      end
      count = count - 1;
   end
   if count == 0 | isemptys(badi)
      break
      paused(1)
   end
end
badi = find(diff(odata(odata_ddayrow,:)) <= 0);
if isemptys(badi)
   if count == origcount
      fprintf('no bad timestamps\n')
   else
      fprintf('no more bad timestamps\n')
   end      
else
   fprintf('FATAL: after %d deletions, ', origcount)
   fprintf('%s data still contain nonmonotonic times\n',    instname )
   return
end

fclose(fid);

%%%%%%%%%%%%%%%%%%%%%%

gdata_ddayrow = r_row(gstruct, cfg.timefield);
gdata_headrow = r_row(gstruct, 'heading');
odata_ddayrow = r_row(ostruct, cfg.timefield);
odata_headrow = r_row(ostruct, 'heading');

badgyro = find(isnan(gdata(gdata_ddayrow,:)));
gdata(:, badgyro) = [];

qcmask = qc_rbin(odata, ostruct);

dday = odata(odata_ddayrow,:);
ughead = 180*unwrap(gdata(gdata_headrow,:)*pi/180)/pi;
gdatahead_ = table1([gdata(gdata_ddayrow,:)' ughead'],dday)';
otherhead= 180*unwrap(odata(odata_headrow,:)*pi/180)/pi;
nanmask = dotmmask(qcmask(odata_headrow,:));

gdatahead_ = rem(  rem(gdatahead_,360) + 720, 360);
otherhead = rem(  rem(otherhead,360) + 720, 360);

if nargout == 1
   outdata = [];
else
   outdata.gyrodday = gdata(gdata_ddayrow,:);
   outdata.gyrohead = ughead;
   outdata.otherdday = odata(odata_ddayrow,:);
   outdata.otherhead = otherhead;
   outdata.other_qcmask = qcmask(odata_headrow,:);
end

% now vectorize the grouping of timestamps in cfg.enddday
index = [1:length(dday)];
spliti = floor(table1([dday' index'], cfg.enddday))';

starti = [1 spliti(2:end)+1];
endi   = [spliti(2:end) length(index)];

if endi(end-1) == endi(end)
   starti(end)=[];
   endi(end)=[];
   cfg.enddday(end)=[];
end

% time ranges should be starti(ii):endi(ii)
% run through timeranges, getting dh


dh = NaN*starti;
numgood = zeros(size(starti));
alldh = rem(180 + gdatahead_ - otherhead.*nanmask, 360) - 180;

absmean = mstdgap(alldh);
badi = find(abs(alldh - absmean) > cfg.maxdh);
nanmask(badi) = NaN;
alldh = rem(180 + gdatahead_ - otherhead.*nanmask, 360) - 180;


for index=1:length(starti)
   ii = starti(index):endi(index);
   if (~isemptys(ii))
      [dh(index), junk, numgood(index)] = mstdgap(alldh(ii));
   end
end


tdh.dday = cfg.enddday;
tdh.ang = dh;
tdh.numgood = numgood;


      
