function mk_ensdday_gyrodh(gyrolist, otherlist, varargin)
% mk_gyrodh(gyrolist, otherlist, varargin);
%       
% output: saves 6xN matrix "data" to "gyrodh.mat":
%       : data =  [dday, filtered_dh, original_dh, meangyro, meanother mask]
%       : dh = gyro-otherhead
%       : mask is 1 if original dh, 0 if interpolated dh
%       :
%       : writes [dday dh] to a named file, default is gyrodh.ang
%       : dh is compatible with codas3 rotate angle_file
%
% input:
% gyrolist is a filelist of gyro rbin files  (as from 'dirs' with fullpath==1)
% otherlist os a filelist of other rbin files with heading
%            (eg. adu.rbin or sea.rbin)
%
% eg, from cal/rotate:
%   gyrolist = dirs('../../rbin/gyro/*hdg.rbin','fullfile',1);
%   otherlist = dirs('../../rbin/seapath1/*sea.rbin','fullfile',1);
%   end_dday = read_asc('../../edit/a0304.tem',4,1);
%   mk_gyrodh(gyrolist, otherlist, 'end_dday', end_dday,...
%                    'othertype', 'seabin');
%
%
%
% varargin:
%
%  name                      what                            default
% -------                ------------                   -----------------
%  'outfile'          output file                            'gyrodh.ang'
%  'end_dday'         end_ensemble times                        [];
%                        (precedence over 'interval')
%  'interval'         output on this interval                  300
%                          in seconds.  If
%                          'end_dday' is set,
%                           use that instead.
%  'timerange'         dday range (subset)                   'all'
%                          or [start_dday end_dday]
%  'showverbose'     look at the output [0 or 1]                0
%  'othertype'       type of other heading data                []   
%                    (for quality control)
%                     'seapath1', 'adu2', 'posmv'
%                     MUST choose one.  
%
%  'verbose'           print every ensemble number              0
%                     0: print percent progress
%                     1: print ensemble number
%  'logfile'          write error messages here            'timestamps.log'
%                                                          (disable with [])
%  'ploterrs'         [0,1] show busy timestamps               0
%  'debug'            [0,1] drop into keyboard                 0
%                      if busy cpu indicated
%
% NOTE - these timestamps are the PC timestamps in the rbin files.  It may 
%        be necessary to add a 'best' to 'timebase' to use the gbin
%        timestamps (if the PC clock is drifting)

%% JH 2003/06/28 
%% JH 2003/12/22 adding qc_rbin for quality control, adding 'debug' and 'logfile'

cfg.outfile = 'gyrodh.ang';
cfg.end_dday = [];
cfg.interval = 300;
cfg.timerange = 'all';
cfg.showverbose = 0;
cfg.othertype = [];
cfg.verbose = 0;
cfg.ploterrs = 0;
cfg.logfile = 'timestamps.log';
cfg.debug = 0;

cfg = fillstruct(cfg, varargin,0);


if strcmp(cfg.othertype, 'adu2')
   cfg.suffix = 'adu';
elseif strcmp(cfg.othertype, 'posmv')
   cfg.suffix = 'pmv';
elseif strcmp(cfg.othertype, 'seapath1')
   cfg.suffix = 'sea';
else
   fprintf('must choose ''othertype'' from the following\n')
   fprintf('''adu2'', ''posmv'', ''seapath1'' \n')
   data=[];
   return
end
   
if (isstr(cfg.timerange) & ~strcmp(cfg.timerange,'all'))
   fprintf('must specify a decimal day pair for timerange\n')
   fprintf('or to use all, specify the string ''all'' \n')
   fprintf('(''all'' is the default)\n')
   data=[];
   return
end

if (~isemptys(cfg.logfile))
   logfile_fid = fopen(cfg.logfile, 'a');
   if logfile_fid == -1
      fprintf('could not open log file %s\n', cfg.logfile)
      return
   end
end   


%get timerange info
fprintf('getting time ranges of files\n')
gyro_trinfo = tr2ilist(gyrolist, 'rowname', 'u_dday');
other_trinfo = tr2ilist(otherlist, 'rowname', 'u_dday');


% find out the stuctures associated with the two rbin files
[tmp1, gstruct]=read_bin(gyro_trinfo{1,1},'last',1);
fclose(gstruct.fid);
gyro_drownum = r_row(gstruct,'heading');

[tmp2, ostruct]=read_bin(other_trinfo{1,1},'last',1);
fclose(ostruct.fid);
other_drownum = r_row(ostruct,'heading');
         
% 
cfg.interval = cfg.interval/86400; %make it dday not seconds
if isstr(cfg.timerange) 
   if isemptys(cfg.end_dday)
      timerange = [gyro_trinfo{1,2}(1)  gyro_trinfo{end,2}(2)]; %all in gyro
      cfg.end_dday = timerange(1)+cfg.interval:cfg.interval:timerange(2);
   else
      timerange = [cfg.end_dday(1) cfg.end_dday(end)];
   end
else
   timerange = cfg.timerange;
   if isemptys(cfg.end_dday)
      cfg.end_dday = timerange(1)+cfg.interval:cfg.interval:timerange(2);
   else
      ii=find( (cfg.end_dday >= timerange(1)-cfg.interval) & ...
               cfg.end_dday <= timerange(2)+cfg.interval);
      cfg.end_dday = cfg.end_dday(ii);
   end
end


% dday, filtered_dh     orig_dh   gyro, other, interpolated_is_0
%      dh=gyro-other
%       
data = NaN*ones(6,length(cfg.end_dday));
NT = length(data(1,:));
tenpercent = fix(NT/10);
dt = mstdgap(diff(cfg.end_dday));

fprintf('getting data and averaging %d %d-second ensembles...\n',...
        NT,round(cfg.interval*86400))
fprintf('this may be timeconsuming...\n')

% do it
for ensi=1:NT
   
   if (cfg.verbose)
      fprintf('%d out of %d\n', ensi, NT)
   else
      if (rem(ensi, tenpercent) == 0)
         fprintf('%d percent done...\n', ceil(100*ensi/NT))
      end
   end
   
   data(1,ensi) = cfg.end_dday(ensi);  %end times
   
   if (ensi == 1)
      timerange = [cfg.end_dday(ensi)-dt, cfg.end_dday(ensi)];
   else
      timerange = [cfg.end_dday(ensi-1), cfg.end_dday(ensi)];
   end
   gyro  = tr2ilist(gyro_trinfo, 'timerange', timerange);
   other = tr2ilist(other_trinfo, 'timerange', timerange);
   
   if (isemptys(gyro) | isemptys(other))
      fprintf('no data found in time range %7.3f - %7.3f\n', ...
              cfg.end_dday(ensi), cfg.end_dday(ensi+1))
   else
      %fix bad times due to ?? cpu cycles used up??
      try
         okdday =  fix_timestamps(gyro, logfile_fid, cfg.ploterrs);
      catch
         if cfg.debug, 
            fprintf('fix_timestamps failed for gyro\n')
            keyboard, 
         end
      end
      
      if (isemptys(okdday))
         fprintf('busy cpu: failed loop for gyro in time range %7.3f - %7.3f\n', ...
                 cfg.end_dday(ensi), cfg.end_dday(ensi+1))
         if ~isemptys(cfg.logfile)
            fprintf(logfile_fid, 'busy cpu: failed loop for gyro in time range %7.3f - %7.3f\n', ...
                    cfg.end_dday(ensi), cfg.end_dday(ensi+1));
         end
         if cfg.debug,  
            fprintf('there are busy CPU timestamps for gyro\n')
            keyboard, 
         end
         %return
         continue
      else
         gyro(1,:) = okdday;
      end
      
      
      %fix bad times due to ?? cpu cycles used up??
      try
         okdday =  fix_timestamps(other, logfile_fid, cfg.ploterrs);
      catch
         if cfg.debug, 
            fprintf('fix_timestamps failed for other\n')
            keyboard, 
         end
      end
      if (isemptys(okdday))
         fprintf('busy cpu: failed loop for other in time range %7.3f - %7.3f\n', ...
                 cfg.end_dday(ensi), cfg.end_dday(ensi+1))
         if ~isemptys(cfg.logfile)
         fprintf(logfile_fid, 'busy cpu: failed loop for other in time range %7.3f - %7.3f\n', ...
                 cfg.end_dday(ensi), cfg.end_dday(ensi+1))
         end
         if cfg.debug,  
            fprintf('there are busy CPU timestamps for other\n')
            keyboard
         end
         %return
         continue
      else
         other(1,:) = okdday;
      end
      
      %% quality control
      other = qc_rbin(other, ostruct, cfg.othertype, cfg.suffix);
      
      other_dday = other(1,:);
      other_head = 180*unwrap(pi*other(other_drownum,:)/180)/pi;
      gyro_dday = gyro(1,:);
      gyro_head = 180*unwrap(pi*gyro(gyro_drownum,:)/180)/pi;
      
      othertab = [other_dday' other_head'];
      othertabhead = table1(othertab, gyro_dday');
      
      dh  =  gyro_head - othertabhead';
      mm = mstdgap(dh);
      
      data(2,ensi) = mm;
      data(3,ensi) = mm;
      data(4,ensi) = mstdgap(gyro_head);
      data(5,ensi) = mstdgap(other_head);
   
      if (cfg.showverbose)
         clg
         
         subplot(211)
         plot(gyro_dday, gyro_head,'g')
         hold on
         plot(other_dday, other_head,'r')
         
         title(sprintf('%7.3f to %7.3f', cfg.end_dday(ensi), cfg.end_dday(ensi+1)))
         
         subplot(212)
         plot(gyro_dday,  dh,'b')
         hold on
         plot(timerange(:), [mm mm]','r')
         keyboard
         
      end
   
   end
   
end

%-----------

% data is:
% [ens_dday mstdgap(dh) mstdgap(dh) mstdgap(gyrohead) mstdgap(otherhead)  mask]
%            cleaned      original

badi = find(isnan(data(2,:)));
if ~isemptys(badi)
   data(2,badi) = table1(data([1 2],:)', data(1,badi))';
end

data(2:5,:) = rem(data(2:5,:) + 720 + 50,360) -50;

% data(2,:) will be output to a file, data(3,:) is the originals

fprintf('running "cleaner" to de-glitch data...\n')

[dh data(6,:)] = cleaner(data(2,:),2,3);
bad_is_0 = data(6,:);

if (~isemptys(find(bad_is_0 == 0)))
   fprintf('note:\n')
   fprintf('%d points were eliminated with cleaner\n',  ...
           length(find(bad_is_0==0)))
   fprintf('deleting them from file, masked as 0 in 6th row of data\n')
end


if (length(find(isnan(dh)))>0)
   fprintf('patching at least one end of dh\n')
   % find the first and last good dh values
   dh1 = dh(min(find(~isnan(dh))));
   dh2 = dh(max(find(~isnan(dh))));
   % put them at the beginning and end with surrounding (fake) times
   dday12 = [data(1,1)-cfg.interval; data(1,:)' ; data(1,end)+cfg.interval];
   dh12   = [dh1                   ; dh' ; dh2];
   % get the mask  of good dh in this matrix
   ggi = find(~isnan(dh12));
   %do it
   
   badmask = isnan(dh) | (data(6,:)==0);
   dh(badmask) = table1( [dday12(ggi) dh12(ggi)], data(1,badmask));
   % will be interpolating over NaNs also
   data(6,find(badmask)) = 0;
end


data(2,:) = dh;


if (~isemptys(cfg.outfile))
   fprintf('printing to file %s\n', cfg.outfile)
   outdata = data([1 2],:)';
   eval(sprintf('save6 %s -ascii  outdata', cfg.outfile))
end


save6 gyrodh.mat data


figure
subplot(211)
plot(data(1,:), data(3,:),'r')
hold on
plot(data(1,:), dh,'k')
yl(-8,8)
xlabel('dday')
ylabel('degrees heading')
title('gyro-other heading: original (red) and final (black) ')

subplot(212)
plot(data(1,:), data(2,:) ,'k')
hold on
badi = find(data(6,:) == 0);
plot(data(1,badi), data(2,badi),'r.')
yl(-8,8)
xlabel('dday')
ylabel('degrees heading')
title('final gyro-other heading, black (interpolated points (red)')
orient tall
print -dpsc gyrodh.ps







