function mk_gyrodh(gps_attname, varargin)
% mk_gyrodh(gps_attname, varargin)
% gps_attname is one of 'ashtech', 'posmv', 'seapath'
% varargin must all be specified, and are 
% 
%  name                  default                             what
% --------             -----------                         ------------
% cfgbase                [required]                         see get_uhdascfg
% cfgpath                [required]                         see get_uhdascfg
% instname               [required]                         see get_uhdascfg
% pingtype               [required]                         see get_uhdascfg
% yearbase               [required]                         (for title)
% cruiseid               [required]                         (for title)
% logfile                mk_gyrodh.log                      write a note here
% time_angle_file        gyrodh.ang                         angle file (for 'rotate')
% dh_ddrange                (all)                           get dh over this time range
%                                                           []         : use all
%                                                           scalar     : use last X days
%                                                           1x2 vector : use this time range
% stats_ddrange            1 day                            get stats over this range
% okcutoff               min(30, .6*max)                    number of good points required

% debug                      0                              drop into keyboard

cfg.cfgbase          = '';
cfg.cfgpath          = '';
cfg.instname         = [];
cfg.pingtype         = [];
cfg.okcutoff         = [];
cfg.dh_ddrange       = [];
cfg.yearbase         = [];
cfg.cruiseid         = '';
cfg.logfile          = 'mk_gyrodh.log';
cfg.time_angle_file  = 'gyrodh.ang';
cfg.stats_ddrange    =  [];   

cfg.debug            = 0;

cfg = fillstruct(cfg, varargin);

if isempty(cfg.cfgbase), fprintf('must specify cfgbase\n'), end
if isempty(cfg.cfgpath), fprintf('must specify cfgpath\n'), end
if isempty(cfg.cruiseid), fprintf('must specify cruiseid\n'), end
if isempty(cfg.pingtype), fprintf('must specify pingtype\n'), end
if isempty(cfg.instname), fprintf('must specify instname\n'), end
if isempty(cfg.yearbase), fprintf('must specify yearbase\n'), end
if isempty(cfg.dh_ddrange), cfg.dh_ddrange = [-10000 10000]; end   %all ddays

try
   delete('mk_gyrodh_diary.txt')
catch
   fprintf('diary file mk_gyrodh_diary.txt does not (yet) exist\n')
end
diary('mk_gyrodh_diary.txt')

fprintf('about to run get_uhdascfg\n')

uhdascfg =get_uhdascfg(...
   'cfgbase', cfg.cfgbase,...
   'cfgpath', cfg.cfgpath,...
   'instname', cfg.instname,...
   'pingtype', cfg.pingtype);

cfg.toofarsecs = 30;

fakeit = 0;

% -----  get gyro heading  -------
inst_info = {'gyro','hdg'};
gyrolist = guess_rfilelist(uhdascfg, cfg.dh_ddrange, inst_info, 'rbin');
if isempty(gyrolist)
   fprintf('no rbin gyro data found\n')
   fakeit = 1;
end
if ~fakeit
   gyro_trinfo = tr2ilist(gyrolist);
   if isempty(gyro_trinfo)
      fprintf('no rbin gyro data found in time range\n')
      fakeit = 1;
   end
end
if ~fakeit
   [gdata, gstruct] = tr2ilist(gyro_trinfo);
end
      
% --- get heading from gps-aided attitude device ("other" heading) ----------
if strcmp(gps_attname, 'ashtech')
   inst_info = {'ashtech','adu'};
elseif  strcmp(gps_attname, 'posmv')
   inst_info = {'posmv','pmv'};
elseif  strcmp(gps_attname, 'seapath')
   inst_info = {'seapath','sea'};
end

otherlist = guess_rfilelist(uhdascfg, cfg.dh_ddrange, inst_info, 'rbin');
if isempty(otherlist)
   fprintf('no rbin gps-aided data found\n')
   fakeit = 1;
end
if ~fakeit 
   other_trinfo = tr2ilist(otherlist);
   if isempty(other_trinfo)
      fprintf('no rbin gps-aided data found in time range\n')
      fakeit = 1;
   end
end
if ~fakeit
   [odata, ostruct] = tr2ilist(other_trinfo);
end


%% get ensemble timestamps
dday_dh = load('scn.hdg');   % recent
dday = dday_dh (:,1);
ii = find( dday >= cfg.dh_ddrange(1) & dday <= cfg.dh_ddrange(2));
dday = dday(ii);

% get the heading corrections
if fakeit
   dh = zeros(size(dday));
else
   o_tdh = mk_avgdh(...
      gdata, gstruct, odata, ostruct, ...
      gps_attname, 'logfile', cfg.logfile);

   maxgood = max(o_tdh.numgood);
   if isempty(cfg.okcutoff)
      okcutoff = .6*min(60, maxgood);
   else
      okcutoff = cfg.okcutoff;
   end
   obadi = find( (o_tdh.numgood < okcutoff) | isnan(o_tdh.ang));
   ogoodi = find(find(o_tdh.numgood > okcutoff));

   % interpolate NaNs 
   o_tdh.ang = fillends(o_tdh.ang);

   clf
   plot(o_tdh.dday, o_tdh.ang, 'k')
   hold on
   plot(o_tdh.dday(ogoodi), o_tdh.ang(ogoodi), 'g.')
   hh=plot(o_tdh.dday(obadi), o_tdh.ang(obadi), 'r+');ms(hh,4)

   if isempty(ogoodi)
      fprintf('CANNOT MAKE %s HEADING CORRECTION\n', gps_attname)
      fprintf('empty starting table\n')
      if cfg.debug
         keyboard
      end
      return
   end   
   if isempty(dday)
      fprintf('CANNOT MAKE %s HEADING CORRECTION\n', gps_attname)
      fprintf('empty independent variable\n')
      if cfg.debug
         keyboard
      end
      return
   end   

   dh = table1([o_tdh.dday(ogoodi)', o_tdh.ang(ogoodi)'], dday);
   dh = fillends(dh);
   title(sprintf('gyro - %s, degrees', gps_attname))

   print -append -dpsc gyrodh_incr.ps
   paused(1)
   close
end

 
%% Now we have a little segment of dh; make the following:

%% (1)  alldh.txt   =  [dday gyro_heading numgood];  % append with each run
alldhfile = 'alldh.txt';
fid = fopen('alldh.txt','a');
alldh = [o_tdh.dday(:) o_tdh.ang(:) o_tdh.numgood(:)];
fprintf(fid, '%10.7f        %5.3f         %4d\n', alldh');
fclose(fid);

%% (2) gyrodh.ang  (time angle file for all data)
clear alldh
load alldh.txt
[sorted_dday, sorted_i] = unique(alldh(:,1));
ddaydh = alldh(sorted_i, [1 2]);
fid = fopen(cfg.time_angle_file,'w');  
fprintf(fid,  '%10.8f      %5.3f\n', ddaydh');
fclose(fid);

%% rewrite alldh.txt with sorted values
fid = fopen('alldh.txt','w');  
ddaydhn = alldh(sorted_i,:);
fprintf(fid, '%10.7f        %5.3f         %4d\n', ddaydhn');
fclose(fid);


%% (3) get statistics for last xx days (cfg.stats_ddrange)
dday    = alldh(sorted_i,1);
dh      = alldh(sorted_i,2);
numgood = alldh(sorted_i,3);

if isempty(cfg.stats_ddrange)
   ii = 1:length(dday);
elseif length(cfg.stats_ddrange) == 1
   ii =  find( dday >= (dday(end) - cfg.stats_ddrange));
else
   ii =  find( dday >= cfg.stats_ddrange(1) & dday <=  cfg.stats_ddrange(2));
end

maxgood = max(numgood);
if isempty(cfg.okcutoff)
   okcutoff = .6*min(60, maxgood);
else
   okcutoff = cfg.okcutoff;
end

print_stats(dday, dh, numgood, ii, okcutoff, cfg.yearbase)

%% (4) plot numgood and dh for whole cruise so far
%% clipped this from ashrot


ndays = 2.0;         % no. of days to plot per panel
tt = dday;


ps_file = 'gyrodh_panels.ps';
figure
good_att_mask = (numgood > okcutoff);
i_good_att = find(good_att_mask);

if exist(ps_file)
  eval(['delete ' ps_file]);
end

axleft = [floor(tt(1)):ndays:tt(end)];
nplots = length(axleft);
npanels = min(6, nplots); % no. of panels per page
npages = ceil(nplots/npanels);

yrange = 1.2 * [min(dh) max(dh)];
ym = 0.1;       % top & bottom margin
ysep = 0.03;    % separation between panels
yp = (1 - 2 * ym - (npanels-1) * ysep) / npanels;  % panel height
xm = 0.15;                                         % side margin
xp = 1 - 2 * xm;                                   % panel width

for ipage = [1:npages]
  figure
  y0 = 1 - ym - yp;
  for ipanel = [1:npanels]
    if (ipage-1) * npanels + ipanel <= nplots
      axes('position', [xm y0 xp yp])
      plot(tt(i_good_att),     dh(i_good_att)    , '.', ...
        tt(~good_att_mask), dh(~good_att_mask), '+')
      axl = axleft(npanels*(ipage-1) + ipanel);
      axis([axl axl+ndays yrange])
      if ipanel == 1
        title(sprintf('%s heading - gyro heading', gps_attname));
        text(axl+ndays, yrange(2), cfg.cruiseid, ...
            'horizontalalignment', 'right', 'verticalalignment', 'bottom')
      end
      ylabel('deg')
      y0 = y0 - yp - ysep;
    end
  end
  xlabel([int2str(cfg.yearbase) ' decimal days'])
  paused(1)
  if (npanels > 2)
    orient('tall')
  else
    orient('landscape')
  end
  date_it('ashrot.m')
  eval(['print -dpsc -append ' ps_file]);
end

diary off

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function print_stats(dday, dh, numgood, ii, okcutoff, yearbase)

dday = dday(ii);
dh = dh(ii);
numgood= numgood(ii);

obadi = find(numgood < okcutoff);
ogoodi = find(find(numgood > okcutoff));

sfid = fopen('ang_stats.txt','w');
   fprintf(sfid, '\nddrange: %10.7f to %10.7f\n', dday(1), dday(end));
   fprintf(sfid, '(%s to %s)\n\n', ...
             to_date(yearbase, dday(1)), to_date(yearbase, dday(end)));
   fprintf(sfid, 'number of good points: %d out of %d (%d%%)\n',...
                      length(ogoodi), length(dday), ...
                      round(100*length(ogoodi)/length(dday)));
   fprintf(sfid, 'min =  %3.2f,  max =  %3.2f\n', min(dh), max(dh));
   [mm, ss] = mstdgap(dh);
   fprintf(sfid, 'mean   =  %3.2f\n', mm);
   fprintf(sfid, 'stddev =  %3.2f\n\n', ss);
fclose(sfid);

