function  qcmask = qc_rbin(bdata, bstruct)
%  qcmask = qc_rbin(bdata, bstruct)
% apply quality control to rbin data, return same data with NaNs for bad data
% qcmask is 1 where bad, 0 where good
%
% input: [bdata, bstruct] are data array and structure (respectively) from
%     bin_extract or read_bin


if nargin ~= 2
   error('qc_rbin now uses two arguments only\n')
end

   


%allowed combitations
ok_inst_suffix = {...
   'gps'       ;
   'gps_sea'   ;
   'sea'       ;
   'hdg'       ;
   'rdi'       ;
   'adu'       ;
   'pmv'       ;
   'psxn20'    ;
   'spd'       };

messagename = bstruct.messagename;
qcmask = zeros(size(bdata));


if sum(strcmp(messagename, ok_inst_suffix)) ~= 1
   fprintf('messagename %s from read_bin not contained in qc_rbin.\n', messagename)
   fprintf('new messagename %s must be added to list and accounted for in the code\n', ...
           messagename)
   help(mfilename)
   return
end


if (isemptys(bdata))
   fprintf('returning empty from qc_rbin\n')
   return
end



if (strcmp(messagename, 'gps'))
   lon_row  = find(strcmp(bstruct.rows, 'lon'));
   lat_row  = find(strcmp(bstruct.rows, 'lat'));
   qual_row = find(strcmp(bstruct.rows, 'quality'));
   badi = find( ...
      (bdata(qual_row, :) == 0) | (bdata(qual_row, :) == 6));
   qcmask([lon_row lat_row],badi) = 1;
   
elseif (strcmp(messagename, 'gps_sea'))
   
   lon_row  = find(strcmp(bstruct.rows, 'lon'));
   lat_row  = find(strcmp(bstruct.rows, 'lat'));
   qual_row = find(strcmp(bstruct.rows, 'quality'));
   badi = find( ...
      (bdata(qual_row, :) == 0) | (bdata(qual_row, :) == 6));
   qcmask([lon_row lat_row],badi) = 1;

   hqual_row = find(strcmp(bstruct.rows, 'horiz_quality'));
   badi = find( bdata(hqual_row, :) ~= 0) ;
   qcmask([lon_row lat_row],badi) = 1;

elseif (strcmp(messagename, 'pmv'))

   %editing thresholds
   acc_heading_cutoff = .25;  %posmv
   acc_roll_cutoff = .25;
   acc_pitch_cutoff = .25;
   
   %posmv attitude
   % use the reaquisition flag to make bad entries into NaNs
   % edit (NaN) out bad headings
   heading_row  = find(strcmp(bstruct.rows, 'heading'));
   qual_row = find(strcmp(bstruct.rows, 'acc_heading'));
   badi = find(bdata(qual_row, :) > acc_heading_cutoff);
   qcmask(heading_row,badi) = 1;
   
   % edit (NaN) out bad pitch
   pitch_row  = find(strcmp(bstruct.rows, 'pitch'));
   qual_row = find(strcmp(bstruct.rows, 'acc_pitch'));
   badi = find(bdata(qual_row, :) > acc_pitch_cutoff);
   qcmask(pitch_row,badi) = 1;
   
   % edit (NaN) out bad roll
   roll_row  = find(strcmp(bstruct.rows, 'roll'));
   qual_row = find(strcmp(bstruct.rows, 'acc_roll'));
   badi = find(bdata(qual_row, :) > acc_roll_cutoff);
   qcmask(roll_row,badi) = 1;
   
   % throw out heave if all 3 are bad (pretty arbitrary)
   heave_row  = find(strcmp(bstruct.rows, 'heave'));
   badi = find(isnan(bdata(heading_row,:)) & ...
               isnan(bdata(pitch_row,:)) & ...
               isnan(bdata(roll_row,:)));
   qcmask(heave_row, badi) = 1;

elseif strcmp(messagename, 'sea')
   %now do attitude
   qual_row = find(strcmp(bstruct.rows, 'head_qual'));
   heading_row = find(strcmp(bstruct.rows, 'heading'));
   badi = find(bdata(qual_row, :) > 1 );
   qcmask(heading_row,badi) = 1;

   qual_row = find(strcmp(bstruct.rows, 'rp_qual'));
   roll_row = find(strcmp(bstruct.rows, 'roll'));
   pitch_row = find(strcmp(bstruct.rows, 'pitch'));
   badi = find(bdata(qual_row, :) > 1 );
   qcmask(roll_row,badi) = 1;
   qcmask(pitch_row,badi) = 1;
   
elseif (strcmp(messagename, 'att') | strcmp(messagename, 'adu'))

   
   maxmrms  = 0.004;          %ashtech
   maxbrms  = 0.03;
   
   % adu2 heading:
   % use the reaquisition flag to make bad entries into NaNs (reaqfl = 1)
   heading_row = find(strcmp(bstruct.rows, 'heading'));
   pitch_row = find(strcmp(bstruct.rows, 'pitch'));
   roll_row = find(strcmp(bstruct.rows, 'roll'));
   
   % (1)
   qual_row = find(strcmp(bstruct.rows, 'reacq'));
   badmask = (bdata(qual_row, :) == 1 );
   % (2)
   mrms_row = find(strcmp(bstruct.rows, 'mrms'));
   badmask = badmask | (bdata(qual_row, :) > maxmrms );
   % (3)
   brms_row = find(strcmp(bstruct.rows, 'brms'));
   badmask = badmask | (bdata(qual_row, :) > maxbrms );
   
   qcmask(heading_row,:) = badmask;
   qcmask(pitch_row,:) = badmask;
   qcmask(roll_row,:) = badmask;

elseif strcmp(messagename, 'sea')

   % combined psxn20 and psxn23 messages
   headqual_row = find(strcmp(bstruct.rows, 'head_qual'));
   heading_row = find(strcmp(bstruct.rows, 'heading'));
   badi = find(bdata(headqual_row, :) ~= 0 );
   qcmask(heading_row,badi) = 1;

   rpqual_row = find(strcmp(bstruct.rows, 'rp_qual'));
   pitch_row = find(strcmp(bstruct.rows, 'pitch'));
   roll_row = find(strcmp(bstruct.rows, 'roll'));
   badi = find(bdata(headqual_row, :) ~= 0 );
   qcmask(pitch_row,badi) = 1;
   qcmask(roll_row,badi) = 1;

end   
   
   
   
qcmask = logical(qcmask);