function raw_lookit(data, config, varargin)
% raw_lookit(data, config, varargin);
%
% show N fields (as specified in cellarray config.fields)
% take out a reference layer if specified
%
% eg: dd = read(nb,..)
%     [data, config] = restruct(nb, dd);
%     rrlookit(data, config)
%
%
% names to add to config from restruct:
%
% name                  default             % comments
%--------             ---------             -------------------------------
% refl_range                                %  remove reference layer average
%                    []                     %  don't do it
%                                           %  [startbin endbin] use REFAVG
% fields      {'umeas', 'vmeas', 'emeas'}   % do these
%  index                 []                 % do it all
%                                           % how to subsample abcissa : 
%                                           %      - empty (use it all)
%                                           %      - scalar (subsample by this) 
%                                           %      - length 2 vector (range)
%                                           %      - length > 2 vector
%                                           %        (indices of abcissa)
% doref              []                     % vector of 1 (yes, take out a refl
%                                           %          if possible) or 0 (don't)
%                                           %       MUST have the same number of
%                                           %        elements as "fields" or 
%                                           %        be empty.  If empty, do 
%                                           %        all or none according to 
%                                           %        refl_range.
% autoscale         0                       % use predefined values in config
%                                           % 1  let it autoscale
%                                           % 2XNfields vector of scale ranges

cfg.refl_range = [];
cfg.fields =  {'umeas', 'vmeas', 'emeas'} ;
cfg.doref  =  [];
cfg.index = [];
cfg.mins = [];
cfg.autoscale = 0;


%these are so config coming in won't barf
cfg.title = [];
cfg.minclipval = [];
cfg.maxclipval = [];
cfg.comment = [];


%override defaults
cfg = fillstruct(cfg, config, 0);
cfg = fillstruct(cfg, varargin, 0);



%make abcissa be minutes
data.mins = 24*60*(data.dday - data.dday(1));
cfg.abc_name = 'mins';


%figure out which fields to plot
numplots = 0;
for fnum = 1:length(cfg.fields)
   if (isfield(data, cfg.fields{fnum}))
      numplots = numplots + 1;
      fields{numplots} = cfg.fields{fnum};
   else
      fprintf('FAILURE: field %s does not exist in structure "data"\n',...
              cfg.fields{fnum})
      return
   end
end


%set scales
if (length(cfg.autoscale) == 1)  %either autoscale everthing or use defaults
   if (~cfg.autoscale)
      if (isfield(cfg, 'minclipval'))
         cfg = rmfield(cfg, 'minclipval');   
      end
      if (isfield(cfg, 'maxclipval'))
         cfg = rmfield(cfg, 'maxclipval')
      end
   end
else %match up scales with fields
   if (length(cfg.autoscale(1,:)) ~= numplots)
      fprintf('FAILURE: "autoscale" must have one element for each field\n')
      return
   end
   for plotnum = 1:numplots
      minclipval = setfield(cfg.minclipval, ...
                     cfg.fields{plotnum}, cfg.autoscale(1,plotnum));
      cfg = setfield(cfg, 'minclipval', minclipval);
      maxclipval = setfield(cfg.maxclipval, ...
                     cfg.fields{plotnum}, cfg.autoscale(2,plotnum));
      cfg = setfield(cfg, 'maxclipval', maxclipval);
      
   end
end

%get indices
abcissa_index = [1:length(data.dday)];
if (~isemptys(cfg.index))
   if (length(cfg.index) == 1)
      abcissa_index = abcissa_index([1:cfg.index:end]);
   elseif  (length(cfg.index) == 2)
      abcissa_index = abcissa_index(cfg.index(1):cfg.index(end));
   else
      abcissa_index = abcissa_index(cfg.index);
   end
end

%deal with the reference layer range
%make cfg.refl_range hold [] or a useable refl range (so we can propage it)
if (~isemptys(cfg.refl_range))
   cfg.refl_range = [max(1, min(cfg.refl_range));
                     min(length(data.depth), max(cfg.refl_range))];
else
   cfg.refl_range = [NaN NaN]';
end

% make a [2 X numplots] matrix with one refl per field, either NaNs or numbers
if (isemptys(cfg.doref))
   % propagate it 
   for plotnum = 1:numplots
      refl_range(:,plotnum) = cfg.refl_range;
   end
else %if doref is specified, use that to fill in refl.
   %exit if badly specified
   if (length(cfg.doref) ~= numplots)
      fprintf('FAILURE: "doref" must have one element for each field\n')
      return
   end
   %otherwise fill it in
   cfg.doref(cfg.doref ==0) = NaN;
   for plotnum = 1:numplots
      refl_range(:, plotnum) = cfg.doref(plotnum)*cfg.refl_range;
   end
end
% make comments
for plotnum = 1:numplots
   comments{plotnum} = cfg.fields{plotnum};
   if (~isnan(sum(refl_range(:,plotnum))))
      comments{plotnum} = sprintf('dereferenced %s', comments{plotnum});
   end
end



clg;whitebg('w')
for plotnum = 1:numplots
   if  (~isnan(sum(refl_range(:,plotnum))))
      ilayer = [refl_range(1,plotnum) : refl_range(2,plotnum)];
      orig = getfield(data, cfg.fields{plotnum});
      [vprofile, trefl] = refavg(orig, ilayer, 2);
      deref = orig - repmat(trefl, length(orig(:,1)), 1);
      data = setfield(data, cfg.fields{plotnum}, deref);
   end
   aplotit(data, cfg, 'comment', comments{plotnum},...
           'index', abcissa_index, 'abc_name', 'mins',...
           'fn', cfg.fields{plotnum},...
           'cur_ax', subplot(numplots, 1, plotnum));
end

date_it

if (length(~isnan(refl_range(1,:))) > 0)
   reflstr = sprintf(' refl bins = [%d - %d], ', ...
                     cfg.refl_range(1), cfg.refl_range(2));
else
   reflstr = ' ';
end


subtitle_comment = ...
    sprintf('raw data,%s(dday %3.3f - %3.3f)', reflstr, ...
            data.dday(abcissa_index(1)), data.dday(abcissa_index(end)));
subtitle(subtitle_comment)
