function  write_configs(cc, varargin)
% write_configs(config_structure, varargin)
% 
%   write configs to output file in current directory
%   TWO files and one matlab file are written
%   outfile_TYPE.cfg  crude dday range and configuration info
%   outfile_TYPE.llt  dday, date, ll range
%   outfile_TYPE.mat  matlab contents of cfg and llt file
%   (TYPE is 'LTA', 'STA', etc from file_suffix)
% 
%   name               what
%   -----              ----------
%   outfile_base       prefix for output files (defaults to 'configs')
%   timesort           sort by data time range (default = 0 (i.e. false))
%
% eg.  cc = get_configs(os,'dirbase','../osdata/cramer075/c185',...
%             'file_suffix','LTA','verbose',1);
%      write_configs(cc,  'outfile_base','cramer075');

cfg.outfile_base = 'configs';
cfg.timesort  = 0;

cfg=fillstruct(cfg, varargin);

if isemptys(cc)
   fprintf('empty config structure\n')
   return
end

cindex          = cc.cindex ;
cnames          = cc.cnames ;
config          = cc.config ;
otherinfo       = cc.otherinfo ;
file_suffix     = cc.file_suffix ;
dirbase         = cc.dirbase;
filelist        = cc.filelist ;
errmsg          = cc.errmsg;
comment         = cc.errmsg;

if isemptys(dirbase)
   dirbase = fileparts(filelist(1).name);
end

if ismember(file_suffix, {'LTA', 'lta', 'STA', 'sta'})
   enstype = 'average';
else
   enstype = 'pings';
end

cfgfile = sprintf('%s_%s.cfg', cfg.outfile_base, file_suffix);
lltfile = sprintf('%s_%s.llt', cfg.outfile_base, file_suffix);
matfile = sprintf('%s_%s.mat', cfg.outfile_base, file_suffix);
   

if (exist(cfgfile,'file')),  delete(cfgfile); end
if (exist(lltfile,'file')),  delete(lltfile); end

cfg_fid = fopen(cfgfile,'w');
llt_fid = fopen(lltfile,'w');


fprintf(cfg_fid,'%s files from %s\n\n',file_suffix, dirbase);
fprintf(cfg_fid, ' startdd  enddd     #ens  #pings  num   cell  blank       bin1m       BT         mode    pc-utc  sec/ens    FILE \n');
fprintf(cfg_fid, '                    this    per   cells size        pulse       maxd      xducer          sec             \n');
fprintf(cfg_fid, '                    file    ens          (m)   (m)   (m)   (m)   (m)  y/n  (m)                          \n\n');

fprintf(llt_fid,'%s files from %s\n\n',file_suffix, dirbase);
fprintf(llt_fid, '     starttime            endtime         startdd  enddd       lon_range               lat_range                 FILE \n\n');


if cfg.timesort
   [sorted_dday, isort] = sort(otherinfo.startdd);
else 
   isort = 1:length(cc.config(:,1));
end
   

for iconfig = isort(:)'   
   
   ifile = round(iconfig/2);

   ssi = getfield(cc.cindex, 'second_set');
   if isnan(cc.config(iconfig, ssi))
      continue
   end
   
   
   if (~isemptys(errmsg{iconfig})), fprintf(cfg_fid, '%s\n', errmsg{iconfig}); end
   if (~isemptys(errmsg{iconfig})), fprintf(llt_fid, '%s\n', errmsg{iconfig}); end
   
   if (~isemptys(comment{iconfig})), fprintf(cfg_fid, '%s\n', comment{iconfig}); end
   if (~isemptys(comment{iconfig})), fprintf(llt_fid, '%s\n', comment{iconfig}); end

   if (config(iconfig,end) == 10)
      if config(iconfig, ssi) == 1
         pingtype = '() nb';
      else
         pingtype = '   nb';
      end
   else
      pingtype    = 'bb   ';
   end

   %------------- lon lat time ---------------
   %starttime endtime startdd enddd lonrange latrange file

   try 
      timestr1 = to_date(otherinfo.yearbase(iconfig), otherinfo.startdd(iconfig),'s');
   catch
      timestr1 = 'time determination';
   end
   
   try
      timestr2 = to_date(otherinfo.yearbase(iconfig), otherinfo.enddd(iconfig),'s');
   catch
      timestr2 = 'FAILURE           ';
   end
   
   [pp,nn,ee]=fileparts(filelist(ifile).name);
   fprintf(llt_fid, '%s  %s  %7.2f %7.2f   %7.2f to %7.2f    %7.2f to %7.2f   %s\n', ...
           timestr1, timestr2,...
           otherinfo.startdd(iconfig),...
           otherinfo.enddd(iconfig),...
           otherinfo.lonrange(iconfig,1),...
           otherinfo.lonrange(iconfig,2),...
           otherinfo.latrange(iconfig,1),...
           otherinfo.latrange(iconfig,2),...
           [nn,ee]);
   
   %------------- configuration  ---------------
   
   %  startdd-enddd            #ens  #pings  num   cell  blank        bin1m 
   %                     this    per   cells size        pulse           
   %                     file    ens          (m)   (m)   (m)    (m)     
   
   
   fprintf(cfg_fid, ' %6.2f - %6.2f    %5d  %4d  %4d   %4d  %4d %4d %4d ',...
           otherinfo.startdd(iconfig),...
           otherinfo.enddd(iconfig),...
           otherinfo.numens(iconfig),...
           floor(config(iconfig, cindex.num_pings)),...
           floor(config(iconfig, cindex.ncells)),...
           floor(config(iconfig, cindex.cell)),...
           floor(config(iconfig, cindex.blank)),...
           floor(config(iconfig, cindex.pulse)),...
           floor(config(iconfig, cindex.bin1distance)));
   
   %    maxd  BT xducer(m) mode pc-utc sec/ens file\n');
   if (config(iconfig,cindex.bt_was_on) == 1)
      btstr = 'BT';
   else
      btstr = ' -';
   end
   
   
   fprintf(cfg_fid, '  %4d  %s %4d     %s ',...
           otherinfo.maxdepth(iconfig),...           
           btstr,...
           floor(config(iconfig, cindex.tr_pressure)),...
           pingtype);
   
   
   if (otherinfo.sec_pc_minus_utc(iconfig) > 99)
      fprintf(cfg_fid, '    %5d  ',  floor(otherinfo.sec_pc_minus_utc(iconfig)));
   else
      fprintf(cfg_fid, '   %5.1f ',  otherinfo.sec_pc_minus_utc(iconfig));
   end
   
   if strcmp(enstype, 'average')
      fprintf(cfg_fid,' %5d  ', floor(otherinfo.sec_per_ens(iconfig)));
   else
      fprintf(cfg_fid,' %4.1f  ', otherinfo.sec_per_ens(iconfig) );
   end
   
   [pp,nn,ee]=fileparts(filelist(ifile).name);
   fprintf(cfg_fid, '%s\n',  [nn,ee]);
   
   
   %-----------------------------------------------------
   
   if (rem(iconfig,10) == 0)
      fprintf(cfg_fid,'\n');
      fprintf(llt_fid,'\n');
   end
end


fprintf(cfg_fid,'\n\n');
fprintf(llt_fid,'\n\n');

fclose(cfg_fid);
fclose(llt_fid);


eval(sprintf('save6 %s cnames cindex config otherinfo', matfile))
fprintf('lon lat time info to %s\n', lltfile)
fprintf('configuration info to %s\n', cfgfile)
fprintf('saving info to %s\n', matfile)


