function r = load_raw(instclass, file, params, varargin)
% function load_raw(instclass, file, params, ...)
%
% Writes the command (*.cmd) and binary (*.bin) files
% required by ldcodas, starting from ue4 or EF RAW nb logging
% Each call to load_raw processes a single input file.
%                 ** Will change to process whole set.
% It also writes out files with (t x y) in a format suitable
% for refabs input.  The first fix from the ensemble goes to
% *.gps1, the last to *.gps2.
%
% Required arguments:
%     instclass:              instrument class (bb, nb, or os)
%     file: name of RAW
%     params: a structure with fields
%         params.h_align      : Heading alignment in degrees;
%                                 RDI sign convention
%         params.interval     : Averaging interval in seconds (default 300)
%         params.ilayer       :  bin0:bin1, reference layer indices
%                                 for averaging.  Give the array of
%                                 indices, not just the first and last bin.
%         params.yearbase     : yearbase (*** required *** (unlike os)
%         params.add_to_depth : transducer depth (required)  ***(unlike os)***
%
%         params.min_pings    : Minimum number of pings for an average;
%                                 only relevant at the end of file with
%                                 "finish" set to 1 (see below)
%         params.min_pg       : Within the reference layer, use only
%                                 the depths with at least this percent good;
%                                 if there are not at least 2 such levels,
%                                 don't use a reference layer, just calculate
%                                 a simple average.
%         params.max_ev       : Error velocity screening criterion, m/s
%         params.use_3_beam   : Not yet implemented; only 4-beam solutions
%                                 are used at present
%         params.bigtarget_amp       : 25
%         params.bigtarget_offset    :  5
%         params.bigtarget_avg       :  2
%                                          (See bigtarget.m)
%
%     >>The first four param fields are required;<<
%     the remainder have defaults.  Read the source to see what
%     they are, or eliminate all uncertainty by specifying all
%     param fields explicitly.
%
%     Other fields will likely be added as additional screening
%     methods are implemented.
%
% Optional named arguments: (keyword, value pairs)
%     "bin_path", bindir    : directory for *.cmd, *.bin
%     "leftover", b         : remaining profiles from last file
%     "finish", num         : 1 to finish, 0 to leave "leftover" for next call
%     "heading_offset", dh  : transducer heading offset in degrees
%
%        (second_set is normally 0, the default; set it to 1 only
%        if you have interleaved BB and NB pings and want to load
%        the NB part of the data set; leave it at 0 for the BB data,
%        or for any case when only BB or NB pings were collected.)



%
% 2000/09/22 EF
% 2001/01/25 JH adding instclass, morphing for NBADCP; get nav fields 
%            from gbin files
%            

global fid_bin fid_cmd

if (nargin <= 4)
   help(mfilename)
   error('too few variables')
end

fid_cmd = 0;  % initialize so it can serve as a "files are open" flag

fid_log = fopen('load_raw.log','a');
cc = clock;
fprintf(fid_log,'running load_raw on %s at %s\n',...
        file, to_date(cc(1), to_day(cc(1), cc)));
clear cc

%set defaults for varargin structure 
cfg.bin_path = './';
%cfg.nav_path = './';     %don't write nav yet
cfg.second_set = 0;
cfg.leftover = [];
cfg.finish = 1;

cfg = fillstruct(cfg, varargin, 0);

% rename parameters to match code
bin_path = cfg.bin_path;
%nav_path = cfg.nav_path;     %don't write nav yet
leftover = cfg.leftover;
finish = cfg.finish;

% The following parameter structure will become an input, but
% for now it is hardwired here.
%  p.h_align =  43.28; %%% Darryl's value for Kaiyo, ky0006L3 later files
%  p.interval = 300;
%  p.ilayer = 2:40;
p.yearbase = 0;     % for string-formatting to_date correctly
p.min_pg = 25;
p.max_ev = 1;
p.use_3_beam = 0;    % 3-beam not yet implemented
p.min_pings = 10;
p.add_to_depth = 0;
p.bigtarget_amp = 25;
p.bigtarget_offset = 5;
p.bigtarget_avg = 2;

if nargin < 4 | ~ischar(file) | ~isstruct(params),
   help load_raw
   fprintf('At least 3 arguments are required:\n')
   fprintf('an instrument class, a filename and a structure.')
   error('too few input arguments ')
end

if (~isfield(params, 'h_align') | ...
    ~isfield(params, 'interval') | ...
    ~isfield(params, 'ilayer') | ...
    ~isfield(params, 'add_to_depth') | ...
    ~isfield(params, 'yearbase'));
   help load_raw
   fprintf('required Parameter fields: \n');
   error('h_align, interval, ilayer, yearbase, add_to_depth ')
end

% Transfer input parameters, overriding defaults.
fn = fieldnames(params);
for ii = 1:length(fn)
   fname = fn{ii};
   p = setfield(p, fname, getfield(params, fname));
end

[a_path, a_file, a_ext] = fileparts(file);
cmdfile = fullfile(bin_path, [a_file  '.cmd']);
binfile = fullfile(bin_path, [a_file  '.bin']);
nav1file = fullfile(nav_path, [a_file  '.gps1']);     %don't write nav yet
nav2file = fullfile(nav_path, [a_file  '.gps2']);     %don't write nav yet

% Read the first and last profile to get the configuration, get start and 
% end dates, initialize the structure  for the average.  This is very fast.
a = read(instclass, file,  'yearbase', p.yearbase, 'ends',2,...
         'vars', 'all'); %no nav in nb
tic
fprintf('reading all times\n')
temp = read(instclass, file,  'yearbase', p.yearbase, 'vars', 'ts');
toc
p.dday = temp.dday;

% reset leftover to [] if we're starting a new configuration segment
if (isemptys(leftover))
   fprintf('leftover is empty\n')
   paused(1)
end
if (a.ens_num(1) == 1)
   leftover = [];
   fprintf('starting a new configuration segment: resetting leftover to []\n')
end


%keep the argument list the same
index_ranges = find_iranges(instclass, leftover,  p); 
NP = size(index_ranges, 2);

% depth: temporary variable just to fill the structure a.depth
% a.depth: depth of bins for average
% r.depth (below): raw depth
depth = mk_depth(a); %a.config.tr_depth set to 0 in read.m
a.depth = round(depth + p.add_to_depth);
NC = length(a.depth);  %from a.config.ncells (i.e. number of cells)
a.config.tr_depth = p.add_to_depth;
config1 = mk_config1(instclass, a);
ii = config1_ind;
config1(ii.hd_offset) = p.h_align;
access = mk_access(instclass, a);    % Doesn't change.

keyboard
for ip = 1:NP
   
   tic
   r = read(instclass, file,  'yearbase',  p.yearbase,...
            'irange', index_ranges(:, ip), 'vars', 'all');
   
   if (isemptys(r.dday))
      disp('r.dday is empty: in load_raw now.  check it out')
      keyboard
   end

   
   %%% fill these somehow %%
   %r = fixtime(nb,  r, nb2gga_coef);
   %r = fillhdg(instclass, fullfile(rawdatapath, rawdataname), r);
   %r = fillnav(instclass, fullfile(rawdatapath, rawdataname), r);
   

   
   r.depth = mk_depth(r); %a.config.tr_depth set to 0 in read.m
   r.depth = round(r.depth + p.add_to_depth);
   fprintf('%d out of %d (%s) \n', ip, NP, file)

   fprintf(fid_log, ...
           '%d out of %d, indices [%d %d], dd: %7.3f - %7.3f (%s %s)\n',...
           ip, NP, index_ranges(1, ip), index_ranges(2, ip), ...
           r.dday([1 end]), to_date(2000, r.dday(1)), ...
           to_date(p.yearbase, r.dday(end)));

   if ip == 1 & ~isemptys(leftover)
      r = catadcp(instclass, leftover, r);
   end

   if ip == NP
      if ~finish | (diff(index_ranges(:, NP)) < p.min_pings - 1),
         break,
      end
   end

   a.num_pings = length(r.dday);

   a = avg(instclass, r, a, p);
   toc
   
   % all the structures:
   ancil1 = mk_ancil1(instclass, r, 1);
   ancil2 = mk_ancil2(instclass, r, 1);
   bt = mk_bt(instclass, r, 1);

   
   nav = ones(4,1) * 1e38;   
   %NAVIGATION structure wants [latitude longitude]
   %but nav structure has [time longitude latitude]

   % The velocity part is not used for anything.
   a.dday = r.dday(end);
   
   if ip == 1,
      %fid_nav1 = fopen(nav1file, 'w');
      %fid_nav2 = fopen(nav2file, 'w');
      fid_cmd = fopen(cmdfile, 'w');
      fid_bin = fopen(binfile, 'w');
      if fid_nav1 == -1, error(['Can''t open ', nav1file]); end
      if fid_nav2 == -1, error(['Can''t open ', nav2file]); end
      if fid_cmd == -1, error(['Can''t open ', cmdfile]); end
      if fid_bin == -1, error(['Can''t open ', binfile]); end
      fprintf(fid_cmd, 'binary_file: %s\n', binfile);
   end

   if ip == 1 | (rem(ip, 400) == 1 &  (NP - ip) > 10),
      fprintf(fid_cmd, 'new_block\n');
      fprintf(fid_cmd, 'dp_mask: 1\n');

      bin_float('DEPTH', depth);
      bin_double('CONFIGURATION_1', config1);
   end
   fprintf(fid_cmd, ['new_profile: ', to_date(a.config.yearbase, a.dday),...
              '  /* %d %d */\n'], r.ens_num([1 end]));
   fprintf(fid_cmd, 'depth_range: %d %d\n', round(depth([1 end])));
   bin_double('ACCESS_VARIABLES', access);
   bin_double('ANCILLARY_1', ancil1);
   bin_double('ANCILLARY_2', ancil2);
   bin_double('BOTTOM_TRACK', bt);
   bin_double('NAVIGATION', nav);
   bin_float('U', a.vel(:, 1));
   bin_float('V', a.vel(:, 2));
   bin_float('W', a.vel(:, 3));
   bin_float('ERROR_VEL', a.vel(:, 4));
   bin_ubyte('AMP_SOUND_SCAT', round(mean(a.amp, 2)));
   bin_ubyte('RAW_AMP', a.amp);
   bin_ubyte('PERCENT_GOOD', a.pg(:, 4));
   bin_ubyte('PERCENT_3_BEAM', a.pg(:, 3));
   bin_ubyte('SPECTRAL_WIDTH', round(mean(a.sw, 2)));
   bin_ubyte('RAW_SPECTRAL_WIDTH', a.sw);
   %% Maybe we should call it something else, but this will do
   %% for now; it is not correlation, either.
   bin_ubyte('PROFILE_FLAGS', zeros(1, NC));
   % .....
   
   %don't write nav now
   %fprintf(fid_nav1, '%12.7f %12.7f %12.7f\n', r.nav.txy1(:, 1));
   %fprintf(fid_nav2, '%12.7f %12.7f %12.7f\n', r.nav.txy2(:, end));
   

end

if finish,
   r = [];
elseif fid_cmd,
   fprintf(fid_cmd, '/* leftover: %d %d */\n', r.ens_num([1 end]));
end
fprintf(fid_log, '/* leftover: %d %d */\n', r.ens_num([1 end]));
fprintf(fid_log, 'leftover dday range is %f - %f (%s - %s)\n', ...
        r.dday([1 end]), to_date(r.config.yearbase,r.dday(1)), ...
        to_date(r.config.yearbase,r.dday(end)));

if fid_cmd
   %fclose(fid_nav1);      %don't write nav now
   %fclose(fid_nav2);      %don't write nav now
   fclose(fid_cmd);
   fclose(fid_bin);
end

fclose(fid_log);
%====================================================

function ir =  find_iranges(instclass, leftover,  p);

yearbase = p.yearbase;
interval = p.interval;

if ~isemptys(leftover)
   dday = [leftover.dday; p.dday];
else
   dday = p.dday;
end

delta = interval/86400;
d0 = dday(1):delta:dday(end);

n = length(d0);


index = [1:length(dday)];
if n == 1,
   ir = [index(1); index(end)];
else
   for ii = 1:n
      ir(1, ii) = index(max(find(dday <= d0(ii))));
   end
   
   ir(2, 1:(n-1)) = ir(1, 2:n) - 1;
   ir(2, end) = index(end);
end


%-----------------------------------------------------------
