function load_avg(instclass, file, yearbase, varargin)
% function load_avg(instclass, file,  yearbase, ...)
%
% Writes the command (*.cmd) and binary (*.bin) files
% required by ldcodas, starting from VMDAS *.STA or *.LTA.
% Each call to load_avg processes a single input file.
%
% If data are from an Ocean Surveyoror a Broad Band adcp, it also writes 
% out files with (t x y) in a formula suitable for refabs input.  
% The first fix from the  ensemble goes to *.gps1, the last to *.gps2.
%
% Required arguments:
%     instclass: (bb or os)
%     file: name of STA or LTA file
%     yearbase: base year for dday
%
% Optional named arguments:
%     "bin_path", bindir    : directory for *.cmd, *.bin
%     "nav_path", navdir    : directory for *.gps1 and *.gps2
%     "irange", [starti  endi] : start and end index
%     "ducer_depth",       : meters to add to recorded depths
%     "second_set", num     : 1 for second set, otherwise 0  (OS only)
%
%
%
% 2000/09/22 EF, written for os
%
% 2001/01/25 JH adding instclass
% 2001/06/05 JH adding bb
% 2003/11/02 JH removing NaNs from navigation files; 
%               change default binpath and navpath from './' to ''
%% 2004/02026 JH : more complete -- delete bad ens from consideration


global fid_bin fid_cmd

bin_path = '';   
nav_path = '';
irange = [1 10000000];
ducer_depth = 0;
second_set = 0;

for iarg = 1:2:(nargin-3)
   arg = varargin{iarg};
   val = varargin{iarg+1};
   switch lower(arg)
      case 'bin_path',     bin_path = val;
      case 'nav_path',     nav_path = val;
      case 'irange',    irange = val;
      case 'ducer_depth', ducer_depth = val;
      case 'second_set',   second_set = val;
      otherwise,           error('invalid argument name')
   end
end

[a_path, a_file, a_ext] = fileparts(file);
cmdfile = fullfile(bin_path, [a_file a_ext '.cmd']);
binfile = fullfile(bin_path, [a_file a_ext '.bin']);
nav1file = fullfile(nav_path, [a_file a_ext '.gps1']);
nav2file = fullfile(nav_path, [a_file a_ext '.gps2']);


%%% this should be irrelevant: second_set is never processed!!
if second_set,
   a = read(instclass, file, 'irange', irange,...
            'yearbase', yearbase,...
            'vars', {'all', 'nav'},   'second_set', 1 );
else
   a = read(instclass, file, 'irange', irange, ...
            'yearbase', yearbase,...
            'vars', {'all', 'nav'});
end

if (length(a.dday) == 1)
   avg_interval = 0;
else
   avg_interval = mstdgap(diff(a.dday*86400));
end

%use gga time at end of ensemble
a.corr_dday = a.nav.txy2(1,:);

%% vmdas bugs
%% (1)  remove these profiles (they have bad navigation)
goodi = find(~isnan(a.nav.txy1(1,:)));
badi =  find(isnan(a.nav.txy1(1,:)));
for ii=1:length(badi)
   fprintf('deleting ensemble %d: bad navigation\n', a.ens_num(badi))
end
a = cutadcp(a, goodi);
%% (2) If navigation is lost, corrected times are not updated.  
%      Remove those profiles.  
intervals = 86400*diff(a.corr_dday);
goodi = find( intervals > .2*(avg_interval));
badi  = find(intervals <= .2*(avg_interval));

for ii=1:length(badi)
   fprintf('deleting ensemble %d (%s): interval (%2.1f sec) too short\n', ...
           a.ens_num(badi(ii)), ...
           to_date(a.config.yearbase, a.corr_dday(badi(ii)),'s'),...
           intervals(badi(ii)));           
end
a = cutadcp(a, goodi);


%??? see a.dday below
if length(a.dday) < 3,
   disp(['Warning: ', file, ' has fewer than 3 ensembles (ignore)'])
   return
end



NP = length(a.heading);

fid_nav1 = fopen(nav1file, 'w');
fid_nav2 = fopen(nav2file, 'w');
if fid_nav1 == -1, error(['Can''t open ', nav1file]); end
if fid_nav2 == -1, error(['Can''t open ', nav2file]); end

%% take out NaNs in navigation files
goodi1 = find(~isnan(sum(a.nav.txy1)));
goodi2 = find(~isnan(sum(a.nav.txy2)));
fprintf(fid_nav1, '%12.7f %12.7f %12.7f\n', a.nav.txy1(:,goodi1));
fprintf(fid_nav2, '%12.7f %12.7f %12.7f\n', a.nav.txy2(:,goodi2));

fclose(fid_nav1);
fclose(fid_nav2);

% all the structures:
a.top_ref_bin   = 0;
a.bot_ref_bin   = 0;
a.ens_threshold = 0;
a.ev_threshold  = 0;

access = mk_access(a);
config1 = mk_config1(a);
c1_index =  config1_ind;
config1(c1_index.avg_interval)   = avg_interval;
config1(c1_index.hd_offset)      = a.config.head_align;
config1(c1_index.heading_bias)   = a.config.head_bias;

ancil1 = mk_ancil1(a);
ancil2 = mk_ancil2(a);
depth = round(a.depth + a.config.tr_pressure);
ND = length(depth);
bt = mk_bt(a);

nav = zeros(4,NP) * NaN;
% NAVIGATION structure from codas3/adcp/include/adcp.h is [latitude longitude]
% Positions shown on the editing stagger plots.
nav(1:2,:) = a.nav.txy2([3 2],:);  
% The velocity part is not used for anything.



fid_cmd = fopen(cmdfile, 'w');
fid_bin = fopen(binfile, 'w');
if fid_cmd == -1, error(['Can''t open ', cmdfile]); end
if fid_bin == -1, error(['Can''t open ', binfile]); end


fprintf(fid_cmd, 'binary_file: %s\n', binfile);

for ip = 1:NP
   if ip == 1 | (rem(ip, 400) == 1 &  (NP - ip) > 10),
      fprintf(fid_cmd, 'new_block\n');
      fprintf(fid_cmd, 'dp_mask: 1\n');

      bin_float('DEPTH', depth);
      bin_double('CONFIGURATION_1', config1);
   end

   fprintf(fid_cmd, ['new_profile: ', ...
                     to_date(a.config.yearbase, a.corr_dday(ip)), '\n']);
   fprintf(fid_cmd, 'depth_range: %d %d\n', round(depth([1 end])));
   bin_double('ACCESS_VARIABLES', access);
   bin_double('ANCILLARY_1', ancil1(:,ip));
   bin_double('ANCILLARY_2', ancil2(:,ip));
   bin_double('BOTTOM_TRACK', bt(:,ip));
   bin_double('NAVIGATION', nav(:,ip));
   bin_float('U', a.vel(:, 1, ip));
   bin_float('V', a.vel(:, 2, ip));
   bin_float('W', a.vel(:, 3, ip));
   bin_float('ERROR_VEL', a.vel(:, 4, ip));
   bin_ubyte('AMP_SOUND_SCAT', round(mean(a.amp(:, :, ip), 2)));
   bin_ubyte('RAW_AMP', squeeze(a.amp(:, :, ip)));
   bin_ubyte('PERCENT_GOOD', a.pg(:, 4, ip));
   bin_ubyte('PERCENT_3_BEAM', a.pg(:, 3, ip));
   bin_ubyte('SPECTRAL_WIDTH', round(mean(a.cor(:, :, ip), 2)));
   bin_ubyte('RAW_SPECTRAL_WIDTH', squeeze(a.cor(:, :, ip)));
   %% Maybe we should call it something else, but this will do
   %% for now; it is not correlation, either.
   bin_ubyte('PROFILE_FLAGS', zeros(1, ND));
   % ......

end

fclose(fid_cmd);
fclose(fid_bin);

save oc407
%----------------------------------------------------
% function bin_float(name, array)
%---------------------------------------------------
% function bin_double(name, array)
%---------------------------------------------------
% function bin_ubyte(name, array)


