disp('The current version is in ../../logging')
disp('Make that version work for nb using this as a guide')
return


function r = load_raw(instclass, file, params,  varargin)
% function load_raw(instclass, file, params, ...)
%
% Writes the command (*.cmd) and binary (*.bin) files
% required by ldcodas, starting from VMDAS *.ENR or *.ENS.
% Each call to load_raw processes a single input file
%                 ** Will change to process whole set.
% It also writes out files with (t x y) in a format suitable
% for refabs input.  The first fix from the ensemble goes to
% *.gps1, the last to *.gps2.
%
% Required arguments:
%     instclass:              instrument class (bb, os)
%     file: name of ENR or ENS file
%     params: a structure with fields
%         params.h_align      : Heading alignment in degrees;
%                                 RDI sign convention
%         params.interval     : Averaging interval in seconds (default 300)
%         params.ilayer       :  bin0:bin1, reference layer indices
%                                 for averaging.  Give the array of
%                                 indices, not just the first and last bin.
%         params.geometry     : 'convex' (OS) 'concave' (NB) 
%         params.yearbase     : yearbase (*** required *** (unlike os)
%         params.min_pings    : Minimum number of pings for an average;
%                                 only relevant at the end of file with
%                                 "finish" set to 1 (see below)
%         params.add_to_depth : meters to add to recorded depth,
%                                 if recorded transducer depth is incorrect
%         params.min_pg       : Within the reference layer, use only
%                                 the depths with at least this percent good;
%                                 if there are not at least 2 such levels,
%                                 don't use a reference layer, just calculate
%                                 a simple average.
%         params.max_ev       : Error velocity screening criterion, m/s
%         params.use_3_beam   : Not yet implemented; only 4-beam solutions
%                                 are used at present
%         params.bigtarget_amp       : 25
%         params.bigtarget_offset    :  5
%         params.bigtarget_avg       :  2
%                                          (See bigtarget.m)
%
%     >>The first three param fields are required;<<
%     the remainder have defaults.  Read the source to see what
%     they are, or eliminate all uncertainty by specifying all
%     param fields explicitly.
%
%     Other fields will likely be added as additional screening
%     methods are implemented.
%
% Optional named arguments: (keyword, value pairs)
%     "ldcodas_path", bindir: directory to write *.cmd, *.bin
%     "nav_path", navdir    : directory for *.gps1 and *.gps2
%     "second_set", num     : 1 for second set, otherwise 0 (see below)
%     "leftover", b         : remaining profiles from last file
%     "finish", num         : 1 to finish, 0 to leave "leftover" for next call
%     "heading_offset", dh  : transducer heading offset in degrees
%
%        Ocean Surveyor only:
%        (second_set is normally 0, the default; set it to 1 only
%        if you have interleaved BB and NB pings and want to load
%        the NB part of the data set; leave it at 0 for the BB data,
%        or for any case when only BB or NB pings were collected.)
%
% 2000/09/22 EF
%

% 2001/01/25 JH adding instclass
% 2002/07/19 JH adding final r read (after index_ranges loop)

global fid_bin fid_cmd

fid_cmd = 0;  % initialize so it can serve as a "files are open" flag

fid_log = fopen('load_raw.log','a');
cc = clock;
fprintf(fid_log,'running load_raw on %s at %s\n',...
        file, to_date(cc(1), to_day(cc(1), cc)));
clear cc


%set defaults for varargin structure 
cfg.ldcodas_path = './';
cfg.nav_path = './';     
cfg.second_set = 0;
cfg.leftover = [];
cfg.finish = 1;

cfg = fillstruct(cfg, varargin, 0);

% rename parameters to match code
ldcodas_path   = cfg.ldcodas_path ;
nav_path       = cfg.nav_path     ;
second_set     = cfg.second_set   ;
leftover       = cfg.leftover     ;
finish          = cfg.finish      ;

% The following parameter structure will become an input, but
% for now it is hardwired here.
%  p.h_align =  43.28; %%% Darryl's value for Kaiyo, ky0006L3 later files
%  p.interval = 300;
%  p.ilayer = 2:40;
p.yearbase = 0;     % for string-formatting to_date correctly
p.interval = 300;
p.geometry = 'convex';
p.h_align = 0;
p.ilayer = [4:12];
p.min_pg = 25;
p.max_ev = 1;
p.use_3_beam = 0;    % 3-beam not yet implemented
p.min_pings = 10;
p.add_to_depth = 0;
p.bigtarget_amp = 25;
p.bigtarget_offset = 5;
p.bigtarget_avg = 2;

if nargin < 4 | ~ischar(file) | ~isstruct(params),
   help load_raw
   fprintf('At least 3 arguments are required:\n')
   fprintf('an instrument class, a filename and a structure.')
   error('too few input arguments ')
end

if (~isfield(params, 'h_align') | ...
    ~isfield(params, 'interval') | ...
    ~isfield(params, 'ilayer') | ...
    ~isfield(params, 'geometry') | ...
    ~isfield(params, 'yearbase')),
   help load_raw
   fprintf('required Parameter fields: \n');
   error('h_align, interval, geometry, ilayer, yearbase ')
end

p = fillstruct(p, params, 0);

[a_path, a_file, a_ext] = fileparts(file);
cmdfile = fullfile(ldcodas_path, [a_file a_ext '.cmd']);
binfile = fullfile(ldcodas_path, [a_file a_ext '.bin']);
nav1file = fullfile(nav_path, [a_file a_ext '.gps1']);
nav2file = fullfile(nav_path, [a_file a_ext '.gps2']);

% Read the first and last profile to get the configuration, get start and 
% end dates, initialize the structure  for the average.  This is very fast.
% AND return the fid
a = read(instclass, file,  'yearbase', p.yearbase, 'ends',1,...
         'second_set', second_set, 'vars', {'all', 'nav'});
finfo = a.info; %pass this in to subsequent reads; 

% reset leftover to [] if we're starting a new configuration segment
if (isemptys(leftover))
   fprintf('leftover is empty\n')
end
if (a.ens_num(1) == 1)
   leftover = [];
   fprintf('starting a new configuration segment: resetting leftover to []\n')
end

%warn the user about how many ensembles they are dealing with:
tmp = read(instclass, file,  'yearbase', p.yearbase, 'second_set', ...
         second_set, 'ends', 2);
fprintf('reading times.  (%d pings in this file)\n', ...
        tmp.ens_num(end) - tmp.ens_num(1))
%now we have to read the times:
tmp = read(instclass, file,  'yearbase', p.yearbase, 'second_set', ...
         second_set, 'vars', 'nav');
p.dday = tmp.nav.txy1(1,:); %start time of ping, corrected to gps

if ~(isemptys(leftover))
   if (p.dday(1) - leftover.dday(end) > .5*(p.interval/86400))
      leftover = [];
   end
end


fprintf('calculating iranges\n')
index_ranges = find_iranges(leftover,  p); 


NP = size(index_ranges, 2);
depth = round(a.depth + p.add_to_depth);
ND = length(depth);
config1 = mk_config1(a);
ii = config1_ind;
config1(ii.hd_offset) = p.h_align;
access = mk_access(a);    % Doesn't change.

for ip = 1:NP
   tic
   fprintf('reading and averaging  %d out of %d ensembles\n', ip, NP)

   ilist = [index_ranges(1, ip) : index_ranges(2, ip)];

   r = read(instclass, file, 'ilist', ilist, ...
               'vars', {'all', 'nav'},   'second_set', second_set );


   if ip == 1 & ~isemptys(leftover)
      r = catadcp(leftover, r);
   end


   fprintf(fid_log, ...
           '%d out of %d, indices [%d %d], dd: %7.3f - %7.3f (%s %s)\n',...
           ip, NP, index_ranges(1, ip), index_ranges(2, ip), ...
           r.dday([1 end]), to_date(p.yearbase, r.dday(1)), ...
           to_date(p.yearbase, r.dday(end)));
   
   if ip == NP
      if ~finish | (diff(index_ranges(:, NP)) < p.min_pings - 1),
         break,
      end
   end
   
   
   a.num_pings = length(r.dday);
   a = avg(r, a, p);
   
   
   % all the structures:
   ancil1 = mk_ancil1(r, 1);
   ancil2 = mk_ancil2(r, 1);
   bt = mk_bt(r, 1);


   %%%% nav and dday are now assuming ens files with good fixes!
   nav = ones(4,1) * 1e38;
   % NAVIGATION structure from codas3/adcp/include/adcp.h is [lat lon]
   % Positions shown on the editing stagger plots.
   nav(1:2) = r.nav.txy2([3 2],end);  

   [mm, ss, nn, mdt] = mstdgap(r.nav.sec_pc_minus_utc);
   a.dday = r.dday(end) - mdt / 86400;

   if ip == 1,
      fid_nav1 = fopen(nav1file, 'w');
      fid_nav2 = fopen(nav2file, 'w');
      fid_cmd = fopen(cmdfile, 'w');
      fid_bin = fopen(binfile, 'w');
      if fid_nav1 == -1, error(['Can''t open ', nav1file]); end
      if fid_nav2 == -1, error(['Can''t open ', nav2file]); end
      if fid_cmd == -1, error(['Can''t open ', cmdfile]); end
      if fid_bin == -1, error(['Can''t open ', binfile]); end
      fprintf(fid_cmd, 'binary_file: %s\n', binfile);
   end

   if ip == 1 | (rem(ip, 400) == 1 &  (NP - ip) > 10),
      fprintf(fid_cmd, 'new_block\n');
      fprintf(fid_cmd, 'dp_mask: 1\n');

      bin_float('DEPTH', depth);
      bin_double('CONFIGURATION_1', config1);
   end
   fprintf(fid_cmd, ['new_profile: ', to_date(a.config.yearbase, a.dday),...
              '  /* %d %d */\n'], r.ens_num([1 end]));
   fprintf(fid_cmd, 'depth_range: %d %d\n', round(depth([1 end])));
   bin_double('ACCESS_VARIABLES', access);
   bin_double('ANCILLARY_1', ancil1);
   bin_double('ANCILLARY_2', ancil2);
   bin_double('BOTTOM_TRACK', bt);
   bin_double('NAVIGATION', nav);
   bin_float('U', a.vel(:, 1));
   bin_float('V', a.vel(:, 2));
   bin_float('W', a.vel(:, 3));
   bin_float('ERROR_VEL', a.vel(:, 4));
   bin_ubyte('AMP_SOUND_SCAT', round(mean(a.amp, 2)));
   bin_ubyte('RAW_AMP', a.amp);
   bin_ubyte('PERCENT_GOOD', a.pg(:, 4));
   bin_ubyte('PERCENT_3_BEAM', a.pg(:, 3));
   bin_ubyte('SPECTRAL_WIDTH', round(mean(a.cor, 2)));
   bin_ubyte('RAW_SPECTRAL_WIDTH', a.cor);
   %% Maybe we should call it something else, but this will do
   %% for now; it is not correlation, either.
   bin_ubyte('PROFILE_FLAGS', zeros(1, ND));
   % ......

   fprintf(fid_nav1, '%12.7f %12.7f %12.7f\n', r.nav.txy1(:, 1));
   fprintf(fid_nav2, '%12.7f %12.7f %12.7f\n', r.nav.txy2(:, end));

   toc

end


if finish,
   r = [];
elseif fid_cmd,
   
   % read the remaining portion of the file
   ilist = [index_ranges(2,NP):length(p.dday)];

   r = read(instclass, file, 'ilist', ilist, ...
               'vars', {'all', 'nav'},   'second_set', second_set );

   if (isemptys(r.dday))
      disp('r.dday is empty: in load_raw now.  check it out')
      keyboard
   end

   fprintf(fid_cmd, '/* leftover: %d %d */\n', r.ens_num([1 end]));
end

fprintf(fid_log, '/* leftover: %d %d */\n', r.ens_num([1 end]));
fprintf(fid_log, 'leftover dday range is %f - %f (%s - %s)\n', ...
        r.dday([1 end]), to_date(r.config.yearbase,r.dday(1)), ...
        to_date(r.config.yearbase,r.dday(end)));

if fid_cmd
   fclose(fid_nav1);
   fclose(fid_nav2);
   fclose(fid_cmd);
   fclose(fid_bin);
end


fclose(fid_log);
%====================================================


function ir =  find_iranges( leftover,  p);
gapfrac = .5;  %fraction of delta to allow between leftover(end) and dday(1)

yearbase = p.yearbase;
interval = p.interval;


dday = p.dday;

delta = interval/86400;

if (isemptys(leftover) | (  (dday(1) - leftover.dday(end)) > gapfrac)  )
   d0 =dday(1) : delta : dday(end);
else
   d0 = leftover.dday(1) : delta : dday(end);
end

d0(find(d0 < dday(2))) = [];   %throw out times where no dday exist
   
n = length(d0);


ir = zeros(2,n+1);

ir(1,1) = 1; 
for ii = 1:n
   ir(1, ii+1) = max(find(dday <= d0(ii)));;
end

ir(2,:) = [ir(1,2:end)-1, length(dday)];
