function d = read(instclass, file_or_struct, varargin)
% read.m: a general function for extracting variables from Sontek
%        adp data files.  This version is specific to the 4-beam "type 2"
%        files, but could easily be modified for "type 1" or others.
% Usage:
%   data = read(instclass, file_or_struct, varargin)
% instclass: son
%
%
% Input:
% file_or_struct:  
%     file:  typical usage for a one-time call to look at the data
%             file is the relative path to full raw file name, eg:
%                   '../raw/adcp_pings/9602078r.019'
%     struct: specialized usage for repeated calls (as in "load_enr")
%             uses info from a previous read OF THE SAME FILE 
%             (read the top of this file for more details)
% 
% assume varargin is a collection of name, values pairs
% 'vars', {'vel', 'amp', 'ts', 'bt', 'einfo', 'all'}
% i.e. aquisition of single data types  in a cellarray called 'vars'
%     include any or all in the cellarray to turn them on
% -- or -- if there is only one variable, it can be a string 
% 'vars', 'ts'         % gives fields: heading, pitch, roll, 
%                      %            temperature, soundspeed
% 'vars', 'einfo'      % gives fields: xflux, yflux, zflux, noise
%                      %            compass temperature, voltage
% -- or --  use the string 'all' to get all variables possible
%       (values returned will be NaN if variable is not available)
% 'vars', 'all'   
%% 
% 'yearbase', [yearbase]      % default is first year in the file
%
% 'pulse', [pulse in meters]  % raw logged files do not include pulse. add it
%                             %   here to get a correct bin1distance.
%
% 'autosuffix'  [0 or 1]      % default is 0 (no autosuffix, i.e. use 
%                             %    filenames verbatim).  if set to 1,
%                             %     looks for ".adp" or ".ADP" suffix.
%
%                             % different ways to specify data extracted:
% 'irange',  [istart istop]   % range of indices to use
% 'step', [step]              % use only every "step" index
% 'ilist', [ index list]      % explicit index list overrides step
% 'ends', [integer 0-4]       % 0 (default - no subsampling)
%                             % 1: get only first profile
%                             % 2: get first and last profile
%                             % 3: not implimented
%                             % 4: get first and last pair of profiles
%                             % NOTE: use of 'ends' overrides range, step,
%                             %    and ilist
%
% 'back_to_front' [0 or 1]    %   find last good index by reading file
%                                 from back to front (default)
%                                 (otherwise back_to_front = 0 starts
%                                 at the beginning of the file -
%                                 slower but more reliable)
%
%
% 'verbose' [0,1]             %default is 0; add verbosity
%
%  NOTE on ranges: default is 1:end
%       You may only use one of the following choices.  
%       -  ilist
%       -  irange (step is optional, defaults to 1) - overrides ilist
%       -  ends  - overrides ilist and irange
% 
% Output:
%     Dimensions:
%        NP: number of profiles extracted
%        NC: number of depth cells
%        NB: number of beams (hardwired here to 4!)
%     Outputs:
%        data.vel: NCxNBxNP array of velocities in m/s
%        timeseries data ('ts' or 'einfo') NPx1
% -------------------------------------
%  typical one-time usage:
%  rawfile =  '../raw/adcp_pings/nbp2001_091_52596.raw';
%  data = read(son, rawfile, 'vars', 'amp');
%  data = read(son, rawfile, 'vars', {'vel', 'ts'});
%  data = read(son, rawfile, 'yearbase', 2001, 'vars', 'all');
%
%  useful restructuring call:
%  [dd, config] = restruct(son, data);



%%% ADITIONAL NOTES %%%
%  NOTE a structure called "info" in the outgoing data structure contains
%         - info.fullrawfile        %the whole path to the file
%         - info.rawfilebase        %the base of the file read 
%         - info.ilist              %the index list used to get the data
%                                   % NOTE: must use name, value pair with
%                                   % name 'ilist' to choose the values to
%                                   % be read
%         - info.nbytes             % number of bytes in a data block (incl cs)
%         - info.config             %
%         - info.last_index         % 
%
%  NOTE file_or_struct - usage with a structure
%          previous call should explicitly output additional info:
%             "d = read(instclass, file_or_strut, 'return_fid', 1, varargin)
%          then call as either
%             "d = read(instclass, d.info , 'return_fid', 1, varargin)
%                                   % does not close raw file upon return
%             "d = read(instclass, d.info , varargin) 
%                                   % closes file upon return
%
%  NOTE ens_num is the raw ensemble value contained in the data.  There could
%       potentially be gaps in ens_num, but it should always increase (with 
%       the single exception of a wrap which occurs at 2^16)
%

%% undocumented features:
%% 'vars' defaults to none, turn on pieces or 'all' exlicitly
%% range defaults to all; choose specific subsets explicitly


%     1998/03/22 Eric Firing
%     1998/04/27 EF added ts2 array.
%     2000/03/17 EF allow input filename with extension
%     2000/04/19 EF add noise output
%     2002/07/04 JH adapt to use class, matching with current rdi read versions
%     2003/12/12 JH fixed bug in read_buf (some uint8 changed to uint16)


%%%%%%%%%%%%


if (nargin < 2)
   help read 
   error('must set instclass and file_or_struct')
end

%set defaults.  all except field "vars" are present in the original
%  cfg structure; fillstruct will update the values listed
%  Field "vars" will be initialized by fillstruct.  set those chosen to 1
cfg.yearbase  = [];  %default: use first year in the file
cfg.pulse     = NaN;
cfg.verbose     = 0;
cfg.step        = 1;
cfg.irange    = []; % default all range specifications to [] (see function
cfg.ilist     = []; % default all range specifications to [] test_range_request
cfg.back_to_front = 1;  %find last_index by reading back to front
cfg.ends      = []; % default all range specifications to []
cfg.vars =      [];% default variables to []
vars            = {'vel', 'amp', 'pg', 'ts', 'bt', 'einfo', 'all'};
cfg.autosuffix = 0;
out.nav         = 0; %initialized "out" as a nonempty structure 


% structure "out" fields from "vars" cellarray initialized as zero
for cii = 1:length(vars)
   out = setfield(out, vars{cii}, 0); 
end
%we can take it back out here; it doesn't exist in sontek data
out = rmfield(out, 'nav');


% update cfg.
if (nargin > 2)%   we'll assume the rest is a set of name,value pairs
               %override input structure with name,value pairs
   cfg = fillstruct(cfg, varargin,0 );
end
%now check whether cellarray 'vars' has been added - use the elements of
% cfg.vars (if it exists) to turn on aquisition of the listed fields
% NOTE: cellarray "vars" does not have "nav", whereas cfg.vars could.
if (isfield(cfg, 'vars'))
   if (~isemptys(cfg.vars))
      if (ischar(cfg.vars))  %if there is only variable chosen, allow a string
         cfg.vars = {cfg.vars};
      end
      
      for cii = 1:length(cfg.vars)
         out = setfield(out, cfg.vars{cii}, 1); 
      end
      
      % out.all (override individual chosen data types and output everything)
      if ismember({'all'}, cfg.vars)
         %they turned it on so get everything
         for cii = 1:length(vars)
            out = setfield(out, vars{cii}, 1);
         end
      end
      out = rmfield(out, 'all');  %now remove: there shouldn't be an out.all
      
      %%% should remove fields that are bad here, and say they are being removed:
      
      
      outnames = fieldnames(out);
      %   fprintf('getting NPx1 fields: dday, ens_num, num_pings\n');
      for fnamei = 1:length(outnames)
         val = getfield(out, outnames{fnamei});
         if (val)
            if (strcmp(outnames{fnamei}, 'ts'))
               % fprintf('getting NPx1 fields: heading, pitch, roll, temperature, soundspeed\n')
            elseif (strcmp(outnames{fnamei}, 'einfo'))
               % fprintf('getting NPx1 diagnostic fields: xflux, yflux, zflux, compass temperature, voltage\n')
            else
               % fprintf('getting field "%s"\n', outnames{fnamei})
            end
         end
      end
      
   end
end


if (cfg.yearbase < 1970)
   fprintf('FATAL: cannot use yearbase %d\n', cfg.yearbase)
   fprintf(' ... must specify yearbase on commandline\n')
   return;
end

params.yearbase = cfg.yearbase;
params.pulse = cfg.pulse;
params.out = out;



% The following are hardwired here, but could be read from
% the file.
params.fileheadlength = 416;
params.profheadlength = 80;


%had to wait for this to get params initialized
if (ischar(file_or_struct))
   file = file_or_struct;

   if (cfg.autosuffix)
      fileroot = file(1:end-4);

      if exist(fileroot) == 2,
         filename = fileroot;
      else
         filename = [fileroot, '.adp'];
         if exist(filename) ~= 2,
            filename = [fileroot, '.ADP'];
         end
      end
   else
      filename = file;
   end
   
   fid = fopen(filename, 'r', 'ieee-le'); % Little-endian data file.
   if fid == -1,
      error(['Can''t open input file: ', file,...
             ', with or without .adp or .ADP']);
   else 
      fclose(fid);
   end
elseif (isstruct(file_or_struct))
   finfo = file_or_struct;
   file       = finfo.fullrawfile;
   nbytes     = finfo.nbytes;             %%%includes checksum
   config     = finfo.config;
   last_index = finfo.last_index;
else
   error('second argument was not an existing file or appropriate structure')
end


%open and close in here
% config tells whether there was BT or not


if (~exist('config', 'var')) % file_or_struct was a file
   [nbytes, last_index, config] = get_config(instclass, params, cfg, file);
end

%reset this if necessary
if ~config.sysconfig.getfilehead
   params.fileheadlength = 0;
end

ilist = get_ilist(cfg, last_index);

%now initialize everything

%%%%%%%%%%%%%%%%%%%%% start of original code %%%%%%%%%%%%%%%%%%%%
NP = length(ilist);  %%1 + floor((stop - start) / step);
NC = config.ncells;
NB = config.nbeams_used;


%% no field for cor
if out.vel, d.vel     = zeros(NC, NB, NP); end
if out.amp, d.amp     = zeros(NC, NB, NP); end
if out.pg, d.pg       = zeros(NC, NB, NP); end
if out.ts,
   d.heading     = zeros(1, NP);
   d.pitch       = zeros(1, NP);
   d.roll        = zeros(1, NP);
   d.soundspeed  = zeros(1, NP); 
   d.temperature = zeros(1, NP);
   d.tr_pressure    = zeros(1, NP);
end                           
                              
if (out.einfo), %compass, power, and noise info
   d.xflux          = zeros(1, NP);
   d.yflux          = zeros(1, NP);
   d.zflux          = zeros(1, NP);
   d.ctemperature   = zeros(1, NP);
   d.voltage        = zeros(1, NP);
   d.noise          = zeros(NB, NP);
end

if (out.bt)
   d.bt.vel = zeros(NB, NP);
   d.bt.pg  = d.bt.vel;
   d.bt.range = d.bt.vel;
   d.bt.status =    NaN*zeros(1,NP);
end

% Always include dday and ens_num.
d.dday = zeros(1, NP);
d.ens_num = zeros(1, NP);
d.num_pings = zeros(1, NP);
d.config = config;


%%%%%%%% now loop through indices and read data

% now loop through ilist
fid = fopen(file, 'r', 'ieee-le');
for ii = 1:length(ilist)

   if (cfg.verbose) ii, end

   fseek(fid, params.fileheadlength, 'bof');
   fseek(fid, nbytes * (ilist(ii) - 1), 'cof');

   [buf, config] = read_buf(instclass, fid, params, config);

   if ischar(buf),
      d.error = buf;
      return
   end
   d.dday(:,ii) = buf.dday;
   d.ens_num(ii) = buf.ens_num;
   d.num_pings(ii) = buf.num_pings;

   if out.ts,
      d.heading(ii) = buf.heading;
      d.pitch(ii)   = buf.pitch;
      d.roll(ii)    = buf.roll;
      d.soundspeed(ii)  = buf.soundspeed;
      d.temperature(ii) = buf.temperature;
      d.tr_pressure(ii) = buf.tr_pressure;
   end
   if (out.einfo), %compass, power, and noise info
      d.xflux(ii)          = buf.xflux;
      d.yflux(ii)          = buf.yflux;
      d.zflux(ii)          = buf.zflux;
      d.ctemperature(ii)   = buf.ctemperature;
      d.voltage(ii)        = buf.voltage;
      d.noise(:,ii)        = buf.noise;
   end

   if (out.bt)
      d.bt.vel(:, ii)      = buf.bt.vel;
      d.bt.range(:, ii)    = buf.bt.range;
      d.bt.pg(:, ii)       = buf.bt.pg;      
      d.bt.status(ii)      = buf.bt.status;
   end
   
   if out.vel, d.vel(:, :, ii)     = buf.vel; end
   if out.amp, d.amp(:, :, ii)     = buf.amp; end
end

%scale bt
if (out.bt)
   d.bt.vel   = d.bt.vel/1000;
   d.bt.range = d.bt.range/100;
end


%%%% add things which will make our lives easier later, or perhaps not.

d.depth =  [d.config.bin1distance + (0:(NC-1)) * d.config.cell]';
d.config.pingtype = 'son';

%take out things which are not useful:
d.config = rmfield(d.config, 'press_xform');


%augment d with some useful information
if (isstr(file_or_struct))
   [fpath, rawfilebase, ext] = fileparts(file);
   d.info.rawfilebase = rawfilebase;
   d.info.fullrawfile = file;
else
   d.info.rawfilebase = finfo.rawfilebase;
   d.info.rawfilebase = finfo.fullrawfile;
end

d.info.ilist = ilist;
d.info.nbytes =     nbytes;  %% includes checksum
d.info.last_index = last_index;
d.info.config =     config;

fclose(fid);
d.error = [];
