function r = load_raw(instclass, file, params, varargin)
% function load_raw(instclass, file, params, ...)
%
% Writes the command (*.cmd) and binary (*.bin) files
% required by ldcodas, starting from ue4 or EF RAW nb logging
% Each call to load_raw processes a single input file.
%                 ** Will change to process whole set.
% It also writes out files with (t x y) in a format suitable
% for refabs input.  The first fix from the ensemble goes to
% *.gps1, the last to *.gps2.
%
% Required arguments:
%   instclass:              instrument class (nb)
%   file: name of RAW
%   params: a structure with fields
%     params.h_align      : Heading alignment in degrees;
%                             RDI sign convention
%     params.interval     : Averaging interval in seconds (default 300)
%     params.ilayer       :  bin0:bin1, reference layer indices
%                             for averaging.  Give the array of
%                             indices, not just the first and last bin.
%     params.geometry     : 'convex' (OS) 'concave' (NB) 
%     params.beamangle    :  30 is default
%     params.velpermute   : permutation of beams (KM adp) default [1 2 3 4]
%     params.yearbase     : yearbase (*** required *** (unlike os)
%     params.add_to_depth : transducer depth (required)  ***(unlike os)***
%
%     params.pingedit     : def:1 (0, do not apply editing)
%     params.use_refavg   : def:1 (0, use simple reference layer average)
%     params.min_pings    : Minimum number of pings for an average;
%                             only relevant at the end of file with
%                             "finish" set to 1 (see below)
%     params.min_pg       : Within the reference layer, use only
%                             the depths with at least this percent good;
%                             if there are not at least 2 such levels,
%                             don't use a reference layer, just calculate
%                             a simple average.
%     params.max_ev       : Error velocity screening criterion, m/s
%     params.use_3_beam   : Not yet implemented; only 4-beam solutions
%                             are used at present
%     params.bigtarget_amp       : 25
%     params.bigtarget_offset    :  5
%     params.bigtarget_avg       :  2
%                                      (See bigtarget.m)
%     params.header         : sontek might need it: from son/read"
%                           % this structure replaces critical information
%     params.header.ncells  %    from the header structure of binary data.
%     params.header.blank   %    (the header structure is already present
%     params.header.cell    %    in data recorder data, but lacking in 
%     params.header.pulse   %    directly logged raw pings.
%
%   >>The first four param fields are required;<<
%   the remainder have defaults.  Read the source to see what
%   they are, or eliminate all uncertainty by specifying all
%   param fields explicitly.
%
%   Other fields will likely be added as additional screening
%   methods are implemented.
%
% Optional named arguments: (keyword, value pairs)
%     "ldcodas_path", bindir: directory to write *.cmd, *.bin
%   "leftover", b         : remaining profiles from last file
%   "finish", num         : 1 to finish, 0 to leave "leftover" for next call
%   "heading_offset", dh  : transducer heading offset in degrees
%   "use_for_heading", str: default 'heading' (from best)
%                         : other options: 'gyro.heading' 
%                         :     'posmv.heading' (NOTE: must add {'posmv'}
%                         :            to 'attstats'
%                         :     'adu2.heading' (NOTE: must add {'adu2'}
%                         :            to 'attstats'
%     "verbose", [0 or 1] : def:0 (1: show raw data -- timeconsuming)
%     "gbindir", path     : path to gbin dirs (default is  '../gbin')
%     "attstats" [cell]   : get attitude statistics (default: none
%                               'attstats', {'adu2'} %ashtech stats
%                               'attstats', {'adu2', 'posmv'} % and posmv
%
%
%  EDITING now exists as a structure of 'params'. undocumented as yet
%
%
% 2000/09/22 EF
% 2001/01/25 JH adding instclass, morphing for NBADCP and gbin;
%                    get nav fields from gbin files
%            


global fid_bin fid_cmd

if (nargin <= 4)
   help(mfilename)
   error('too few variables')
end

fid_cmd = 0;  % initialize so it can serve as a "files are open" flag

fid_log = fopen('load_raw.log','a');
cc = clock;
fprintf(fid_log,'running load_raw on %s at %s\n',...
        file, to_date(cc(1), to_day(cc(1), cc)));
clear cc

%set defaults for varargin structure 
cfg.ldcodas_path = './';
cfg.nav_path = './';     
cfg.att_path = './';
cfg.use_for_heading = 'heading';
cfg.second_set = 0;
cfg.leftover = [];
cfg.finish = 1;
cfg.verbose = 0;
cfg.gbindir = '../gbin';
cfg.attstats = [];
cfg.header = [];
cfg.path_ext = {'time',    'best';   
                'time',    'tim'};

cfg = fillstruct(cfg, varargin, 0);

% rename parameters to match code
ldcodas_path = cfg.ldcodas_path;
nav_path = cfg.nav_path;     %don't write nav yet
leftover = cfg.leftover;
finish = cfg.finish;



% get statistics if requested: have to use path_ext in get_serfields
for ii=1:length(cfg.attstats)
   if (strcmp(cfg.attstats(ii), 'posmv'))
      cfg.path_ext(2 + ii, 1) = cfg.attstats(ii);
      cfg.path_ext(2 + ii, 2) = {'pmv'};
   elseif (strcmp(cfg.attstats(ii), 'adu2'))
      cfg.path_ext(2 + ii, 1) = cfg.attstats(ii);
      cfg.path_ext(2 + ii, 2) = {'adu'};
   else
      fprintf('cannot add attitude statistics for unsupported type "%s"\n',...
              cfg.attstats{ii});
   end
end



% The following parameter structure will become an input, but
% for now it is hardwired here.
%  p.h_bias  = 0;
%  p.h_align =  43.28; %%% Darryl's value for Kaiyo, ky0006L3 later files
%  p.interval = 300;
%  p.ilayer = 2:40;
p.h_bias = 0;      %hardwire here.  angle is dealt with in h_align
p.yearbase = 0;     % for string-formatting to_date correctly
p.min_pg = 25;
p.max_ev = 1;
p.use_3_beam = 0;    % 3-beam not yet implemented
p.min_pings = 10;
p.add_to_depth = 0;
p.bigtarget_amp = 25;
p.bigtarget_offset = 5;
p.bigtarget_avg = 2;
p.pingedit = 1;          %default is to apply pingedit editing
p.use_refavg = 1;         %default is to use refavg reference layer (else simple)

%set up editing parameters
p.editcfg.rl_range = [2 20];
p.editcfg.fieldname = 'cuv';
p.editcfg.cor_cutoff = [];
p.editcfg.sbr_bincheck = [];
p.editcfg.sbr_binreject = [];
p.editcfg.sbr_corcutoff = [];
p.editcfg.err_cutoff = .5;
p.editcfg.errstd_cutoff = 1;
p.editcfg.weakprof_binsreq = round(.5* (diff(p.editcfg.rl_range)+1));
p.editcfg.weakprof_testrange = p.editcfg.rl_range;
p.editcfg.refavg_bincutoff = .5;   %toss things outside of this limit
p.editcfg.refavg_binstddev = 3;

if nargin < 4 | ~ischar(file) | ~isstruct(params),
   help load_raw
   fprintf('At least 3 arguments are required:\n')
   fprintf('an instrument class, a filename and a structure.')
   error('too few input arguments ')
end

if (~isfield(params, 'header') | ...
    ~isfield(params, 'h_align') | ...
    ~isfield(params, 'interval') | ...
    ~isfield(params, 'ilayer') | ...
    ~isfield(params, 'add_to_depth') | ...
    ~isfield(params, 'yearbase'));
   help son/load_raw
   fprintf('required Parameter fields: \n');
   error('header, h_align, interval,  ilayer, yearbase, add_to_depth ')
end

% Transfer input parameters, overriding defaults.
header = params.header;
params = rmfield(params, 'header');
fn = fieldnames(params);
for ii = 1:length(fn)
   fname = fn{ii};
   p = setfield(p, fname, getfield(params, fname));
end

[a_path, rawfilebase, a_ext] = fileparts(file);
cmdfile = fullfile(ldcodas_path, [rawfilebase  '.cmd']);
binfile = fullfile(ldcodas_path, [rawfilebase  '.bin']);
nav1file = fullfile(nav_path, [rawfilebase  '.gps1']);  
nav2file = fullfile(nav_path, [rawfilebase  '.gps2']);  
attfile =  fullfile(nav_path, [rawfilebase  '.attstats.bin']);  

if (   (exist(nav1file, 'file') == 2 ) | ...
       (exist(nav2file, 'file') == 2 ) | ...
       (exist(cmdfile, 'file') == 2 ) | ...
       (exist(binfile, 'file') == 2 ) )
   fprintf('at least one of the following files already exists:\n')
   fprintf('%s, %s, %s, %s.  Not processing.\n',...
           nav1file, nav2file, cmdfile, binfile)
   r = [];
   return
end
      

% Read the first and last profile to get the configuration, get start and 
% end dates, initialize the structure  for the average.  This is very fast.
% AND return the fid
a = read(instclass, file,  'yearbase', p.yearbase, 'ends', 1,...
         'header', header,...
         'vars', 'all'); %no nav in nb or son
finfo = a.info; %pass this in to subsequent reads; 

time_path = fullfile(cfg.gbindir, 'time');
timbin_filename = fullfile(time_path,  sprintf('%s.tim.gbin', rawfilebase));
tim_bindata = read_bin(timbin_filename);
if (isemptys(tim_bindata))
   r = [];
   return
end

   
p.dday = tim_bindata(2,:);  %best time


% reset leftover to [] if we're starting a new configuration segment
if (isemptys(leftover))
   fprintf('leftover is empty\n')
end
if (a.ens_num(1) == 1)
   leftover = [];
   fprintf('starting a new configuration segment: resetting leftover to []\n')
end

if ~(isemptys(leftover))
   if (p.dday(1) - leftover.dday(end) > .5*(p.interval/86400))
      leftover = [];
   end
end


fprintf('calculating index ranges\n')
%% NOTE: these are index ranges to read; 
%% indices less than 1 refer to leftovers (already read)
%% read will ignore those
[index_ranges, leftover_irange] = find_iranges(leftover,  p); 
NP = size(index_ranges, 2);


% note on the proliferation of "depth" variables
% depth: temporary variable just to fill the structure a.depth
% a.depth: depth of bins for average
% r.depth (below): raw depth
% override a.config.tr_depth (starts as 0 in @son/read.m)
a.config.tr_depth = p.add_to_depth;
a.depth = round(mk_depth(a, a.config.tr_depth)); 
NC = length(a.depth);  %from a.config.ncells (i.e. number of cells)
config1 = mk_config1(a);
c1_index =  config1_ind;
config1(c1_index.avg_interval)   = p.interval;
config1(c1_index.hd_offset)      = p.h_align;
config1(c1_index.heading_bias)   = p.h_bias;
access = mk_access(a);    % Doesn't change.

%get all the auxilliary data for this raw file
% ("data" can simply be data.info.rawfilenames and data.info.ilist)
%must replace ilist!!!
a.info.ilist = [1:length(p.dday)];
aux_struct = get_serfields(a, 'gbindir', cfg.gbindir, ...
                              'path_ext', cfg.path_ext);


if (isemptys(aux_struct))
   fprintf('could not complete auxilliary (nav) fill\n')
   r = [];   return;
end
aux_fnames = fieldnames(aux_struct);


if (~ismember('adu2', cfg.attstats))
   noashtech = 1;
else
   noashtech = 0;
end

if (~ismember('posmv', cfg.attstats))
   noashtech = 1;
else
   noashtech = 0;
end


if (isemptys(leftover))
   leftoverpings = 0;
else
   leftoverpings = length(leftover.dday);
end

fprintf(fid_log, '(%d pings from leftover) + (%d pings from %s) = %d\n', ...
        leftoverpings, length(p.dday),   rawfilebase,...
        leftoverpings + length(p.dday));


%%   diagnostics -------------------
if (isemptys(leftover))
   alldday = p.dday;
else
   alldday = [leftover.dday  p.dday];
end
sec_res = mstdgap(86400*diff(p.dday));
%% print indices and time duration (seconds)
fprintf('indices:   [start   end]        duration\n')
for ip=1:NP
   irange = [index_ranges(1,ip), index_ranges(2,ip)];
   if (irange(2) < 1)
      secs = NaN;
   else
      irange(1) = max(irange(1),1);
      secs = floor(86400*(p.dday(irange(2)) - p.dday(irange(1))));
   end
   fprintf('indices %6d %6d %6d\n', irange(1), irange(2), secs)
   if (secs > p.interval + sec_res)
      warning(sprintf('averaging interval at ip = %d is too big: %f secs\n',...
                      ip, secs))
      fprintf(fid_log, ...
              'averaging interval at ip = %d is too big: %f secs\n',...
              ip, secs)
      %keyboard
   end
end

if (~isemptys(leftover_irange))
   irange = [leftover_irange(1), leftover_irange(2)];
   fprintf('leftover: %6d %6d %6d\n',   irange(1), irange(2),...
        floor(86400*(alldday(irange(2)) - alldday(irange(1)))))
end


% read and average the full profiles
for ip = 1:NP

   tic

   ilist = [index_ranges(1, ip) : index_ranges(2, ip)];
   
   if (max(ilist) < 1) %the leftover passed in was the whole ensemble
      r = [];
   else
      r = read(instclass, finfo,  'yearbase',  p.yearbase, ...
               'header', header,...
               'ilist', ilist , 'vars', 'all');
   end

   if (~isemptys(r))
   
      %% this extra stuff is necessary for NB data
      r.depth = mk_depth(r); %a.config.tr_depth set to 0 in read.m
      r.depth = round(r.depth + p.add_to_depth);
      
      
      fprintf('%d out of %d (%s) \n', ip, NP, file)
      
      
      aux = cutadcp(aux_struct, ilist(ilist>0));
      
      %assign aux fields to r: (eg. aduheading, pitch, roll,...)
      
      for fnamei = 1:length(aux_fnames);
         fname = aux_fnames{fnamei};
         r = setfield(r, fname, getfield(aux, fname));
      end

      %% choose correct heading here

      if (~strcmp(cfg.use_for_heading, 'heading'))
         inst_struct = getfield(r, cfg.use_for_heading);  %eg "gyro"
         r = setfield(r, 'heading', getfield(inst_struct, 'heading'));
      end
   end
      
   %now that 'r' is set up, cat it to 'leftover' if appropriate
   if ip == 1 & ~isemptys(leftover)
      r = catadcp(leftover, r);
   end

   
   fprintf(fid_log, ...
           '%d out of %d, indices [%d %d], dd: %10.7f - %10.7f (%s %s)\n',...
           ip, NP, index_ranges(1, ip), index_ranges(2, ip), ...
           r.dday([1 end]), to_date(p.yearbase, r.dday(1)), ...
           to_date(p.yearbase, r.dday(end)));

    a.num_pings = length(r.dday);

    
    if ~noashtech
       if (~isemptys(leftover))   
          %get ashtech statistics
          [attstats, att_rownames] = att_stats(r.adu2);
       end
   end
   
   p.verbose = cfg.verbose;
   a = avg(instclass, r, a, p);
   
   % all the structures:
   ancil1 = mk_ancil1(r, 1);
   ancil2 = mk_ancil2(r, 1);
   

   nav = ones(4,1) * 1e38;   
   %NAVIGATION structure wants [latitude longitude]
   %but nav structure has [time longitude latitude]
   
   % official codas timestamp is at the END of the averaging interval
   a.dday = r.dday(end);
   
   if ip == 1,
      fid_nav1 = fopen(nav1file, 'w');
      fid_nav2 = fopen(nav2file, 'w');
      fid_cmd = fopen(cmdfile, 'w');
      fid_bin = fopen(binfile, 'w');
      if fid_nav1 == -1, error(['Can''t open ', nav1file]); end
      if fid_nav2 == -1, error(['Can''t open ', nav2file]); end
      if fid_cmd == -1, error(['Can''t open ', cmdfile]); end
      if fid_bin == -1, error(['Can''t open ', binfile]); end
      fprintf(fid_cmd, 'binary_file: %s\n', binfile);
   end
   
   if ip == 1 | (rem(ip, 400) == 1 &  (NP - ip) > 10),
      fprintf(fid_cmd, 'new_block\n');
      fprintf(fid_cmd, 'dp_mask: 1\n');
      
      bin_float('DEPTH', a.depth);
      bin_double('CONFIGURATION_1', config1);
   end
   fprintf(fid_cmd, ['new_profile: ', to_date(a.config.yearbase, a.dday),...
                     '  /* %d %d */\n'], r.ens_num([1 end]));
   fprintf(fid_cmd, 'depth_range: %d %d\n', round(a.depth([1 end])));
   bin_double('ACCESS_VARIABLES', access);
   bin_double('ANCILLARY_1', ancil1);
   bin_double('ANCILLARY_2', ancil2);
   bin_double('BOTTOM_TRACK', a.bt);
   bin_double('NAVIGATION', nav);
   bin_float('U', a.vel(:, 1));
   bin_float('V', a.vel(:, 2));
   bin_float('W', a.vel(:, 3));
   bin_float('ERROR_VEL', a.vel(:, 4));
   bin_ubyte('AMP_SOUND_SCAT', round(mean(a.amp, 2)));
   bin_ubyte('RAW_AMP', a.amp);
   bin_ubyte('PERCENT_GOOD', a.pg(:, 4));
   bin_ubyte('PERCENT_3_BEAM', a.pg(:, 3));
   bin_ubyte('SPECTRAL_WIDTH', round(mean(a.sw, 2)));
   bin_ubyte('RAW_SPECTRAL_WIDTH', a.sw);
   %% Maybe we should call it something else, but this will do
   %% for now; it is not correlation, either.
   bin_ubyte('PROFILE_FLAGS', zeros(1, NC));
   % .....
   
   %make sure there are values for navigation 
   %txy1 is the first good fix of the ensemble
   posi = min(find(~isnan(sum(r.nav.txy1))));
   if (~isemptys(posi))
      fprintf(fid_nav1, '%12.7f %12.7f %12.7f\n', r.nav.txy1(1, 1),...
              r.nav.txy1(2, posi), r.nav.txy1(3, posi));
   else
      fprintf(fid_nav1, '%%%12.7f \n', r.nav.txy1(1, 1));
   end
   %txy2 is the last good fix of the ensemble
   posi = max(find(~isnan(sum(r.nav.txy2))));
   if (~isemptys(posi))
      fprintf(fid_nav2, '%12.7f %12.7f %12.7f\n', r.nav.txy1(1, end),...
              r.nav.txy1(2, posi), r.nav.txy1(3, posi));
   else
      fprintf(fid_nav2, '%%%12.7f\n', r.nav.txy1(1, end));
   end
   
   if (~noashtech)
      if (ip == 1)
         attstruct = write_bin(attfile, attstats, 'att_stats', att_rownames); 
      else
         attstruct = write_bin(attstruct, attstats);
      end
   end

   toc
end


%close the att file
if (~noashtech)
   if (exist('attstruct'))
      fclose(attstruct.fid);
      fprintf('closing the att bin file %s\n', attfile)
   else
      if (isemptys(index_ranges))
         fprintf(fid_log, 'No full profiles\n');
      end
      fprintf(fid_log, 'NOTE: no ashtech file to close\n');
   end
end


if finish,
   fprintf(fid_log, 'finishing: r=[];\n');
   r = [];
else
   %we already read, averaged,  and wrote info for all full ensembles that 
   % existed.  Now 
   
   if (isemptys(leftover_irange))
      r = [];
      fprintf(fid_log, 'no leftover\n');
   else
      ilist = [leftover_irange(1):leftover_irange(end)];
      
      r = read(instclass, finfo,  'yearbase',  p.yearbase, ...
               'header', header,...
               'ilist', ilist , 'vars', 'all');
      
      if (isemptys(r.dday))
         fprintf('r.dday is empty: in load_raw now.  check it out')
         keyboard
      end
      
      %% this extra stuff is necessary for NB data
      r.depth = mk_depth(r); %a.config.tr_depth set to 0 in read.m
      r.depth = round(r.depth + p.add_to_depth);
      fprintf('%d out of %d (%s) \n', ip, NP, file)
      
      aux = cutadcp(aux_struct, ilist);
   
      for fnamei = 1:length(aux_fnames);
         fname = aux_fnames{fnamei};
         r = setfield(r, fname, getfield(aux, fname));
      end
      
      %now that 'r' is set up, cat it to 'leftover' if appropriate
      if (NP == 0) & ~isemptys(leftover)
         r = catadcp(leftover, r);
         fprintf(fid_log, 'there WAS a leftover from the previous file,  \n');
         fprintf(fid_log, 'but the new file only had %d pings: passing the \n');
         fprintf(fid_log, 'leftover through as a bigger leftover\n');
      end
   
   
      fprintf(fid_log, 'leftover, ilist = [%d  %d]\n', ilist(1), ilist(end));
      fprintf(fid_log, 'leftover dday range is %10.7f - %10.7f (%s - %s)\n', ...
              r.dday([1 end]), to_date(r.config.yearbase,r.dday(1)), ...
              to_date(r.config.yearbase,r.dday(end)));

      if (fid_cmd)
         fprintf(fid_cmd, '/* leftover: %d %d */\n', r.ens_num([1 end]));
      else
         fid_cmd = fopen(cmdfile, 'a');
         fprintf('leftover generated from this read.\n')
         fclose(fid_cmd);
         fid_cmd = 0;
      end
   end
end


if fid_cmd
   fclose(fid_nav1);      %don't write nav now
   fclose(fid_nav2);      %don't write nav now
   fclose(fid_cmd);
   fclose(fid_bin);
end

fclose(fid_log);
%====================================================

function [ir, lr] =  find_iranges( leftover,  p);
% ir is 2xN index ranges for profile averages
% the last index range may be incomplete (gets used as lefover)

gapfrac = .5;  %fraction of delta to allow between leftover(end) and dday(1)


yearbase = p.yearbase;
interval = p.interval;

delta = interval/86400;

% use all days together
if (isemptys(leftover) | ((p.dday(1) - leftover.dday(end)) > gapfrac))
   alldday = p.dday;
   first_ipday = 1;
else
   alldday = [leftover.dday  p.dday];
   first_ipday = length(leftover.dday) + 1;
end

% make sure we include leftovers in the allocation
d0 = alldday(1) : delta : alldday(end);
% but get rid of first one if it is before p.dday
n = length(d0);
ir = zeros(2,n);

% lock in the starting indices
for ii = 1:n
   ir(1, ii) = min(find(alldday >= d0(ii)));
end
if (ir(1,1) ~= 1)
   fprintf('something went wrong with determining ensemble indices\n')
   keyboard
   paused
end

ir(1,:) = ir(1,:) - first_ipday + 1;
ir(2,:) = [ir(1,2:end)-1, length(p.dday)];



%now move the last indices to lr if it is not a whole ensemble
lr = [];
if (    (86400*(alldday(ir(2,end)) - alldday(ir(1,end)))) < interval)
   lr = ir(:,end); %lefover range
   ir(:,end)=[];   %index range
end

 



%-----------------------------------------------------------


