function [xyze, xyz_mat] = son_beam_xyz(vel,  varargin);

% function [xyze, xyz_mat] = son_beam_xyz(vel, 'angle', 30, 'orientation', 'down')
%
%      vel can be: beam velocity from adp.m,
%                  a vector of beam coordinates
%                  a matrix, the columns of which are beam coord vectors
%
%      'angle' and 'orientation' argument pairs are optional;
%                defaults are as above.  orientation must be 'up'
%                or 'down'.
%      'direction', 'inverse' converts from xyze to beam
%                   'normal' (default) from beam to xyze
%
%      'instrument', 'son' (default: use generic transformation matrix)
%                    'kmson' (only for Kilo Moana Sontek (overrides angle)

% JH 2002/09/11 changed to fillstruct; added 'kmson' option


cfg.angle = 30;
cfg.direction = 'normal';
cfg.orientation = 'down';
cfg.instrument = 'son';

cfg = fillstruct(cfg, varargin, 0);

%now deal with inputs
beam_angle = cfg.angle*pi/180;
%start with defaults; change if necessary
orientation_up = 0;
if (strcmp(cfg.orientation, 'up'))
   orientation_up = 1;
elseif (~strcmp(cfg.orientation, 'down'))
   help(mfilename)
   error(['Invalid value of orientation: ', cfg.orientation]);   
end
%start with defaults; change if necessary
xyze_to_beam = 0;
if strcmp(cfg.direction, 'inverse')
   xyze_to_beam = 1;
elseif ~strcmp(cfg.direction, 'normal')
   help(mfilename)
   error('Invalid direction; must be normal or inverse');
end


if strcmp(cfg.instrument, 'kmson')
   xyz_mat = [-0.9487   0.9568       0        0;
                0         0      0.9527   -0.9504;
              -0.2970  -0.2904  -0.2929   -0.2948;
               0.2970   0.2904  -0.2929   -0.2948];
else
   cc = 1/(4*cos(beam_angle));
   ss = 1/(2*sin(beam_angle));
   xyz_mat = [  ss,  0, -ss,   0 ; ...     %xyz_mat * [vel1    = [u v w e]
                0 , ss,   0, -ss ; ...     %           vel2     
                -cc, -cc, -cc, -cc; ...     %           vel3     
                cc, -cc,  cc, -cc ];       %           vel4]    
end


if orientation_up,
   xyz_mat([2 3],:) = -xyz_mat([2 3],:); % reverse y and z
end

if xyze_to_beam,
   xyz_mat = inv(xyz_mat);
end

sv = size(vel);
ndims = length(sv);
if ndims > 3,
   error('Too many dimensions in the first input argument')
elseif ndims == 3
   xyze = zeros(sv);
   for ii = 1:sv(3)
      xyze(:,:,ii) = (xyz_mat * squeeze(vel(:,:,ii)).').';
   end
   return
else % has 2 dimensions even if it is empty
   if isemptys(vel),
      xyze = [];
      return
   end
   if sv(1) == 1, vel = vel(:); end
   if sv(1) ~= 4,
      error('First dimension of vel must be 4 (for 4 beams).')
   end
   xyze = xyz_mat * vel;
end

