function cfg = get_uhdascfg(varargin)
% cfg = get_uhdascfg(varargin)
% Use this program to initialize a structure for use with uhdas processing
% (updating gbin files, extracting navigated data, and processing). 
%
% Defaults are for underway logging; all fields are initialzed
% If a config file is specified, fields are updated with those values
% Values from 'name', [value] pairs are updated last.
%
% Use 'cfgbase' to specify the prefix for the [cfgbase]_cfg.m file
% Use 'cfgpath' to specify a full path to the [cfgbase]_cfg.m file
% Use 'uhdas_dir' as a shortcut to uhdas_dir/raw/config/[cfgbase]_cfg.m 
%========================================================================
%
% varargin:
%  name                         what                       default
%
% silent             supress useful messages                 0
%  ---------  instrument information -------
% instname             'son125', 'os38', 'nb150'           'os38'
% instclass            son, os, nb, bb                        os
% pingtype             'son', 'nb', 'bb',
%                         (if os: 'first', 'second')        'first'
%
%  --------- locations and paths -----------
%
% 'uhdas_dir'        path to directory with               '/home/adcp/cruise'
%                          raw/, rbin/ gbin/, etc
%
% 'cfgbase'          prefix for config files                [] (use defaults)
%                    [cfgbase]_cfg.m (for path info)
%                    [cfgbase]_proc.m (for params)
%                    [cfgbase]_disp.m (for display)
%
% 'cfgpath'          directory with [cfgbase]_cfg.m        /home/adcp/config
%
%  'rawdir'          holds raw and serial acquisition dirs [uhdas_dir]/raw
%  'rbindir'         holds rbin dirs                       [uhdas_dir]/rbin
%  'gbindirbase'     holds "gbin" dir                      [uhdas_dir]/gbin
%  'gbindir'         holes gbin dirs                [uhdas_dir]/gbin/instname 
%  'logfpath'        dir for messages generated      (current working directory)
%                      during gbin or processing
%
% ---- for acquiring navigated data --------
% 'ddrange'        select time range this way:              default=600
%                  []            (choose by clicking)      
%                  scalar        (interval length:
%                                 previous N seconds) 
%                  1x2 vector     [startday endday]
%
% 'interval'      if ddrange == [], select this way:
%                 scalar        click once, get N secs       600
%                 []            click twice for ddrange
%
% 'showfig'             show a diagnostic figure              0
%
% 'allparams'    structure with allparams.instname.pingtype.field
%                   (if present, can use get_params to extract)
% 'params'        structure for specific instrument fields: 
%                  eg. Kilo Moana example OS
%
%                  params.yearbase = (current year)
%                  params.ducer_depth = 7
%                  params.h_align = -45
%                  params.beamangle = [];              (required for nb)
%                                          
%                  params.editcfg                          (optional)
%                                                          editcfg is from  get_pedefaults
%                                                          and can be used to override
%                                                          defaults from get_pedefaults
%
%
% ----- making gbin files ----- ("help mk_gbin")
%
% 'whichones'              'all', 'last', 'none'    'none'
% 'redo'                redo gbin files 0 or 1        1
%
%  path_ext        gbin files to make             {'time','best';
%                                                  'time','tim'};  
%  msg_info        directory and rbin suffix
%                                                {'posmv', 'gps',  'gps'; 
%                                                 'gyro',   'hdg',  'hdg';    
%                                                 'adu2',   'adu',  'adu';    
%                                                 'posmv',  'pmv',  'pmv'};   
%  best.rules          make gbin 'best' with these fields:
%                                       ps','heading','pitch','roll','heave'};
%  best.gps_rule       position rule                       {'pcode', 'gps'}; 
%  best.heading_rule   heading rule                        {'gyro',  'hdg'};
%  best.pitch_rule     pitch rule (not used)               {'posmv', 'pmv'};
%  best.roll_rule      roll rule (not used)                {'posmv', 'pmv'};
%  best.heave_rule     heave rule (not used)               {'posmv', 'pmv'};
%
% -----  plots -------------
%   figdir         output directory                  '/home/adcp/www/figures'
%   cruisename     for labels                         'adcp'
%   otherheading   for lastens                       {}  % use gyro
    
% JH 2004/01/12
%
% If a new parameter is going to be added to [cfgbase]_cfg.m or 
% [cfgbase]_proc.m  it must be in this program before get_uhdascfg will 
% successfully update fields using those programs.  

silent = 0;

cfg.verbose  = 0;
cfg.debug    = 0;
% in general
cfg.instname = 'os38';
cfg.instclass = os;
cfg.pingtype = 'first';

cfg.uhdas_dir = '/home/adcp/cruise';  
cfg.cfgpath = '/home/adcp/config';
cfg.cfgbase = 'cruise';
%% put these in uhdas_dir if specified
cfg.rawdir =      'raw';
cfg.rbindir =     'rbin';
cfg.gbindirbase = 'gbin';
cfg.gbindir =     fullfile(cfg.gbindirbase, cfg.instname);
cfg.logfpath =    '';
cfg.gbinlogfile = 'mk_gbin.log';

% for getsome_adcp
cfg.ddrange = [];
cfg.interval = 300;
cfg.showfig = 0;
cc=clock;
cfg.params.yearbase = cc(1);  % os
cfg.params.ducer_depth = 7;  %
cfg.params.h_align = 45;    %
cfg.beamangle = [];             % beam angle
cfg.allparams = [];          % could fill it from [cfgbase]_cfg.m


% for update_gbin
cfg.whichones = 'none';
cfg.redo = 1;
cfg.msg_info=   {'pcode', 'gps',  'gps';   %see help mk_gbin
                 'gyro',   'hdg',  'hdg';    
                 'adu2',   'adu',  'adu';    
                 'adu2',   'adu',  'pgs';    
                 'posmv',  'pmv',  'pmv';   
                 'posmv',  'pmv',  'gps'};   

cfg.path_ext =  {'time',    'best';    %  default   gbin files to load
                 'time',    'tim'};      %  default

for sernum = 1:length(cfg.msg_info(:,1))
   cfg.path_ext{2 + sernum,1} = cfg.msg_info{sernum, 1};
   cfg.path_ext{2 + sernum,2} = cfg.msg_info{sernum, 2};
end


cfg.best.rules = {'gps','heading','pitch','roll','heave'};
cfg.best.gps_rule       =     {'pcode', 'gps'}; 
cfg.best.heading_rule   =     {'gyro',  'hdg'};
cfg.best.pitch_rule     =     {'posmv', 'pmv'};
cfg.best.roll_rule      =     {'posmv', 'pmv'};
cfg.best.heave_rule     =     {'posmv', 'pmv'};

%  plots
cfg.figdir  =     '/home/adcp/www/figures';
cfg.cruiseid = 'ADCP_cruise'; %title
cfg.otherheading = {};


%%% oh this is ugly!
%%% replace instrument class with a string so varargin os all a character arrays
for ii=1:length(varargin) 
   if (isa(varargin{ii}, 'nb') | ...
       isa(varargin{ii}, 'bb') | ...
       isa(varargin{ii}, 'son') | ...
       isa(varargin{ii}, 'os'))
      varargin{ii} = inst2str(varargin{ii});
      break
   end
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% use varargin 'silent' to set verbosity
%            then remove 'silent' from varargin
hits =  ismember('silent', varargin);
if length(find(hits)) == 1
   silent = 0
   varargin([find(hits), find(hits)+1]) = [];
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% use varargin 'uhdas_dir' to set raw, rbin, etc
%            then remove uhdas_dir from varargin
%% get arguments from varargin; remove them. 
hits =  ismember('uhdas_dir', varargin);
if length(find(hits)) == 1
   cfg.uhdas_dir = varargin{find(hits) + 1};
   varargin([find(hits), find(hits)+1]) = [];
  
   %% update path 
   cfg.rawdir =      fullfile(cfg.uhdas_dir,    cfg.rawdir);
   cfg.rbindir =     fullfile(cfg.uhdas_dir,    cfg.rbindir);
   cfg.gbindirbase = fullfile(cfg.uhdas_dir,    cfg.gbindirbase);
   cfg.gbindir =     fullfile(cfg.uhdas_dir,    cfg.gbindir);
%   cfg.logfpath =    fullfile(cfg.uhdas_dir,    cfg.logfpath);
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% use varargin 'instname', 'pingtype', and 'instclass'
%            then remove those fields from varargin

hits =  ismember(varargin, 'instclass');
if length(find(hits)) == 1
   cfg.instclass = str2inst(varargin{find(hits) + 1});
   varargin([find(hits), find(hits)+1]) = [];
end

hits =  ismember(varargin, 'instname');
if length(find(hits)) == 1
   cfg.instname = varargin{find(hits) + 1};
   varargin([find(hits), find(hits)+1]) = [];
end

hits =  ismember(varargin, 'pingtype');
if length(find(hits)) == 1
   cfg.pingtype = varargin{find(hits) + 1};
   varargin([find(hits), find(hits)+1]) = [];
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% use varargin 'cfgpath' and 'cfgbase' to find config file 
%            then remove those fields from varargin
% run the config files
% then override with any other varargin 


%-------get path and param info -------------------------
%% (1)  get arguments from varargin; remove them.
%get cfgbase match
hits =  ismember(varargin, 'cfgbase');
if length(find(hits)) == 1
   cfg.cfgbase = varargin{find(hits) + 1};
   varargin([find(hits), find(hits)+1]) = [];
end
%get cfgpath match
hits =  ismember(varargin, 'cfgpath');
if length(find(hits)) == 1
   cfg.cfgpath = varargin{find(hits) + 1};
   varargin([find(hits), find(hits)+1]) = [];
end


%%  (3) now use heirarchy to fill in cfg_path
if isemptys(cfg.cfgpath) 
   cfg.cfgpath = fullfile('/home', 'adcp', 'cruise');
else
   % use it
end
if ~exist(cfg.cfgpath,'dir')
   if ~silent
      fprintf('path to config files "%s" does not exist:\n', cfg.cfgpath)
      fprintf('using with defaults\n\n')
   end
else

   % override some directories and param info
   startdir = pwd;
   chdir(cfg.cfgpath)
   if ~silent
      fprintf('\ngetting config files from "%s"\n\n', pwd)
   end
   
   
   cfgfile =   [cfg.cfgbase,  '_cfg.m'];
   paramfile = [cfg.cfgbase, '_proc.m'];
   dispfile =  [cfg.cfgbase, '_disp.m'];
   
   if ~exist(cfgfile,'file')
      if ~silent
         fprintf('no configuration file "%s" found\n', cfgfile);
         fprintf('using defaults\n\n')
      end
   else
      if ~silent
         fprintf('running "%s"\n', cfgfile)
      end
      cfgfile(end-1:end)=[];
      eval(cfgfile)
   end
   
   if ~exist(paramfile,'file')
      if ~silent
         fprintf('no proc file "%s" found\n', paramfile);
         fprintf('using defaults\n\n')
      end
   else
      if ~silent
         fprintf('running "%s"\n', paramfile)
      end
      paramfile(end-1:end)=[];
      eval(paramfile)
   end
   
   
   if ~exist(dispfile,'file')
      if ~silent
         fprintf('no display file "%s" found\n', dispfile);
         fprintf('using defaults\n\n')
      end
   else
      if ~silent
         fprintf('running "%s"\n', dispfile)
      end
      dispfile(end-1:end)=[];
      eval(dispfile)
   end

   chdir(startdir)
end


if exist('best');          cfg.best             = best;          end
if exist('cruiseid');      cfg.cruiseid         = cruiseid;      end
if exist('otherheading');  cfg.otherheading     = otherheading;      end
if exist('figpath');       cfg.figpath          = figpath;       end
if exist('rawdir');        cfg.rawdir           = rawdir;        end
if exist('rbindir');       cfg.rbindir          = rbindir;       end
if exist('gbindirbase');   cfg.gbindirbase      = gbindirbase;   end
if exist('logfpath');      cfg.logfpath         = logfpath;      end
if exist('msg_info');      cfg.msg_info         = msg_info;      
   cfg.path_ext =  {'time',    'best';    %  default   gbin files to load
                    'time',    'tim'};      %  default
   for sernum = 1:length(cfg.msg_info(:,1))
      cfg.path_ext{2 + sernum,1} = cfg.msg_info{sernum, 1};
      cfg.path_ext{2 + sernum,2} = cfg.msg_info{sernum, 2};
   end
end
if exist('rawdir');        cfg.rawdir           = rawdir;        end
if exist('uhdas_dir');     cfg.uhdas_dir        = uhdas_dir;     end
if exist('allparams');     cfg.allparams        = allparams;     end

  
cfg.gbindir = fullfile(cfg.gbindirbase,cfg.instname);   

cfg.params = get_params(cfg.allparams, cfg.instname, cfg.pingtype);
if exist('yearbase');  cfg.params.yearbase = yearbase;  end 
   
if isfield(cfg.params,'yearbase')
   cfg.yearbase = cfg.params.yearbase;
end

cfg = fillstruct(cfg, varargin);

