function [dd, config, gpsinfo] = getsome_adcp(cfg, varargin)
% [dd, config, gpsinfo] = getsome_adcp(cfg, varargin)
%
% extract raw navigated data from specified instrument
% can be used for looking at data during logging (default, last 10 minutes)
%     or to access some at a specified time of the cruise 
% (this is just a wrapper for mk_gbin + get_rndata)
%
% usage:
% cfg = get_uhdascfg(varargin)  % help get_uhdascfg for details
% [dd,config] = getsome_adcp(cfg);                         % get 300s
% [dd,config] = getsome_adcp(cfg,'interval',[],'step',50); % subsample
%
% NOTE (can override any parameters in cfg with name value pairs)
%
% varargin:
%  name                         what                       default
%
%  ---------  instrument information -------
% instname             'son125', 'os38', (ship-specific)     'os38'
% instclass            son, os, nb, bb                        os
% pingtype             'son', 'nb', 'bb',
%                         (if os: 'first', 'second')        'first'
%
%  --------- locations and paths -----------
%
% 'uhdas_dir'          path to directory with               '/home/adcp/cruise'
%                          raw/, rbin/ gbin/, etc
%
%  'rawdir'          holds raw and serial acquisition dirs [uhdas_dir]/raw
%  'gbindirbase'     holds "gbin" dir                      [uhdas_dir]/gbin
%
% ---- for acquiring navigated data --------
% 'ddrange'        select time range this way: 
%                  []            (choose by clicking on map)
%                  scalar        (get previous N seconds) default=600
%                  1x2 vector     [startday endday]
%
% 'interval'      if ddrange == [], select this way:
%                 scalar        click once, get N secs       600
%                 []            click twice for ddrange
% 'step'          integer       retrieve every Nth ping        1
%
% 'showfig'             show a diagnostic figure              0
%
% 'params'        structure for specific instrument fields: 
%                                eg. Kilo Moana example OS
%
%
%                  params.yearbase = (current year)
%                  params.ducer_depth = 7
%                  params.h_align = -45
%                  params.beamangle = [];        (required for nb)
%


if (~isstruct(cfg))
   help(mfilename)
   fprintf('error. first argumant not a structure:.\n')
   return
end

cfg.step = 1;
cfg = fillstruct(cfg,varargin);
dd=[]; config = [];
if ~isfield(cfg.params, 'beamangle')
   cfg.params.beamangle = [];
end

   
globstr = sprintf('%s/time/*.tim.gbin', cfg.gbindir)
filelist = ...
    dirs(globstr,'fullfile',1);
if isemptys(filelist)
   fprintf('no files found with %s\n', globstr)
   return
else
   trlist = tr2ilist(filelist,'rowname', 'bestdday');
end

% click on map
if isemptys (cfg.ddrange)
   [ddrange, gpsinfo] = mk_rbinmap(cfg);
   cfg.ddrange = ddrange(:)';
elseif length(cfg.ddrange) == 1
   cc = clock; 
   now_dday = to_day(cfg.params.yearbase, cc);
   cfg.ddrange = now_dday + [-abs(cfg.ddrange)/86400 0] ;
else
   if length(cfg.ddrange) ~= 2
      fprintf('''ddrange'' must be empty, scalar, or 2xN\n')
      help(mfilename)
      return
   end
end


% get data; steal logic from load_raw to choose first or secondset
if isfield(cfg, 'msg_info')
   serfields = cfg.path_ext;
end


[dd, config] =  get_rndata(...
   cfg.instclass, ...
   cfg.ddrange,...
   'beamangle', cfg.params.beamangle,...
   'gbindir', cfg.gbindir,...
   'rawfpath',fullfile(cfg.uhdas_dir,'raw'),...
   'pingtype', cfg.pingtype,...
   'pingdir', cfg.instname,...
   'h_align',  cfg.params.h_align, ...
   'yearbase', cfg.params.yearbase,...
   'ducer_depth',  cfg.params.ducer_depth,...
   'path_ext', serfields,...
   'step', cfg.step,...
   'add_aux', 1,...
   'return3d',1,...
   'convex', 0);


if cfg.showfig & ~isempty(dd)

   figure(333)

   subplot(231)
   plot(dd.corr_dday, dd.umeas(2,:),'r')
   title('umeas')
   
   subplot(232)
   plot(dd.corr_dday, dd.vmeas(2,:),'b')
   title('vmeas')
 
   subplot(233)
   plot(dd.corr_dday, unwrap_lon(dd.heading),'k.')
   hold on
   plot(dd.corr_dday, dd.cog,'m+')
   title('heading (k), cog(m)')
   
   
   subplot(234)
   plot([-5 5], [0 0]','k')
   hold on
   plot([0 0]',[-5 5]','k')
   plot( dd.umeas(2,:), dd.vmeas(2,:),'ko')
   axis('equal')
   maguv = mstdgap(abs(dd.umeas(2,:) + i*dd.vmeas(2,:)));
   plot(maguv*cos((90-dd.heading)*pi/180), ...
        maguv*sin((90-dd.heading)*pi/180),'c.')
   axis('equal')
   
   
   subplot(235)
   plot(mstdgap(dd.umeas'),dd.depth,'r')
   title('umeas')
   ra(gca);
   
   subplot(236)
   plot(mstdgap(dd.vmeas'),dd.depth,'b')
   title('vmeas')
   ra(gca);
end
