function data = run_autovect(varargin)
% data = run_autovect(varargin)
%
% options are (originally from polar plots web page)
%    use_tsg   [0 1]              %default 0
%                                 % if it is a string, it is the path
%                                 % to a n ascii file with [dday temp]
%    numdays   [float or 'all']   %default 3
%    use_contdata  [0 1]          %default 0  (use vector data)
%    datadir   path to read and write *_xy.mat   %default ''
%    figdir    path to  output         %default ''
%    mvec_scale  [data units per inch]  % default 1
%    cruisesid  [string]          % optional
%    comment    [string]          %optional (goes into titlestr)
%    outfile  [string]            % default ''
%    rlbins     [reflayer bins]   % default is 1:2
%    shaftwidth    [linewidth]    % default is 2
%    headlength    [pixels?]      % default is 10 (matches shaftwidth 2)
%    firstdepth   [first depth]   % default is 31
%    savedata_name   [name]       % saves data structure to datadir, name.mat
%    axesfontsize [num]           % default is 14; 
%    axesfontweight [string]      % default is 'bold'; could say 'normal'
%    yearbase                    % yearbase; for titles

%
% contour data are every bin, (8m apart) 15minute averages
% vector data are:
% [firstdepth-50],[50-75], [75-125], [125-175], [175-225], [225-275]...


% the first collection of options was designed for daily plotting
% on polar ships; called by programs in /home/adcp/plots 
% eg. run_3dayplots.py

% JH 2001/04/06

cfg.use_tsg      = 0;
cfg.numdays      = 3;
cfg.use_contdata = 0;
cfg.savedata_name= '';
cfg.mvec_scale   = 0;
cfg.cruiseid     = '';
cfg.comment      = '';
cfg.datadir      = '';
cfg.figdir       = '';
cfg.outfile      = '';
cfg.rlbins       = [1 2];
cfg.shaftwidth   = 2;
cfg.headlength   = 10;
cfg.firstdepth   = 31;
cfg.axesfontsize = 14;     % or 10 or 12
cfg.axesfontweight = 'bold';  % or normal
cfg.yearbase  = [];


cfg = fillstruct(cfg, varargin, 0);


if (isemptys(cfg.yearbase))
   fprintf('run_autovect: must specify yearbase\n')
   return
end

if (isemptys(cfg.cruiseid))
   titlestr = '';
else
   titlestr = cfg.cruiseid;
end
if (~isemptys(cfg.comment))
   if (isemptys(titlestr))
      titlestr = cfg.comment;      
   else
      titlestr = sprintf('%s (%s)', titlestr, cfg.comment)
   end
end
if (~exist('titlestr','var')) | isemptys(titlestr)
   titlestr = 'velocity vectors';
end
%get temperature if appropriate ------------------------------
if isstr(cfg.use_tsg) %this is the file, try to use it
   if exist(cfg.use_tsg)
      day_temp = load(cfg.use_tsg);
      tsg_dday = day_temp(:,1);
      t1 = day_temp(:,2);
      cfg.use_tsg = 1;
   else
      cfg.use_tsg = 0;
   end
else
   try
      if ~exist('tsg.dat', 'file')
         if (cfg.use_tsg)
            fprintf('WARNING: can''t find tsg.dat\n')
         end
         cfg.use_tsg = 0;
      end
      
      if (cfg.use_tsg)
         load tsg.dat
         tsg_dday = (tsg(:,1)+ 2000) -  (tsg(1,1) + 2000) + ...
             (tsg(:,2) - 1)  + ...
             tsg(:,3)/24 + ...
             tsg(:,4)/(24*60) + ...
             tsg(:,5)/86400;
         t1 = tsg(:,7);
      end
   catch
      fprintf('WARNING: problem with tsg.dat; will not use tsg\n')
      cfg.use_tsg = 0
   end
end

% get the data -----------------------------------------------

if (cfg.use_contdata)
   data = load_adcp('cont', cfg.datadir);
else
   data = load_adcp('vect', cfg.datadir);
end

   
if (isnumeric(cfg.numdays))
   ii = find(data.dday >= (data.dday(end) - cfg.numdays));
   data.dday = data.dday(ii);
   data.x = data.x(ii);
   data.y = data.y(ii);
   data.u = data.u(:,ii);
   data.v = data.v(:,ii);
end



%strings for subtitle ----------
mostr1 = to_date(cfg.yearbase, data.dday(1),'s'); 
mostr2 = to_date(cfg.yearbase, data.dday(end),'s');
if  data.dday(end) - data.dday(1) > 10
   % use days only 
   mostr1 = mostr1(1:10);   
   mostr2 = mostr2(1:10);   
end
if data.dday(end) > 365
   mostr2 = mostr(1:end);
else
   mostr2 = mostr2(6:end);
end


%get the reference layer ----------
if (length(cfg.rlbins) > 1)
   uref = mstdgap(data.u(cfg.rlbins,:));
   vref = mstdgap(data.v(cfg.rlbins,:));
else
   uref = data.u(cfg.rlbins,:);
   vref = data.v(cfg.rlbins,:);
end
zmin  = data.z(cfg.rlbins(1));
zmax =  data.z(cfg.rlbins(end)+1);



% set up vector length
if (cfg.mvec_scale == 0)
   oki = find(~isnan(data.u+i*data.v));
   if (max(abs(data.u(oki) + i*data.v(oki))) > 1)
      cfg.mvec_scale = 2;
   elseif (max(abs(data.u(oki) + i*data.v(oki))) > .5)
      cfg.mvec_scale = 1;
   else
      cfg.mvec_scale = .5;   
   end
end


if cfg.use_tsg
   T = table1( [tsg_dday t1], data.dday);
else
   T = NaN*data.dday;
end


autovect(data.x, data.y, uref, vref, 'T', T, 'Tstep', [],...
         'shaftwidth', cfg.shaftwidth,...
         'headlength', cfg.headlength,...
         'axesfontsize', cfg.axesfontsize,...
         'axesfontweight', cfg.axesfontweight);



%finish it up ------------------------------
date_it

title(sprintf('%s (%s-%s),  [%d-%dm]',...
              titlestr, mostr1, mostr2, ...
              floor(zmin), floor(zmax)));


if ~isemptys(cfg.outfile)
   orient portrait
   %%% clean this up!! Python should be translating the figures
   filenamebase = sprintf('%s/%s', cfg.figdir, cfg.outfile)
   print(gcf, '-dpsc', filenamebase)
   %gsconvert(filenamebase, 'ps', 'gif')
   eval(sprintf('!gsconvert.py %s.ps %s.jpg\n', filenamebase, filenamebase))
end


if (~isemptys(cfg.savedata_name) & ~isemptys(cfg.datadir))
   command = sprintf('save6 %s/%s data', cfg.datadir, cfg.savedata_name)
   eval(command)

   command = sprintf('!chmod a+r %s/%s data.mat', cfg.datadir, cfg.savedata_name)
   eval(command)
end

%---------------------------------------------------------------
