function update_gbin(cfg, varargin)
% update gbin files.  Info comes from get_uhdascfg
%     cfg = get_uhdascfg(varargin); 
%     update_gbin(cfg)
%
% NOTE (can override any parameters in cfg with name value pairs)
% eg, update_gbin(cfg, 'whichones','last')
%
% varargin:
%  name               :           what              :  default
%                     :                             :
%  ---------  instrument information -------        :
%                     :                             :
% instname            :     'nb150', 'os38'         : 'os38'
%                     :      (ship-specific)        :
%                     :                             :
% instclass           :     son, os, nb, bb         :   os
% log_yearbase        :    for uhdas2001            : same as yearbase
%                     :                             :
%  --------- locations and paths -----------        :
%                     :                             :
% 'uhdas_dir'         :  path to directory with     :  '/home/adcp/cruise'
%                     :     raw/, rbin/ gbin/, etc  :
%                     :                             :
%  'rawdir'           :   holds raw and serial      :  [uhdas_dir]/raw
%                     :      acquisition dirs       :
%  'rbindir'          :   holds rbin dirs           :  [uhdas_dir]/rbin
%  'gbindir'          :   holds  dirs ('time', etc) :  [uhdas_dir]/gbin/instname
%  'logfpath'         :   dir for processing and    :  [uhdas_dir]/proc/proclog
%                     :       gbin messages         :
%  'gbinlogfile',     :   file to write info        :  mk_gbin.log
%                     :                             :
% ------------ debugging -------------------        :
%  'debug             :   drop into keyboard        :  (do not)
% ----- making gbin files ----- ("help mk_gbin")    :
%                     :                             :
% 'whichones'         :   nteger                    :   redo the last N in list
% 'whichones'         :   'none'                    :   'none' (i.e. just do
%                     :                             :       new ones)
%                     :   'all'                     :  
%                     :   'recover'                 :  for use in incremental 
%                     :                             :     mode: (redo last 2 
%                     :                             :     already done, and 
%                     :                             :     update the rest
%                     :   'last'                    :  last 2 only
%                     :                             :
%                     :   filelist                  :  (structure of *.raw
%                     :                             :   from 'dirs' (see below)
%                     :                             :
% 'redo'              :   redo gbin files 0 or 1    :     1
%                     :                             :
%                     :                             :
%  path_ext           :  gbin files to make         : {'time','best';
%                     :                             :  'time','tim'};     
%  msg_info           :  directory and rbin suffix  :
%                     :                             :{'posmv', 'gps',  'gps'; 
%                     :                             : 'gyro',   'hdg',  'hdg';  
%                     :                             : 'adu2',   'adu',  'adu';  
%                     :                             : 'posmv',  'pmv',  'pmv'};
%                     :                             :
%                     :                             : make these fields 'best'
%  best.rules         :  make gbin 'best'           : {'gps','heading',
%                     :                             :'pitch','roll','heave'};
%                     :                             :
%                     :                             : from these sources:
%  best.gps_rule      :  position rule              :  {'posmv', 'gps'}; 
%  best.heading_rule  :  heading rule               :  {'gyro',  'hdg'};
%  best.pitch_rule    :  pitch rule (not used)      :  {'posmv', 'pmv'};
%  best.roll_rule     :  roll rule (not used)       :  {'posmv', 'pmv'};
%  best.heave_rule    :  heave rule (not used)      :  {'posmv', 'pmv'};
%
% NOTE: 
%---------------------------
%
% eg.
%   
%   cfg =get_uhdascfg('cfgbase', cruiseid);
%   filelist =  dirs(fullfile(cfg.rawdir,cfg.instname,'*.raw'),'fullfile',1);
%   update_gbin(cfg,'whichones', filelist(end-5:end),'redo',1,'logfpath','.')
% %- or - 
%   update_gbin(cfg,'whichones','last','logfpath','.')
%


%JH 2004/01/10

if ~isstruct(cfg)
   help(mfilename)
   fprintf('error. first argumant not a structure:.\n')
   return
end


cfg.log_yearbase = [];
cfg = fillstruct(cfg,varargin);



if isstr(cfg.whichones)
   if strcmp(cfg.whichones, 'none')
      if cfg.redo, conjunction = 'but'; else , conjunction = 'and'; end;
      fprintf('''redo'' is %d %s ''whichones'' is ''none''.\n',...
              cfg.redo, conjunction)
      fprintf('not updating gbin.\n')
      return
   end
end


if isemptys(cfg.yearbase)
   cc=clock;
   cfg.yearbase = cc(1);
end



mkgbin = cfg.msg_info;


if (~isemptys(cfg.instclass) & ~isemptys(cfg.instname))
   if ~isemptys(cfg.whichones)
      if isstruct(cfg.whichones)
         % just use it
         filelist = cfg.whichones;
      elseif iscell(cfg.whichones)
         for iic = 1:length(cfg.whichones)
            filelist(iic).name = cfg.whichones{iic};
         end
      else
         %generate the file list, then use specified parts
         globstr = sprintf('%s/raw/%s/*.raw', cfg.uhdas_dir, cfg.instname);
         filelist = dirs(globstr,'fullfile',1);
         if isstr(cfg.whichones) %it is a string: 'last', 'all', or 'recover'
            if strcmp(cfg.whichones, 'last')
               if length(filelist) == 0
                  fprintf('no raw files found\n')
                  return
               elseif length(filelist) == 1
                  filelist = filelist(end);
               else
                  filelist = filelist(end-1:end);
               end
            elseif strcmp(cfg.whichones, 'recover')
               % figure out which ones were done, redo the last two,
               %   and update the rest.  usually this will only be the 
               %   last 2, but if there is a gap in updating, this will
               %   ensure all are caught up
               donemask = NaN*ones(size(filelist));
               for ifile = 1:length(filelist)
                  [pp,filebase,ee]=fileparts(filelist(ifile).name);
                  gbinfile = fullfile(cfg.gbindir, 'time', ...
                                      sprintf('%s.tim.gbin', filebase));
                  donemask(ifile) = exist(gbinfile);
               end
               if sum(donemask) == 0 % none done
                  lastdone = 0;
               else
                  lastdone = max(find(donemask));
               end
               dothese = max(1,lastdone-1):length(donemask);
               filelist = filelist(dothese);
               donemask = donemask(dothese);
               %print file names
               redoing = find(donemask);
               for redoi=1:length(redoing)
                  [pp,filebase,ee]=fileparts(filelist(redoing(redoi)).name);
                  fprintf('about to REDO gbin file from %s.raw\n', filebase)
               end
               doing =  find(~donemask);
               for doi = 1:length(doing)
                  [pp,filebase,ee]=fileparts(filelist(doing(doi)).name);
                  fprintf('about to do gbin file from %s.raw\n', filebase)
               end

            elseif ~strcmp(cfg.whichones, 'all')
               fprintf('"whichones" incorrectly specified. \n')
               fprintf('valid string values are:\n')
               fprintf('     ''all'', ''none'', ''last'', ''recover''\n')
               fprintf(' or use a structure from ''dirs''\n')
               return
            end
         else
            %assume integer
            filelist = filelist(end - [abs(cfg.whichones) 0]);      
         end
      end
      
      %% now we have the file list -- make the files
      
      mk_gbin(cfg.instclass, cfg.yearbase, 'filelist', filelist,...
              'msg_info', cfg.msg_info,...
              'redo',cfg.redo,...
              'rbindir', cfg.rbindir,...
              'gbindir', cfg.gbindir,...
              'rawdir',  cfg.rawdir,...
              'instname', cfg.instname,...
              'log_yearbase', cfg.log_yearbase,...
              'logfpath', cfg.logfpath,...
              'gbinlogfile', cfg.gbinlogfile,...
              'best', cfg.best, ...
              'mkgbin',cfg.msg_info,...
              'verbose', cfg.verbose,...
              'debug', cfg.debug);
   else
      fprintf('no %s files found in %s\n', cfg.instname, globstr)
   end
   
end


