function [enu, xyze] = beam_enu(instclass, vel, varargin);
% function [enu, xyze] = beam_enu(instclass, vel, H, P, R, ...);
%    or    [enu, xyze] = beam_enu(instclass, vel, HPR, ...);
%      instclass is nb, bb, os, son
%      vel can be: beam velocity from adp.m,
%                  a vector of beam coordinates
%                  a matrix, the columns of which are beam coord vectors
%      H,P,R, HPR are COLUMN vectors (Nx1 or Nx3)
%
%      Optional key/value pairs:
%
%      'beamangle'     degrees from vertical (default 30)
%      'convex'    1  (0 for concave)
%
%      'direction', 'inverse' converts from xyze to beam
%                   'normal' (default) from beam to xyze
%
%      'instrument': default is empty,
%                    use 'kmson' for Kilo Moana Sontek (overrides beamangle)
%                    use 'kmos'  for Kilo Moana OS38 (overrides beam order)
%      'orientation': 'down' (default)
%                     'up'
%      'error':  0 (default) enu omits error velocity; 1 to include it.
%
%      'tiltsensor': source of tilts 'gimballed' (for shipboard units)
%                                     'strapped' for lowered adcps
%                     irrelevant for sontek instruments


% 99/10/22 modified for orientation and angle inputs.
% 2000/09/25 modified to optionally include error velocity in enu array.
% 03/02  JH - combine sontek and rdi
% 2003/04/28 EF - change help text, add 'direction' option
% 2003/09/26 JH add tiltsensor (for rdi ladcp)


%cfg.geometry = 'convex';

cfg.convex = 1;
cfg.beamangle = 30;
cfg.instrument = [];              %set to 'kmson' if kilo moana sontek
cfg.orientation = 'down';
cfg.error = 0;                   %include error velocity
cfg.direction = 'normal';
cfg.tiltsensor = 'gimballed';

if (rem(nargin,2) == 0)
   help(mfilename)
   fprintf('Cannot parse input arguments.\n')
   enu=[]; xyze=[];
   return
end


%find where varargin starts
vstart = length(varargin) + 1;  % in case there are no key/value pairs
for ii = 1:length(varargin)
   if isstr(varargin{ii}),
      vstart = ii;
      break
   end
end


if (vstart == 2)
   hpr = varargin{1};
   Heading = hpr(:,1);
   Pitch = hpr(:,2);
   Roll = hpr(:,3);
elseif (vstart == 4)
   Heading = varargin{1};
   Pitch = varargin{2};
   Roll = varargin{3};
   [Hrows, Hcols] = size(Heading);
   if (Hrows == 1),       Heading = Heading.'; end  %should be column vector
   if length(Pitch) == 1,  Pitch = Pitch + zeros(size(Heading));   end
   if length(Roll) == 1,   Roll  = Roll  + zeros(size(Heading));   end

else
   help(mfilename)
   fprintf('Cannot parse input arguments.\n')
   enu=[]; xyze=[];
   return
end

%trim off HPR, use the rest for arguments
varargin = varargin(vstart:end);
if ~isemptys(varargin)
   cfg = fillstruct(cfg, varargin, 0);
end


if strcmp(cfg.direction, 'normal'),
   xyze = beam_xyz(instclass, vel, ...
                   'beamangle', cfg.beamangle,...
                   'instrument', cfg.instrument,...
                   'orientation', cfg.orientation,...
                   'convex', cfg.convex);

   
   if (length(size(xyze)) == 2)
      enu = xyz_enu(instclass, xyze(1:3, :), Heading, Pitch, Roll,...
                    'tiltsensor',cfg.tiltsensor);
      if cfg.error,
         enu = [enu;  xyze(4,:)];
      end
   else % Nbins x Nbeams x Nprofs
      enu = xyz_enu(instclass, xyze(:, 1:3, :), Heading, Pitch, Roll,...
                    'tiltsensor',cfg.tiltsensor);
      if cfg.error,
         enu = cat(2, enu, xyze(:,4,:));
      end
   end

else, % inverse direction
   if (length(size(vel)) == 2)
      xyze = xyz_enu(instclass, vel(1:3,:), Heading, Pitch, Roll, ...
                     'direction', 'inverse',...
                     'tiltsensor',cfg.tiltsensor);
                     
      xyze = [xyze; vel(4,:)];
   else
      xyze = xyz_enu(instclass, vel(:, 1:3, :), Heading, Pitch, Roll, ...
                     'direction', 'inverse',...
                     'tiltsensor',cfg.tiltsensor);
      
      xyze = cat(2, xyze, vel(:, 4, :));
   end

   enu = beam_xyz(instclass, xyze, ...
                'beamangle', cfg.beamangle,...
                'instrument', cfg.instrument,...
                'orientation', cfg.orientation,...
                'convex', cfg.convex, ...
                'direction', 'inverse');
   % 'enu' is really 'beam' here
end


