function [xyze, xyz_mat] = beam_xyz(instclass, vel,  varargin);
%
% [xyze, xyz_mat] = beam_xyz(instclass, vel, 'beamangle', 30, ...
%                            'convex', 1, 'direction', 'normal')
%
%      Transform velocities (RDI or Sontek ADCP) from beam to ship coords
%
%      vel can be: beam velocity from adp.m,
%                  a vector of beam coordinates
%                  a matrix, the columns of which are beam coord vectors
%
%      'beamangle'     degrees from vertical (default 30)
%      'convex'    1  (0 for concave)
%
%      'direction', 'inverse' converts from xyze to beam
%                   'normal' (default) from beam to xyze
%
%      'instrument': default is empty,
%                    use 'kmson' for Kilo Moana Sontek (overrides beamangle)
%      'orientation': 'down' (default)
%                     'up'


%  00/01/31 EF
%         modified matlab/ladp/beam_xyz.m, which was written for
%         Sontek data.
%         Based on "ADCP Coordinate Transformation", Copyright 1988 by
%         RD Instruments. pp 10-11
%
% 03/02  JH - combine sontek and rdi
% 2003/04/29 EF fixed inverse transformation matrix for upward-looking case.
% 2003/04/30 EF fixed another bug in the up-looking case.


cfg.beamangle = 30;
cfg.direction = 'normal';
cfg.orientation = 'down';
cfg.instrument = [];
cfg.convex = 1;

cfg = fillstruct(cfg, varargin, 0);


% rename some things: and do some error checking
%
if strcmp(cfg.direction, 'normal')
   xyze_to_beam = 0; % default: beam to xyze
else
   xyze_to_beam = 1; %  xyze to beam
end
%
angle_rad = cfg.beamangle*pi/180;
%
if (cfg.convex)
   convex = 1;
else
   convex = 0;
end
%
orientation_up = 0; %set default
if (strcmp(cfg.orientation, 'up'))
   orientation_up = 1;
elseif (~strcmp(cfg.orientation, 'down'))
   help(mfilename)
   error(['Invalid value of orientation: ', cfg.orientation]);
end


%% if it is an RDI instrument:
if (isa(instclass, 'nb') | isa(instclass, 'bb') | isa(instclass, 'os'))
   a = 1/(2*sin(angle_rad));
   b = 1/(4*cos(angle_rad));
   d = a/sqrt(2);

   xyz_mat = [  a, -a,  0,  0; ...
                0,  0, -a,  a; ...
                b,  b,  b,  b; ...
                d,  d, -d, -d ];

   if strcmp(cfg.instrument, 'kmos')
      newvel = zeros(size(vel));
      newvel(:,4,:) = vel(:,1,:);
      newvel(:,3,:) = vel(:,2,:);
      newvel(:,2,:) = vel(:,3,:);
      newvel(:,1,:) = vel(:,4,:);
      vel = newvel;
   end
   
   if ~convex,   % concave (shipboard NB or BB)
      xyz_mat([1, 2], :) = -xyz_mat([1, 2], :);
   end

   if orientation_up,
      xyz_mat([1 3],:) = -xyz_mat([1 3],:); % reverse x and z
   end
else
   %sontek
   % Kilo Moana

   if strcmp(cfg.instrument, 'kmson')
      xyz_mat = [ 0.9568  -0.9487         0        0;
                  0       0           0.9527   -0.9504;
                 -0.2904  -0.2970    -0.2929   -0.2948;
                  0.2904  0.2970     -0.2929   -0.2948];
   else
      cc = 1/(4*cos(angle_rad));
      ss = 1/(2*sin(angle_rad));
      xyz_mat = [  ss,  0,  -ss,   0 ; ...   %xyz_mat * [vel1    = [u v w e]
                   0 , ss,   0,   -ss ; ...   %          vel2
                   -cc, -cc, -cc, -cc; ...  %           vel3
                   cc, -cc,  cc, -cc ];     %           vel4]
   end

   if orientation_up,
      xyz_mat([2 3],:) = -xyz_mat([2 3],:); % reverse y and z
   end
end


if xyze_to_beam,
   xyz_mat = inv(xyz_mat);
end

% do the math

sv = size(vel);
ndims = length(sv);
if ndims > 3,
   error('Too many dimensions in the first input argument')
elseif ndims == 3
   xyze = zeros(sv);
   for ii = 1:sv(3)
      xyze(:,:,ii) = (xyz_mat * squeeze(vel(:,:,ii)).').';
   end
   return
else % has 2 dimensions even if it is empty
   if isemptys(vel),
      xyze = [];
      return
   end
   if sv(1) == 1, vel = vel(:); end
   if size(vel, 1) ~= 4,
      error('First dimension of vel must be 4 (for 4 beams).')
   end
   xyze = xyz_mat * vel;
end

