function [enu, RotMat] = xyz_enu(instclass, vec, varargin);
%function [enu, RotMat] = xyz_enu(instclass, vec, H, P, R, ...);
%  or     [enu, RotMat] = xyz_enu(instclass, vec, HPR, ...);
% 
% convert velocities from horizontal (ship coord) to earth coord
%
% instclass is 'nb', 'bb', 'os', 'son'
% vec must have 3 rows; the column dimension should be time, corresponding
% to the row dimension of H, P, R, or HPR.
% If there is an additional leading dimension to vec, it will be interpreted
% as cell number, as when vec is xyze(:,1:3,:)
%
% varargin key/value  pairs
%   (name)           (value)               (what)
%  'direction'      'normal'     (default)  to go from xyz to enu (default),
%                   'inverse'               to go from enu to xyz.
%
%  'tiltsensor'                             how tilts are defined
%                   'gimballed'             (default, eg. as on vmadcp)  
%                   'strapped'              as on an rdi lowered adcp        
%                                           (irrelevant for sontek)

%% 00/01/31 EF
%  Considerably modified for RDI from a version for Sontek profilers.
%  This version follows the RDI booklet, "ADCP Coordinate Transformation",
%  copyright 1998.
%%%%% Incomplete: does not provide for modified tilt; carefully check
%%%%% all H, P, R definitions for any particular application, referring
%%%%% to page 15.

% 03/01/13 JH - combine rdi and sontek
% 2003/04/28 EF - changed to varargin style, key/value for 'direction'
%% 2003/09/26 JH - added 'tiltsensor' for lowered ADCP
% 2003/12/12 JH/EF separated treatment of heading and pitch for RDI 
%                  and sontek: sontek was using hdg, now changed to 90-hdg


cfg.direction = 'normal';
cfg.tiltsensor = 'gimballed';

DegToRads = pi/180;

if (rem(nargin,2) == 0)
   help(mfilename)
   fprintf('Cannot parse input arguments.\n')
   enu=[]; RotMat=[];
   return
end


%find where varargin starts
vstart = length(varargin) + 1;  % in case there are no key/value pairs
for ii = 1:length(varargin)
   if isstr(varargin{ii}),
      vstart = ii;
      break
   end
end

if (vstart == 2)
   hpr = varargin{1};
   Heading = hpr(:,1);
   Pitch = hpr(:,2);
   Roll = hpr(:,3);
elseif (vstart == 4)
   Heading = varargin{1};
   Pitch = varargin{2};
   Roll = varargin{3};
   [Hrows, Hcols] = size(Heading);
   if (Hrows == 1),       Heading = Heading.'; end  %should be column vector
   if length(Pitch) == 1,  Pitch = Pitch + zeros(size(Heading));   end
   if length(Roll) == 1,   Roll  = Roll  + zeros(size(Heading));   end

else
   help(mfilename)
   fprintf('Cannot parse input arguments.\n')
   enu=[]; RotMat=[];
   return
end

%trim off HPR, use the rest for arguments
varargin = varargin(vstart:end);
if ~isemptys(varargin)
   cfg = fillstruct(cfg, varargin, 0);
end

enu_to_xyz = strcmp(cfg.direction, 'inverse');


if (isa(instclass, 'nb') | isa(instclass, 'bb') | isa(instclass, 'os'))
   dr =  DegToRads * Roll(:);
   dp =  DegToRads * Pitch(:);
   dh =  DegToRads * Heading(:);
elseif isa(instclass, 'son')
   dr =   DegToRads * Roll(:);
   dp = - DegToRads * Pitch(:);
   dh =   DegToRads * (90 - Heading(:));
else
   fprintf('instrument class is not defined\n')
   instclass
   enu=[];
   RotMat=[];
   return
end
   
cr = cos(dr);
sr = sin(dr);
ch = cos(dh);
sh = sin(dh);
cp = cos(dp); %gimballed default; might get overridden in RDI section
sp = sin(dp); % "


%%% error checking:
if isemptys(vec)
   vec = zeros(3,1);
end
sv = size(vec);
ndims = length(sv);
if ndims > 3
   error('The first argument must have no more than 3 dimensions.')
end

if ndims == 2 & sv(2) == 3 & sv(1) ~= 3
   vec = vec.';
   sv = size(vec);
end

if sv(ndims - 1) ~= 3
   error('One dimension of vec must be 3')
end

n = max([length(dh), length(dp), length(dr)]);
if n == 1,
   n = sv(end);
end
if ndims == 2 & sv(2) == 1 & n ~= 1,
   vec = vec(:,ones(n,1));
   sv(2) = n;
end
if sv(end) ~= n
   error('Dimension mismatch between vec and H, P, R')
end

if n > 1,
   if length(dh) == 1, dh = dh(ones(n,1)); end
   if length(dp) == 1, dp = dp(ones(n,1)); end
   if length(dr) == 1, dr = dr(ones(n,1)); end
end
if (length(dh)~= n | length(dp) ~= n |  length(dr) ~= n),
   error('Lengths of H, P, R don''t match.')
end

enu = zeros(sv);


% do the math
if (isa(instclass, 'nb') | isa(instclass, 'bb') | isa(instclass, 'os'))
   
   %% added JH 2003/09/26: for lowered ADCP with strapped tilt sensors
   if strcmp(cfg.tiltsensor, 'strapped')
      gimbalpitch = atan(tan(dp) .* cr);
      sp = sin(gimbalpitch);
      cp = cos(gimbalpitch);
   end

   
   for ii = 1:n
      MH = [ch(ii), sh(ii), 0; ...
            -sh(ii), ch(ii), 0; ...
            0, 0, 1];

      MP = [1, 0, 0; ...
            0, cp(ii), -sp(ii); ...
            0, sp(ii), cp(ii)];

      MR = [cr(ii), 0, sr(ii); ...
            0, 1, 0; ...
            -sr(ii), 0, cr(ii)];

      RM = MH * MP * MR;

      if enu_to_xyz
         RM = inv(RM);
      end
      RotMat = RM;
      
      if ndims == 3,
         enu(:,:,ii) = (RM * squeeze(vec(:,:,ii)).').';
         % dims of vec, from adp via beam_xyz, are cell, component, ping.
         % We have to transpose after squeezing so that the row dimension
         % is component; that is, each column is the 3 velocity components
         % for a given cell.
      else
         enu(:,ii) =  RM * vec(:,ii);
      end
   end %rdi

elseif isa(instclass, 'son')

   srsp = sr.*sp;
   srsp = sqrt(1.0-srsp.*srsp);
   dp = asin(sp.*cr./srsp);
   cp = cos(dp);
   sp = sin(dp);

   RotMat = zeros(3,3,n);
   RotMat(1,1,:) =  ch.*cp - sh.*sp.*sr;
   RotMat(1,2,:) =  -sh.*cr;
   RotMat(1,3,:) =  ch.*sp + sh.*cp.*sr;

   RotMat(2,1,:) =  sh.*cp + ch.*sp.*sr;
   RotMat(2,2,:) =  ch.*cr;
   RotMat(2,3,:) =  sh.*sp - ch.*cp.*sr;

   RotMat(3,1,:) =  -sp.*cr;
   RotMat(3,2,:) =  sr;
   RotMat(3,3,:) =  cp.*cr;


   for ii = 1:n
      RM = squeeze(RotMat(:,:,ii));
      if enu_to_xyz
         RM = inv(RM);
      end
      if ndims == 3,
         enu(:,:,ii) = (RM * squeeze(vec(:,:,ii)).').';
         % dims of vec, from adp via beam_xyz, are cell, component, ping.
         % We have to transpose after squeezing so that the row dimension
         % is component; that is, each column is the 3 velocity components
         % for a given cell.
      else
         enu(:,ii) =  RM * vec(:,ii);
      end
   end
else
   fprintf('instrument class is not defined\n')
   instclass
   enu=[];
   RotMat=[];
   return
end

