function  write_ensblk(instclass, filelist, params, varargin)
% function write_ensblk(instclass, rawfilelist, params, ...)
%
% Writes the command (*.cmd) and log (*.log) and binary (*.bin) files
% required by ldcodas, starting from UHDAS  logging
% This file breaks the correspondence between raw file base and 
% cmd filename.  Instead, .cmd files correspond to blocks
%
% It also writes out files with (t x y) in a format suitable
% for refabs input.  The first fix from the ensemble goes to
% *.gps1, the last to *.gps2.
%
% Required arguments:
%   instclass:              instrument class (nb, bb, os, son)
%   file: name of RAW
%   params: a structure with fields
%     params.h_align      : Heading alignment in degrees;
%                             RDI sign convention
%     params.velscale      : multiply velocities by this factor
%     params.interval     : Averaging interval in seconds (default 300)
%     params.end_dday     : Averaging interval end times (as from a 
%                             codas3 database; eg. to match pingdata times)  
%                             default empty (DISABLED).  
%                             Overrides params.interval if set.  
%     params.geometry     : 'convex' (OS) 'concave' (NB) 
%     params.yearbase     : yearbase (required for nbuh and sonuh)
%     params.ducer_depth      : add to transducer pressure;
%                             (nbuh and sonuh start with tr_pressure = 0)
%
%     params.use_refavg   : def:1 (0, use simple reference layer average)
%     params.min_pings    : Minimum number of pings for an average;
%                             only relevant at the end of file with
%                             "finish" set to 1 (see below)
%     params.pingedit     :         level of pingediting to apply
%                                   'none' (or 0)   no pingediting at all
%                                   'ampbt'         amp and bottom only
%                                   'all'  (or 1)   amp and bottom, and water
%                                   'debug'         return both answers
%                                                   'none' and 'all' as
%                                                  a.enu_noedit, a.enul_pedit
%                                                   (a.enu is a.enu_pingedit)
%     params.min_pg       : Within the reference layer, use only
%                             the depths with at least this percent good;
%                             if there are not at least 2 such levels,
%                             don't use a reference layer, just calculate
%                             a simple average.
%                             NOTE: decrease this significantly if pingediting
%     params.use_3_beam   : Not yet implemented; only 4-beam solutions
%                             are used at present
%     params.instrument   : used only to select specific irregular transformations
%                         :        (two exist: "kmson", "kmos")
%
%   >>The first four param fields are required;<<
%   the remainder have defaults.  Read the source code (pingedit, write_ensblk.m) to 
%   see what they are, or eliminate all uncertainty by specifying all
%   param fields explicitly.
%
%   Other fields will likely be added as additional screening
%   methods are implemented.
%
% Optional named arguments: (keyword, value pairs)
%   "ldcodas_path", bindir: directory to write *.cmd, *.bin
%   "pingtype",           : (default = 'first')
%                         :  options -
%                         : 'first' (same as 'second_set' == 0) only use bb pings
%                         : 'bb'    (same as 'second_set' == 0) only use bb pings
%                         : 'second' (same as 'second_set' == 1) only use 2nd set
%                         : 'nb'    use logic to test if file is nb only or 
%                         :         interleaved.  Only use nb pings
%  showverbose", [0 or 1] : def:0 (1: show raw data -- timeconsuming)
%
%        (second_set is normally 0, the default; set it to 1 only
%        if you have interleaved BB and NB pings and want to load
%        the NB part of the data set; leave it at 0 for the BB data,
%        or for any case when only BB or NB pings were collected.)
%
%  pingediting parameters come from get_pedefaults.m and are overridden as
%  elements of a substructure 'editcfg', eg:
%      params.editcfg.rl_bins = [1:40];
%
% 2000/09/22 EF
% 2001/01/25 JH adding instclass, morphing for NBADCP and gbin;
%                    get nav fields from gbin files for sonuh and nbuh
% 2004/06 JH break change correspondence (old: raw file <--> cmd file)
%                                        (new: cmd file <--> blk file)
% (also breaking the following nice touches in load_raw:
%             use with VmDAS 
%             choice of actual timeranges for ensembles

%%%% undocumented: (for testing only) 
%
% 'max_BLKprofiles'       :     number of profiles allowed per block.
%                               (make smaller to test 'newday')



global fid_cmd fid_bin
fid_cmd = 0;
fid_bin = 0;

if (nargin <= 4)
   help(mfilename)
   error('too few variables')
end

fid_genlog = fopen('write_ensblk.log','a');

%set defaults for varargin structure 
cfg.redo = 0;
cfg.ldcodas_path = './';
cfg.nav_path = './';     
cfg.att_path = './';
cfg.use_for_heading = 'heading';
cfg.pingtype = []; %defaults to 'first';
cfg.verbose = 0;
cfg.bailout_at = [];
cfg.gbindir = '../gbin';
cfg.path_ext = {'time',    'best';   
                'time',    'tim'};
p.use_3_beam = 0;    % 3-beam not yet implemented
p.min_pings = 10;
p.h_bias = 0; %hardwire here; deal with offset in h_align

cfg.max_BLKprofiles = 144;  % max number of profiles to use per block
                            % holds 12 hours of 5-minute data
                            % (still big enough for many days of data)
                            % max = 399

cfg = fillstruct(cfg, varargin, 0);
% rename parameters to match code
ldcodas_path = cfg.ldcodas_path;

%%
if nargin < 4 | (~iscell(filelist) & ~isstruct(filelist)) | ~isstruct(params),
   help write_ensblk.m
   fprintf('At least 3 arguments are required:\n')
   fprintf('an instrument class, a filename and a structure.')
   error('too few input arguments ')
end


if (~isfield(params, 'h_align') | ...
    ~isfield(params, 'velscale') | ...    
    ~isfield(params, 'interval') | ...
    ~isfield(params, 'ducer_depth') | ...
    ~isfield(params, 'yearbase'));
   help(mfilename)
   fprintf('''params'' requires more fields.  You have only: \n');
   params
   fprintf('must have: h_align, velscale, interval, yearbase, ducer_depth \n')
   fclose(fid_genlog);  fid_genlog = 0;
   return
end
if isa(instclass, 'nb') & ~isfield(params, 'beamangle')
   fprintf('NB instrument must have params.beamangle explicitly set\n')
   fprintf('exiting\n')
   fclose(fid_genlog);  fid_genlog = 0;
   return
end

if (isemptys(params.yearbase))
   r=[];
   fprintf('MUST specify yearbase\n')
   fclose(fid_genlog);  fid_genlog = 0;
   return
end

% Transfer input parameters, overriding defaults.
editcfg = get_pedefaults(instclass, cfg.pingtype);
if isfield(params, 'editcfg')
   editcfg = fillstruct(editcfg, params.editcfg, 0);
   params = rmfield(params, 'editcfg');
end
params.editcfg = editcfg;


editlog = 'editcfg.log';
fid_editlog = fopen(editlog,'a');
diary(editlog)
  fprintf('running write_ensblk at %s\n',  nowstr)
  params
  params.editcfg
diary off


%% use params to pass variables around
%% this is just conveniece for matlab.  
%%We could pass things in as variables instead...
params.instclass = instclass;
params.pingtype = cfg.pingtype;
params.path_ext = cfg.path_ext;
params.gbindir = cfg.gbindir;


%%%%%%%%%%%%%%% deal with filelist %%%%%%%%%%%%%%%%
%% make sure filelist is a cellarray; 
%% extract rawdir and collapse to rawbases
if isstruct(filelist)
   rawfilelist = cell(length(filelist),1);
   for ifile=1:length(filelist)
      rawfilelist{ifile} = filelist(ifile).name;
   end
   filelist = rawfilelist;
end
% now parse into filebase and directory
% assume everything comes from the same directory
rawdir = fileparts(filelist{1});
rawbaselist = cell(size(filelist));
for ifile = 1:length(rawbaselist)
   [junk, rawbaselist{ifile}] = fileparts(filelist{ifile});
end


%%%%%%%%%%%%%%%%  get ensemble info %%%%%%%%%%%%%
% find last blk log file (assumes ens_blk???.log)
% NOTE ensemble numbers start with 1, profile and blk begin with 0 
last_ensinfo = get_last_ensinfo(instclass, rawdir, rawbaselist )

%% profnum is comes from last_ensinfo at the outset but is controlled by 
%%     write_ensblk.m, not passed around in structures
profnum = last_ensinfo.profnum;
blknum  = last_ensinfo.blknum;
blkbase_str = sprintf('ens_blk%03d', blknum);           

if last_ensinfo.newdb
   newdb = 1;
else
   newdb = 0;
end
last_ensinfo = rmfield(last_ensinfo, 'newdb');
last_ensinfo = rmfield(last_ensinfo, 'profnum');
last_ensinfo = rmfield(last_ensinfo, 'blknum');
last_ensinfo = rmfield(last_ensinfo, 'blkbase_str');


%% get iM, index matrix, contains columns:
%% ifile         (index into rawbaselist)
%% instr_dday    (instrument dday from log file)
%% ensnum        (ensemble number, from log file)
%% ifilej        (index into file(ifile))
%%              LATER REFINEMENT: (only deal with files not yet used)

% returns rawbaselist for onlu data files with accompanying log files
[iM, iMc, rawbaselist] = get_iM(rawdir, rawbaselist, last_ensinfo.ifile);
%% get "pria"  - Ping Requested Is Available; throw out useless iM
pria =  get_pria(rawdir, rawbaselist, iM, iMc, params);
iM(find(~pria),:) = [];
if isempty(iM)
   fprintf('iM empty: no matching data found\n')
   return
end
pria(~pria) = [];
%% add second_set: pria now contains 1 (for first set), 2 (for second set)


iMc{end+1} = 'second_set';
iM = [iM pria-1];

ens_col   = find(ismember(iMc, 'ensnum'));
ifile_col = find(ismember(iMc, 'ifile'));


if newdb
   cursor = 1;
else
   cursor = find( ...
      (iM(:,ifile_col) == last_ensinfo.ifile) & ...
      (iM(:,ens_col) == last_ensinfo.lastens));
   if isempty(cursor)
      fprintf('cannot find cursor in iM\n')
      return
   end
end

fprintf(fid_genlog, '%s: getting last_ens_info\n', nowstr);
fprintf(fid_genlog, '         ensemble time range                 block      rawfilebase       lastens secs prof\n');
fprintf(fid_genlog, ' --------------------------------------      ----      -----------       -------  --- ----\n');

firstrun = 1;

%--------------------------------------------------------------------------

%% do it:
while (1)
   tic

   %--------------------------------------------------------------------------
   [ensinfo, last_ensinfo, newblock, cursor] = ...
       get_next_ensinfo(rawdir, rawbaselist, iM, iMc,  cursor, params);

      
   if isemptys(ensinfo)
      fprintf('no data remaining\n')                                        
      return                                                                
   end                                                                      

   if firstrun
      dbinfo = setup_dbinfo(rawdir, last_ensinfo, params);  
      firstrun = 0;
   end

   % test: is this a good ensemble??   
   secs_in_ens = 0;
   for segi = 1:length(ensinfo)
      secs_in_ens = secs_in_ens + ensinfo(segi).secs_read;
   end
   
   if secs_in_ens > .3*params.interval    %%% set this up top
      good_enough = 1;
   else
      good_enough = 0;
      message = 'too short';
      continue
   end
   
   if good_enough
      %% get the data here; call avg; return raw and averaged data
      [a, r] = mk_rawens(rawdir, ensinfo, params);
   end
   
   if isemptys(a)
      good_enough = 0;
      message = 'gbin failed ??';
   end
  
   if ~good_enough
      fprintf(fid_genlog, '%s: lastens = %d,  is not good enough (%s)\n',...
              ensinfo(end).rawfilebase, ...             
              ensinfo(end).lastens,...
              message);
      fprintf( '%s: lastens = %d,  is not good enough (%s)\n',...
               ensinfo(end).rawfilebase, ...             
               ensinfo(end).lastens,...
               message);
      paused(1)
      continue
   end
   
     
   %% evidently we have good data: procede
   %increment the blockfile counter and reset profile number if necessary

   fopen_if_notexist   
   
   if newblock | (profnum >  cfg.max_BLKprofiles)  |  newdb == 1;
      if newdb
         newdb = 0;
         blknum = 1
      else
         blknum = blknum + 1;   
      end
      
      % newblock
      % reset these 
      blkbase_str = sprintf('ens_blk%03d', blknum);           
      profnum = 0;
      dbinfo = setup_dbinfo(rawdir, ensinfo(end), params);  
   
      % close the old files
      fclose_if_exist
      fopen_if_notexist
      
      %% write the new block info to the cmd file
      fprintf(fid_cmd, 'binary_file: %s\n', binfile);
      fprintf(fid_cmd, 'new_block\n');
      fprintf(fid_cmd, 'dp_mask: 1\n');
      bin_float('DEPTH', round(dbinfo.depth));
      bin_double('CONFIGURATION_1', dbinfo.config1);
      %% write the new block info to the log file
      fprintf(fid_log, '% rawfilebase    last ensemble   secs(ens)   profile_num\n');
      fprintf(fid_log, '% -----------   -------------    -------    ------------\n');
      %% notify user
      fprintf( '------------   New Block\n');
  
   else  %could be jumping in here on the Nth call to write_ensblk.
      % set file names; open files if they don't exist yet.
      profnum = profnum + 1;
      fopen_if_notexist
            
   end
   
   
   %%%%%%%%%%%%%%%%%
   
   fprintf(fid_log, '%20s   %10d  %10d %5d\n', ...
           ensinfo(end).rawfilebase, ...
           ensinfo(end).lastens, ...
           secs_in_ens,...
           profnum);

   fprintf(fid_genlog, '%s to %s,  blk%03d %3s  %15s   %8d  %3d %5d\n', ...
             to_date(params.yearbase, r.corr_dday(1)), ...
             to_date(params.yearbase, r.corr_dday(end)), ...
             blknum,...
             r.config.pingtype,...
             ensinfo(end).rawfilebase, ...
             ensinfo(end).lastens, ...
             secs_in_ens,...
             profnum);
   
   fprintf(fid_editlog, '%s to %s,  blk%03d %3s  %15s   %8d  %3d %5d\n', ...
             to_date(params.yearbase, r.corr_dday(1)), ...
             to_date(params.yearbase, r.corr_dday(end)), ...
             blknum,...
             r.config.pingtype,...
             ensinfo(end).rawfilebase, ...
             ensinfo(end).lastens, ...
             secs_in_ens,...
             profnum);
   
   fprintf( '%s to %s, blk%03d %03s  %15s   %8d  %3d %5d\n', ...
            to_date(params.yearbase, r.corr_dday(1)), ...
            to_date(params.yearbase, r.corr_dday(end)), ...
            blknum,...
            r.config.pingtype,...
            ensinfo(end).rawfilebase, ...
            ensinfo(end).lastens, ...
            secs_in_ens,...
            profnum);
   %%%%%%%%%%%%%%%%%
   
   % all the structures:
   ancil1 = mk_ancil1(r, 1);
   ancil2 = mk_ancil2(r, 1);
   
   %NAVIGATION structure wants [latitude longitude]
   %but nav structure has [time longitude latitude]
   nav = ones(4,1) * 1e38;   

   fprintf(fid_cmd, ['new_profile: ', ...
                     to_date(a.config.yearbase, a.lastdday),...
                     '  /* %d %d */\n'], r.ens_num([1 end]));
   fprintf(fid_cmd, 'depth_range: %d %d\n', round(a.depth([1 end])));
   bin_double('ACCESS_VARIABLES', dbinfo.access);
   bin_double('ANCILLARY_1', ancil1);
   bin_double('ANCILLARY_2', ancil2);
   bin_double('BOTTOM_TRACK', a.bt);
   bin_double('NAVIGATION', nav);
   bin_float('U', a.enu(:, 1));
   bin_float('V', a.enu(:, 2));
   bin_float('W', a.enu(:, 3));
   bin_float('ERROR_VEL', a.enu(:, 4));
   bin_ubyte('AMP_SOUND_SCAT', round(mean(a.amp, 2)));
   bin_ubyte('RAW_AMP', a.amp);
   bin_ubyte('PERCENT_GOOD', a.pg(:, 4));
   bin_ubyte('PERCENT_3_BEAM', a.pg(:, 3));
   
   %% Maybe we should call it something else, but this will do
   %% for now; it is not correlation, either.
   if (~isfield(a, 'sw'))
      if (isfield(a, 'cor'))
         a.sw = a.cor;
      else
         a.sw = zeros(size(a.amp));
      end
   end
   bin_ubyte('SPECTRAL_WIDTH', round(mean(a.sw, 2)));
   bin_ubyte('RAW_SPECTRAL_WIDTH', a.sw);
   bin_ubyte('PROFILE_FLAGS', zeros(1, length(a.depth)));
   % .....
   
   %make sure there are values for navigation 
   %txy1 is the first good fix of the ensemble
   posi = min(find(~isnan(sum(r.nav.txy1))));
   if (~isemptys(posi))
      fprintf(fid_nav1, '%12.7f %12.7f %12.7f\n', r.nav.txy1(1, posi),...
              r.nav.txy1(2, posi), r.nav.txy1(3, posi));
   end
   %txy2 is the last good fix of the ensemble
   posi = max(find(~isnan(sum(r.nav.txy2))));
   if (~isemptys(posi))
      fprintf(fid_nav2, '%12.7f %12.7f %12.7f\n', r.nav.txy1(1, posi),...
              r.nav.txy1(2, posi), r.nav.txy1(3, posi));
   end

   elapsed_time = toc;
   fprintf('time used: %4.2fs\n', elapsed_time)
   
   
   %% for manual switching of editing paramters, 
   if ~isempty(cfg.bailout_at)
     if a.lastdday >= cfg.bailout_at
        break
     end
  end
   
  %------------------------------------------------------------------------
  
  
end % of ensembles

if exist('fid_genlog'),  if fid_genlog,  fclose(fid_genlog);  fid_genlog  = 0; end, end
if exist('fid_editlog'), if fid_editlog, fclose(fid_editlog); fid_editlog = 0; end, end
