#!/usr/bin/env python

''' this module calls matlab with everything which was put into string

    1) inside python:

    from call_matlab import MatlabEngine

    M = MatlabEngine(**kwargs)

    runstr, output = M.matlab(mcmd, **kwargs)

    where mcmd is a string with matlab command(s),
          mpath1,
          matlabpath  are *lists* of paths to add to the matlab path

      **kwargs may include:
          matlabpath = mpath, a *list* of paths to add to the matlab path
          verbose = 0 for minimal output,
                    1 for normal output (default),
                    2 for pwd and matlabpath output in addition.
          keepvars = 0 (default) causes "clear all"
                        and "close all" to be called
                        before mcmd is executed.
                     1 retains the previous matlab variables and plots.
          timeout = 1000 (default) is a timeout in seconds
                        for completing mcmd

      kwargs specified in the MatlabEngine call set defaults;
      they may be overridden for any given command string by
      specifying them in the call to M.matlab(...)

      See the source for additional keyword arguments that may be
      used in the MatlabEngine constructor.

    M.matlab(...) may be called repeatedly without restarting Matlab.

    Any Matlab error will trigger a python exception and stop
    the execution of commands in mcmd.

    Note that mcmd can be a multi-line string with a complete
    script in it, **provided** each line is a complete
    command that would yield a prompt if typed in on the
    command line.  In other words, anything that requires
    an "end" must have that "end" on the same line as the
    beginning of the construct.

    Ordinarily mpath would be used only when instantiating the
    MatlabEngine; only rarely will it be desirable to use the
    matlabpath argument to change the paths in a call to M.matlab().

    2) Or, this module can be used from the command line, or in a
    script in any language:

    call_matlab.py m_command m_path

    where m_command is a matlab command (usually an m-file,
    with the ".m" omitted; it must be in the current working
    directory), and m_path is a matlab search path specified
    as a path list delimited by ":" characters, like the
    MATLABPATH environment variable.

    The m_path argument is optional; note that the order of
    the arguments is reversed relative to the matlab python
    function arguments.

    With COM, the diary command opens the file, but
    it seems that nothing is written to it.

    It would be nice to see output as it is generated by
    the command being run instead of only at the end, but
    it appears that this is at best difficult in Linux and
    impossible in Windows.

   There is a 15-second timeout for matlab startup; if a
   prompt has not been received, a KeyboardInterrupt goes to
   the main thread of the program.

   2004/05/01 EF major rewrite to use win32com.
   The method used to get output under unix seems like
   an ugly kluge, but I don\'t know a better way to do it.
   The problem is how to know when the Matlab process has
   finished writing output from the current command.


   Interesting Gotcha:  MatlabEngine requires commands to be completed
   in a single line, so if there are 'for', 'if', or '...'
   it will fail.  To get around this, write the code to disk
   and run the program by name.  (disadvantage: no line-by-line output
   goes to the screen anymore.)
   The problem is how to know when the Matlab process has
   finished writing output from the current command.

   2004/06/24 EF more modifications--simplified the non-Win
   parts, took out the "ugly kluge", added timer using threads
   2004/07/08 EF fixed blank line bug

   NOTE: requires version 2.3.x or higher

    2006/03/26 EF Improved error handling.
    Now requires 2.4.x or higher.

'''

import os, sys, time
if not (sys.version_info[0] >= 2 and sys.version_info[1] >= 4):
    raise ImportError("Python version >= 2.4 is required for call_matlab")

from threading import Timer
from thread import interrupt_main

from select import select
import fcntl, signal
from subprocess import Popen, PIPE, STDOUT

defaultcmd = '''
format compact
pos = get(0,'defaultFigurePosition')
set(0,'defaultFigurePosition', [0 pos(2:4)])
'''

class MatlabEngine:
   def __init__(self, paths = None,
                matcmd = '/usr/local/bin/matlab -nojvm -nosplash',
                startupcmd = defaultcmd,
                verbose = 0,
                keepvars = 0,
                timeout = 1000):
      self._verbose = verbose
      self._keepvars = keepvars
      self._timeout = timeout
      self._opened = False
      if sys.platform == 'win32':
         import win32com.client
         self.mat = win32com.client.Dispatch('Matlab.Application.Single')
         self._opened = True
      else:

         self.P = Popen(matcmd, bufsize=0, stdin=PIPE, stdout=PIPE,
                            stderr=STDOUT, shell=True)
         self.fin, self.fout = self.P.stdin, self.P.stdout

         #self.fin, self.fout = os.popen4(matcmd, 'r+', 0)

         self.mat_in, self.mat_out = [f.fileno() for f in (self.fin, self.fout)]
         for f in self.mat_in, self.mat_out:
            flags = fcntl.fcntl(f, fcntl.F_GETFL)
            fcntl.fcntl(f, fcntl.F_SETFL, flags | os.O_NONBLOCK)
         self._opened = True
         try:
            s = self.run_string("", timeout = 15) # Wait for prompt
         except KeyboardInterrupt:
            output = "".join(self.out)
            self.close_pipes()
            raise LicenseManagerError(output)
         if self._verbose:
            print s
      if paths is not None:
         for path in paths:
            self.run_string('addpath %s' % path)
      self.run_multiline(startupcmd)

   def close_pipes(self):
      if self._opened:
         self.fin.close()
         self.fout.close()
         self._opened = False


   def __del__(self):
      if sys.platform == 'win32':
         pass    # 'quit' causes an error
      else:
         if self._opened:
            self.run_string('quit', timeout = 0)
            time.sleep(0.5)
         try:
             os.kill(self.P.pid, signal.SIGKILL)
             while 1:
                try: os.waitpid(self.P.pid, os.WNOHANG)
                except os.error: break
         except Exception, err:
             print err

   def run_string(self, cmd, timeout = None):
      ''' If cmd is an empty string, then we send nothing;
      just wait for a prompt.
      '''
      if timeout is None:
         timeout = self._timeout
      cmd_orig = cmd
      cmd1 = cmd.rstrip()
      # Replace single quotes with double quotes.
      c = cmd1.split("'")
      cmd1 = "''".join(c)
      # The following will trigger a python exception and show the
      # Matlab error.
      cmd2 = "disp(''MATLAB_SCRIPT_ERROR''); disp([''LASTERR: '', lasterr])"
      # Matlab 'eval' with two string arguments executes the first,
      # then the second if there was any error in the first.
      cmd = "eval('%s','%s')\n" % (cmd1, cmd2)
      #print "Command: ", cmd
      self.out = []
      if timeout > 0:
         TO = Timer(timeout, self.timeout_action)
         TO.start()
      if sys.platform == 'win32' and cmd_orig != '':
         result = self.mat.Execute(cmd)
      else:
         if cmd_orig != '':
            os.write(self.mat_in, cmd)
         if timeout == 0:      # We need this when sending "quit".
            return ''
         while 1:
            fin, fout, ferr = select((self.mat_out,), (), (), 0.1)
            if fin:
               c = os.read(self.mat_out, 1000)#self.mat_out.read()
               self.out.append(c)
               if ''.join(self.out[-3:]).endswith('>> '):
                  break
         result = ''.join(self.out)
      TO.cancel()
      return result

   def timeout_action(self):
      print "Timout in call_matlab"
      interrupt_main()

   def run_multiline(self, cmd, timeout = None):
      for line in cmd.splitlines():
         line = line.strip()
         if line != '':
            self.run_string(line, timeout)

   def matlab(self, commands, matlabpath = None,
               verbose = None, keepvars = None, timeout = None):
      if not self._opened:
         raise IOError('Connection to matlab is broken')
      if verbose is None:
         verbose = self._verbose
      if keepvars is None:
         keepvars = self._keepvars
      if timeout is None:
         timeout = self._timeout
      cmdlist = []
      if matlabpath is not None:
         cmdlist.append("addpath %s;" % " ".join(matlabpath))
      if not keepvars:
         cmdlist.append('clear all\n close all')
      cmdlist.append('cd %s' % os.getcwd())
      if verbose > 1:
         cmdlist.append('pwd\n path')
      cmdlist.append(commands)
      runstr = '\n'.join(cmdlist)

      outlist = []
      try:
         for line in runstr.splitlines():
            if line == '':
               continue
            if verbose:
               print '--> ', line
            s = self.run_string(line, timeout)
            if verbose:
               print '<-- ', s.rstrip()
            outlist.append(s)
         output = '\n'.join(outlist)
      except KeyboardInterrupt:
         self.close_pipes()
         print 'about to kill ', self.P.pid
         try:
             os.kill(self.P.pid, signal.SIGKILL)
             while 1:
                try: os.waitpid(self.P.pid, os.WNOHANG)
                except os.error: break
         except Exception, err:
             print err
         raise MatlabTimeoutError(timeout, line, '\n'.join(outlist))


      if output.find('MATLAB_SCRIPT_ERROR') != -1:
         raise MatlabError(runstr, output)

      return (runstr, output)



class MatlabError(Exception):
   def __init__(self, commands, output):
      self.commands = commands
      self.output = output
      ii = output.find('LASTERR:')
      if ii > -1:
         self.lasterr = output[ii:]
      else:
         self.lasterr = ''
      Exception.__init__(self, commands, output)

   def _indent(self, s):
      sl = s.split('\n')
      sl = [(' '*8)+l for l in sl]
      sl[0] = sl[0][4:]
      return '\n'.join(sl) + '\n'

   def __str__(self):
      '''Print the matlab string being executed, and the captured output'''
      s1 = 'Executing string:\n%s' % self.commands
      s2 = 'With output:\n%s'% self.output
      return '\n' + self._indent(s1) + self._indent(s2)

class LicenseManagerError(Exception):
   def __init__(self, output):
      self.output = output
      Exception.__init__(self, output)

class MatlabTimeoutError(MatlabError):
   def __init__(self, timeout, commands, output):
      self.timeout = timeout
      self.commands = commands
      self.output = output
      Exception.__init__(self, timeout, commands, output)

   def __str__(self):
      s0 = '\nTimeout time: %f\n' % self.timeout
      return s0 + MatlabError.__str__(self)


if __name__ == "__main__":
   if len(sys.argv) == 3:
      mcmd, mpath = sys.argv[1:3]
   elif len(sys.argv) == 2:
      mcmd = sys.argv[1]
      mpath = None
   else:
      print __doc__
      sys.exit()
   if mpath is not None:
      mpath = mpath.split(':')
   M = MatlabEngine(mpath, verbose = 1)

   try:
      runstr, output = M.matlab(mcmd)
      print "We ran:\n" + runstr
      print "Output was:\n" + output
   except MatlabTimeoutError:
      raise
   except MatlabError, err:
      print "Error running string: \n", err.commands
      print "Output was: \n", err.output
      print "Error was: \n", err.lasterr
      print "Exception prints as:\n", err
      print "Re-raising the exception"
      raise

## The except clause above illustrates various ways of
## dealing with the exception and accessing the information
## it contains.



