#! /usr/bin/env python

''' Convert postscript files to image files using ghostscript
    and the netpbm package.

    Usage: convertps.py [options] file [file...]

    Input filenames will be assumed to refer to postscript
    files or encapsulated postscript.
    (Add later: support for .ps.gz.)

    options:

        -t --type             output type: jpg, png, or gif
                                 default is png
        -a --antialias        antialias graphics; text is antialiased
                                 in any case.

        -d --directory        output directory
                                 default is current working directory
        -o --outbase          output file name base if different from
                                 input name (only works on the first
                                 file converted)

        -m --margin           pixels to add to postscript bounding box
                                 default is 6
        -p --pixels_per_inch  pixels per inch (like -r option to gs)
                                 default is 72
        -g --gs_options       string for additional options for gs

        -n --netpbm_options   string for additional options for netpbm

        -r --rotate           cw        rotate 90 degrees cw
                              ccw                         ccw
                              180              180 degrees

        -b --bounding_box     calculate bounding box instead of using
                                 the one in the file; this is useful
                                 when an image is extracted from a pdf
                                 file with Acroread, for example,
                                 because the image is centered on
                                 a full-size page in the ps file.

    The bounding box of the postscript file is used to
    calculate the size of the image file, and the amount by
    which the image must be shifted on the page to eliminate
    all but a small margin.  If a bounding box is not found
    in the file, it is calculated.  Note, however, that the
    positioning of the image in the output file may still
    end up wrong if the postscript file includes
    "initgraphics", which cancels our attempt to position
    the image.  GRADS gxps does this; use gxeps instead,
    which also makes a much more compact file.

    To make the image smaller on the screen, specify fewer than
    72 pixels per inch; to make it larger, specify more.

    If the requested output directory does not exist, it will
    be created.

    2004/03/17 EF
      Added antialiasing; text antialiasing is always on, graphics
      is turned on with the -a option.  It makes the file much bigger,
      but quite a bit nicer looking.  It also slows the conversion.

    2004/07/02 EF
      Fixed bug: now it works even if an eps file lacks a "showpage"
      command.  Also, changed so that the input postscript file is
      read only once.

    2005/10/04 EF
      Added -b option; added image interpolation to antialiasing
      option.

    2005/11/09 EF
      Added pnmquant for gif files; this needs to be improved
      so that it is only done when necessary.  We might also want
      an option of specifying a single mapfile for all plots in
      a series.

'''


import sys, os, os.path, re, string, getopt, tempfile, shutil


class convert_Error(Exception):
   pass


#The following two dictionaries are put here so we can check
# our input arguments in the script.
rotate_opts = {'cw' : '-cw', 'ccw' : '-ccw', '180': '-rotate180'}

netpbm_dict = {'gif' : 'ppmtogif',
               'png' : 'pnmtopng',
               'jpg' : 'pnmtojpeg'}


def gs(infilename, outfilename, out_format, gs_options = '', margin = 6,
       ppi = 72, antialias_graphics = False, bounding_box = False):

   gsdev_dict = {'jpg' : 'jpeg',
                'pnm' : 'pnmraw'}

   # EPS files may or may not end in a "showpage"; it could be
   # hidden by a redefinition, so we can't just search for it.
   # Fortunately, having a double "showpage" doesn't hurt, so
   # we will simply add one to every eps file (identified via
   # its extension).  (If there is no "showpage", gs will not
   # produce any output.)
   base, ext = os.path.splitext(infilename)
   ps = open(infilename).read()
   if ext[-3:] == 'eps':
      ps += '\nshowpage\n'

   bb_pat = re.compile('%%BoundingBox:\s*(\d+)\s*(\d+)\s*(\d+)\s*(\d+)')
   bbox_str = ''
   try:
      bbox_str = bb_pat.search(ps).groups()
   except:
      print '''convertps.py:
                This file lacks a bounding box; we will calculate it,
                but the postscript file may have other problems such
                an "initgraphics" statement that will wreck our
                attempt to position the image.
            '''
   if bounding_box or bbox_str == '':
      gs_cmd = 'gs -q -sDEVICE=bbox -'
      print gs_cmd
      (p_in, p_out, p_err) = os.popen3(gs_cmd)
      p_in.write(ps)
      p_in.close()
      p_out.close()
      gs_out = p_err.read()
      print gs_out
      bbox_str = bb_pat.search(gs_out).groups()

   (llx, lly, urx, ury) = map(int, bbox_str)

   width = urx - llx + 2 * margin
   height = ury - lly + 2 * margin
   shiftleft = llx - margin
   shiftdown = lly - margin

   if ppi != 72:
      sc = ppi/72.0
      width = int(width * sc)
      height = int(height * sc)

   gs_shift = '%d neg %d neg translate\n' % (shiftleft, shiftdown)
   print gs_shift
   gs_geometry = '-g%dx%d' % (width, height)
   gs_out = '-sOutputFile=' + outfilename
   gs_dev = '-sDEVICE=' + gsdev_dict[out_format]
   gs_aa = '-dTextAlphaBits=4'
   if antialias_graphics:
      gs_aa = gs_aa + ' -dGraphicsAlphaBits=4 -dDOINTERPOLATE'
   gs_cmd = string.join(('gs', '-q', gs_dev, gs_aa, gs_out,
                              '-r%d' % ppi,
                              gs_options, gs_geometry, '-'))
                              # join takes a tuple, hence double parens
   print gs_cmd
   os.popen(gs_cmd, 'w').write(gs_shift + ps)

def netpbm(infilename, outfilename, out_format, np_options = ''):

   cmd = string.join((netpbm_dict[out_format], np_options, infilename, '>',
                     outfilename))
   print cmd
   os.system(cmd)

def pnm_rotate(fn, rotate):
   if not rotate:
      return
   o = rotate_opts[rotate]
   newfn = tempfile.mktemp()
   cmd = string.join(('pnmflip', o, fn, '>', newfn))
   print cmd
   os.system(cmd)
   shutil.copyfile(newfn, fn)
   os.remove(newfn)

def pnmquant(fn, N=256):
    '''Limit the number of colors; like netpbm pnmquant program
    '''
    mapfn = tempfile.mktemp()
    newfn = tempfile.mktemp()
    cmd = string.join(('pnmcolormap', str(N), fn, '>', mapfn))
    print cmd
    os.system(cmd)
    cmd = string.join(('pnmremap', '-fs', '-mapfile', mapfn, fn, '>', newfn))
    print cmd
    os.system(cmd)
    shutil.copyfile(newfn, fn)
    os.remove(newfn)
    os.remove(mapfn)


def convert(infilename, outfilename, outformat,
            gs_options = '', np_options = '',
            margin = 6, pixels_per_inch = 72,
            rotate = None, antialias_graphics = False,
            bounding_box = False):
   tmpfilename = tempfile.mktemp()
   gs(infilename, tmpfilename, 'pnm', gs_options, margin,
      pixels_per_inch, antialias_graphics, bounding_box)
   pnm_rotate(tmpfilename, rotate)
   if outformat == 'gif':
      pnmquant(tmpfilename)
   netpbm(tmpfilename, outfilename, outformat, np_options)
   os.remove(tmpfilename)



if __name__ == '__main__':

   shortopts = "t:d:m:g:n:p:r:o:ab"
   longopts = ["type=", "directory=", "margin=",
               "gs_options=", "netpbm_options=",
               "pixels_per_inch=",
               "rotate=","outbase=", "antialias",
               "bounding_box"]
   try:
      opts, args = getopt.getopt(sys.argv[1:], shortopts, longopts)
      if not args:
         raise(getopt.GetoptError(None, None))
   except getopt.GetoptError:
      print __doc__
      sys.exit(2)

   type = 'png'
   directory = './'
   margin = 6
   gs_options = ''
   netpbm_options = ''
   pixels_per_inch = 72
   rotate = None
   outbase = ''
   antialias_graphics = False
   bounding_box = False
   for o, a in opts:
      if o in ("-t", "--type"):
         if a in netpbm_dict.keys():
            type = a
         else:
            print '\nInput argument error: "-t" arg is incorrect\n'
            print __doc__
            sys.exit(-1)

      if o in ("-d", "--directory"):
         directory = a
      if o in ("-m", "--margin"):
         margin = int(a)
      if o in ("-g", "--gs_options"):
         gs_options = a
      if o in ("-n", "--netpbm_options"):
         netpbm_options = a
      if o in ("-p", "--pixels_per_inch"):
         try:
            pixels_per_inch = int(a)
         except:
            print '\nInput argument error: "-p" arg should be an integer\n'
            print __doc__
            sys.exit(-1)
      if o in ("-r", "--rotate"):
         if a in rotate_opts.keys():
            rotate = a
         else:
            print '\nInput argument error: "-r" arg is incorrect\n'
            print __doc__
            sys.exit(-1)
      if o in ("-o", "--outbase"):
         outbase = a
      if o in ("-a", "--antialias"):
         antialias_graphics = True
      if o in ("-b", "--bounding_box"):
         bounding_box = True




   if not os.path.isdir(directory):
      print "Output directory %s does not exist yet." % directory
      os.makedirs(directory)
      print "Created output directory: %s" % directory

   filesconverted = 0
   for infilename in args:
      dir, name = os.path.split(infilename)
      if (filesconverted == 0) and (outbase != ''):
         outbase, outext = os.path.splitext(outbase)
         outfilename = os.path.join(directory, outbase + '.' + type)
      else:
         namebase, ext = os.path.splitext(name)
         outfilename = os.path.join(directory, namebase + '.' + type)
      convert(infilename, outfilename, type,
              gs_options, netpbm_options,
              margin, pixels_per_inch,
              rotate, antialias_graphics,
              bounding_box)
      filesconverted = filesconverted + 1



