#!/usr/bin/python

'''A mechanism for repeatedly running a set of tasks.

   I originally intended to use threadgroup for this, but
   rsync won't run from a python thread (at least as of
   python 2.3) because it relies on signals, and python
   blocks signals when starting a thread.

   This module uses signals to provide a timeout, so that
   a hung task will be deleted from the queue instead of
   blocking the queue.  Therefore it must be used only in
   the main thread--signals do not work in secondary threads.
'''

# rsync doesn't work in a python thread, so here is a non-threaded
# alternative to threadgroup.

# 2005/01/10 EF added force arg to run method; previously,
#               the at_exit flag was not being honored.

import sys, os, os.path, commands, time
import signal
import logging

L = logging.getLogger('repeater')


class Timeout(Exception):
   pass

def timeout_handler(signum, frame):
   raise Timeout()

class periodic:
   def __init__(self, function, args = (), kwargs = {},
                interval = 60, initial = None, at_exit = 0,
                name = "unnamed"):
      self.function = function
      self.args = args
      self.kwargs = kwargs
      self.interval = interval
      self.at_exit = at_exit
      self.name = name
      self.started = time.time()
      self.time_to_run = self.started + interval
      if initial:
         t = ((int(self.started) // 86400) * 86400) + initial
         while t < self.started + 0.2 * interval:
            t += interval
         self.time_to_run = t
      L.info('periodic: %s interval is %d seconds', self.name, self.interval)

   def run(self, force=0):
      now = time.time()
      if (now >= self.time_to_run) or force:
         self.time_to_run += self.interval
         L.info("task %s starting" % (self.name,))
         out = apply(self.function, self.args, self.kwargs)
         if type(out) is str and len(out) > 2:
            L.info("task %s output:\n  %s" % (self.name, out))
         ended = time.time()
         L.info("task %s ended" % (self.name,))
         while ended > self.time_to_run:
            self.time_to_run += self.interval

class periodic_system(periodic):
   def __init__(self, cmd, **kw):
      L.info("periodic_system: %s" % (cmd,))
      function = commands.getoutput
      args = (cmd,)
      periodic.__init__(self, function, args = args, **kw)


class repeater:
   def __init__(self, condition = None, poll_interval = 1,
                timeout = 3600, flagfile = None):
      logging.basicConfig(format=
               '%(asctime)s %(levelname)-8s %(name)-12s %(message)s')
      if condition is None:
         self.condition = lambda : 1 # loop forever
      else:
         self.condition = condition
      self.poll_interval = poll_interval
      self.timeout = timeout
      self.flagfile = flagfile
      self.tasks = []
      self.running = 1

   def add(self, task):
      self.tasks.append(task)

   def start(self):
      alarm_handler = signal.getsignal(signal.SIGALRM)
      signal.signal(signal.SIGALRM, timeout_handler)
      while self.condition() and self.running:
         ibad = None
         if self.flagfile:
            os.utime(self.flagfile, None)
         for i, task in enumerate(self.tasks):
            if self.condition() and self.running:
               signal.alarm(self.timeout)
               try:
                  task.run()
               except Timeout:
                  L.warning("Timeout: %s" % (task.name,))
                  ibad = i
               except:
                  L.exception('non-timeout exception in task %s' % (task.name,))
               signal.alarm(0)
         if ibad is not None:
            L.warning("Deleted task %s" % (self.tasks[ibad].name,))
            del(self.tasks[ibad])
            if len(self.tasks) == 0:
               L.warning("No tasks left; quitting")
               self.running = 0
         time.sleep(self.poll_interval)
      for task in self.tasks:
         if task.at_exit:
            try:
               L.info("Running task %s at exit" % (task.name))
               task.run(force=1)
            except:
               L.exception('exception in at_exit run of %s' % (task.name,))
      signal.signal(signal.SIGALRM, alarm_handler)

   def stop(self):
      '''Stop the loop.
      This might be useful only in a gui context, which might
      in turn require other changes.
      '''
      self.running = 0


#####################################################################

def main():
   '''Example for testing.

   This illustrates capturing python output and output
   from system commands, and redirecting it to a file.
   It also shows the use of a sentinal file created by
   the program, the deletion of which stops the loop.
   '''
   L = logging.getLogger()
   logging.basicConfig(format=
            '--> %(asctime)s %(name)-12s %(levelname)-8s %(message)s <--')

   L.setLevel(logging.DEBUG)
   sys.stdout = open('test.tmp', 'w')
   open("/tmp/repeater_test", 'w').close() # make a flag file
   def file_present():
      return os.path.exists("/tmp/repeater_test")
   def printer(s):
      print s, time.time()
   L.info('Starting the repeater example.')
   R = repeater(condition = file_present, poll_interval = 0.2, timeout = 5)
   #R.add(periodic_system("ls -ltr | tail -2 >> test2.tmp", interval = 2))
   R.add(periodic_system("ls -ltr | tail -2", interval = 2, name="lister"))
   R.add(periodic(printer, args = ("The time is",), interval=10,
                                          initial=10, at_exit=1,
                                          name = "time printer"))
   R.add(periodic_system("sleep 6", interval = 10, name = "sleeper"))
   R.start()
   sys.stdout.close()

if __name__ == "__main__":
   main()


