#!/usr/bin/env python
'''Record or restore symbolic links in a directory tree.

The motivation is that mercurial, as of version 0.7, does
not handle symbolic links, but the CODAS3 tree uses symlinks
extensively.
'''

usage = '''
       symlinker.py command [options]
   or: symlinker.py [options] command

           command is "save" to record links in a file, or
                      "restore" to make links from the file contents.

    Note: the stored paths will be relative to the location of the
    file in which the links are written; if you move that file,
    you will not be able to restore the links.

    Example:
        cd /home/currents/programs/codas3
        symlinker.py save

    results in all symlinks in the codas3 tree being recorded in
    a file /home/currents/programs/codas3/.linklist

        cd /home/currents/programs/codas3
        symlinker.py restore

    results in all the symlinks in that file being restored.

    On a system that does not support symlinks, the file will
    be copied instead.
'''
import sys, os, os.path, shutil

class Symlink:
    '''Facilitates storing information about a symbolic link,
    or restoring a link from such stored information.
    '''
    def __init__(self, path=None, text=None):
        '''Initialize with either a symlink path or the
        line of text describing a symlink provided by this class.
        '''
        self.path = path
        if path is not None:
            self.evaluate()
        elif text is not None:
            self.parse(text)
        else:
            raise Exception, "Either path or text must be specified."

    def __str__(self):
        '''Return a text string suitable for saving the symlink info
        in a file, and for human reading/editing.
        '''
        return "%6s %s %s  -->  %s" % (self.status,
                                          self.rel_abs,
                                          self.path,
                                          self.target)

    def evaluate(self):
        '''Read an existing symlink, and store it with ancillary information.
        '''
        self.target = os.readlink(self.path)
        if os.path.exists(self.path):
            self.status = "GOOD"
        else:
            self.status = "BROKEN"
        if os.path.isabs(self.target):
            self.rel_abs = "ABS"
        else:
            self.rel_abs = "REL"

    def parse(self, text):
        '''Extract and store symlink information from a line of text.
        Typically this will be used when reading a line written using
        the Symlink.__str__() method.
        '''
        fields = text.split()
        if len(fields) != 5:
            return False
        (status, rel_abs, path, arrow, target) = fields
        if status not in ("GOOD", "BROKEN"):
            return False
        if rel_abs not in ("REL", "ABS"):
            return False
        (self.status, self.rel_abs, self.path, self.target) = (status,
                                                  rel_abs, path, target)
        return True

    def write_link(self):
        '''Make the symbolic link in the filesystem.
        First delete an existing link of the same name if it is
        not identical to the desired link.
        '''
        if self.path is None:
            return
        if os.path.islink(self.path):
            if os.readlink(self.path) == self.target:
                return
            else:
                os.remove(self.path)
        os.symlink(self.target, self.path)
        self.evaluate()

    def copy_file(self):
        '''Copy the file instead of writing the symlink.
        This is intended for systems lacking symlinks.
        '''
        if self.path is None:
            return
        try:
            shutil.copy2(self.target, self.path)
        except IOError:
            pass

    if hasattr(os, 'symlink'):
        link_or_copy = write_link
    else:
        link_or_copy = copy_file

def relative_path(path, origin):
    '''
    Return path transformed so it is relative to the origin.

    Examples:
    >>> from symlinker import relative_path
    >>> relative_path('/home/adcp/config', '/home')
    'adcp/config'
    >>> relative_path('/home', '/home/adcp/config')
    '../../'

    This is UNIX-specific.
    '''
    pa = os.path.abspath(path).split('/')
    oa = os.path.abspath(origin).split('/')
    n_common = 0
    for index in range(min(len(pa), len(oa))):
        if pa[index] == oa[index]:
            n_common += 1
        else:
            break
    del(pa[:n_common])
    del(oa[:n_common])
    s = (len(oa) * '../') + '/'.join(pa)
    if len(s) == 0:
        s = './'
    return s


def modify_linkfilepath(basedir, linkfilepath):
    '''If linkfilepath is a filename only, with no path, then
    append it to the basedir; otherwise leave it as-is.
    '''
    lfp, lfn = os.path.split(linkfilepath)
    if lfp == '':
        linkfilepath = os.path.join(basedir, lfn)
    return linkfilepath

def record_links(basedir, linkfilepath, warn=True, skip_hg=False):
    '''Traverse basedir, writing a line to linkfilepath for
    each symlink found.  If warn is True, write the line to
    stdout (as well as the linkfilepath) if the link is broken.
    '''
    linkfilepath = modify_linkfilepath(basedir, linkfilepath)
    linkfile = open(linkfilepath, 'w')
    cwd = os.getcwd()
    linkfiledir = os.path.dirname(linkfilepath)
    relbasedir = relative_path(basedir, linkfiledir)
    os.chdir(linkfiledir)
    try:
        for root, dirs, files in os.walk(relbasedir):
            for f in files+dirs:
                pf = os.path.join(root, f)
                if os.path.islink(pf):
                    sl = Symlink(path = pf)
                    if warn and sl.status == "BROKEN":
                        print sl
                    linkfile.write(str(sl) + '\n')
            if skip_hg:
                for i in range(len(dirs)-1, -1, -1):
                    if '.hg' in os.listdir(os.path.join(root,dirs[i])):
                        del(dirs[i])
    finally:
        os.chdir(cwd)
        linkfile.close()


def restore_links(linkfilepath, warn=True):
    cwd = os.getcwd()
    linklines = open(linkfilepath).readlines()
    links = []
    for line in linklines:
        sl = Symlink(text=line)
        if sl.path is not None:
            links.append(sl)
    os.chdir(os.path.dirname(linkfilepath))
    try:
        for sl in links:
            #sl.write_link()
            sl.link_or_copy()
            if warn and sl.status == "BROKEN":
                print sl
    finally:
        os.chdir(cwd)


from optparse import OptionParser

class OP(OptionParser):
    '''Modified to print a full help message in case of parse error.
    '''
    def error(self, msg):
        self.print_help()
        print msg
        sys.exit()

lf_help = '''filename (with or without path) of file
in which links are listed; default is '.linklist'.
If no path is included, the file will be in BASEDIR;
to put the file in the current working directory,
specify, e.g., "./linklist"'''

def main():
    parser = OP(usage=usage)
    parser.add_option('-d', '--directory', dest='basedir',
                      default='./',
                      help="trunk of directory tree; default is ./")
    parser.add_option('-l', '--linkfile', dest='linkfile',
                      default='.linklist',
                      help=lf_help)
    parser.add_option('--nowarn', action='store_false', dest='warn',
                      default=True,
                      help="do not warn of broken links")
    parser.add_option('--skip_hg', action='store_true', dest='skip_hg',
                      default=False,
                      help="skip any subdirectory containing .hg")


    options, args = parser.parse_args()

    badargs = [arg for arg in args if arg not in ("save", "restore")]
    if len(badargs) > 0:
        parser.error("Unrecognized command: %s" % ' '.join(badargs))
    if len(args) > 1:
        parser.error("Too many commands given; only one is allowed.")
    if len(args) == 0:
        parser.error("")

    if args[0] == "save":
        record_links(options.basedir, options.linkfile, options.warn,
                        options.skip_hg)

    if args[0] == "restore":
        lfp = modify_linkfilepath(options.basedir, options.linkfile)
        restore_links(lfp, options.warn)

if __name__ == '__main__':
    main()


