function gps = upktiogagps( fname )
%
% gps = upktiogagps( fname )
%
% Returns the structure 'gps' with fields
%
%    yd        yearday                (0.5 = 1200, 1 Jan)
%    date      yy/mm/dd               (string)
%    time      hh:mm:ss  (gmt)        (string)
%    lat       latitude               (deg)
%    lon       longitude              (deg)
%    gdate     gps date  (ddmmyy)     (integer)
%    ut        gps universal time     ( ? )
%    cog       course over ground     (deg)
%    sog       speed over ground      (cm/s ? )
%    gyro      gyro output            (deg ? )
%    hrh       heading                (degT ? )



gps.yd = [];                                % reserve a slot for yd

[gps.date,gps.time,gps.lat,gps.lon,gps.gdate, ...
 gps.ut,gps.cog,gps.sog,gps.gyro,gps.hrh] =  ...
textread(fname, ...
       '%s%s%*f%*f%*f%f%f%*d%d%d%f%f%f%f%*[^\n]', ...
       'headerlines',1,'delimiter',',');


gps.date = char( gps.date );                % convert to char array
gps.time = char( gps.time );

yy = str2int( gps.date(:,1:4)  );           % convert to integers
mo = str2int( gps.date(:,6:7)  );
dd = str2int( gps.date(:,9:10) );

hh = str2int( gps.time(:,1:2) );
mm = str2int( gps.time(:,4:5) );
ss = str2int( gps.time(:,7:8) );

mmss = mm + ss/60;                          % decimal minutes

gps.yd = date2yd( yy, mo, dd, hh, mmss );   % compute yearday



% __________________________ date2yd ____________________________

function Yd = date2yd( yy, mo, dd, hh, mi )
%
% Yd = date2yd( yy, mo, dd, hh, mi )  or  Yd = date2yd( [yy,mo,dd,hh,mi] )
%
% Returns the fractional yearday given the month, day, year and (optionally)
% hour and minutes.  The date & time information may be passed as separate
% vectors, or as a single array.  If the arguments are passed separately,
% and all dates are in a single year, yy may be a scalar.  If mi is absent,
% we assume hh is the decimal hour.  If hours (hh) also are not specified,
% an integer yearday is returned.
%
% For example:  date2yd( 1994, 1, 1, 6 ) = 0.25
%     whereas:  date2yd( 1994, 1, 1 )    = 1
%
% See also yd2date                                          [3.5, 4.0, 5.1]

% Speed: 2 x datenum, 120 x non-vectorized   N.Galbraith & E.Terray 3/94, 1/98

Hours_Flag = 0;

if( nargin == 1 )                                % if a single array is passed
    if( length(yy(1,:)) > 3 )                    %    get the hours, if
        hh = yy(:,4);                            %    specified, and set the
        Hours_Flag = 1;                          %    hours flag to true
        if( length(yy(1,:)) > 4 )                %    if minutes explicitly
            hh = hh + yy(:,5)/60;                %    given, add them to hours
        end
    end
    mo = yy(:,2);                                %    then extract the other
    dd = yy(:,3);                                %    date & time parameters
    yy = yy(:,1);                                %    note: must do yy last!!
else
   if(any(mo < 1 | mo > 12)),  error('months < 1 or > 12' ); end
   if(any(dd < 1 | dd > 31)),  error('days < 1 or > 31'   ); end
   % if(any(hh < 0 | hh > 24)),  error('hours < 0 or > 24'  ); end
   % if(any(mi < 0 | mi > 60)),  error('minutes < 0 or > 60'); end
end

if( nargin > 3 ),  Hours_Flag = 1;   end         % if hours are given, set flag
if( nargin > 4 ),  hh = hh + mi/60;  end         % if mi given, add to hours

[nr,nc] = size( mo );
if( length(yy) == 1 )                            % make sure yy has the same
    yy = yy(ones(nr,nc));                        %   length as mo, dd etc.
end
                                       % now do the conversion to yearday

mdays = [0 31 28 31 30 31 30 31 31 30 31 30];    % days in months Jan thru Nov
mdays = cumsum( mdays ) - 1;                     % cumulative # days, less 1
if( nr > nc ),  mdays = mdays(:);  end           % force row or column form

isleap = ( (rem(yy,4) == 0) & (mo > 2) );        % 1 if a leap year, else 0

if( Hours_Flag )                                 % if hours are specified
    Yd = mdays(mo) + dd + hh/24 + isleap;        %    use this form
else                                             % else if no hours
    mdays = mdays + 1;                           %    bump the days by 1
    Yd = mdays(mo) + dd + isleap;                %    so that Jan 1 is Day 1
end

